/* Generated by Cython 3.0.12 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "sQQideR.DatabaseContext",
        "sources": [
            "sQQideR/DatabaseContext.pyx"
        ]
    },
    "module_name": "sQQideR.DatabaseContext"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_12" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x03000CF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__;  (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__sQQideR__DatabaseContext
#define __PYX_HAVE_API__sQQideR__DatabaseContext
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "sQQideR\\\\DatabaseContext.pyx",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords;
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr;
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql;
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl;
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr;

/* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords {
  PyObject_HEAD
  PyObject *__pyx_v_pattern;
};


/* "sQQideR/DatabaseContext.pyx":497
 *                     match_score = 3
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 */
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *__pyx_outer_scope;
  PyObject *__pyx_genexpr_arg_0;
  PyObject *__pyx_v_k;
};


/* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql {
  PyObject_HEAD
  PyObject *__pyx_v_used_aliases;
};


/* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl {
  PyObject_HEAD
  PyObject *__pyx_v_natural_language_query;
};


/* "sQQideR/DatabaseContext.pyx":758
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):             # <<<<<<<<<<<<<<
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)
 * 
 */
struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr {
  PyObject_HEAD
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *__pyx_outer_scope;
  PyObject *__pyx_v_word;
};

/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif PY_MAJOR_VERSION < 3
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyString_CheckExact(s)) ? PyUnicode_FromEncodedObject(s, NULL, "strict") :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right)
    #endif
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_ConcatInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_ConcatInPlaceSafe(left, right) ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_ConcatInPlace(left, right))

/* SliceTupleAndList.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
#else
#define __Pyx_PyList_GetSlice(seq, start, stop)   PySequence_GetSlice(seq, start, stop)
#define __Pyx_PyTuple_GetSlice(seq, start, stop)  PySequence_GetSlice(seq, start, stop)
#endif

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* PyIntCompare.proto */
static CYTHON_INLINE int __Pyx_PyInt_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* py_dict_keys.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* RaiseUnboundLocalError.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* RaiseClosureNameError.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* py_dict_items.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* pyfrozenset_new.proto */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* PySetContains.proto */
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);
#endif

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchInspect.proto */
static PyObject* __Pyx_patch_inspect(PyObject* module);

/* PatchAsyncIO.proto */
static PyObject* __Pyx_patch_asyncio(PyObject* module);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CoroutineBase.proto */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */

/* Module declarations from "sQQideR.DatabaseContext" */
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "sQQideR.DatabaseContext"
extern int __pyx_module_is_main_sQQideR__DatabaseContext;
int __pyx_module_is_main_sQQideR__DatabaseContext = 0;

/* Implementation of "sQQideR.DatabaseContext" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_print;
static PyObject *__pyx_builtin_open;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_sorted;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_max;
/* #### Code section: string_decls ### */
static const char __pyx_k_G[] = "G";
static const char __pyx_k_b[] = "\\b";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_e[] = "e";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_j[] = "j";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_ON[] = " ON ";
static const char __pyx_k__2[] = ", ";
static const char __pyx_k__5[] = ")";
static const char __pyx_k__7[] = "\"\n                ";
static const char __pyx_k__8[] = ": ";
static const char __pyx_k__9[] = "\n";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_nx[] = "nx";
static const char __pyx_k_os[] = "os";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_AND[] = " AND ";
static const char __pyx_k_Any[] = "Any";
static const char __pyx_k__10[] = "```";
static const char __pyx_k__12[] = "";
static const char __pyx_k__15[] = ".";
static const char __pyx_k__20[] = ":";
static const char __pyx_k__23[] = "'";
static const char __pyx_k__27[] = " ";
static const char __pyx_k__28[] = " = ?";
static const char __pyx_k__29[] = ".*";
static const char __pyx_k__31[] = "?";
static const char __pyx_k__35[] = "\n\n";
static const char __pyx_k__36[] = "### ";
static const char __pyx_k__37[] = "| ------ | --------- | ----------- |\n";
static const char __pyx_k__38[] = "| ";
static const char __pyx_k__39[] = " | ";
static const char __pyx_k__40[] = " |\n";
static const char __pyx_k__41[] = "* **";
static const char __pyx_k__42[] = "** \342\206\222 **";
static const char __pyx_k__43[] = "**\n";
static const char __pyx_k__44[] = "`\n";
static const char __pyx_k__45[] = "* ";
static const char __pyx_k__48[] = "*";
static const char __pyx_k__55[] = "_";
static const char __pyx_k_day[] = "day";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_max[] = "max";
static const char __pyx_k_msg[] = "msg";
static const char __pyx_k_rel[] = "rel";
static const char __pyx_k_run[] = "run";
static const char __pyx_k_sql[] = "sql";
static const char __pyx_k_src[] = "src";
static const char __pyx_k_str[] = "str";
static const char __pyx_k_sub[] = "sub";
static const char __pyx_k_tbl[] = "tbl";
static const char __pyx_k_Dict[] = "Dict";
static const char __pyx_k_FROM[] = "FROM ";
static const char __pyx_k_JOIN[] = "JOIN ";
static const char __pyx_k_List[] = "List";
static const char __pyx_k_None[] = "None";
static const char __pyx_k_User[] = "User";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_bool[] = "bool";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_date[] = "date";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_find[] = "find";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_json[] = "json";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_load[] = "load";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_node[] = "node";
static const char __pyx_k_open[] = "open";
static const char __pyx_k_path[] = "path";
static const char __pyx_k_prev[] = "prev";
static const char __pyx_k_role[] = "role";
static const char __pyx_k_rule[] = "rule:";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_tags[] = "tags";
static const char __pyx_k_term[] = "term:";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_word[] = "word";
static const char __pyx_k_Found[] = "  - Found ";
static const char __pyx_k_Tuple[] = "Tuple";
static const char __pyx_k_Union[] = "Union";
static const char __pyx_k_WHERE[] = "WHERE ";
static const char __pyx_k_alias[] = "alias";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_edges[] = "edges";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_graph[] = "graph";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_loads[] = "loads";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_nodes[] = "nodes";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_query[] = "query";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_rules[] = "rules";
static const char __pyx_k_score[] = "score";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_table[] = "table:";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_Models[] = "Models";
static const char __pyx_k_SELECT[] = "SELECT ";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_column[] = "column:";
static const char __pyx_k_domain[] = "domain";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_escape[] = "escape";
static const char __pyx_k_exists[] = "exists";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_output[] = "output";
static const char __pyx_k_parsed[] = "parsed";
static const char __pyx_k_prompt[] = "prompt";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_rule_2[] = "rule";
static const char __pyx_k_schema[] = "schema";
static const char __pyx_k_search[] = "search";
static const char __pyx_k_simple[] = "simple";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_source[] = "source";
static const char __pyx_k_suffix[] = "suffix";
static const char __pyx_k_tables[] = "tables";
static const char __pyx_k_target[] = "target";
static const char __pyx_k_term_2[] = "term";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_DiGraph[] = "DiGraph";
static const char __pyx_k_ai_base[] = "ai_base";
static const char __pyx_k_aliases[] = "aliases";
static const char __pyx_k_asyncio[] = "asyncio";
static const char __pyx_k_columns[] = "columns";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_concept[] = "concept:";
static const char __pyx_k_content[] = "content";
static const char __pyx_k_context[] = "context";
static const char __pyx_k_defines[] = "defines";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_edges_2[] = " edges";
static const char __pyx_k_example[] = "example:";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_indexed[] = "indexed";
static const char __pyx_k_inspect[] = "inspect";
static const char __pyx_k_is_user[] = "is_user";
static const char __pyx_k_keyword[] = "keyword:";
static const char __pyx_k_matched[] = "matched";
static const char __pyx_k_matches[] = "matches";
static const char __pyx_k_missing[] = "missing";
static const char __pyx_k_node_id[] = "node_id";
static const char __pyx_k_os_path[] = "os.path";
static const char __pyx_k_pattern[] = "pattern";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_replace[] = "replace";
static const char __pyx_k_results[] = "results";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_rule_id[] = "rule_id";
static const char __pyx_k_service[] = "service";
static const char __pyx_k_table_2[] = "table";
static const char __pyx_k_targets[] = "targets";
static const char __pyx_k_term_id[] = "term_id";
static const char __pyx_k_unknown[] = "unknown";
static const char __pyx_k_verbose[] = "verbose";
static const char __pyx_k_version[] = " (version ";
static const char __pyx_k_List_str[] = "List[str]";
static const char __pyx_k_Optional[] = "Optional";
static const char __pyx_k_add_edge[] = "add_edge";
static const char __pyx_k_add_node[] = "add_node";
static const char __pyx_k_col_data[] = "col_data";
static const char __pyx_k_col_name[] = "col_name";
static const char __pyx_k_column_2[] = "column";
static const char __pyx_k_concepts[] = "concepts";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_encoding[] = "encoding";
static const char __pyx_k_end_node[] = "end_node";
static const char __pyx_k_endswith[] = "endswith";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_has_node[] = "has_node";
static const char __pyx_k_keywords[] = "keywords";
static const char __pyx_k_markdown[] = "markdown";
static const char __pyx_k_metadata[] = "metadata";
static const char __pyx_k_neighbor[] = "neighbor";
static const char __pyx_k_networkx[] = "networkx";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_response[] = "response";
static const char __pyx_k_set_name[] = "__set_name__";
static const char __pyx_k_src_name[] = "src_name";
static const char __pyx_k_table_id[] = "table_id";
static const char __pyx_k_tgt_name[] = "tgt_name";
static const char __pyx_k_to_table[] = "to_table";
static const char __pyx_k_Assistant[] = "Assistant";
static const char __pyx_k_SQLPrompt[] = "SQLPrompt";
static const char __pyx_k_all_rules[] = "all_rules";
static const char __pyx_k_concept_2[] = "concept";
static const char __pyx_k_data_type[] = "data_type";
static const char __pyx_k_edge_data[] = "edge_data";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_example_2[] = "example";
static const char __pyx_k_file_path[] = "file_path";
static const char __pyx_k_find_path[] = "find_path";
static const char __pyx_k_get_alias[] = "get_alias";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_keyword_2[] = "keyword";
static const char __pyx_k_md_output[] = "md_output";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_neighbors[] = "neighbors";
static const char __pyx_k_node_data[] = "node_data";
static const char __pyx_k_nodes_and[] = " nodes and ";
static const char __pyx_k_row_count[] = "row_count";
static const char __pyx_k_rule_data[] = "rule_data";
static const char __pyx_k_rule_desc[] = "rule_desc";
static const char __pyx_k_rule_node[] = "rule_node";
static const char __pyx_k_sql_lines[] = "sql_lines";
static const char __pyx_k_version_2[] = "version";
static const char __pyx_k_IGNORECASE[] = "IGNORECASE";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_ai_context[] = "ai_context";
static const char __pyx_k_all_tables[] = "all_tables";
static const char __pyx_k_cleaned_at[] = "cleaned_at";
static const char __pyx_k_complexity[] = "complexity";
static const char __pyx_k_concept_id[] = "concept_id";
static const char __pyx_k_context_md[] = "context_md";
static const char __pyx_k_definition[] = "definition";
static const char __pyx_k_example_id[] = "example_id";
static const char __pyx_k_from_table[] = "from_table";
static const char __pyx_k_has_column[] = "has_column";
static const char __pyx_k_implements[] = "implements";
static const char __pyx_k_join_found[] = "join_found";
static const char __pyx_k_keyword_id[] = "keyword_id";
static const char __pyx_k_node_props[] = "node_props";
static const char __pyx_k_node_rules[] = "node_rules";
static const char __pyx_k_node_score[] = "node_score";
static const char __pyx_k_nx_DiGraph[] = "nx.DiGraph";
static const char __pyx_k_prev_table[] = "prev_table";
static const char __pyx_k_query_text[] = "query_text";
static const char __pyx_k_references[] = "references";
static const char __pyx_k_related_to[] = "related_to";
static const char __pyx_k_start_node[] = "start_node";
static const char __pyx_k_startswith[] = "startswith";
static const char __pyx_k_statistics[] = "statistics";
static const char __pyx_k_table_data[] = "table_data";
static const char __pyx_k_table_name[] = "table_name";
static const char __pyx_k_tables_and[] = " tables and ";
static const char __pyx_k_ChatMessage[] = "ChatMessage";
static const char __pyx_k_Description[] = "  * Description: ";
static const char __pyx_k_ID_contains[] = "ID contains '";
static const char __pyx_k_ai_enhanced[] = "ai_enhanced";
static const char __pyx_k_build_graph[] = "build_graph";
static const char __pyx_k_col_node_id[] = "col_node_id";
static const char __pyx_k_d_4_d_2_d_2[] = "'(\\d{4}-\\d{2}-\\d{2})'";
static const char __pyx_k_description[] = "description";
static const char __pyx_k_first_table[] = "first_table";
static const char __pyx_k_from_tables[] = "from_tables";
static const char __pyx_k_governed_by[] = "governed_by";
static const char __pyx_k_match_score[] = "match_score";
static const char __pyx_k_node_scores[] = "node_scores";
static const char __pyx_k_output_data[] = "output_data";
static const char __pyx_k_table_count[] = "table_count";
static const char __pyx_k_table_names[] = "table_names";
static const char __pyx_k_table_nodes[] = "table_nodes";
static const char __pyx_k_terminology[] = "terminology";
static const char __pyx_k_to_table_id[] = "to_table_id";
static const char __pyx_k_user_prompt[] = "user_prompt";
static const char __pyx_k_Raw_response[] = "Raw response: ";
static const char __pyx_k_best_example[] = "best_example";
static const char __pyx_k_chat_history[] = "chat_history";
static const char __pyx_k_concept_data[] = "concept_data";
static const char __pyx_k_example_data[] = "example_data";
static const char __pyx_k_example_node[] = "example_node";
static const char __pyx_k_generate_sql[] = "generate_sql";
static const char __pyx_k_get_response[] = "get_response";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_join_clauses[] = "join_clauses";
static const char __pyx_k_match_reason[] = "match_reason";
static const char __pyx_k_node_matches[] = "node_matches";
static const char __pyx_k_previous_sql[] = "previous_sql";
static const char __pyx_k_relationship[] = "relationship";
static const char __pyx_k_sorted_nodes[] = "sorted_nodes";
static const char __pyx_k_unique_rules[] = "unique_rules";
static const char __pyx_k_used_aliases[] = "used_aliases";
static const char __pyx_k_Generated_SQL[] = "\nGenerated SQL: ";
static const char __pyx_k_Name_contains[] = "Name contains '";
static const char __pyx_k_adapted_query[] = "adapted_query";
static const char __pyx_k_asyncio_tasks[] = "asyncio.tasks";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_current_table[] = "current_table";
static const char __pyx_k_example_nodes[] = "example_nodes";
static const char __pyx_k_first_newline[] = "first_newline";
static const char __pyx_k_from_table_id[] = "from_table_id";
static const char __pyx_k_get_edge_data[] = "get_edge_data";
static const char __pyx_k_init_subclass[] = "__init_subclass__";
static const char __pyx_k_load_metadata[] = "load_metadata";
static const char __pyx_k_metadata_file[] = "metadata_file";
static const char __pyx_k_output_format[] = "output_format";
static const char __pyx_k_process_query[] = "process_query";
static const char __pyx_k_relationships[] = "relationships";
static const char __pyx_k_s_LIMIT_s_d_s[] = "\\s+LIMIT\\s+\\d+\\s*;?";
static const char __pyx_k_select_clause[] = "select_clause";
static const char __pyx_k_shortest_path[] = "shortest_path";
static const char __pyx_k_where_clauses[] = "where_clauses";
static const char __pyx_k_Business_Rules[] = "## Business Rules\n\n";
static const char __pyx_k_Join_condition[] = "  * Join condition: `";
static const char __pyx_k_NetworkXNoPath[] = "NetworkXNoPath";
static const char __pyx_k_best_guess_sql[] = "best_guess_sql";
static const char __pyx_k_business_rules[] = "business_rules";
static const char __pyx_k_clean_response[] = "clean_response";
static const char __pyx_k_context_prompt[] = "context_prompt";
static const char __pyx_k_domain_results[] = "domain_results";
static const char __pyx_k_enhanced_query[] = "enhanced_query";
static const char __pyx_k_example_scores[] = "example_scores";
static const char __pyx_k_filtered_rules[] = "filtered_rules";
static const char __pyx_k_join_condition[] = "join_condition";
static const char __pyx_k_query_examples[] = "query_examples";
static const char __pyx_k_selected_query[] = "selected_query";
static const char __pyx_k_template_query[] = "template_query";
static const char __pyx_k_JSONDecodeError[] = "JSONDecodeError";
static const char __pyx_k_Markdown_Output[] = "\nMarkdown Output:";
static const char __pyx_k_Relevant_Tables[] = "## Relevant Tables\n\n";
static const char __pyx_k_condition_nodes[] = "condition_nodes";
static const char __pyx_k_database_schema[] = "database_schema";
static const char __pyx_k_example_queries[] = "example_queries";
static const char __pyx_k_generate_output[] = "generate_output";
static const char __pyx_k_history_context[] = "history_context";
static const char __pyx_k_relationships_2[] = " relationships";
static const char __pyx_k_Business_Domains[] = "**Business Domains**: ";
static const char __pyx_k_Cleaned_response[] = "Cleaned response: ";
static const char __pyx_k_Graph_built_with[] = "Graph built with ";
static const char __pyx_k_List_ChatMessage[] = "List[ChatMessage]";
static const char __pyx_k_No_SQL_generated[] = "No SQL generated";
static const char __pyx_k_Processing_query[] = "\nProcessing query: ";
static const char __pyx_k_Relevant_Columns[] = "## Relevant Columns\n\n";
static const char __pyx_k_buisness_domains[] = "buisness_domains";
static const char __pyx_k_business_domains[] = "business_domains";
static const char __pyx_k_cleaning_version[] = "cleaning_version";
static const char __pyx_k_database_objects[] = "database_objects";
static const char __pyx_k_AI_Enhanced_Query[] = "## AI Enhanced Query\n\n";
static const char __pyx_k_AI_enhanced_query[] = "AI enhanced query: ";
static const char __pyx_k_Business_Concepts[] = "## Business Concepts\n\n";
static const char __pyx_k_FileNotFoundError[] = "FileNotFoundError";
static const char __pyx_k_Optional_List_str[] = "Optional[List[str]]";
static const char __pyx_k_business_concepts[] = "business_concepts";
static const char __pyx_k_column_statistics[] = "column_statistics";
static const char __pyx_k_get_related_rules[] = "get_related_rules";
static const char __pyx_k_metadata_cleaning[] = "metadata_cleaning";
static const char __pyx_k_metadata_sections[] = "metadata_sections";
static const char __pyx_k_required_sections[] = "required_sections";
static const char __pyx_k_AI_history_context[] = "AI history context: ";
static const char __pyx_k_Extracted_keywords[] = "Extracted keywords: ";
static const char __pyx_k_List_Tuple_str_str[] = "List[Tuple[str, str]]";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_format_as_markdown[] = "format_as_markdown";
static const char __pyx_k_get_schema_summary[] = "_get_schema_summary";
static const char __pyx_k_relationship_count[] = "relationship_count";
static const char __pyx_k_search_by_keywords[] = "search_by_keywords";
static const char __pyx_k_Table_Relationships[] = "## Table Relationships\n\n";
static const char __pyx_k_Tagged_with_keyword[] = "Tagged with keyword '";
static const char __pyx_k_database_statistics[] = "database_statistics";
static const char __pyx_k_normalized_keywords[] = "normalized_keywords";
static const char __pyx_k_Description_contains[] = "Description contains '";
static const char __pyx_k_Loaded_metadata_from[] = "Loaded metadata from ";
static const char __pyx_k_Using_template_query[] = "Using template query: ";
static const char __pyx_k_chat_history_context[] = "chat_history_context";
static const char __pyx_k_generate_sql_from_nl[] = "generate_sql_from_nl";
static const char __pyx_k_AI_extracted_keywords[] = "AI extracted keywords: ";
static const char __pyx_k_AI_identified_domains[] = "AI identified domains: ";
static const char __pyx_k_process_query_with_ai[] = "process_query_with_ai";
static const char __pyx_k_DatabaseContextService[] = "DatabaseContextService";
static const char __pyx_k_Name_partially_matches[] = "Name partially matches '";
static const char __pyx_k_natural_language_query[] = "natural_language_query";
static const char __pyx_k_related_business_rules[] = "related_business_rules";
static const char __pyx_k_Metadata_file_not_found[] = "Metadata file not found: ";
static const char __pyx_k_Metadata_was_cleaned_at[] = "  - Metadata was cleaned at ";
static const char __pyx_k_sQQideR_DatabaseContext[] = "sQQideR.DatabaseContext";
static const char __pyx_k_AI_SQL_generation_failed[] = "AI SQL generation failed: ";
static const char __pyx_k_Error_parsing_AI_response[] = "Error parsing AI response: ";
static const char __pyx_k_Database_Context_for_Query[] = "# Database Context for Query: ";
static const char __pyx_k_Metadata_contains_sections[] = "  - Metadata contains sections: ";
static const char __pyx_k_The_previous_SQL_query_was[] = "\n\nThe previous SQL query was:\n";
static const char __pyx_k_sQQideR_DatabaseContext_pyx[] = "sQQideR\\DatabaseContext.pyx";
static const char __pyx_k_Column_Data_Type_Description[] = "| Column | Data Type | Description |\n";
static const char __pyx_k_No_tables_selected_for_query[] = "No tables selected for query";
static const char __pyx_k_DatabaseContextService___init[] = "DatabaseContextService.__init__";
static const char __pyx_k_Description_partially_matches[] = "Description partially matches '";
static const char __pyx_k_Error_enhancing_query_with_AI[] = "Error enhancing query with AI: ";
static const char __pyx_k_Service_for_providing_database[] = "\n    Service for providing database context from natural language queries.\n    \n    This class encapsulates the functionality to build a graph from database metadata,\n    search it based on natural language queries, and generate AI-friendly context\n    that can be used to aid in SQL generation or database exploration.\n    ";
static const char __pyx_k_Using_basic_keyword_extraction[] = "Using basic keyword extraction: ";
static const char __pyx_k_1_Extract_the_key_business_doma[] = "\n                1. Extract the key business domains this relates to (from: scheduling, resource management, critical path, risk assessment, etc.)\n                2. Extract important keywords for database searching (specific terms, metrics, or concepts mentioned)\n                3. Rewrite the question to be more precise and aligned with database terminology\n                4. Consider the conversation history in your analysis\n                \n                Return your analysis in JSON format with these fields:\n                {\n                    \"keywords\": [\"keyword1\", \"keyword2\", ...],\n                    \"business_domains\": [\"domain1\", \"domain2\", ...],\n                    \"enhanced_query\": \"improved query text\",\n                    \"history_context\": \"brief summary of relevant context from history\"\n                }\n                \n                Only return the JSON itself - do not include any code block formatting or other text.\n                ";
static const char __pyx_k_Consider_this_conversation_hist[] = "\n\nConsider this conversation history:\n";
static const char __pyx_k_DatabaseContextService_generate[] = "DatabaseContextService.generate_sql.<locals>.get_alias";
static const char __pyx_k_Database_Context_Service_A_serv[] = "\nDatabase Context Service\n\nA service for analyzing database metadata and generating context from natural language queries.\nThis service helps bridge the gap between natural language and SQL by providing rich context\nabout database objects relevant to a user query.\n";
static const char __pyx_k_Falling_back_to_rule_based_appr[] = ". Falling back to rule-based approach.";
static const char __pyx_k_ON_Missing_join_condition_verif[] = " ON /* Missing join condition - verify relationship */";
static const char __pyx_k_Show_all_projects_with_negative[] = "Show all projects with negative float";
static const char __pyx_k_WARNING_Missing_required_sectio[] = "  - WARNING: Missing required sections: ";
static const char __pyx_k_You_are_an_expert_in_project_ma[] = "\n                You are an expert in project management and database systems. Analyze this question:\n                \n                \"";
static const char __pyx_k_process_query_with_chat_history[] = "process_query_with_chat_history";
static const char __pyx_k_Attempting_to_load_enriched_proj[] = "Attempting to load enriched_project_metadata_cleaned.json...";
static const char __pyx_k_DatabaseContextService__get_sche[] = "DatabaseContextService._get_schema_summary";
static const char __pyx_k_DatabaseContextService_build_gra[] = "DatabaseContextService.build_graph";
static const char __pyx_k_DatabaseContextService_find_path[] = "DatabaseContextService.find_path";
static const char __pyx_k_DatabaseContextService_format_as[] = "DatabaseContextService.format_as_markdown";
static const char __pyx_k_DatabaseContextService_get_relat[] = "DatabaseContextService.get_related_rules";
static const char __pyx_k_DatabaseContextService_load_meta[] = "DatabaseContextService.load_metadata";
static const char __pyx_k_DatabaseContextService_process_q[] = "DatabaseContextService.process_query_with_chat_history";
static const char __pyx_k_DatabaseContextService_search_by[] = "DatabaseContextService.search_by_keywords.<locals>.genexpr";
static const char __pyx_k_Failed_to_load_metadata_file_Ple[] = "Failed to load metadata file. Please ensure the file exists.";
static const char __pyx_k_Find_activities_with_excessive_l[] = "Find activities with excessive lag";
static const char __pyx_k_Graph_not_built_Call_build_graph[] = "Graph not built. Call build_graph first.";
static const char __pyx_k_List_relationships_with_lag_grea[] = "List relationships with lag greater than 44 days";
static const char __pyx_k_No_metadata_provided_Call_load_m[] = "No metadata provided. Call load_metadata first or provide metadata.";
static const char __pyx_k_Tuple_List_Tuple_str_Dict_Dict_s[] = "Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]";
static const char __pyx_k_What_are_the_activities_on_the_c[] = "What are the activities on the critical path?";
static const char __pyx_k_enriched_project_metadata_cleane[] = "enriched_project_metadata_cleaned.json";
static const char __pyx_k_1_Extract_the_key_business_doma_2[] = "\"\n                \n                1. Extract the key business domains this relates to (from: scheduling, resource management, critical path, risk assessment, etc.)\n                2. Extract important keywords for database searching (specific terms, metrics, or concepts mentioned)\n                3. Rewrite the question to be more precise and aligned with database terminology\n                \n                Return your analysis in JSON format with these fields:\n                {\n                    \"keywords\": [\"keyword1\", \"keyword2\", ...],\n                    \"business_domains\": [\"domain1\", \"domain2\", ...],\n                    \"enhanced_query\": \"improved query text\"\n                }\n                \n                Only return the JSON itself - do not include any code block formatting or other text.\n                ";
static const char __pyx_k_DatabaseContextService_generate_2[] = "DatabaseContextService.generate_sql_from_nl.<locals>.genexpr";
static const char __pyx_k_DatabaseContextService_generate_3[] = "DatabaseContextService.generate_sql_from_nl.<locals>.<lambda>";
static const char __pyx_k_DatabaseContextService_generate_4[] = "DatabaseContextService.generate_sql";
static const char __pyx_k_DatabaseContextService_generate_5[] = "DatabaseContextService.generate_sql_from_nl";
static const char __pyx_k_DatabaseContextService_generate_6[] = "DatabaseContextService.generate_output";
static const char __pyx_k_DatabaseContextService_process_q_2[] = "DatabaseContextService.process_query";
static const char __pyx_k_DatabaseContextService_process_q_3[] = "DatabaseContextService.process_query_with_ai";
static const char __pyx_k_DatabaseContextService_search_by_2[] = "DatabaseContextService.search_by_keywords.<locals>.<lambda>";
static const char __pyx_k_DatabaseContextService_search_by_3[] = "DatabaseContextService.search_by_keywords";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_metadata_file, PyObject *__pyx_v_verbose, PyObject *__pyx_v_ai_base); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_2load_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_filename); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_4process_query_with_chat_history(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_chat_history, PyObject *__pyx_v_previous_sql, PyObject *__pyx_v_output_format); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_6process_query(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_output_format, PyObject *__pyx_v_chat_history, PyObject *__pyx_v_previous_sql); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_8build_graph(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_metadata); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_genexpr(PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda1(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_10search_by_keywords(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_keywords); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12get_related_rules(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_node_id); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_14find_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_start_node, PyObject *__pyx_v_end_node); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_get_alias(PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_16generate_sql(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_table_nodes, PyObject *__pyx_v_condition_nodes); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda2(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_1genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18generate_sql_from_nl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_natural_language_query); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20process_query_with_ai(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_output_format); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_22generate_output(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, CYTHON_UNUSED PyObject *__pyx_v_results, CYTHON_UNUSED PyObject *__pyx_v_matches, PyObject *__pyx_v_tables, PyObject *__pyx_v_columns, PyObject *__pyx_v_concepts, PyObject *__pyx_v_rules, CYTHON_UNUSED PyObject *__pyx_v_sql, PyObject *__pyx_v_all_tables); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_24format_as_markdown(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, PyObject *__pyx_v_output_data); /* proto */
static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_26_get_schema_summary(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get = {0, 0, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items = {0, 0, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_keys = {0, 0, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyList_Type_copy = {0, 0, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type_lower = {0, 0, 0, 0, 0};
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  PyObject *__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords;
  PyObject *__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr;
  PyObject *__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql;
  PyObject *__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl;
  PyObject *__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr;
  #endif
  PyTypeObject *__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords;
  PyTypeObject *__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr;
  PyTypeObject *__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql;
  PyTypeObject *__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl;
  PyTypeObject *__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr;
  PyObject *__pyx_kp_u_1_Extract_the_key_business_doma;
  PyObject *__pyx_kp_u_1_Extract_the_key_business_doma_2;
  PyObject *__pyx_kp_u_AI_Enhanced_Query;
  PyObject *__pyx_kp_u_AI_SQL_generation_failed;
  PyObject *__pyx_kp_u_AI_enhanced_query;
  PyObject *__pyx_kp_u_AI_extracted_keywords;
  PyObject *__pyx_kp_u_AI_history_context;
  PyObject *__pyx_kp_u_AI_identified_domains;
  PyObject *__pyx_kp_u_AND;
  PyObject *__pyx_n_s_Any;
  PyObject *__pyx_n_u_Assistant;
  PyObject *__pyx_kp_u_Attempting_to_load_enriched_proj;
  PyObject *__pyx_kp_u_Business_Concepts;
  PyObject *__pyx_kp_u_Business_Domains;
  PyObject *__pyx_kp_u_Business_Rules;
  PyObject *__pyx_n_s_ChatMessage;
  PyObject *__pyx_kp_u_Cleaned_response;
  PyObject *__pyx_kp_u_Column_Data_Type_Description;
  PyObject *__pyx_kp_u_Consider_this_conversation_hist;
  PyObject *__pyx_n_s_DatabaseContextService;
  PyObject *__pyx_n_s_DatabaseContextService___init;
  PyObject *__pyx_n_s_DatabaseContextService__get_sche;
  PyObject *__pyx_n_s_DatabaseContextService_build_gra;
  PyObject *__pyx_n_s_DatabaseContextService_find_path;
  PyObject *__pyx_n_s_DatabaseContextService_format_as;
  PyObject *__pyx_n_s_DatabaseContextService_generate;
  PyObject *__pyx_n_s_DatabaseContextService_generate_2;
  PyObject *__pyx_n_s_DatabaseContextService_generate_3;
  PyObject *__pyx_n_s_DatabaseContextService_generate_4;
  PyObject *__pyx_n_s_DatabaseContextService_generate_5;
  PyObject *__pyx_n_s_DatabaseContextService_generate_6;
  PyObject *__pyx_n_s_DatabaseContextService_get_relat;
  PyObject *__pyx_n_s_DatabaseContextService_load_meta;
  PyObject *__pyx_n_s_DatabaseContextService_process_q;
  PyObject *__pyx_n_s_DatabaseContextService_process_q_2;
  PyObject *__pyx_n_s_DatabaseContextService_process_q_3;
  PyObject *__pyx_n_s_DatabaseContextService_search_by;
  PyObject *__pyx_n_s_DatabaseContextService_search_by_2;
  PyObject *__pyx_n_s_DatabaseContextService_search_by_3;
  PyObject *__pyx_kp_u_Database_Context_for_Query;
  PyObject *__pyx_kp_u_Description;
  PyObject *__pyx_kp_u_Description_contains;
  PyObject *__pyx_kp_u_Description_partially_matches;
  PyObject *__pyx_n_s_DiGraph;
  PyObject *__pyx_n_s_Dict;
  PyObject *__pyx_kp_u_Error_enhancing_query_with_AI;
  PyObject *__pyx_kp_u_Error_parsing_AI_response;
  PyObject *__pyx_kp_u_Extracted_keywords;
  PyObject *__pyx_kp_u_FROM;
  PyObject *__pyx_kp_u_Failed_to_load_metadata_file_Ple;
  PyObject *__pyx_kp_u_Falling_back_to_rule_based_appr;
  PyObject *__pyx_n_s_FileNotFoundError;
  PyObject *__pyx_kp_u_Find_activities_with_excessive_l;
  PyObject *__pyx_kp_u_Found;
  PyObject *__pyx_n_s_G;
  PyObject *__pyx_kp_u_Generated_SQL;
  PyObject *__pyx_kp_u_Graph_built_with;
  PyObject *__pyx_kp_u_Graph_not_built_Call_build_graph;
  PyObject *__pyx_kp_u_ID_contains;
  PyObject *__pyx_n_s_IGNORECASE;
  PyObject *__pyx_kp_u_JOIN;
  PyObject *__pyx_n_s_JSONDecodeError;
  PyObject *__pyx_kp_u_Join_condition;
  PyObject *__pyx_n_s_List;
  PyObject *__pyx_kp_s_List_ChatMessage;
  PyObject *__pyx_kp_s_List_Tuple_str_str;
  PyObject *__pyx_kp_u_List_relationships_with_lag_grea;
  PyObject *__pyx_kp_s_List_str;
  PyObject *__pyx_kp_u_Loaded_metadata_from;
  PyObject *__pyx_kp_u_Markdown_Output;
  PyObject *__pyx_kp_u_Metadata_contains_sections;
  PyObject *__pyx_kp_u_Metadata_file_not_found;
  PyObject *__pyx_kp_u_Metadata_was_cleaned_at;
  PyObject *__pyx_n_s_Models;
  PyObject *__pyx_kp_u_Name_contains;
  PyObject *__pyx_kp_u_Name_partially_matches;
  PyObject *__pyx_n_s_NetworkXNoPath;
  PyObject *__pyx_kp_u_No_SQL_generated;
  PyObject *__pyx_kp_u_No_metadata_provided_Call_load_m;
  PyObject *__pyx_kp_u_No_tables_selected_for_query;
  PyObject *__pyx_kp_u_None;
  PyObject *__pyx_kp_u_ON;
  PyObject *__pyx_kp_u_ON_Missing_join_condition_verif;
  PyObject *__pyx_n_s_Optional;
  PyObject *__pyx_kp_s_Optional_List_str;
  PyObject *__pyx_kp_u_Processing_query;
  PyObject *__pyx_kp_u_Raw_response;
  PyObject *__pyx_kp_u_Relevant_Columns;
  PyObject *__pyx_kp_u_Relevant_Tables;
  PyObject *__pyx_kp_u_SELECT;
  PyObject *__pyx_n_s_SQLPrompt;
  PyObject *__pyx_kp_s_Service_for_providing_database;
  PyObject *__pyx_kp_u_Show_all_projects_with_negative;
  PyObject *__pyx_kp_u_Table_Relationships;
  PyObject *__pyx_kp_u_Tagged_with_keyword;
  PyObject *__pyx_kp_u_The_previous_SQL_query_was;
  PyObject *__pyx_n_s_Tuple;
  PyObject *__pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s;
  PyObject *__pyx_n_s_Union;
  PyObject *__pyx_n_u_User;
  PyObject *__pyx_kp_u_Using_basic_keyword_extraction;
  PyObject *__pyx_kp_u_Using_template_query;
  PyObject *__pyx_n_s_ValueError;
  PyObject *__pyx_kp_u_WARNING_Missing_required_sectio;
  PyObject *__pyx_kp_u_WHERE;
  PyObject *__pyx_kp_u_What_are_the_activities_on_the_c;
  PyObject *__pyx_kp_u_You_are_an_expert_in_project_ma;
  PyObject *__pyx_kp_u__10;
  PyObject *__pyx_kp_u__12;
  PyObject *__pyx_kp_u__15;
  PyObject *__pyx_kp_u__2;
  PyObject *__pyx_kp_u__20;
  PyObject *__pyx_kp_u__23;
  PyObject *__pyx_kp_u__27;
  PyObject *__pyx_kp_u__28;
  PyObject *__pyx_kp_u__29;
  PyObject *__pyx_n_s__31;
  PyObject *__pyx_kp_u__31;
  PyObject *__pyx_kp_u__35;
  PyObject *__pyx_kp_u__36;
  PyObject *__pyx_kp_u__37;
  PyObject *__pyx_kp_u__38;
  PyObject *__pyx_kp_u__39;
  PyObject *__pyx_kp_u__40;
  PyObject *__pyx_kp_u__41;
  PyObject *__pyx_kp_u__42;
  PyObject *__pyx_kp_u__43;
  PyObject *__pyx_kp_u__44;
  PyObject *__pyx_kp_u__45;
  PyObject *__pyx_n_s__48;
  PyObject *__pyx_kp_u__5;
  PyObject *__pyx_n_s__55;
  PyObject *__pyx_kp_u__7;
  PyObject *__pyx_kp_u__8;
  PyObject *__pyx_kp_u__9;
  PyObject *__pyx_n_s_adapted_query;
  PyObject *__pyx_n_s_add_edge;
  PyObject *__pyx_n_s_add_node;
  PyObject *__pyx_n_s_ai_base;
  PyObject *__pyx_n_s_ai_context;
  PyObject *__pyx_n_u_ai_enhanced;
  PyObject *__pyx_n_s_alias;
  PyObject *__pyx_n_s_aliases;
  PyObject *__pyx_n_s_all_rules;
  PyObject *__pyx_n_s_all_tables;
  PyObject *__pyx_n_s_append;
  PyObject *__pyx_n_s_args;
  PyObject *__pyx_n_s_asyncio;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_asyncio_tasks;
  PyObject *__pyx_kp_u_b;
  PyObject *__pyx_n_s_base;
  PyObject *__pyx_n_s_best_example;
  PyObject *__pyx_n_u_best_guess_sql;
  PyObject *__pyx_n_s_bool;
  PyObject *__pyx_n_s_build_graph;
  PyObject *__pyx_n_s_buisness_domains;
  PyObject *__pyx_n_u_business_concepts;
  PyObject *__pyx_n_u_business_domains;
  PyObject *__pyx_n_u_business_rules;
  PyObject *__pyx_n_s_c;
  PyObject *__pyx_n_s_chat_history;
  PyObject *__pyx_n_u_chat_history_context;
  PyObject *__pyx_n_s_class_getitem;
  PyObject *__pyx_n_s_clean_response;
  PyObject *__pyx_n_s_cleaned_at;
  PyObject *__pyx_n_u_cleaned_at;
  PyObject *__pyx_n_u_cleaning_version;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_close;
  PyObject *__pyx_n_s_col_data;
  PyObject *__pyx_n_s_col_name;
  PyObject *__pyx_n_s_col_node_id;
  PyObject *__pyx_kp_u_column;
  PyObject *__pyx_n_s_column_2;
  PyObject *__pyx_n_u_column_2;
  PyObject *__pyx_n_u_column_statistics;
  PyObject *__pyx_n_s_columns;
  PyObject *__pyx_n_u_columns;
  PyObject *__pyx_n_s_compile;
  PyObject *__pyx_n_s_complexity;
  PyObject *__pyx_n_u_complexity;
  PyObject *__pyx_kp_u_concept;
  PyObject *__pyx_n_s_concept_2;
  PyObject *__pyx_n_u_concept_2;
  PyObject *__pyx_n_s_concept_data;
  PyObject *__pyx_n_s_concept_id;
  PyObject *__pyx_n_s_concepts;
  PyObject *__pyx_n_u_concepts;
  PyObject *__pyx_n_s_condition_nodes;
  PyObject *__pyx_n_s_content;
  PyObject *__pyx_n_s_context;
  PyObject *__pyx_n_s_context_md;
  PyObject *__pyx_n_s_context_prompt;
  PyObject *__pyx_n_s_copy;
  PyObject *__pyx_n_s_current_table;
  PyObject *__pyx_n_s_d;
  PyObject *__pyx_kp_u_d_4_d_2_d_2;
  PyObject *__pyx_n_s_data;
  PyObject *__pyx_n_u_data;
  PyObject *__pyx_n_u_data_type;
  PyObject *__pyx_n_u_database_objects;
  PyObject *__pyx_n_u_database_schema;
  PyObject *__pyx_n_u_database_statistics;
  PyObject *__pyx_n_u_date;
  PyObject *__pyx_n_s_datetime;
  PyObject *__pyx_n_u_day;
  PyObject *__pyx_n_u_defines;
  PyObject *__pyx_n_s_definition;
  PyObject *__pyx_n_s_description;
  PyObject *__pyx_n_u_description;
  PyObject *__pyx_n_s_dict;
  PyObject *__pyx_kp_u_disable;
  PyObject *__pyx_n_s_doc;
  PyObject *__pyx_n_s_domain;
  PyObject *__pyx_n_u_domain;
  PyObject *__pyx_n_s_domain_results;
  PyObject *__pyx_n_s_e;
  PyObject *__pyx_n_s_edge_data;
  PyObject *__pyx_n_s_edges;
  PyObject *__pyx_kp_u_edges_2;
  PyObject *__pyx_kp_u_enable;
  PyObject *__pyx_n_s_encoding;
  PyObject *__pyx_n_s_end_node;
  PyObject *__pyx_n_s_endswith;
  PyObject *__pyx_n_s_enhanced_query;
  PyObject *__pyx_n_u_enhanced_query;
  PyObject *__pyx_kp_u_enriched_project_metadata_cleane;
  PyObject *__pyx_n_s_enter;
  PyObject *__pyx_n_s_enumerate;
  PyObject *__pyx_n_s_escape;
  PyObject *__pyx_kp_u_example;
  PyObject *__pyx_n_s_example_2;
  PyObject *__pyx_n_u_example_2;
  PyObject *__pyx_n_s_example_data;
  PyObject *__pyx_n_s_example_id;
  PyObject *__pyx_n_s_example_node;
  PyObject *__pyx_n_s_example_nodes;
  PyObject *__pyx_n_s_example_queries;
  PyObject *__pyx_n_s_example_scores;
  PyObject *__pyx_n_s_exists;
  PyObject *__pyx_n_s_exit;
  PyObject *__pyx_n_s_f;
  PyObject *__pyx_n_s_file_path;
  PyObject *__pyx_n_s_filename;
  PyObject *__pyx_n_s_filtered_rules;
  PyObject *__pyx_n_s_find;
  PyObject *__pyx_n_s_find_path;
  PyObject *__pyx_n_s_first_newline;
  PyObject *__pyx_n_s_first_table;
  PyObject *__pyx_n_s_flags;
  PyObject *__pyx_n_s_format_as_markdown;
  PyObject *__pyx_n_u_from_table;
  PyObject *__pyx_n_s_from_table_id;
  PyObject *__pyx_n_s_from_tables;
  PyObject *__pyx_kp_u_gc;
  PyObject *__pyx_n_s_generate_output;
  PyObject *__pyx_n_s_generate_sql;
  PyObject *__pyx_n_s_generate_sql_from_nl;
  PyObject *__pyx_n_s_genexpr;
  PyObject *__pyx_n_s_get;
  PyObject *__pyx_n_s_get_alias;
  PyObject *__pyx_n_s_get_edge_data;
  PyObject *__pyx_n_s_get_related_rules;
  PyObject *__pyx_n_u_get_response;
  PyObject *__pyx_n_s_get_schema_summary;
  PyObject *__pyx_n_u_governed_by;
  PyObject *__pyx_n_s_graph;
  PyObject *__pyx_n_u_has_column;
  PyObject *__pyx_n_s_has_node;
  PyObject *__pyx_n_s_history_context;
  PyObject *__pyx_n_u_history_context;
  PyObject *__pyx_n_s_i;
  PyObject *__pyx_n_u_id;
  PyObject *__pyx_n_u_implements;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_u_indexed;
  PyObject *__pyx_n_s_init;
  PyObject *__pyx_n_s_init_subclass;
  PyObject *__pyx_n_s_initializing;
  PyObject *__pyx_n_s_inspect;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_n_s_is_user;
  PyObject *__pyx_kp_u_isenabled;
  PyObject *__pyx_n_s_items;
  PyObject *__pyx_n_s_j;
  PyObject *__pyx_n_s_join;
  PyObject *__pyx_n_s_join_clauses;
  PyObject *__pyx_n_s_join_condition;
  PyObject *__pyx_n_u_join_condition;
  PyObject *__pyx_n_s_join_found;
  PyObject *__pyx_n_s_json;
  PyObject *__pyx_n_u_json;
  PyObject *__pyx_n_s_k;
  PyObject *__pyx_n_s_key;
  PyObject *__pyx_n_s_keys;
  PyObject *__pyx_kp_u_keyword;
  PyObject *__pyx_n_s_keyword_2;
  PyObject *__pyx_n_u_keyword_2;
  PyObject *__pyx_n_s_keyword_id;
  PyObject *__pyx_n_s_keywords;
  PyObject *__pyx_n_u_keywords;
  PyObject *__pyx_n_s_load;
  PyObject *__pyx_n_s_load_metadata;
  PyObject *__pyx_n_s_loads;
  PyObject *__pyx_n_s_lower;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_u_main;
  PyObject *__pyx_n_u_markdown;
  PyObject *__pyx_n_s_match_reason;
  PyObject *__pyx_n_s_match_score;
  PyObject *__pyx_n_s_matched;
  PyObject *__pyx_n_s_matches;
  PyObject *__pyx_n_s_max;
  PyObject *__pyx_n_s_md_output;
  PyObject *__pyx_n_s_metaclass;
  PyObject *__pyx_n_s_metadata;
  PyObject *__pyx_n_u_metadata_cleaning;
  PyObject *__pyx_n_s_metadata_file;
  PyObject *__pyx_n_s_metadata_sections;
  PyObject *__pyx_n_s_missing;
  PyObject *__pyx_n_s_module;
  PyObject *__pyx_n_s_msg;
  PyObject *__pyx_n_s_n;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_u_name;
  PyObject *__pyx_n_s_name_2;
  PyObject *__pyx_n_s_natural_language_query;
  PyObject *__pyx_n_s_neighbor;
  PyObject *__pyx_n_s_neighbors;
  PyObject *__pyx_n_s_networkx;
  PyObject *__pyx_n_s_node;
  PyObject *__pyx_n_s_node_data;
  PyObject *__pyx_n_s_node_id;
  PyObject *__pyx_n_s_node_matches;
  PyObject *__pyx_n_s_node_props;
  PyObject *__pyx_n_s_node_rules;
  PyObject *__pyx_n_s_node_score;
  PyObject *__pyx_n_s_node_scores;
  PyObject *__pyx_n_s_nodes;
  PyObject *__pyx_kp_u_nodes_and;
  PyObject *__pyx_n_s_normalized_keywords;
  PyObject *__pyx_n_s_nx;
  PyObject *__pyx_kp_s_nx_DiGraph;
  PyObject *__pyx_n_s_open;
  PyObject *__pyx_n_s_os;
  PyObject *__pyx_n_s_os_path;
  PyObject *__pyx_n_s_output;
  PyObject *__pyx_n_s_output_data;
  PyObject *__pyx_n_s_output_format;
  PyObject *__pyx_n_s_parsed;
  PyObject *__pyx_n_s_path;
  PyObject *__pyx_n_s_pattern;
  PyObject *__pyx_n_s_prepare;
  PyObject *__pyx_n_s_prev;
  PyObject *__pyx_n_s_prev_table;
  PyObject *__pyx_n_s_previous_sql;
  PyObject *__pyx_n_s_print;
  PyObject *__pyx_n_s_process_query;
  PyObject *__pyx_n_s_process_query_with_ai;
  PyObject *__pyx_n_s_process_query_with_chat_history;
  PyObject *__pyx_n_s_prompt;
  PyObject *__pyx_n_s_qualname;
  PyObject *__pyx_n_s_query;
  PyObject *__pyx_n_u_query;
  PyObject *__pyx_n_u_query_examples;
  PyObject *__pyx_n_s_query_text;
  PyObject *__pyx_n_s_r;
  PyObject *__pyx_n_u_r;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_re;
  PyObject *__pyx_n_u_references;
  PyObject *__pyx_n_s_rel;
  PyObject *__pyx_n_u_related_business_rules;
  PyObject *__pyx_n_u_related_to;
  PyObject *__pyx_n_s_relationship;
  PyObject *__pyx_n_s_relationship_count;
  PyObject *__pyx_n_u_relationships;
  PyObject *__pyx_kp_u_relationships_2;
  PyObject *__pyx_n_s_replace;
  PyObject *__pyx_n_s_required_sections;
  PyObject *__pyx_n_s_response;
  PyObject *__pyx_n_s_results;
  PyObject *__pyx_n_s_return;
  PyObject *__pyx_n_s_reverse;
  PyObject *__pyx_n_s_role;
  PyObject *__pyx_n_s_row_count;
  PyObject *__pyx_n_u_row_count;
  PyObject *__pyx_kp_u_rule;
  PyObject *__pyx_n_s_rule_2;
  PyObject *__pyx_n_u_rule_2;
  PyObject *__pyx_n_s_rule_data;
  PyObject *__pyx_n_s_rule_desc;
  PyObject *__pyx_n_s_rule_id;
  PyObject *__pyx_n_s_rule_node;
  PyObject *__pyx_n_s_rules;
  PyObject *__pyx_n_s_run;
  PyObject *__pyx_n_s_s;
  PyObject *__pyx_n_s_sQQideR_DatabaseContext;
  PyObject *__pyx_kp_s_sQQideR_DatabaseContext_pyx;
  PyObject *__pyx_kp_u_s_LIMIT_s_d_s;
  PyObject *__pyx_n_s_schema;
  PyObject *__pyx_n_s_score;
  PyObject *__pyx_n_s_search;
  PyObject *__pyx_n_s_search_by_keywords;
  PyObject *__pyx_n_s_select_clause;
  PyObject *__pyx_n_s_selected_query;
  PyObject *__pyx_n_s_self;
  PyObject *__pyx_n_s_send;
  PyObject *__pyx_n_s_service;
  PyObject *__pyx_n_s_set_name;
  PyObject *__pyx_n_s_shortest_path;
  PyObject *__pyx_n_u_simple;
  PyObject *__pyx_n_s_sorted;
  PyObject *__pyx_n_s_sorted_nodes;
  PyObject *__pyx_n_u_source;
  PyObject *__pyx_n_s_spec;
  PyObject *__pyx_n_s_split;
  PyObject *__pyx_n_s_sql;
  PyObject *__pyx_n_s_sql_lines;
  PyObject *__pyx_n_u_src;
  PyObject *__pyx_n_s_src_name;
  PyObject *__pyx_n_s_start_node;
  PyObject *__pyx_n_s_startswith;
  PyObject *__pyx_n_u_statistics;
  PyObject *__pyx_n_s_str;
  PyObject *__pyx_n_s_strip;
  PyObject *__pyx_n_s_sub;
  PyObject *__pyx_n_s_suffix;
  PyObject *__pyx_n_s_super;
  PyObject *__pyx_n_s_t;
  PyObject *__pyx_kp_u_table;
  PyObject *__pyx_n_s_table_2;
  PyObject *__pyx_n_u_table_2;
  PyObject *__pyx_n_s_table_count;
  PyObject *__pyx_n_s_table_data;
  PyObject *__pyx_n_s_table_id;
  PyObject *__pyx_n_s_table_name;
  PyObject *__pyx_n_s_table_names;
  PyObject *__pyx_n_s_table_nodes;
  PyObject *__pyx_n_s_tables;
  PyObject *__pyx_n_u_tables;
  PyObject *__pyx_kp_u_tables_and;
  PyObject *__pyx_n_u_tags;
  PyObject *__pyx_n_s_target;
  PyObject *__pyx_n_u_target;
  PyObject *__pyx_n_s_targets;
  PyObject *__pyx_n_s_tbl;
  PyObject *__pyx_n_s_template_query;
  PyObject *__pyx_kp_u_term;
  PyObject *__pyx_n_s_term_2;
  PyObject *__pyx_n_u_term_2;
  PyObject *__pyx_n_s_term_id;
  PyObject *__pyx_n_u_terminology;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_tgt_name;
  PyObject *__pyx_n_s_throw;
  PyObject *__pyx_n_u_to_table;
  PyObject *__pyx_n_s_to_table_id;
  PyObject *__pyx_n_s_type;
  PyObject *__pyx_n_u_type;
  PyObject *__pyx_n_s_typing;
  PyObject *__pyx_n_s_unique_rules;
  PyObject *__pyx_n_u_unknown;
  PyObject *__pyx_n_s_used_aliases;
  PyObject *__pyx_n_s_user_prompt;
  PyObject *__pyx_kp_u_utf_8;
  PyObject *__pyx_n_s_values;
  PyObject *__pyx_n_s_verbose;
  PyObject *__pyx_kp_u_version;
  PyObject *__pyx_n_s_version_2;
  PyObject *__pyx_n_s_where_clauses;
  PyObject *__pyx_n_s_word;
  PyObject *__pyx_n_s_x;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_2;
  PyObject *__pyx_int_3;
  PyObject *__pyx_int_4;
  PyObject *__pyx_int_5;
  PyObject *__pyx_int_neg_1;
  PyObject *__pyx_int_neg_3;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__3;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_slice__11;
  PyObject *__pyx_tuple__13;
  PyObject *__pyx_tuple__14;
  PyObject *__pyx_tuple__16;
  PyObject *__pyx_tuple__17;
  PyObject *__pyx_tuple__18;
  PyObject *__pyx_tuple__19;
  PyObject *__pyx_tuple__21;
  PyObject *__pyx_tuple__22;
  PyObject *__pyx_tuple__24;
  PyObject *__pyx_tuple__26;
  PyObject *__pyx_tuple__30;
  PyObject *__pyx_tuple__32;
  PyObject *__pyx_tuple__33;
  PyObject *__pyx_tuple__34;
  PyObject *__pyx_tuple__46;
  PyObject *__pyx_tuple__47;
  PyObject *__pyx_tuple__49;
  PyObject *__pyx_tuple__50;
  PyObject *__pyx_tuple__52;
  PyObject *__pyx_tuple__53;
  PyObject *__pyx_tuple__56;
  PyObject *__pyx_tuple__58;
  PyObject *__pyx_tuple__59;
  PyObject *__pyx_tuple__61;
  PyObject *__pyx_tuple__62;
  PyObject *__pyx_tuple__64;
  PyObject *__pyx_tuple__65;
  PyObject *__pyx_tuple__67;
  PyObject *__pyx_tuple__69;
  PyObject *__pyx_tuple__71;
  PyObject *__pyx_tuple__73;
  PyObject *__pyx_tuple__75;
  PyObject *__pyx_tuple__77;
  PyObject *__pyx_tuple__78;
  PyObject *__pyx_tuple__80;
  PyObject *__pyx_tuple__81;
  PyObject *__pyx_tuple__83;
  PyObject *__pyx_tuple__85;
  PyObject *__pyx_tuple__86;
  PyObject *__pyx_tuple__87;
  PyObject *__pyx_tuple__88;
  PyObject *__pyx_tuple__89;
  PyObject *__pyx_tuple__90;
  PyObject *__pyx_codeobj__25;
  PyObject *__pyx_codeobj__51;
  PyObject *__pyx_codeobj__54;
  PyObject *__pyx_codeobj__57;
  PyObject *__pyx_codeobj__60;
  PyObject *__pyx_codeobj__63;
  PyObject *__pyx_codeobj__66;
  PyObject *__pyx_codeobj__68;
  PyObject *__pyx_codeobj__70;
  PyObject *__pyx_codeobj__72;
  PyObject *__pyx_codeobj__74;
  PyObject *__pyx_codeobj__76;
  PyObject *__pyx_codeobj__79;
  PyObject *__pyx_codeobj__82;
  PyObject *__pyx_codeobj__84;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords);
  Py_CLEAR(clear_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords);
  Py_CLEAR(clear_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr);
  Py_CLEAR(clear_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql);
  Py_CLEAR(clear_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql);
  Py_CLEAR(clear_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl);
  Py_CLEAR(clear_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl);
  Py_CLEAR(clear_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr);
  Py_CLEAR(clear_module_state->__pyx_kp_u_1_Extract_the_key_business_doma);
  Py_CLEAR(clear_module_state->__pyx_kp_u_1_Extract_the_key_business_doma_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_Enhanced_Query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_SQL_generation_failed);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_enhanced_query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_extracted_keywords);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_history_context);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AI_identified_domains);
  Py_CLEAR(clear_module_state->__pyx_kp_u_AND);
  Py_CLEAR(clear_module_state->__pyx_n_s_Any);
  Py_CLEAR(clear_module_state->__pyx_n_u_Assistant);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Attempting_to_load_enriched_proj);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Business_Concepts);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Business_Domains);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Business_Rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_ChatMessage);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Cleaned_response);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Column_Data_Type_Description);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Consider_this_conversation_hist);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService___init);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService__get_sche);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_build_gra);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_find_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_format_as);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate_3);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate_4);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate_5);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_generate_6);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_get_relat);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_load_meta);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_process_q);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_process_q_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_process_q_3);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_search_by);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_search_by_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_DatabaseContextService_search_by_3);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Database_Context_for_Query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Description);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Description_contains);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Description_partially_matches);
  Py_CLEAR(clear_module_state->__pyx_n_s_DiGraph);
  Py_CLEAR(clear_module_state->__pyx_n_s_Dict);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Error_enhancing_query_with_AI);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Error_parsing_AI_response);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Extracted_keywords);
  Py_CLEAR(clear_module_state->__pyx_kp_u_FROM);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Failed_to_load_metadata_file_Ple);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Falling_back_to_rule_based_appr);
  Py_CLEAR(clear_module_state->__pyx_n_s_FileNotFoundError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Find_activities_with_excessive_l);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Found);
  Py_CLEAR(clear_module_state->__pyx_n_s_G);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Generated_SQL);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Graph_built_with);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Graph_not_built_Call_build_graph);
  Py_CLEAR(clear_module_state->__pyx_kp_u_ID_contains);
  Py_CLEAR(clear_module_state->__pyx_n_s_IGNORECASE);
  Py_CLEAR(clear_module_state->__pyx_kp_u_JOIN);
  Py_CLEAR(clear_module_state->__pyx_n_s_JSONDecodeError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Join_condition);
  Py_CLEAR(clear_module_state->__pyx_n_s_List);
  Py_CLEAR(clear_module_state->__pyx_kp_s_List_ChatMessage);
  Py_CLEAR(clear_module_state->__pyx_kp_s_List_Tuple_str_str);
  Py_CLEAR(clear_module_state->__pyx_kp_u_List_relationships_with_lag_grea);
  Py_CLEAR(clear_module_state->__pyx_kp_s_List_str);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Loaded_metadata_from);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Markdown_Output);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Metadata_contains_sections);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Metadata_file_not_found);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Metadata_was_cleaned_at);
  Py_CLEAR(clear_module_state->__pyx_n_s_Models);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Name_contains);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Name_partially_matches);
  Py_CLEAR(clear_module_state->__pyx_n_s_NetworkXNoPath);
  Py_CLEAR(clear_module_state->__pyx_kp_u_No_SQL_generated);
  Py_CLEAR(clear_module_state->__pyx_kp_u_No_metadata_provided_Call_load_m);
  Py_CLEAR(clear_module_state->__pyx_kp_u_No_tables_selected_for_query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_None);
  Py_CLEAR(clear_module_state->__pyx_kp_u_ON);
  Py_CLEAR(clear_module_state->__pyx_kp_u_ON_Missing_join_condition_verif);
  Py_CLEAR(clear_module_state->__pyx_n_s_Optional);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Optional_List_str);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Processing_query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Raw_response);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Relevant_Columns);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Relevant_Tables);
  Py_CLEAR(clear_module_state->__pyx_kp_u_SELECT);
  Py_CLEAR(clear_module_state->__pyx_n_s_SQLPrompt);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Service_for_providing_database);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Show_all_projects_with_negative);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Table_Relationships);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Tagged_with_keyword);
  Py_CLEAR(clear_module_state->__pyx_kp_u_The_previous_SQL_query_was);
  Py_CLEAR(clear_module_state->__pyx_n_s_Tuple);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_Union);
  Py_CLEAR(clear_module_state->__pyx_n_u_User);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Using_basic_keyword_extraction);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Using_template_query);
  Py_CLEAR(clear_module_state->__pyx_n_s_ValueError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_WARNING_Missing_required_sectio);
  Py_CLEAR(clear_module_state->__pyx_kp_u_WHERE);
  Py_CLEAR(clear_module_state->__pyx_kp_u_What_are_the_activities_on_the_c);
  Py_CLEAR(clear_module_state->__pyx_kp_u_You_are_an_expert_in_project_ma);
  Py_CLEAR(clear_module_state->__pyx_kp_u__10);
  Py_CLEAR(clear_module_state->__pyx_kp_u__12);
  Py_CLEAR(clear_module_state->__pyx_kp_u__15);
  Py_CLEAR(clear_module_state->__pyx_kp_u__2);
  Py_CLEAR(clear_module_state->__pyx_kp_u__20);
  Py_CLEAR(clear_module_state->__pyx_kp_u__23);
  Py_CLEAR(clear_module_state->__pyx_kp_u__27);
  Py_CLEAR(clear_module_state->__pyx_kp_u__28);
  Py_CLEAR(clear_module_state->__pyx_kp_u__29);
  Py_CLEAR(clear_module_state->__pyx_n_s__31);
  Py_CLEAR(clear_module_state->__pyx_kp_u__31);
  Py_CLEAR(clear_module_state->__pyx_kp_u__35);
  Py_CLEAR(clear_module_state->__pyx_kp_u__36);
  Py_CLEAR(clear_module_state->__pyx_kp_u__37);
  Py_CLEAR(clear_module_state->__pyx_kp_u__38);
  Py_CLEAR(clear_module_state->__pyx_kp_u__39);
  Py_CLEAR(clear_module_state->__pyx_kp_u__40);
  Py_CLEAR(clear_module_state->__pyx_kp_u__41);
  Py_CLEAR(clear_module_state->__pyx_kp_u__42);
  Py_CLEAR(clear_module_state->__pyx_kp_u__43);
  Py_CLEAR(clear_module_state->__pyx_kp_u__44);
  Py_CLEAR(clear_module_state->__pyx_kp_u__45);
  Py_CLEAR(clear_module_state->__pyx_n_s__48);
  Py_CLEAR(clear_module_state->__pyx_kp_u__5);
  Py_CLEAR(clear_module_state->__pyx_n_s__55);
  Py_CLEAR(clear_module_state->__pyx_kp_u__7);
  Py_CLEAR(clear_module_state->__pyx_kp_u__8);
  Py_CLEAR(clear_module_state->__pyx_kp_u__9);
  Py_CLEAR(clear_module_state->__pyx_n_s_adapted_query);
  Py_CLEAR(clear_module_state->__pyx_n_s_add_edge);
  Py_CLEAR(clear_module_state->__pyx_n_s_add_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_ai_base);
  Py_CLEAR(clear_module_state->__pyx_n_s_ai_context);
  Py_CLEAR(clear_module_state->__pyx_n_u_ai_enhanced);
  Py_CLEAR(clear_module_state->__pyx_n_s_alias);
  Py_CLEAR(clear_module_state->__pyx_n_s_aliases);
  Py_CLEAR(clear_module_state->__pyx_n_s_all_rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_all_tables);
  Py_CLEAR(clear_module_state->__pyx_n_s_append);
  Py_CLEAR(clear_module_state->__pyx_n_s_args);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_tasks);
  Py_CLEAR(clear_module_state->__pyx_kp_u_b);
  Py_CLEAR(clear_module_state->__pyx_n_s_base);
  Py_CLEAR(clear_module_state->__pyx_n_s_best_example);
  Py_CLEAR(clear_module_state->__pyx_n_u_best_guess_sql);
  Py_CLEAR(clear_module_state->__pyx_n_s_bool);
  Py_CLEAR(clear_module_state->__pyx_n_s_build_graph);
  Py_CLEAR(clear_module_state->__pyx_n_s_buisness_domains);
  Py_CLEAR(clear_module_state->__pyx_n_u_business_concepts);
  Py_CLEAR(clear_module_state->__pyx_n_u_business_domains);
  Py_CLEAR(clear_module_state->__pyx_n_u_business_rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_c);
  Py_CLEAR(clear_module_state->__pyx_n_s_chat_history);
  Py_CLEAR(clear_module_state->__pyx_n_u_chat_history_context);
  Py_CLEAR(clear_module_state->__pyx_n_s_class_getitem);
  Py_CLEAR(clear_module_state->__pyx_n_s_clean_response);
  Py_CLEAR(clear_module_state->__pyx_n_s_cleaned_at);
  Py_CLEAR(clear_module_state->__pyx_n_u_cleaned_at);
  Py_CLEAR(clear_module_state->__pyx_n_u_cleaning_version);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_col_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_col_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_col_node_id);
  Py_CLEAR(clear_module_state->__pyx_kp_u_column);
  Py_CLEAR(clear_module_state->__pyx_n_s_column_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_column_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_column_statistics);
  Py_CLEAR(clear_module_state->__pyx_n_s_columns);
  Py_CLEAR(clear_module_state->__pyx_n_u_columns);
  Py_CLEAR(clear_module_state->__pyx_n_s_compile);
  Py_CLEAR(clear_module_state->__pyx_n_s_complexity);
  Py_CLEAR(clear_module_state->__pyx_n_u_complexity);
  Py_CLEAR(clear_module_state->__pyx_kp_u_concept);
  Py_CLEAR(clear_module_state->__pyx_n_s_concept_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_concept_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_concept_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_concept_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_concepts);
  Py_CLEAR(clear_module_state->__pyx_n_u_concepts);
  Py_CLEAR(clear_module_state->__pyx_n_s_condition_nodes);
  Py_CLEAR(clear_module_state->__pyx_n_s_content);
  Py_CLEAR(clear_module_state->__pyx_n_s_context);
  Py_CLEAR(clear_module_state->__pyx_n_s_context_md);
  Py_CLEAR(clear_module_state->__pyx_n_s_context_prompt);
  Py_CLEAR(clear_module_state->__pyx_n_s_copy);
  Py_CLEAR(clear_module_state->__pyx_n_s_current_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_d);
  Py_CLEAR(clear_module_state->__pyx_kp_u_d_4_d_2_d_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_data);
  Py_CLEAR(clear_module_state->__pyx_n_u_data);
  Py_CLEAR(clear_module_state->__pyx_n_u_data_type);
  Py_CLEAR(clear_module_state->__pyx_n_u_database_objects);
  Py_CLEAR(clear_module_state->__pyx_n_u_database_schema);
  Py_CLEAR(clear_module_state->__pyx_n_u_database_statistics);
  Py_CLEAR(clear_module_state->__pyx_n_u_date);
  Py_CLEAR(clear_module_state->__pyx_n_s_datetime);
  Py_CLEAR(clear_module_state->__pyx_n_u_day);
  Py_CLEAR(clear_module_state->__pyx_n_u_defines);
  Py_CLEAR(clear_module_state->__pyx_n_s_definition);
  Py_CLEAR(clear_module_state->__pyx_n_s_description);
  Py_CLEAR(clear_module_state->__pyx_n_u_description);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict);
  Py_CLEAR(clear_module_state->__pyx_kp_u_disable);
  Py_CLEAR(clear_module_state->__pyx_n_s_doc);
  Py_CLEAR(clear_module_state->__pyx_n_s_domain);
  Py_CLEAR(clear_module_state->__pyx_n_u_domain);
  Py_CLEAR(clear_module_state->__pyx_n_s_domain_results);
  Py_CLEAR(clear_module_state->__pyx_n_s_e);
  Py_CLEAR(clear_module_state->__pyx_n_s_edge_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_edges);
  Py_CLEAR(clear_module_state->__pyx_kp_u_edges_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enable);
  Py_CLEAR(clear_module_state->__pyx_n_s_encoding);
  Py_CLEAR(clear_module_state->__pyx_n_s_end_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_endswith);
  Py_CLEAR(clear_module_state->__pyx_n_s_enhanced_query);
  Py_CLEAR(clear_module_state->__pyx_n_u_enhanced_query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enriched_project_metadata_cleane);
  Py_CLEAR(clear_module_state->__pyx_n_s_enter);
  Py_CLEAR(clear_module_state->__pyx_n_s_enumerate);
  Py_CLEAR(clear_module_state->__pyx_n_s_escape);
  Py_CLEAR(clear_module_state->__pyx_kp_u_example);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_example_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_nodes);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_queries);
  Py_CLEAR(clear_module_state->__pyx_n_s_example_scores);
  Py_CLEAR(clear_module_state->__pyx_n_s_exists);
  Py_CLEAR(clear_module_state->__pyx_n_s_exit);
  Py_CLEAR(clear_module_state->__pyx_n_s_f);
  Py_CLEAR(clear_module_state->__pyx_n_s_file_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_filename);
  Py_CLEAR(clear_module_state->__pyx_n_s_filtered_rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_find);
  Py_CLEAR(clear_module_state->__pyx_n_s_find_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_first_newline);
  Py_CLEAR(clear_module_state->__pyx_n_s_first_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_flags);
  Py_CLEAR(clear_module_state->__pyx_n_s_format_as_markdown);
  Py_CLEAR(clear_module_state->__pyx_n_u_from_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_from_table_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_from_tables);
  Py_CLEAR(clear_module_state->__pyx_kp_u_gc);
  Py_CLEAR(clear_module_state->__pyx_n_s_generate_output);
  Py_CLEAR(clear_module_state->__pyx_n_s_generate_sql);
  Py_CLEAR(clear_module_state->__pyx_n_s_generate_sql_from_nl);
  Py_CLEAR(clear_module_state->__pyx_n_s_genexpr);
  Py_CLEAR(clear_module_state->__pyx_n_s_get);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_alias);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_edge_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_related_rules);
  Py_CLEAR(clear_module_state->__pyx_n_u_get_response);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_schema_summary);
  Py_CLEAR(clear_module_state->__pyx_n_u_governed_by);
  Py_CLEAR(clear_module_state->__pyx_n_s_graph);
  Py_CLEAR(clear_module_state->__pyx_n_u_has_column);
  Py_CLEAR(clear_module_state->__pyx_n_s_has_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_history_context);
  Py_CLEAR(clear_module_state->__pyx_n_u_history_context);
  Py_CLEAR(clear_module_state->__pyx_n_s_i);
  Py_CLEAR(clear_module_state->__pyx_n_u_id);
  Py_CLEAR(clear_module_state->__pyx_n_u_implements);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_u_indexed);
  Py_CLEAR(clear_module_state->__pyx_n_s_init);
  Py_CLEAR(clear_module_state->__pyx_n_s_init_subclass);
  Py_CLEAR(clear_module_state->__pyx_n_s_initializing);
  Py_CLEAR(clear_module_state->__pyx_n_s_inspect);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_user);
  Py_CLEAR(clear_module_state->__pyx_kp_u_isenabled);
  Py_CLEAR(clear_module_state->__pyx_n_s_items);
  Py_CLEAR(clear_module_state->__pyx_n_s_j);
  Py_CLEAR(clear_module_state->__pyx_n_s_join);
  Py_CLEAR(clear_module_state->__pyx_n_s_join_clauses);
  Py_CLEAR(clear_module_state->__pyx_n_s_join_condition);
  Py_CLEAR(clear_module_state->__pyx_n_u_join_condition);
  Py_CLEAR(clear_module_state->__pyx_n_s_join_found);
  Py_CLEAR(clear_module_state->__pyx_n_s_json);
  Py_CLEAR(clear_module_state->__pyx_n_u_json);
  Py_CLEAR(clear_module_state->__pyx_n_s_k);
  Py_CLEAR(clear_module_state->__pyx_n_s_key);
  Py_CLEAR(clear_module_state->__pyx_n_s_keys);
  Py_CLEAR(clear_module_state->__pyx_kp_u_keyword);
  Py_CLEAR(clear_module_state->__pyx_n_s_keyword_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_keyword_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_keyword_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_keywords);
  Py_CLEAR(clear_module_state->__pyx_n_u_keywords);
  Py_CLEAR(clear_module_state->__pyx_n_s_load);
  Py_CLEAR(clear_module_state->__pyx_n_s_load_metadata);
  Py_CLEAR(clear_module_state->__pyx_n_s_loads);
  Py_CLEAR(clear_module_state->__pyx_n_s_lower);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_u_main);
  Py_CLEAR(clear_module_state->__pyx_n_u_markdown);
  Py_CLEAR(clear_module_state->__pyx_n_s_match_reason);
  Py_CLEAR(clear_module_state->__pyx_n_s_match_score);
  Py_CLEAR(clear_module_state->__pyx_n_s_matched);
  Py_CLEAR(clear_module_state->__pyx_n_s_matches);
  Py_CLEAR(clear_module_state->__pyx_n_s_max);
  Py_CLEAR(clear_module_state->__pyx_n_s_md_output);
  Py_CLEAR(clear_module_state->__pyx_n_s_metaclass);
  Py_CLEAR(clear_module_state->__pyx_n_s_metadata);
  Py_CLEAR(clear_module_state->__pyx_n_u_metadata_cleaning);
  Py_CLEAR(clear_module_state->__pyx_n_s_metadata_file);
  Py_CLEAR(clear_module_state->__pyx_n_s_metadata_sections);
  Py_CLEAR(clear_module_state->__pyx_n_s_missing);
  Py_CLEAR(clear_module_state->__pyx_n_s_module);
  Py_CLEAR(clear_module_state->__pyx_n_s_msg);
  Py_CLEAR(clear_module_state->__pyx_n_s_n);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_u_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_name_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_natural_language_query);
  Py_CLEAR(clear_module_state->__pyx_n_s_neighbor);
  Py_CLEAR(clear_module_state->__pyx_n_s_neighbors);
  Py_CLEAR(clear_module_state->__pyx_n_s_networkx);
  Py_CLEAR(clear_module_state->__pyx_n_s_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_matches);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_props);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_score);
  Py_CLEAR(clear_module_state->__pyx_n_s_node_scores);
  Py_CLEAR(clear_module_state->__pyx_n_s_nodes);
  Py_CLEAR(clear_module_state->__pyx_kp_u_nodes_and);
  Py_CLEAR(clear_module_state->__pyx_n_s_normalized_keywords);
  Py_CLEAR(clear_module_state->__pyx_n_s_nx);
  Py_CLEAR(clear_module_state->__pyx_kp_s_nx_DiGraph);
  Py_CLEAR(clear_module_state->__pyx_n_s_open);
  Py_CLEAR(clear_module_state->__pyx_n_s_os);
  Py_CLEAR(clear_module_state->__pyx_n_s_os_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_output);
  Py_CLEAR(clear_module_state->__pyx_n_s_output_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_output_format);
  Py_CLEAR(clear_module_state->__pyx_n_s_parsed);
  Py_CLEAR(clear_module_state->__pyx_n_s_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_pattern);
  Py_CLEAR(clear_module_state->__pyx_n_s_prepare);
  Py_CLEAR(clear_module_state->__pyx_n_s_prev);
  Py_CLEAR(clear_module_state->__pyx_n_s_prev_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_previous_sql);
  Py_CLEAR(clear_module_state->__pyx_n_s_print);
  Py_CLEAR(clear_module_state->__pyx_n_s_process_query);
  Py_CLEAR(clear_module_state->__pyx_n_s_process_query_with_ai);
  Py_CLEAR(clear_module_state->__pyx_n_s_process_query_with_chat_history);
  Py_CLEAR(clear_module_state->__pyx_n_s_prompt);
  Py_CLEAR(clear_module_state->__pyx_n_s_qualname);
  Py_CLEAR(clear_module_state->__pyx_n_s_query);
  Py_CLEAR(clear_module_state->__pyx_n_u_query);
  Py_CLEAR(clear_module_state->__pyx_n_u_query_examples);
  Py_CLEAR(clear_module_state->__pyx_n_s_query_text);
  Py_CLEAR(clear_module_state->__pyx_n_s_r);
  Py_CLEAR(clear_module_state->__pyx_n_u_r);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_re);
  Py_CLEAR(clear_module_state->__pyx_n_u_references);
  Py_CLEAR(clear_module_state->__pyx_n_s_rel);
  Py_CLEAR(clear_module_state->__pyx_n_u_related_business_rules);
  Py_CLEAR(clear_module_state->__pyx_n_u_related_to);
  Py_CLEAR(clear_module_state->__pyx_n_s_relationship);
  Py_CLEAR(clear_module_state->__pyx_n_s_relationship_count);
  Py_CLEAR(clear_module_state->__pyx_n_u_relationships);
  Py_CLEAR(clear_module_state->__pyx_kp_u_relationships_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_replace);
  Py_CLEAR(clear_module_state->__pyx_n_s_required_sections);
  Py_CLEAR(clear_module_state->__pyx_n_s_response);
  Py_CLEAR(clear_module_state->__pyx_n_s_results);
  Py_CLEAR(clear_module_state->__pyx_n_s_return);
  Py_CLEAR(clear_module_state->__pyx_n_s_reverse);
  Py_CLEAR(clear_module_state->__pyx_n_s_role);
  Py_CLEAR(clear_module_state->__pyx_n_s_row_count);
  Py_CLEAR(clear_module_state->__pyx_n_u_row_count);
  Py_CLEAR(clear_module_state->__pyx_kp_u_rule);
  Py_CLEAR(clear_module_state->__pyx_n_s_rule_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_rule_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_rule_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_rule_desc);
  Py_CLEAR(clear_module_state->__pyx_n_s_rule_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_rule_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_rules);
  Py_CLEAR(clear_module_state->__pyx_n_s_run);
  Py_CLEAR(clear_module_state->__pyx_n_s_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_sQQideR_DatabaseContext);
  Py_CLEAR(clear_module_state->__pyx_kp_s_sQQideR_DatabaseContext_pyx);
  Py_CLEAR(clear_module_state->__pyx_kp_u_s_LIMIT_s_d_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_schema);
  Py_CLEAR(clear_module_state->__pyx_n_s_score);
  Py_CLEAR(clear_module_state->__pyx_n_s_search);
  Py_CLEAR(clear_module_state->__pyx_n_s_search_by_keywords);
  Py_CLEAR(clear_module_state->__pyx_n_s_select_clause);
  Py_CLEAR(clear_module_state->__pyx_n_s_selected_query);
  Py_CLEAR(clear_module_state->__pyx_n_s_self);
  Py_CLEAR(clear_module_state->__pyx_n_s_send);
  Py_CLEAR(clear_module_state->__pyx_n_s_service);
  Py_CLEAR(clear_module_state->__pyx_n_s_set_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_shortest_path);
  Py_CLEAR(clear_module_state->__pyx_n_u_simple);
  Py_CLEAR(clear_module_state->__pyx_n_s_sorted);
  Py_CLEAR(clear_module_state->__pyx_n_s_sorted_nodes);
  Py_CLEAR(clear_module_state->__pyx_n_u_source);
  Py_CLEAR(clear_module_state->__pyx_n_s_spec);
  Py_CLEAR(clear_module_state->__pyx_n_s_split);
  Py_CLEAR(clear_module_state->__pyx_n_s_sql);
  Py_CLEAR(clear_module_state->__pyx_n_s_sql_lines);
  Py_CLEAR(clear_module_state->__pyx_n_u_src);
  Py_CLEAR(clear_module_state->__pyx_n_s_src_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_start_node);
  Py_CLEAR(clear_module_state->__pyx_n_s_startswith);
  Py_CLEAR(clear_module_state->__pyx_n_u_statistics);
  Py_CLEAR(clear_module_state->__pyx_n_s_str);
  Py_CLEAR(clear_module_state->__pyx_n_s_strip);
  Py_CLEAR(clear_module_state->__pyx_n_s_sub);
  Py_CLEAR(clear_module_state->__pyx_n_s_suffix);
  Py_CLEAR(clear_module_state->__pyx_n_s_super);
  Py_CLEAR(clear_module_state->__pyx_n_s_t);
  Py_CLEAR(clear_module_state->__pyx_kp_u_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_table_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_count);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_data);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_names);
  Py_CLEAR(clear_module_state->__pyx_n_s_table_nodes);
  Py_CLEAR(clear_module_state->__pyx_n_s_tables);
  Py_CLEAR(clear_module_state->__pyx_n_u_tables);
  Py_CLEAR(clear_module_state->__pyx_kp_u_tables_and);
  Py_CLEAR(clear_module_state->__pyx_n_u_tags);
  Py_CLEAR(clear_module_state->__pyx_n_s_target);
  Py_CLEAR(clear_module_state->__pyx_n_u_target);
  Py_CLEAR(clear_module_state->__pyx_n_s_targets);
  Py_CLEAR(clear_module_state->__pyx_n_s_tbl);
  Py_CLEAR(clear_module_state->__pyx_n_s_template_query);
  Py_CLEAR(clear_module_state->__pyx_kp_u_term);
  Py_CLEAR(clear_module_state->__pyx_n_s_term_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_term_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_term_id);
  Py_CLEAR(clear_module_state->__pyx_n_u_terminology);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_tgt_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_throw);
  Py_CLEAR(clear_module_state->__pyx_n_u_to_table);
  Py_CLEAR(clear_module_state->__pyx_n_s_to_table_id);
  Py_CLEAR(clear_module_state->__pyx_n_s_type);
  Py_CLEAR(clear_module_state->__pyx_n_u_type);
  Py_CLEAR(clear_module_state->__pyx_n_s_typing);
  Py_CLEAR(clear_module_state->__pyx_n_s_unique_rules);
  Py_CLEAR(clear_module_state->__pyx_n_u_unknown);
  Py_CLEAR(clear_module_state->__pyx_n_s_used_aliases);
  Py_CLEAR(clear_module_state->__pyx_n_s_user_prompt);
  Py_CLEAR(clear_module_state->__pyx_kp_u_utf_8);
  Py_CLEAR(clear_module_state->__pyx_n_s_values);
  Py_CLEAR(clear_module_state->__pyx_n_s_verbose);
  Py_CLEAR(clear_module_state->__pyx_kp_u_version);
  Py_CLEAR(clear_module_state->__pyx_n_s_version_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_where_clauses);
  Py_CLEAR(clear_module_state->__pyx_n_s_word);
  Py_CLEAR(clear_module_state->__pyx_n_s_x);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_2);
  Py_CLEAR(clear_module_state->__pyx_int_3);
  Py_CLEAR(clear_module_state->__pyx_int_4);
  Py_CLEAR(clear_module_state->__pyx_int_5);
  Py_CLEAR(clear_module_state->__pyx_int_neg_1);
  Py_CLEAR(clear_module_state->__pyx_int_neg_3);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__3);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_slice__11);
  Py_CLEAR(clear_module_state->__pyx_tuple__13);
  Py_CLEAR(clear_module_state->__pyx_tuple__14);
  Py_CLEAR(clear_module_state->__pyx_tuple__16);
  Py_CLEAR(clear_module_state->__pyx_tuple__17);
  Py_CLEAR(clear_module_state->__pyx_tuple__18);
  Py_CLEAR(clear_module_state->__pyx_tuple__19);
  Py_CLEAR(clear_module_state->__pyx_tuple__21);
  Py_CLEAR(clear_module_state->__pyx_tuple__22);
  Py_CLEAR(clear_module_state->__pyx_tuple__24);
  Py_CLEAR(clear_module_state->__pyx_tuple__26);
  Py_CLEAR(clear_module_state->__pyx_tuple__30);
  Py_CLEAR(clear_module_state->__pyx_tuple__32);
  Py_CLEAR(clear_module_state->__pyx_tuple__33);
  Py_CLEAR(clear_module_state->__pyx_tuple__34);
  Py_CLEAR(clear_module_state->__pyx_tuple__46);
  Py_CLEAR(clear_module_state->__pyx_tuple__47);
  Py_CLEAR(clear_module_state->__pyx_tuple__49);
  Py_CLEAR(clear_module_state->__pyx_tuple__50);
  Py_CLEAR(clear_module_state->__pyx_tuple__52);
  Py_CLEAR(clear_module_state->__pyx_tuple__53);
  Py_CLEAR(clear_module_state->__pyx_tuple__56);
  Py_CLEAR(clear_module_state->__pyx_tuple__58);
  Py_CLEAR(clear_module_state->__pyx_tuple__59);
  Py_CLEAR(clear_module_state->__pyx_tuple__61);
  Py_CLEAR(clear_module_state->__pyx_tuple__62);
  Py_CLEAR(clear_module_state->__pyx_tuple__64);
  Py_CLEAR(clear_module_state->__pyx_tuple__65);
  Py_CLEAR(clear_module_state->__pyx_tuple__67);
  Py_CLEAR(clear_module_state->__pyx_tuple__69);
  Py_CLEAR(clear_module_state->__pyx_tuple__71);
  Py_CLEAR(clear_module_state->__pyx_tuple__73);
  Py_CLEAR(clear_module_state->__pyx_tuple__75);
  Py_CLEAR(clear_module_state->__pyx_tuple__77);
  Py_CLEAR(clear_module_state->__pyx_tuple__78);
  Py_CLEAR(clear_module_state->__pyx_tuple__80);
  Py_CLEAR(clear_module_state->__pyx_tuple__81);
  Py_CLEAR(clear_module_state->__pyx_tuple__83);
  Py_CLEAR(clear_module_state->__pyx_tuple__85);
  Py_CLEAR(clear_module_state->__pyx_tuple__86);
  Py_CLEAR(clear_module_state->__pyx_tuple__87);
  Py_CLEAR(clear_module_state->__pyx_tuple__88);
  Py_CLEAR(clear_module_state->__pyx_tuple__89);
  Py_CLEAR(clear_module_state->__pyx_tuple__90);
  Py_CLEAR(clear_module_state->__pyx_codeobj__25);
  Py_CLEAR(clear_module_state->__pyx_codeobj__51);
  Py_CLEAR(clear_module_state->__pyx_codeobj__54);
  Py_CLEAR(clear_module_state->__pyx_codeobj__57);
  Py_CLEAR(clear_module_state->__pyx_codeobj__60);
  Py_CLEAR(clear_module_state->__pyx_codeobj__63);
  Py_CLEAR(clear_module_state->__pyx_codeobj__66);
  Py_CLEAR(clear_module_state->__pyx_codeobj__68);
  Py_CLEAR(clear_module_state->__pyx_codeobj__70);
  Py_CLEAR(clear_module_state->__pyx_codeobj__72);
  Py_CLEAR(clear_module_state->__pyx_codeobj__74);
  Py_CLEAR(clear_module_state->__pyx_codeobj__76);
  Py_CLEAR(clear_module_state->__pyx_codeobj__79);
  Py_CLEAR(clear_module_state->__pyx_codeobj__82);
  Py_CLEAR(clear_module_state->__pyx_codeobj__84);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords);
  Py_VISIT(traverse_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords);
  Py_VISIT(traverse_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr);
  Py_VISIT(traverse_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql);
  Py_VISIT(traverse_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql);
  Py_VISIT(traverse_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl);
  Py_VISIT(traverse_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl);
  Py_VISIT(traverse_module_state->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr);
  Py_VISIT(traverse_module_state->__pyx_kp_u_1_Extract_the_key_business_doma);
  Py_VISIT(traverse_module_state->__pyx_kp_u_1_Extract_the_key_business_doma_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_Enhanced_Query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_SQL_generation_failed);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_enhanced_query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_extracted_keywords);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_history_context);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AI_identified_domains);
  Py_VISIT(traverse_module_state->__pyx_kp_u_AND);
  Py_VISIT(traverse_module_state->__pyx_n_s_Any);
  Py_VISIT(traverse_module_state->__pyx_n_u_Assistant);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Attempting_to_load_enriched_proj);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Business_Concepts);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Business_Domains);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Business_Rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_ChatMessage);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Cleaned_response);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Column_Data_Type_Description);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Consider_this_conversation_hist);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService___init);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService__get_sche);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_build_gra);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_find_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_format_as);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate_3);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate_4);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate_5);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_generate_6);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_get_relat);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_load_meta);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_process_q);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_process_q_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_process_q_3);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_search_by);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_search_by_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_DatabaseContextService_search_by_3);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Database_Context_for_Query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Description);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Description_contains);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Description_partially_matches);
  Py_VISIT(traverse_module_state->__pyx_n_s_DiGraph);
  Py_VISIT(traverse_module_state->__pyx_n_s_Dict);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Error_enhancing_query_with_AI);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Error_parsing_AI_response);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Extracted_keywords);
  Py_VISIT(traverse_module_state->__pyx_kp_u_FROM);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Failed_to_load_metadata_file_Ple);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Falling_back_to_rule_based_appr);
  Py_VISIT(traverse_module_state->__pyx_n_s_FileNotFoundError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Find_activities_with_excessive_l);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Found);
  Py_VISIT(traverse_module_state->__pyx_n_s_G);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Generated_SQL);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Graph_built_with);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Graph_not_built_Call_build_graph);
  Py_VISIT(traverse_module_state->__pyx_kp_u_ID_contains);
  Py_VISIT(traverse_module_state->__pyx_n_s_IGNORECASE);
  Py_VISIT(traverse_module_state->__pyx_kp_u_JOIN);
  Py_VISIT(traverse_module_state->__pyx_n_s_JSONDecodeError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Join_condition);
  Py_VISIT(traverse_module_state->__pyx_n_s_List);
  Py_VISIT(traverse_module_state->__pyx_kp_s_List_ChatMessage);
  Py_VISIT(traverse_module_state->__pyx_kp_s_List_Tuple_str_str);
  Py_VISIT(traverse_module_state->__pyx_kp_u_List_relationships_with_lag_grea);
  Py_VISIT(traverse_module_state->__pyx_kp_s_List_str);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Loaded_metadata_from);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Markdown_Output);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Metadata_contains_sections);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Metadata_file_not_found);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Metadata_was_cleaned_at);
  Py_VISIT(traverse_module_state->__pyx_n_s_Models);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Name_contains);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Name_partially_matches);
  Py_VISIT(traverse_module_state->__pyx_n_s_NetworkXNoPath);
  Py_VISIT(traverse_module_state->__pyx_kp_u_No_SQL_generated);
  Py_VISIT(traverse_module_state->__pyx_kp_u_No_metadata_provided_Call_load_m);
  Py_VISIT(traverse_module_state->__pyx_kp_u_No_tables_selected_for_query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_None);
  Py_VISIT(traverse_module_state->__pyx_kp_u_ON);
  Py_VISIT(traverse_module_state->__pyx_kp_u_ON_Missing_join_condition_verif);
  Py_VISIT(traverse_module_state->__pyx_n_s_Optional);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Optional_List_str);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Processing_query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Raw_response);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Relevant_Columns);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Relevant_Tables);
  Py_VISIT(traverse_module_state->__pyx_kp_u_SELECT);
  Py_VISIT(traverse_module_state->__pyx_n_s_SQLPrompt);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Service_for_providing_database);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Show_all_projects_with_negative);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Table_Relationships);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Tagged_with_keyword);
  Py_VISIT(traverse_module_state->__pyx_kp_u_The_previous_SQL_query_was);
  Py_VISIT(traverse_module_state->__pyx_n_s_Tuple);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_Union);
  Py_VISIT(traverse_module_state->__pyx_n_u_User);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Using_basic_keyword_extraction);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Using_template_query);
  Py_VISIT(traverse_module_state->__pyx_n_s_ValueError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_WARNING_Missing_required_sectio);
  Py_VISIT(traverse_module_state->__pyx_kp_u_WHERE);
  Py_VISIT(traverse_module_state->__pyx_kp_u_What_are_the_activities_on_the_c);
  Py_VISIT(traverse_module_state->__pyx_kp_u_You_are_an_expert_in_project_ma);
  Py_VISIT(traverse_module_state->__pyx_kp_u__10);
  Py_VISIT(traverse_module_state->__pyx_kp_u__12);
  Py_VISIT(traverse_module_state->__pyx_kp_u__15);
  Py_VISIT(traverse_module_state->__pyx_kp_u__2);
  Py_VISIT(traverse_module_state->__pyx_kp_u__20);
  Py_VISIT(traverse_module_state->__pyx_kp_u__23);
  Py_VISIT(traverse_module_state->__pyx_kp_u__27);
  Py_VISIT(traverse_module_state->__pyx_kp_u__28);
  Py_VISIT(traverse_module_state->__pyx_kp_u__29);
  Py_VISIT(traverse_module_state->__pyx_n_s__31);
  Py_VISIT(traverse_module_state->__pyx_kp_u__31);
  Py_VISIT(traverse_module_state->__pyx_kp_u__35);
  Py_VISIT(traverse_module_state->__pyx_kp_u__36);
  Py_VISIT(traverse_module_state->__pyx_kp_u__37);
  Py_VISIT(traverse_module_state->__pyx_kp_u__38);
  Py_VISIT(traverse_module_state->__pyx_kp_u__39);
  Py_VISIT(traverse_module_state->__pyx_kp_u__40);
  Py_VISIT(traverse_module_state->__pyx_kp_u__41);
  Py_VISIT(traverse_module_state->__pyx_kp_u__42);
  Py_VISIT(traverse_module_state->__pyx_kp_u__43);
  Py_VISIT(traverse_module_state->__pyx_kp_u__44);
  Py_VISIT(traverse_module_state->__pyx_kp_u__45);
  Py_VISIT(traverse_module_state->__pyx_n_s__48);
  Py_VISIT(traverse_module_state->__pyx_kp_u__5);
  Py_VISIT(traverse_module_state->__pyx_n_s__55);
  Py_VISIT(traverse_module_state->__pyx_kp_u__7);
  Py_VISIT(traverse_module_state->__pyx_kp_u__8);
  Py_VISIT(traverse_module_state->__pyx_kp_u__9);
  Py_VISIT(traverse_module_state->__pyx_n_s_adapted_query);
  Py_VISIT(traverse_module_state->__pyx_n_s_add_edge);
  Py_VISIT(traverse_module_state->__pyx_n_s_add_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_ai_base);
  Py_VISIT(traverse_module_state->__pyx_n_s_ai_context);
  Py_VISIT(traverse_module_state->__pyx_n_u_ai_enhanced);
  Py_VISIT(traverse_module_state->__pyx_n_s_alias);
  Py_VISIT(traverse_module_state->__pyx_n_s_aliases);
  Py_VISIT(traverse_module_state->__pyx_n_s_all_rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_all_tables);
  Py_VISIT(traverse_module_state->__pyx_n_s_append);
  Py_VISIT(traverse_module_state->__pyx_n_s_args);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_tasks);
  Py_VISIT(traverse_module_state->__pyx_kp_u_b);
  Py_VISIT(traverse_module_state->__pyx_n_s_base);
  Py_VISIT(traverse_module_state->__pyx_n_s_best_example);
  Py_VISIT(traverse_module_state->__pyx_n_u_best_guess_sql);
  Py_VISIT(traverse_module_state->__pyx_n_s_bool);
  Py_VISIT(traverse_module_state->__pyx_n_s_build_graph);
  Py_VISIT(traverse_module_state->__pyx_n_s_buisness_domains);
  Py_VISIT(traverse_module_state->__pyx_n_u_business_concepts);
  Py_VISIT(traverse_module_state->__pyx_n_u_business_domains);
  Py_VISIT(traverse_module_state->__pyx_n_u_business_rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_c);
  Py_VISIT(traverse_module_state->__pyx_n_s_chat_history);
  Py_VISIT(traverse_module_state->__pyx_n_u_chat_history_context);
  Py_VISIT(traverse_module_state->__pyx_n_s_class_getitem);
  Py_VISIT(traverse_module_state->__pyx_n_s_clean_response);
  Py_VISIT(traverse_module_state->__pyx_n_s_cleaned_at);
  Py_VISIT(traverse_module_state->__pyx_n_u_cleaned_at);
  Py_VISIT(traverse_module_state->__pyx_n_u_cleaning_version);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_col_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_col_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_col_node_id);
  Py_VISIT(traverse_module_state->__pyx_kp_u_column);
  Py_VISIT(traverse_module_state->__pyx_n_s_column_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_column_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_column_statistics);
  Py_VISIT(traverse_module_state->__pyx_n_s_columns);
  Py_VISIT(traverse_module_state->__pyx_n_u_columns);
  Py_VISIT(traverse_module_state->__pyx_n_s_compile);
  Py_VISIT(traverse_module_state->__pyx_n_s_complexity);
  Py_VISIT(traverse_module_state->__pyx_n_u_complexity);
  Py_VISIT(traverse_module_state->__pyx_kp_u_concept);
  Py_VISIT(traverse_module_state->__pyx_n_s_concept_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_concept_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_concept_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_concept_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_concepts);
  Py_VISIT(traverse_module_state->__pyx_n_u_concepts);
  Py_VISIT(traverse_module_state->__pyx_n_s_condition_nodes);
  Py_VISIT(traverse_module_state->__pyx_n_s_content);
  Py_VISIT(traverse_module_state->__pyx_n_s_context);
  Py_VISIT(traverse_module_state->__pyx_n_s_context_md);
  Py_VISIT(traverse_module_state->__pyx_n_s_context_prompt);
  Py_VISIT(traverse_module_state->__pyx_n_s_copy);
  Py_VISIT(traverse_module_state->__pyx_n_s_current_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_d);
  Py_VISIT(traverse_module_state->__pyx_kp_u_d_4_d_2_d_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_data);
  Py_VISIT(traverse_module_state->__pyx_n_u_data);
  Py_VISIT(traverse_module_state->__pyx_n_u_data_type);
  Py_VISIT(traverse_module_state->__pyx_n_u_database_objects);
  Py_VISIT(traverse_module_state->__pyx_n_u_database_schema);
  Py_VISIT(traverse_module_state->__pyx_n_u_database_statistics);
  Py_VISIT(traverse_module_state->__pyx_n_u_date);
  Py_VISIT(traverse_module_state->__pyx_n_s_datetime);
  Py_VISIT(traverse_module_state->__pyx_n_u_day);
  Py_VISIT(traverse_module_state->__pyx_n_u_defines);
  Py_VISIT(traverse_module_state->__pyx_n_s_definition);
  Py_VISIT(traverse_module_state->__pyx_n_s_description);
  Py_VISIT(traverse_module_state->__pyx_n_u_description);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict);
  Py_VISIT(traverse_module_state->__pyx_kp_u_disable);
  Py_VISIT(traverse_module_state->__pyx_n_s_doc);
  Py_VISIT(traverse_module_state->__pyx_n_s_domain);
  Py_VISIT(traverse_module_state->__pyx_n_u_domain);
  Py_VISIT(traverse_module_state->__pyx_n_s_domain_results);
  Py_VISIT(traverse_module_state->__pyx_n_s_e);
  Py_VISIT(traverse_module_state->__pyx_n_s_edge_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_edges);
  Py_VISIT(traverse_module_state->__pyx_kp_u_edges_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enable);
  Py_VISIT(traverse_module_state->__pyx_n_s_encoding);
  Py_VISIT(traverse_module_state->__pyx_n_s_end_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_endswith);
  Py_VISIT(traverse_module_state->__pyx_n_s_enhanced_query);
  Py_VISIT(traverse_module_state->__pyx_n_u_enhanced_query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enriched_project_metadata_cleane);
  Py_VISIT(traverse_module_state->__pyx_n_s_enter);
  Py_VISIT(traverse_module_state->__pyx_n_s_enumerate);
  Py_VISIT(traverse_module_state->__pyx_n_s_escape);
  Py_VISIT(traverse_module_state->__pyx_kp_u_example);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_example_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_nodes);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_queries);
  Py_VISIT(traverse_module_state->__pyx_n_s_example_scores);
  Py_VISIT(traverse_module_state->__pyx_n_s_exists);
  Py_VISIT(traverse_module_state->__pyx_n_s_exit);
  Py_VISIT(traverse_module_state->__pyx_n_s_f);
  Py_VISIT(traverse_module_state->__pyx_n_s_file_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_filename);
  Py_VISIT(traverse_module_state->__pyx_n_s_filtered_rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_find);
  Py_VISIT(traverse_module_state->__pyx_n_s_find_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_first_newline);
  Py_VISIT(traverse_module_state->__pyx_n_s_first_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_flags);
  Py_VISIT(traverse_module_state->__pyx_n_s_format_as_markdown);
  Py_VISIT(traverse_module_state->__pyx_n_u_from_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_from_table_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_from_tables);
  Py_VISIT(traverse_module_state->__pyx_kp_u_gc);
  Py_VISIT(traverse_module_state->__pyx_n_s_generate_output);
  Py_VISIT(traverse_module_state->__pyx_n_s_generate_sql);
  Py_VISIT(traverse_module_state->__pyx_n_s_generate_sql_from_nl);
  Py_VISIT(traverse_module_state->__pyx_n_s_genexpr);
  Py_VISIT(traverse_module_state->__pyx_n_s_get);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_alias);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_edge_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_related_rules);
  Py_VISIT(traverse_module_state->__pyx_n_u_get_response);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_schema_summary);
  Py_VISIT(traverse_module_state->__pyx_n_u_governed_by);
  Py_VISIT(traverse_module_state->__pyx_n_s_graph);
  Py_VISIT(traverse_module_state->__pyx_n_u_has_column);
  Py_VISIT(traverse_module_state->__pyx_n_s_has_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_history_context);
  Py_VISIT(traverse_module_state->__pyx_n_u_history_context);
  Py_VISIT(traverse_module_state->__pyx_n_s_i);
  Py_VISIT(traverse_module_state->__pyx_n_u_id);
  Py_VISIT(traverse_module_state->__pyx_n_u_implements);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_u_indexed);
  Py_VISIT(traverse_module_state->__pyx_n_s_init);
  Py_VISIT(traverse_module_state->__pyx_n_s_init_subclass);
  Py_VISIT(traverse_module_state->__pyx_n_s_initializing);
  Py_VISIT(traverse_module_state->__pyx_n_s_inspect);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_user);
  Py_VISIT(traverse_module_state->__pyx_kp_u_isenabled);
  Py_VISIT(traverse_module_state->__pyx_n_s_items);
  Py_VISIT(traverse_module_state->__pyx_n_s_j);
  Py_VISIT(traverse_module_state->__pyx_n_s_join);
  Py_VISIT(traverse_module_state->__pyx_n_s_join_clauses);
  Py_VISIT(traverse_module_state->__pyx_n_s_join_condition);
  Py_VISIT(traverse_module_state->__pyx_n_u_join_condition);
  Py_VISIT(traverse_module_state->__pyx_n_s_join_found);
  Py_VISIT(traverse_module_state->__pyx_n_s_json);
  Py_VISIT(traverse_module_state->__pyx_n_u_json);
  Py_VISIT(traverse_module_state->__pyx_n_s_k);
  Py_VISIT(traverse_module_state->__pyx_n_s_key);
  Py_VISIT(traverse_module_state->__pyx_n_s_keys);
  Py_VISIT(traverse_module_state->__pyx_kp_u_keyword);
  Py_VISIT(traverse_module_state->__pyx_n_s_keyword_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_keyword_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_keyword_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_keywords);
  Py_VISIT(traverse_module_state->__pyx_n_u_keywords);
  Py_VISIT(traverse_module_state->__pyx_n_s_load);
  Py_VISIT(traverse_module_state->__pyx_n_s_load_metadata);
  Py_VISIT(traverse_module_state->__pyx_n_s_loads);
  Py_VISIT(traverse_module_state->__pyx_n_s_lower);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_u_main);
  Py_VISIT(traverse_module_state->__pyx_n_u_markdown);
  Py_VISIT(traverse_module_state->__pyx_n_s_match_reason);
  Py_VISIT(traverse_module_state->__pyx_n_s_match_score);
  Py_VISIT(traverse_module_state->__pyx_n_s_matched);
  Py_VISIT(traverse_module_state->__pyx_n_s_matches);
  Py_VISIT(traverse_module_state->__pyx_n_s_max);
  Py_VISIT(traverse_module_state->__pyx_n_s_md_output);
  Py_VISIT(traverse_module_state->__pyx_n_s_metaclass);
  Py_VISIT(traverse_module_state->__pyx_n_s_metadata);
  Py_VISIT(traverse_module_state->__pyx_n_u_metadata_cleaning);
  Py_VISIT(traverse_module_state->__pyx_n_s_metadata_file);
  Py_VISIT(traverse_module_state->__pyx_n_s_metadata_sections);
  Py_VISIT(traverse_module_state->__pyx_n_s_missing);
  Py_VISIT(traverse_module_state->__pyx_n_s_module);
  Py_VISIT(traverse_module_state->__pyx_n_s_msg);
  Py_VISIT(traverse_module_state->__pyx_n_s_n);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_u_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_name_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_natural_language_query);
  Py_VISIT(traverse_module_state->__pyx_n_s_neighbor);
  Py_VISIT(traverse_module_state->__pyx_n_s_neighbors);
  Py_VISIT(traverse_module_state->__pyx_n_s_networkx);
  Py_VISIT(traverse_module_state->__pyx_n_s_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_matches);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_props);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_score);
  Py_VISIT(traverse_module_state->__pyx_n_s_node_scores);
  Py_VISIT(traverse_module_state->__pyx_n_s_nodes);
  Py_VISIT(traverse_module_state->__pyx_kp_u_nodes_and);
  Py_VISIT(traverse_module_state->__pyx_n_s_normalized_keywords);
  Py_VISIT(traverse_module_state->__pyx_n_s_nx);
  Py_VISIT(traverse_module_state->__pyx_kp_s_nx_DiGraph);
  Py_VISIT(traverse_module_state->__pyx_n_s_open);
  Py_VISIT(traverse_module_state->__pyx_n_s_os);
  Py_VISIT(traverse_module_state->__pyx_n_s_os_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_output);
  Py_VISIT(traverse_module_state->__pyx_n_s_output_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_output_format);
  Py_VISIT(traverse_module_state->__pyx_n_s_parsed);
  Py_VISIT(traverse_module_state->__pyx_n_s_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_pattern);
  Py_VISIT(traverse_module_state->__pyx_n_s_prepare);
  Py_VISIT(traverse_module_state->__pyx_n_s_prev);
  Py_VISIT(traverse_module_state->__pyx_n_s_prev_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_previous_sql);
  Py_VISIT(traverse_module_state->__pyx_n_s_print);
  Py_VISIT(traverse_module_state->__pyx_n_s_process_query);
  Py_VISIT(traverse_module_state->__pyx_n_s_process_query_with_ai);
  Py_VISIT(traverse_module_state->__pyx_n_s_process_query_with_chat_history);
  Py_VISIT(traverse_module_state->__pyx_n_s_prompt);
  Py_VISIT(traverse_module_state->__pyx_n_s_qualname);
  Py_VISIT(traverse_module_state->__pyx_n_s_query);
  Py_VISIT(traverse_module_state->__pyx_n_u_query);
  Py_VISIT(traverse_module_state->__pyx_n_u_query_examples);
  Py_VISIT(traverse_module_state->__pyx_n_s_query_text);
  Py_VISIT(traverse_module_state->__pyx_n_s_r);
  Py_VISIT(traverse_module_state->__pyx_n_u_r);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_re);
  Py_VISIT(traverse_module_state->__pyx_n_u_references);
  Py_VISIT(traverse_module_state->__pyx_n_s_rel);
  Py_VISIT(traverse_module_state->__pyx_n_u_related_business_rules);
  Py_VISIT(traverse_module_state->__pyx_n_u_related_to);
  Py_VISIT(traverse_module_state->__pyx_n_s_relationship);
  Py_VISIT(traverse_module_state->__pyx_n_s_relationship_count);
  Py_VISIT(traverse_module_state->__pyx_n_u_relationships);
  Py_VISIT(traverse_module_state->__pyx_kp_u_relationships_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_replace);
  Py_VISIT(traverse_module_state->__pyx_n_s_required_sections);
  Py_VISIT(traverse_module_state->__pyx_n_s_response);
  Py_VISIT(traverse_module_state->__pyx_n_s_results);
  Py_VISIT(traverse_module_state->__pyx_n_s_return);
  Py_VISIT(traverse_module_state->__pyx_n_s_reverse);
  Py_VISIT(traverse_module_state->__pyx_n_s_role);
  Py_VISIT(traverse_module_state->__pyx_n_s_row_count);
  Py_VISIT(traverse_module_state->__pyx_n_u_row_count);
  Py_VISIT(traverse_module_state->__pyx_kp_u_rule);
  Py_VISIT(traverse_module_state->__pyx_n_s_rule_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_rule_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_rule_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_rule_desc);
  Py_VISIT(traverse_module_state->__pyx_n_s_rule_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_rule_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_rules);
  Py_VISIT(traverse_module_state->__pyx_n_s_run);
  Py_VISIT(traverse_module_state->__pyx_n_s_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_sQQideR_DatabaseContext);
  Py_VISIT(traverse_module_state->__pyx_kp_s_sQQideR_DatabaseContext_pyx);
  Py_VISIT(traverse_module_state->__pyx_kp_u_s_LIMIT_s_d_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_schema);
  Py_VISIT(traverse_module_state->__pyx_n_s_score);
  Py_VISIT(traverse_module_state->__pyx_n_s_search);
  Py_VISIT(traverse_module_state->__pyx_n_s_search_by_keywords);
  Py_VISIT(traverse_module_state->__pyx_n_s_select_clause);
  Py_VISIT(traverse_module_state->__pyx_n_s_selected_query);
  Py_VISIT(traverse_module_state->__pyx_n_s_self);
  Py_VISIT(traverse_module_state->__pyx_n_s_send);
  Py_VISIT(traverse_module_state->__pyx_n_s_service);
  Py_VISIT(traverse_module_state->__pyx_n_s_set_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_shortest_path);
  Py_VISIT(traverse_module_state->__pyx_n_u_simple);
  Py_VISIT(traverse_module_state->__pyx_n_s_sorted);
  Py_VISIT(traverse_module_state->__pyx_n_s_sorted_nodes);
  Py_VISIT(traverse_module_state->__pyx_n_u_source);
  Py_VISIT(traverse_module_state->__pyx_n_s_spec);
  Py_VISIT(traverse_module_state->__pyx_n_s_split);
  Py_VISIT(traverse_module_state->__pyx_n_s_sql);
  Py_VISIT(traverse_module_state->__pyx_n_s_sql_lines);
  Py_VISIT(traverse_module_state->__pyx_n_u_src);
  Py_VISIT(traverse_module_state->__pyx_n_s_src_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_start_node);
  Py_VISIT(traverse_module_state->__pyx_n_s_startswith);
  Py_VISIT(traverse_module_state->__pyx_n_u_statistics);
  Py_VISIT(traverse_module_state->__pyx_n_s_str);
  Py_VISIT(traverse_module_state->__pyx_n_s_strip);
  Py_VISIT(traverse_module_state->__pyx_n_s_sub);
  Py_VISIT(traverse_module_state->__pyx_n_s_suffix);
  Py_VISIT(traverse_module_state->__pyx_n_s_super);
  Py_VISIT(traverse_module_state->__pyx_n_s_t);
  Py_VISIT(traverse_module_state->__pyx_kp_u_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_table_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_count);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_data);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_names);
  Py_VISIT(traverse_module_state->__pyx_n_s_table_nodes);
  Py_VISIT(traverse_module_state->__pyx_n_s_tables);
  Py_VISIT(traverse_module_state->__pyx_n_u_tables);
  Py_VISIT(traverse_module_state->__pyx_kp_u_tables_and);
  Py_VISIT(traverse_module_state->__pyx_n_u_tags);
  Py_VISIT(traverse_module_state->__pyx_n_s_target);
  Py_VISIT(traverse_module_state->__pyx_n_u_target);
  Py_VISIT(traverse_module_state->__pyx_n_s_targets);
  Py_VISIT(traverse_module_state->__pyx_n_s_tbl);
  Py_VISIT(traverse_module_state->__pyx_n_s_template_query);
  Py_VISIT(traverse_module_state->__pyx_kp_u_term);
  Py_VISIT(traverse_module_state->__pyx_n_s_term_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_term_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_term_id);
  Py_VISIT(traverse_module_state->__pyx_n_u_terminology);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_tgt_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_throw);
  Py_VISIT(traverse_module_state->__pyx_n_u_to_table);
  Py_VISIT(traverse_module_state->__pyx_n_s_to_table_id);
  Py_VISIT(traverse_module_state->__pyx_n_s_type);
  Py_VISIT(traverse_module_state->__pyx_n_u_type);
  Py_VISIT(traverse_module_state->__pyx_n_s_typing);
  Py_VISIT(traverse_module_state->__pyx_n_s_unique_rules);
  Py_VISIT(traverse_module_state->__pyx_n_u_unknown);
  Py_VISIT(traverse_module_state->__pyx_n_s_used_aliases);
  Py_VISIT(traverse_module_state->__pyx_n_s_user_prompt);
  Py_VISIT(traverse_module_state->__pyx_kp_u_utf_8);
  Py_VISIT(traverse_module_state->__pyx_n_s_values);
  Py_VISIT(traverse_module_state->__pyx_n_s_verbose);
  Py_VISIT(traverse_module_state->__pyx_kp_u_version);
  Py_VISIT(traverse_module_state->__pyx_n_s_version_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_where_clauses);
  Py_VISIT(traverse_module_state->__pyx_n_s_word);
  Py_VISIT(traverse_module_state->__pyx_n_s_x);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_1);
  Py_VISIT(traverse_module_state->__pyx_int_2);
  Py_VISIT(traverse_module_state->__pyx_int_3);
  Py_VISIT(traverse_module_state->__pyx_int_4);
  Py_VISIT(traverse_module_state->__pyx_int_5);
  Py_VISIT(traverse_module_state->__pyx_int_neg_1);
  Py_VISIT(traverse_module_state->__pyx_int_neg_3);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__3);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_slice__11);
  Py_VISIT(traverse_module_state->__pyx_tuple__13);
  Py_VISIT(traverse_module_state->__pyx_tuple__14);
  Py_VISIT(traverse_module_state->__pyx_tuple__16);
  Py_VISIT(traverse_module_state->__pyx_tuple__17);
  Py_VISIT(traverse_module_state->__pyx_tuple__18);
  Py_VISIT(traverse_module_state->__pyx_tuple__19);
  Py_VISIT(traverse_module_state->__pyx_tuple__21);
  Py_VISIT(traverse_module_state->__pyx_tuple__22);
  Py_VISIT(traverse_module_state->__pyx_tuple__24);
  Py_VISIT(traverse_module_state->__pyx_tuple__26);
  Py_VISIT(traverse_module_state->__pyx_tuple__30);
  Py_VISIT(traverse_module_state->__pyx_tuple__32);
  Py_VISIT(traverse_module_state->__pyx_tuple__33);
  Py_VISIT(traverse_module_state->__pyx_tuple__34);
  Py_VISIT(traverse_module_state->__pyx_tuple__46);
  Py_VISIT(traverse_module_state->__pyx_tuple__47);
  Py_VISIT(traverse_module_state->__pyx_tuple__49);
  Py_VISIT(traverse_module_state->__pyx_tuple__50);
  Py_VISIT(traverse_module_state->__pyx_tuple__52);
  Py_VISIT(traverse_module_state->__pyx_tuple__53);
  Py_VISIT(traverse_module_state->__pyx_tuple__56);
  Py_VISIT(traverse_module_state->__pyx_tuple__58);
  Py_VISIT(traverse_module_state->__pyx_tuple__59);
  Py_VISIT(traverse_module_state->__pyx_tuple__61);
  Py_VISIT(traverse_module_state->__pyx_tuple__62);
  Py_VISIT(traverse_module_state->__pyx_tuple__64);
  Py_VISIT(traverse_module_state->__pyx_tuple__65);
  Py_VISIT(traverse_module_state->__pyx_tuple__67);
  Py_VISIT(traverse_module_state->__pyx_tuple__69);
  Py_VISIT(traverse_module_state->__pyx_tuple__71);
  Py_VISIT(traverse_module_state->__pyx_tuple__73);
  Py_VISIT(traverse_module_state->__pyx_tuple__75);
  Py_VISIT(traverse_module_state->__pyx_tuple__77);
  Py_VISIT(traverse_module_state->__pyx_tuple__78);
  Py_VISIT(traverse_module_state->__pyx_tuple__80);
  Py_VISIT(traverse_module_state->__pyx_tuple__81);
  Py_VISIT(traverse_module_state->__pyx_tuple__83);
  Py_VISIT(traverse_module_state->__pyx_tuple__85);
  Py_VISIT(traverse_module_state->__pyx_tuple__86);
  Py_VISIT(traverse_module_state->__pyx_tuple__87);
  Py_VISIT(traverse_module_state->__pyx_tuple__88);
  Py_VISIT(traverse_module_state->__pyx_tuple__89);
  Py_VISIT(traverse_module_state->__pyx_tuple__90);
  Py_VISIT(traverse_module_state->__pyx_codeobj__25);
  Py_VISIT(traverse_module_state->__pyx_codeobj__51);
  Py_VISIT(traverse_module_state->__pyx_codeobj__54);
  Py_VISIT(traverse_module_state->__pyx_codeobj__57);
  Py_VISIT(traverse_module_state->__pyx_codeobj__60);
  Py_VISIT(traverse_module_state->__pyx_codeobj__63);
  Py_VISIT(traverse_module_state->__pyx_codeobj__66);
  Py_VISIT(traverse_module_state->__pyx_codeobj__68);
  Py_VISIT(traverse_module_state->__pyx_codeobj__70);
  Py_VISIT(traverse_module_state->__pyx_codeobj__72);
  Py_VISIT(traverse_module_state->__pyx_codeobj__74);
  Py_VISIT(traverse_module_state->__pyx_codeobj__76);
  Py_VISIT(traverse_module_state->__pyx_codeobj__79);
  Py_VISIT(traverse_module_state->__pyx_codeobj__82);
  Py_VISIT(traverse_module_state->__pyx_codeobj__84);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#define __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords __pyx_mstate_global->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords
#define __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr __pyx_mstate_global->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr
#define __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql __pyx_mstate_global->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql
#define __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl __pyx_mstate_global->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl
#define __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr __pyx_mstate_global->__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr
#endif
#define __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords __pyx_mstate_global->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords
#define __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr __pyx_mstate_global->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr
#define __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql __pyx_mstate_global->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql
#define __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl __pyx_mstate_global->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl
#define __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr __pyx_mstate_global->__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr
#define __pyx_kp_u_1_Extract_the_key_business_doma __pyx_mstate_global->__pyx_kp_u_1_Extract_the_key_business_doma
#define __pyx_kp_u_1_Extract_the_key_business_doma_2 __pyx_mstate_global->__pyx_kp_u_1_Extract_the_key_business_doma_2
#define __pyx_kp_u_AI_Enhanced_Query __pyx_mstate_global->__pyx_kp_u_AI_Enhanced_Query
#define __pyx_kp_u_AI_SQL_generation_failed __pyx_mstate_global->__pyx_kp_u_AI_SQL_generation_failed
#define __pyx_kp_u_AI_enhanced_query __pyx_mstate_global->__pyx_kp_u_AI_enhanced_query
#define __pyx_kp_u_AI_extracted_keywords __pyx_mstate_global->__pyx_kp_u_AI_extracted_keywords
#define __pyx_kp_u_AI_history_context __pyx_mstate_global->__pyx_kp_u_AI_history_context
#define __pyx_kp_u_AI_identified_domains __pyx_mstate_global->__pyx_kp_u_AI_identified_domains
#define __pyx_kp_u_AND __pyx_mstate_global->__pyx_kp_u_AND
#define __pyx_n_s_Any __pyx_mstate_global->__pyx_n_s_Any
#define __pyx_n_u_Assistant __pyx_mstate_global->__pyx_n_u_Assistant
#define __pyx_kp_u_Attempting_to_load_enriched_proj __pyx_mstate_global->__pyx_kp_u_Attempting_to_load_enriched_proj
#define __pyx_kp_u_Business_Concepts __pyx_mstate_global->__pyx_kp_u_Business_Concepts
#define __pyx_kp_u_Business_Domains __pyx_mstate_global->__pyx_kp_u_Business_Domains
#define __pyx_kp_u_Business_Rules __pyx_mstate_global->__pyx_kp_u_Business_Rules
#define __pyx_n_s_ChatMessage __pyx_mstate_global->__pyx_n_s_ChatMessage
#define __pyx_kp_u_Cleaned_response __pyx_mstate_global->__pyx_kp_u_Cleaned_response
#define __pyx_kp_u_Column_Data_Type_Description __pyx_mstate_global->__pyx_kp_u_Column_Data_Type_Description
#define __pyx_kp_u_Consider_this_conversation_hist __pyx_mstate_global->__pyx_kp_u_Consider_this_conversation_hist
#define __pyx_n_s_DatabaseContextService __pyx_mstate_global->__pyx_n_s_DatabaseContextService
#define __pyx_n_s_DatabaseContextService___init __pyx_mstate_global->__pyx_n_s_DatabaseContextService___init
#define __pyx_n_s_DatabaseContextService__get_sche __pyx_mstate_global->__pyx_n_s_DatabaseContextService__get_sche
#define __pyx_n_s_DatabaseContextService_build_gra __pyx_mstate_global->__pyx_n_s_DatabaseContextService_build_gra
#define __pyx_n_s_DatabaseContextService_find_path __pyx_mstate_global->__pyx_n_s_DatabaseContextService_find_path
#define __pyx_n_s_DatabaseContextService_format_as __pyx_mstate_global->__pyx_n_s_DatabaseContextService_format_as
#define __pyx_n_s_DatabaseContextService_generate __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate
#define __pyx_n_s_DatabaseContextService_generate_2 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate_2
#define __pyx_n_s_DatabaseContextService_generate_3 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate_3
#define __pyx_n_s_DatabaseContextService_generate_4 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate_4
#define __pyx_n_s_DatabaseContextService_generate_5 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate_5
#define __pyx_n_s_DatabaseContextService_generate_6 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_generate_6
#define __pyx_n_s_DatabaseContextService_get_relat __pyx_mstate_global->__pyx_n_s_DatabaseContextService_get_relat
#define __pyx_n_s_DatabaseContextService_load_meta __pyx_mstate_global->__pyx_n_s_DatabaseContextService_load_meta
#define __pyx_n_s_DatabaseContextService_process_q __pyx_mstate_global->__pyx_n_s_DatabaseContextService_process_q
#define __pyx_n_s_DatabaseContextService_process_q_2 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_process_q_2
#define __pyx_n_s_DatabaseContextService_process_q_3 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_process_q_3
#define __pyx_n_s_DatabaseContextService_search_by __pyx_mstate_global->__pyx_n_s_DatabaseContextService_search_by
#define __pyx_n_s_DatabaseContextService_search_by_2 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_search_by_2
#define __pyx_n_s_DatabaseContextService_search_by_3 __pyx_mstate_global->__pyx_n_s_DatabaseContextService_search_by_3
#define __pyx_kp_u_Database_Context_for_Query __pyx_mstate_global->__pyx_kp_u_Database_Context_for_Query
#define __pyx_kp_u_Description __pyx_mstate_global->__pyx_kp_u_Description
#define __pyx_kp_u_Description_contains __pyx_mstate_global->__pyx_kp_u_Description_contains
#define __pyx_kp_u_Description_partially_matches __pyx_mstate_global->__pyx_kp_u_Description_partially_matches
#define __pyx_n_s_DiGraph __pyx_mstate_global->__pyx_n_s_DiGraph
#define __pyx_n_s_Dict __pyx_mstate_global->__pyx_n_s_Dict
#define __pyx_kp_u_Error_enhancing_query_with_AI __pyx_mstate_global->__pyx_kp_u_Error_enhancing_query_with_AI
#define __pyx_kp_u_Error_parsing_AI_response __pyx_mstate_global->__pyx_kp_u_Error_parsing_AI_response
#define __pyx_kp_u_Extracted_keywords __pyx_mstate_global->__pyx_kp_u_Extracted_keywords
#define __pyx_kp_u_FROM __pyx_mstate_global->__pyx_kp_u_FROM
#define __pyx_kp_u_Failed_to_load_metadata_file_Ple __pyx_mstate_global->__pyx_kp_u_Failed_to_load_metadata_file_Ple
#define __pyx_kp_u_Falling_back_to_rule_based_appr __pyx_mstate_global->__pyx_kp_u_Falling_back_to_rule_based_appr
#define __pyx_n_s_FileNotFoundError __pyx_mstate_global->__pyx_n_s_FileNotFoundError
#define __pyx_kp_u_Find_activities_with_excessive_l __pyx_mstate_global->__pyx_kp_u_Find_activities_with_excessive_l
#define __pyx_kp_u_Found __pyx_mstate_global->__pyx_kp_u_Found
#define __pyx_n_s_G __pyx_mstate_global->__pyx_n_s_G
#define __pyx_kp_u_Generated_SQL __pyx_mstate_global->__pyx_kp_u_Generated_SQL
#define __pyx_kp_u_Graph_built_with __pyx_mstate_global->__pyx_kp_u_Graph_built_with
#define __pyx_kp_u_Graph_not_built_Call_build_graph __pyx_mstate_global->__pyx_kp_u_Graph_not_built_Call_build_graph
#define __pyx_kp_u_ID_contains __pyx_mstate_global->__pyx_kp_u_ID_contains
#define __pyx_n_s_IGNORECASE __pyx_mstate_global->__pyx_n_s_IGNORECASE
#define __pyx_kp_u_JOIN __pyx_mstate_global->__pyx_kp_u_JOIN
#define __pyx_n_s_JSONDecodeError __pyx_mstate_global->__pyx_n_s_JSONDecodeError
#define __pyx_kp_u_Join_condition __pyx_mstate_global->__pyx_kp_u_Join_condition
#define __pyx_n_s_List __pyx_mstate_global->__pyx_n_s_List
#define __pyx_kp_s_List_ChatMessage __pyx_mstate_global->__pyx_kp_s_List_ChatMessage
#define __pyx_kp_s_List_Tuple_str_str __pyx_mstate_global->__pyx_kp_s_List_Tuple_str_str
#define __pyx_kp_u_List_relationships_with_lag_grea __pyx_mstate_global->__pyx_kp_u_List_relationships_with_lag_grea
#define __pyx_kp_s_List_str __pyx_mstate_global->__pyx_kp_s_List_str
#define __pyx_kp_u_Loaded_metadata_from __pyx_mstate_global->__pyx_kp_u_Loaded_metadata_from
#define __pyx_kp_u_Markdown_Output __pyx_mstate_global->__pyx_kp_u_Markdown_Output
#define __pyx_kp_u_Metadata_contains_sections __pyx_mstate_global->__pyx_kp_u_Metadata_contains_sections
#define __pyx_kp_u_Metadata_file_not_found __pyx_mstate_global->__pyx_kp_u_Metadata_file_not_found
#define __pyx_kp_u_Metadata_was_cleaned_at __pyx_mstate_global->__pyx_kp_u_Metadata_was_cleaned_at
#define __pyx_n_s_Models __pyx_mstate_global->__pyx_n_s_Models
#define __pyx_kp_u_Name_contains __pyx_mstate_global->__pyx_kp_u_Name_contains
#define __pyx_kp_u_Name_partially_matches __pyx_mstate_global->__pyx_kp_u_Name_partially_matches
#define __pyx_n_s_NetworkXNoPath __pyx_mstate_global->__pyx_n_s_NetworkXNoPath
#define __pyx_kp_u_No_SQL_generated __pyx_mstate_global->__pyx_kp_u_No_SQL_generated
#define __pyx_kp_u_No_metadata_provided_Call_load_m __pyx_mstate_global->__pyx_kp_u_No_metadata_provided_Call_load_m
#define __pyx_kp_u_No_tables_selected_for_query __pyx_mstate_global->__pyx_kp_u_No_tables_selected_for_query
#define __pyx_kp_u_None __pyx_mstate_global->__pyx_kp_u_None
#define __pyx_kp_u_ON __pyx_mstate_global->__pyx_kp_u_ON
#define __pyx_kp_u_ON_Missing_join_condition_verif __pyx_mstate_global->__pyx_kp_u_ON_Missing_join_condition_verif
#define __pyx_n_s_Optional __pyx_mstate_global->__pyx_n_s_Optional
#define __pyx_kp_s_Optional_List_str __pyx_mstate_global->__pyx_kp_s_Optional_List_str
#define __pyx_kp_u_Processing_query __pyx_mstate_global->__pyx_kp_u_Processing_query
#define __pyx_kp_u_Raw_response __pyx_mstate_global->__pyx_kp_u_Raw_response
#define __pyx_kp_u_Relevant_Columns __pyx_mstate_global->__pyx_kp_u_Relevant_Columns
#define __pyx_kp_u_Relevant_Tables __pyx_mstate_global->__pyx_kp_u_Relevant_Tables
#define __pyx_kp_u_SELECT __pyx_mstate_global->__pyx_kp_u_SELECT
#define __pyx_n_s_SQLPrompt __pyx_mstate_global->__pyx_n_s_SQLPrompt
#define __pyx_kp_s_Service_for_providing_database __pyx_mstate_global->__pyx_kp_s_Service_for_providing_database
#define __pyx_kp_u_Show_all_projects_with_negative __pyx_mstate_global->__pyx_kp_u_Show_all_projects_with_negative
#define __pyx_kp_u_Table_Relationships __pyx_mstate_global->__pyx_kp_u_Table_Relationships
#define __pyx_kp_u_Tagged_with_keyword __pyx_mstate_global->__pyx_kp_u_Tagged_with_keyword
#define __pyx_kp_u_The_previous_SQL_query_was __pyx_mstate_global->__pyx_kp_u_The_previous_SQL_query_was
#define __pyx_n_s_Tuple __pyx_mstate_global->__pyx_n_s_Tuple
#define __pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s __pyx_mstate_global->__pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s
#define __pyx_n_s_Union __pyx_mstate_global->__pyx_n_s_Union
#define __pyx_n_u_User __pyx_mstate_global->__pyx_n_u_User
#define __pyx_kp_u_Using_basic_keyword_extraction __pyx_mstate_global->__pyx_kp_u_Using_basic_keyword_extraction
#define __pyx_kp_u_Using_template_query __pyx_mstate_global->__pyx_kp_u_Using_template_query
#define __pyx_n_s_ValueError __pyx_mstate_global->__pyx_n_s_ValueError
#define __pyx_kp_u_WARNING_Missing_required_sectio __pyx_mstate_global->__pyx_kp_u_WARNING_Missing_required_sectio
#define __pyx_kp_u_WHERE __pyx_mstate_global->__pyx_kp_u_WHERE
#define __pyx_kp_u_What_are_the_activities_on_the_c __pyx_mstate_global->__pyx_kp_u_What_are_the_activities_on_the_c
#define __pyx_kp_u_You_are_an_expert_in_project_ma __pyx_mstate_global->__pyx_kp_u_You_are_an_expert_in_project_ma
#define __pyx_kp_u__10 __pyx_mstate_global->__pyx_kp_u__10
#define __pyx_kp_u__12 __pyx_mstate_global->__pyx_kp_u__12
#define __pyx_kp_u__15 __pyx_mstate_global->__pyx_kp_u__15
#define __pyx_kp_u__2 __pyx_mstate_global->__pyx_kp_u__2
#define __pyx_kp_u__20 __pyx_mstate_global->__pyx_kp_u__20
#define __pyx_kp_u__23 __pyx_mstate_global->__pyx_kp_u__23
#define __pyx_kp_u__27 __pyx_mstate_global->__pyx_kp_u__27
#define __pyx_kp_u__28 __pyx_mstate_global->__pyx_kp_u__28
#define __pyx_kp_u__29 __pyx_mstate_global->__pyx_kp_u__29
#define __pyx_n_s__31 __pyx_mstate_global->__pyx_n_s__31
#define __pyx_kp_u__31 __pyx_mstate_global->__pyx_kp_u__31
#define __pyx_kp_u__35 __pyx_mstate_global->__pyx_kp_u__35
#define __pyx_kp_u__36 __pyx_mstate_global->__pyx_kp_u__36
#define __pyx_kp_u__37 __pyx_mstate_global->__pyx_kp_u__37
#define __pyx_kp_u__38 __pyx_mstate_global->__pyx_kp_u__38
#define __pyx_kp_u__39 __pyx_mstate_global->__pyx_kp_u__39
#define __pyx_kp_u__40 __pyx_mstate_global->__pyx_kp_u__40
#define __pyx_kp_u__41 __pyx_mstate_global->__pyx_kp_u__41
#define __pyx_kp_u__42 __pyx_mstate_global->__pyx_kp_u__42
#define __pyx_kp_u__43 __pyx_mstate_global->__pyx_kp_u__43
#define __pyx_kp_u__44 __pyx_mstate_global->__pyx_kp_u__44
#define __pyx_kp_u__45 __pyx_mstate_global->__pyx_kp_u__45
#define __pyx_n_s__48 __pyx_mstate_global->__pyx_n_s__48
#define __pyx_kp_u__5 __pyx_mstate_global->__pyx_kp_u__5
#define __pyx_n_s__55 __pyx_mstate_global->__pyx_n_s__55
#define __pyx_kp_u__7 __pyx_mstate_global->__pyx_kp_u__7
#define __pyx_kp_u__8 __pyx_mstate_global->__pyx_kp_u__8
#define __pyx_kp_u__9 __pyx_mstate_global->__pyx_kp_u__9
#define __pyx_n_s_adapted_query __pyx_mstate_global->__pyx_n_s_adapted_query
#define __pyx_n_s_add_edge __pyx_mstate_global->__pyx_n_s_add_edge
#define __pyx_n_s_add_node __pyx_mstate_global->__pyx_n_s_add_node
#define __pyx_n_s_ai_base __pyx_mstate_global->__pyx_n_s_ai_base
#define __pyx_n_s_ai_context __pyx_mstate_global->__pyx_n_s_ai_context
#define __pyx_n_u_ai_enhanced __pyx_mstate_global->__pyx_n_u_ai_enhanced
#define __pyx_n_s_alias __pyx_mstate_global->__pyx_n_s_alias
#define __pyx_n_s_aliases __pyx_mstate_global->__pyx_n_s_aliases
#define __pyx_n_s_all_rules __pyx_mstate_global->__pyx_n_s_all_rules
#define __pyx_n_s_all_tables __pyx_mstate_global->__pyx_n_s_all_tables
#define __pyx_n_s_append __pyx_mstate_global->__pyx_n_s_append
#define __pyx_n_s_args __pyx_mstate_global->__pyx_n_s_args
#define __pyx_n_s_asyncio __pyx_mstate_global->__pyx_n_s_asyncio
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_asyncio_tasks __pyx_mstate_global->__pyx_n_s_asyncio_tasks
#define __pyx_kp_u_b __pyx_mstate_global->__pyx_kp_u_b
#define __pyx_n_s_base __pyx_mstate_global->__pyx_n_s_base
#define __pyx_n_s_best_example __pyx_mstate_global->__pyx_n_s_best_example
#define __pyx_n_u_best_guess_sql __pyx_mstate_global->__pyx_n_u_best_guess_sql
#define __pyx_n_s_bool __pyx_mstate_global->__pyx_n_s_bool
#define __pyx_n_s_build_graph __pyx_mstate_global->__pyx_n_s_build_graph
#define __pyx_n_s_buisness_domains __pyx_mstate_global->__pyx_n_s_buisness_domains
#define __pyx_n_u_business_concepts __pyx_mstate_global->__pyx_n_u_business_concepts
#define __pyx_n_u_business_domains __pyx_mstate_global->__pyx_n_u_business_domains
#define __pyx_n_u_business_rules __pyx_mstate_global->__pyx_n_u_business_rules
#define __pyx_n_s_c __pyx_mstate_global->__pyx_n_s_c
#define __pyx_n_s_chat_history __pyx_mstate_global->__pyx_n_s_chat_history
#define __pyx_n_u_chat_history_context __pyx_mstate_global->__pyx_n_u_chat_history_context
#define __pyx_n_s_class_getitem __pyx_mstate_global->__pyx_n_s_class_getitem
#define __pyx_n_s_clean_response __pyx_mstate_global->__pyx_n_s_clean_response
#define __pyx_n_s_cleaned_at __pyx_mstate_global->__pyx_n_s_cleaned_at
#define __pyx_n_u_cleaned_at __pyx_mstate_global->__pyx_n_u_cleaned_at
#define __pyx_n_u_cleaning_version __pyx_mstate_global->__pyx_n_u_cleaning_version
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_close __pyx_mstate_global->__pyx_n_s_close
#define __pyx_n_s_col_data __pyx_mstate_global->__pyx_n_s_col_data
#define __pyx_n_s_col_name __pyx_mstate_global->__pyx_n_s_col_name
#define __pyx_n_s_col_node_id __pyx_mstate_global->__pyx_n_s_col_node_id
#define __pyx_kp_u_column __pyx_mstate_global->__pyx_kp_u_column
#define __pyx_n_s_column_2 __pyx_mstate_global->__pyx_n_s_column_2
#define __pyx_n_u_column_2 __pyx_mstate_global->__pyx_n_u_column_2
#define __pyx_n_u_column_statistics __pyx_mstate_global->__pyx_n_u_column_statistics
#define __pyx_n_s_columns __pyx_mstate_global->__pyx_n_s_columns
#define __pyx_n_u_columns __pyx_mstate_global->__pyx_n_u_columns
#define __pyx_n_s_compile __pyx_mstate_global->__pyx_n_s_compile
#define __pyx_n_s_complexity __pyx_mstate_global->__pyx_n_s_complexity
#define __pyx_n_u_complexity __pyx_mstate_global->__pyx_n_u_complexity
#define __pyx_kp_u_concept __pyx_mstate_global->__pyx_kp_u_concept
#define __pyx_n_s_concept_2 __pyx_mstate_global->__pyx_n_s_concept_2
#define __pyx_n_u_concept_2 __pyx_mstate_global->__pyx_n_u_concept_2
#define __pyx_n_s_concept_data __pyx_mstate_global->__pyx_n_s_concept_data
#define __pyx_n_s_concept_id __pyx_mstate_global->__pyx_n_s_concept_id
#define __pyx_n_s_concepts __pyx_mstate_global->__pyx_n_s_concepts
#define __pyx_n_u_concepts __pyx_mstate_global->__pyx_n_u_concepts
#define __pyx_n_s_condition_nodes __pyx_mstate_global->__pyx_n_s_condition_nodes
#define __pyx_n_s_content __pyx_mstate_global->__pyx_n_s_content
#define __pyx_n_s_context __pyx_mstate_global->__pyx_n_s_context
#define __pyx_n_s_context_md __pyx_mstate_global->__pyx_n_s_context_md
#define __pyx_n_s_context_prompt __pyx_mstate_global->__pyx_n_s_context_prompt
#define __pyx_n_s_copy __pyx_mstate_global->__pyx_n_s_copy
#define __pyx_n_s_current_table __pyx_mstate_global->__pyx_n_s_current_table
#define __pyx_n_s_d __pyx_mstate_global->__pyx_n_s_d
#define __pyx_kp_u_d_4_d_2_d_2 __pyx_mstate_global->__pyx_kp_u_d_4_d_2_d_2
#define __pyx_n_s_data __pyx_mstate_global->__pyx_n_s_data
#define __pyx_n_u_data __pyx_mstate_global->__pyx_n_u_data
#define __pyx_n_u_data_type __pyx_mstate_global->__pyx_n_u_data_type
#define __pyx_n_u_database_objects __pyx_mstate_global->__pyx_n_u_database_objects
#define __pyx_n_u_database_schema __pyx_mstate_global->__pyx_n_u_database_schema
#define __pyx_n_u_database_statistics __pyx_mstate_global->__pyx_n_u_database_statistics
#define __pyx_n_u_date __pyx_mstate_global->__pyx_n_u_date
#define __pyx_n_s_datetime __pyx_mstate_global->__pyx_n_s_datetime
#define __pyx_n_u_day __pyx_mstate_global->__pyx_n_u_day
#define __pyx_n_u_defines __pyx_mstate_global->__pyx_n_u_defines
#define __pyx_n_s_definition __pyx_mstate_global->__pyx_n_s_definition
#define __pyx_n_s_description __pyx_mstate_global->__pyx_n_s_description
#define __pyx_n_u_description __pyx_mstate_global->__pyx_n_u_description
#define __pyx_n_s_dict __pyx_mstate_global->__pyx_n_s_dict
#define __pyx_kp_u_disable __pyx_mstate_global->__pyx_kp_u_disable
#define __pyx_n_s_doc __pyx_mstate_global->__pyx_n_s_doc
#define __pyx_n_s_domain __pyx_mstate_global->__pyx_n_s_domain
#define __pyx_n_u_domain __pyx_mstate_global->__pyx_n_u_domain
#define __pyx_n_s_domain_results __pyx_mstate_global->__pyx_n_s_domain_results
#define __pyx_n_s_e __pyx_mstate_global->__pyx_n_s_e
#define __pyx_n_s_edge_data __pyx_mstate_global->__pyx_n_s_edge_data
#define __pyx_n_s_edges __pyx_mstate_global->__pyx_n_s_edges
#define __pyx_kp_u_edges_2 __pyx_mstate_global->__pyx_kp_u_edges_2
#define __pyx_kp_u_enable __pyx_mstate_global->__pyx_kp_u_enable
#define __pyx_n_s_encoding __pyx_mstate_global->__pyx_n_s_encoding
#define __pyx_n_s_end_node __pyx_mstate_global->__pyx_n_s_end_node
#define __pyx_n_s_endswith __pyx_mstate_global->__pyx_n_s_endswith
#define __pyx_n_s_enhanced_query __pyx_mstate_global->__pyx_n_s_enhanced_query
#define __pyx_n_u_enhanced_query __pyx_mstate_global->__pyx_n_u_enhanced_query
#define __pyx_kp_u_enriched_project_metadata_cleane __pyx_mstate_global->__pyx_kp_u_enriched_project_metadata_cleane
#define __pyx_n_s_enter __pyx_mstate_global->__pyx_n_s_enter
#define __pyx_n_s_enumerate __pyx_mstate_global->__pyx_n_s_enumerate
#define __pyx_n_s_escape __pyx_mstate_global->__pyx_n_s_escape
#define __pyx_kp_u_example __pyx_mstate_global->__pyx_kp_u_example
#define __pyx_n_s_example_2 __pyx_mstate_global->__pyx_n_s_example_2
#define __pyx_n_u_example_2 __pyx_mstate_global->__pyx_n_u_example_2
#define __pyx_n_s_example_data __pyx_mstate_global->__pyx_n_s_example_data
#define __pyx_n_s_example_id __pyx_mstate_global->__pyx_n_s_example_id
#define __pyx_n_s_example_node __pyx_mstate_global->__pyx_n_s_example_node
#define __pyx_n_s_example_nodes __pyx_mstate_global->__pyx_n_s_example_nodes
#define __pyx_n_s_example_queries __pyx_mstate_global->__pyx_n_s_example_queries
#define __pyx_n_s_example_scores __pyx_mstate_global->__pyx_n_s_example_scores
#define __pyx_n_s_exists __pyx_mstate_global->__pyx_n_s_exists
#define __pyx_n_s_exit __pyx_mstate_global->__pyx_n_s_exit
#define __pyx_n_s_f __pyx_mstate_global->__pyx_n_s_f
#define __pyx_n_s_file_path __pyx_mstate_global->__pyx_n_s_file_path
#define __pyx_n_s_filename __pyx_mstate_global->__pyx_n_s_filename
#define __pyx_n_s_filtered_rules __pyx_mstate_global->__pyx_n_s_filtered_rules
#define __pyx_n_s_find __pyx_mstate_global->__pyx_n_s_find
#define __pyx_n_s_find_path __pyx_mstate_global->__pyx_n_s_find_path
#define __pyx_n_s_first_newline __pyx_mstate_global->__pyx_n_s_first_newline
#define __pyx_n_s_first_table __pyx_mstate_global->__pyx_n_s_first_table
#define __pyx_n_s_flags __pyx_mstate_global->__pyx_n_s_flags
#define __pyx_n_s_format_as_markdown __pyx_mstate_global->__pyx_n_s_format_as_markdown
#define __pyx_n_u_from_table __pyx_mstate_global->__pyx_n_u_from_table
#define __pyx_n_s_from_table_id __pyx_mstate_global->__pyx_n_s_from_table_id
#define __pyx_n_s_from_tables __pyx_mstate_global->__pyx_n_s_from_tables
#define __pyx_kp_u_gc __pyx_mstate_global->__pyx_kp_u_gc
#define __pyx_n_s_generate_output __pyx_mstate_global->__pyx_n_s_generate_output
#define __pyx_n_s_generate_sql __pyx_mstate_global->__pyx_n_s_generate_sql
#define __pyx_n_s_generate_sql_from_nl __pyx_mstate_global->__pyx_n_s_generate_sql_from_nl
#define __pyx_n_s_genexpr __pyx_mstate_global->__pyx_n_s_genexpr
#define __pyx_n_s_get __pyx_mstate_global->__pyx_n_s_get
#define __pyx_n_s_get_alias __pyx_mstate_global->__pyx_n_s_get_alias
#define __pyx_n_s_get_edge_data __pyx_mstate_global->__pyx_n_s_get_edge_data
#define __pyx_n_s_get_related_rules __pyx_mstate_global->__pyx_n_s_get_related_rules
#define __pyx_n_u_get_response __pyx_mstate_global->__pyx_n_u_get_response
#define __pyx_n_s_get_schema_summary __pyx_mstate_global->__pyx_n_s_get_schema_summary
#define __pyx_n_u_governed_by __pyx_mstate_global->__pyx_n_u_governed_by
#define __pyx_n_s_graph __pyx_mstate_global->__pyx_n_s_graph
#define __pyx_n_u_has_column __pyx_mstate_global->__pyx_n_u_has_column
#define __pyx_n_s_has_node __pyx_mstate_global->__pyx_n_s_has_node
#define __pyx_n_s_history_context __pyx_mstate_global->__pyx_n_s_history_context
#define __pyx_n_u_history_context __pyx_mstate_global->__pyx_n_u_history_context
#define __pyx_n_s_i __pyx_mstate_global->__pyx_n_s_i
#define __pyx_n_u_id __pyx_mstate_global->__pyx_n_u_id
#define __pyx_n_u_implements __pyx_mstate_global->__pyx_n_u_implements
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_u_indexed __pyx_mstate_global->__pyx_n_u_indexed
#define __pyx_n_s_init __pyx_mstate_global->__pyx_n_s_init
#define __pyx_n_s_init_subclass __pyx_mstate_global->__pyx_n_s_init_subclass
#define __pyx_n_s_initializing __pyx_mstate_global->__pyx_n_s_initializing
#define __pyx_n_s_inspect __pyx_mstate_global->__pyx_n_s_inspect
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_n_s_is_user __pyx_mstate_global->__pyx_n_s_is_user
#define __pyx_kp_u_isenabled __pyx_mstate_global->__pyx_kp_u_isenabled
#define __pyx_n_s_items __pyx_mstate_global->__pyx_n_s_items
#define __pyx_n_s_j __pyx_mstate_global->__pyx_n_s_j
#define __pyx_n_s_join __pyx_mstate_global->__pyx_n_s_join
#define __pyx_n_s_join_clauses __pyx_mstate_global->__pyx_n_s_join_clauses
#define __pyx_n_s_join_condition __pyx_mstate_global->__pyx_n_s_join_condition
#define __pyx_n_u_join_condition __pyx_mstate_global->__pyx_n_u_join_condition
#define __pyx_n_s_join_found __pyx_mstate_global->__pyx_n_s_join_found
#define __pyx_n_s_json __pyx_mstate_global->__pyx_n_s_json
#define __pyx_n_u_json __pyx_mstate_global->__pyx_n_u_json
#define __pyx_n_s_k __pyx_mstate_global->__pyx_n_s_k
#define __pyx_n_s_key __pyx_mstate_global->__pyx_n_s_key
#define __pyx_n_s_keys __pyx_mstate_global->__pyx_n_s_keys
#define __pyx_kp_u_keyword __pyx_mstate_global->__pyx_kp_u_keyword
#define __pyx_n_s_keyword_2 __pyx_mstate_global->__pyx_n_s_keyword_2
#define __pyx_n_u_keyword_2 __pyx_mstate_global->__pyx_n_u_keyword_2
#define __pyx_n_s_keyword_id __pyx_mstate_global->__pyx_n_s_keyword_id
#define __pyx_n_s_keywords __pyx_mstate_global->__pyx_n_s_keywords
#define __pyx_n_u_keywords __pyx_mstate_global->__pyx_n_u_keywords
#define __pyx_n_s_load __pyx_mstate_global->__pyx_n_s_load
#define __pyx_n_s_load_metadata __pyx_mstate_global->__pyx_n_s_load_metadata
#define __pyx_n_s_loads __pyx_mstate_global->__pyx_n_s_loads
#define __pyx_n_s_lower __pyx_mstate_global->__pyx_n_s_lower
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_u_main __pyx_mstate_global->__pyx_n_u_main
#define __pyx_n_u_markdown __pyx_mstate_global->__pyx_n_u_markdown
#define __pyx_n_s_match_reason __pyx_mstate_global->__pyx_n_s_match_reason
#define __pyx_n_s_match_score __pyx_mstate_global->__pyx_n_s_match_score
#define __pyx_n_s_matched __pyx_mstate_global->__pyx_n_s_matched
#define __pyx_n_s_matches __pyx_mstate_global->__pyx_n_s_matches
#define __pyx_n_s_max __pyx_mstate_global->__pyx_n_s_max
#define __pyx_n_s_md_output __pyx_mstate_global->__pyx_n_s_md_output
#define __pyx_n_s_metaclass __pyx_mstate_global->__pyx_n_s_metaclass
#define __pyx_n_s_metadata __pyx_mstate_global->__pyx_n_s_metadata
#define __pyx_n_u_metadata_cleaning __pyx_mstate_global->__pyx_n_u_metadata_cleaning
#define __pyx_n_s_metadata_file __pyx_mstate_global->__pyx_n_s_metadata_file
#define __pyx_n_s_metadata_sections __pyx_mstate_global->__pyx_n_s_metadata_sections
#define __pyx_n_s_missing __pyx_mstate_global->__pyx_n_s_missing
#define __pyx_n_s_module __pyx_mstate_global->__pyx_n_s_module
#define __pyx_n_s_msg __pyx_mstate_global->__pyx_n_s_msg
#define __pyx_n_s_n __pyx_mstate_global->__pyx_n_s_n
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_u_name __pyx_mstate_global->__pyx_n_u_name
#define __pyx_n_s_name_2 __pyx_mstate_global->__pyx_n_s_name_2
#define __pyx_n_s_natural_language_query __pyx_mstate_global->__pyx_n_s_natural_language_query
#define __pyx_n_s_neighbor __pyx_mstate_global->__pyx_n_s_neighbor
#define __pyx_n_s_neighbors __pyx_mstate_global->__pyx_n_s_neighbors
#define __pyx_n_s_networkx __pyx_mstate_global->__pyx_n_s_networkx
#define __pyx_n_s_node __pyx_mstate_global->__pyx_n_s_node
#define __pyx_n_s_node_data __pyx_mstate_global->__pyx_n_s_node_data
#define __pyx_n_s_node_id __pyx_mstate_global->__pyx_n_s_node_id
#define __pyx_n_s_node_matches __pyx_mstate_global->__pyx_n_s_node_matches
#define __pyx_n_s_node_props __pyx_mstate_global->__pyx_n_s_node_props
#define __pyx_n_s_node_rules __pyx_mstate_global->__pyx_n_s_node_rules
#define __pyx_n_s_node_score __pyx_mstate_global->__pyx_n_s_node_score
#define __pyx_n_s_node_scores __pyx_mstate_global->__pyx_n_s_node_scores
#define __pyx_n_s_nodes __pyx_mstate_global->__pyx_n_s_nodes
#define __pyx_kp_u_nodes_and __pyx_mstate_global->__pyx_kp_u_nodes_and
#define __pyx_n_s_normalized_keywords __pyx_mstate_global->__pyx_n_s_normalized_keywords
#define __pyx_n_s_nx __pyx_mstate_global->__pyx_n_s_nx
#define __pyx_kp_s_nx_DiGraph __pyx_mstate_global->__pyx_kp_s_nx_DiGraph
#define __pyx_n_s_open __pyx_mstate_global->__pyx_n_s_open
#define __pyx_n_s_os __pyx_mstate_global->__pyx_n_s_os
#define __pyx_n_s_os_path __pyx_mstate_global->__pyx_n_s_os_path
#define __pyx_n_s_output __pyx_mstate_global->__pyx_n_s_output
#define __pyx_n_s_output_data __pyx_mstate_global->__pyx_n_s_output_data
#define __pyx_n_s_output_format __pyx_mstate_global->__pyx_n_s_output_format
#define __pyx_n_s_parsed __pyx_mstate_global->__pyx_n_s_parsed
#define __pyx_n_s_path __pyx_mstate_global->__pyx_n_s_path
#define __pyx_n_s_pattern __pyx_mstate_global->__pyx_n_s_pattern
#define __pyx_n_s_prepare __pyx_mstate_global->__pyx_n_s_prepare
#define __pyx_n_s_prev __pyx_mstate_global->__pyx_n_s_prev
#define __pyx_n_s_prev_table __pyx_mstate_global->__pyx_n_s_prev_table
#define __pyx_n_s_previous_sql __pyx_mstate_global->__pyx_n_s_previous_sql
#define __pyx_n_s_print __pyx_mstate_global->__pyx_n_s_print
#define __pyx_n_s_process_query __pyx_mstate_global->__pyx_n_s_process_query
#define __pyx_n_s_process_query_with_ai __pyx_mstate_global->__pyx_n_s_process_query_with_ai
#define __pyx_n_s_process_query_with_chat_history __pyx_mstate_global->__pyx_n_s_process_query_with_chat_history
#define __pyx_n_s_prompt __pyx_mstate_global->__pyx_n_s_prompt
#define __pyx_n_s_qualname __pyx_mstate_global->__pyx_n_s_qualname
#define __pyx_n_s_query __pyx_mstate_global->__pyx_n_s_query
#define __pyx_n_u_query __pyx_mstate_global->__pyx_n_u_query
#define __pyx_n_u_query_examples __pyx_mstate_global->__pyx_n_u_query_examples
#define __pyx_n_s_query_text __pyx_mstate_global->__pyx_n_s_query_text
#define __pyx_n_s_r __pyx_mstate_global->__pyx_n_s_r
#define __pyx_n_u_r __pyx_mstate_global->__pyx_n_u_r
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_re __pyx_mstate_global->__pyx_n_s_re
#define __pyx_n_u_references __pyx_mstate_global->__pyx_n_u_references
#define __pyx_n_s_rel __pyx_mstate_global->__pyx_n_s_rel
#define __pyx_n_u_related_business_rules __pyx_mstate_global->__pyx_n_u_related_business_rules
#define __pyx_n_u_related_to __pyx_mstate_global->__pyx_n_u_related_to
#define __pyx_n_s_relationship __pyx_mstate_global->__pyx_n_s_relationship
#define __pyx_n_s_relationship_count __pyx_mstate_global->__pyx_n_s_relationship_count
#define __pyx_n_u_relationships __pyx_mstate_global->__pyx_n_u_relationships
#define __pyx_kp_u_relationships_2 __pyx_mstate_global->__pyx_kp_u_relationships_2
#define __pyx_n_s_replace __pyx_mstate_global->__pyx_n_s_replace
#define __pyx_n_s_required_sections __pyx_mstate_global->__pyx_n_s_required_sections
#define __pyx_n_s_response __pyx_mstate_global->__pyx_n_s_response
#define __pyx_n_s_results __pyx_mstate_global->__pyx_n_s_results
#define __pyx_n_s_return __pyx_mstate_global->__pyx_n_s_return
#define __pyx_n_s_reverse __pyx_mstate_global->__pyx_n_s_reverse
#define __pyx_n_s_role __pyx_mstate_global->__pyx_n_s_role
#define __pyx_n_s_row_count __pyx_mstate_global->__pyx_n_s_row_count
#define __pyx_n_u_row_count __pyx_mstate_global->__pyx_n_u_row_count
#define __pyx_kp_u_rule __pyx_mstate_global->__pyx_kp_u_rule
#define __pyx_n_s_rule_2 __pyx_mstate_global->__pyx_n_s_rule_2
#define __pyx_n_u_rule_2 __pyx_mstate_global->__pyx_n_u_rule_2
#define __pyx_n_s_rule_data __pyx_mstate_global->__pyx_n_s_rule_data
#define __pyx_n_s_rule_desc __pyx_mstate_global->__pyx_n_s_rule_desc
#define __pyx_n_s_rule_id __pyx_mstate_global->__pyx_n_s_rule_id
#define __pyx_n_s_rule_node __pyx_mstate_global->__pyx_n_s_rule_node
#define __pyx_n_s_rules __pyx_mstate_global->__pyx_n_s_rules
#define __pyx_n_s_run __pyx_mstate_global->__pyx_n_s_run
#define __pyx_n_s_s __pyx_mstate_global->__pyx_n_s_s
#define __pyx_n_s_sQQideR_DatabaseContext __pyx_mstate_global->__pyx_n_s_sQQideR_DatabaseContext
#define __pyx_kp_s_sQQideR_DatabaseContext_pyx __pyx_mstate_global->__pyx_kp_s_sQQideR_DatabaseContext_pyx
#define __pyx_kp_u_s_LIMIT_s_d_s __pyx_mstate_global->__pyx_kp_u_s_LIMIT_s_d_s
#define __pyx_n_s_schema __pyx_mstate_global->__pyx_n_s_schema
#define __pyx_n_s_score __pyx_mstate_global->__pyx_n_s_score
#define __pyx_n_s_search __pyx_mstate_global->__pyx_n_s_search
#define __pyx_n_s_search_by_keywords __pyx_mstate_global->__pyx_n_s_search_by_keywords
#define __pyx_n_s_select_clause __pyx_mstate_global->__pyx_n_s_select_clause
#define __pyx_n_s_selected_query __pyx_mstate_global->__pyx_n_s_selected_query
#define __pyx_n_s_self __pyx_mstate_global->__pyx_n_s_self
#define __pyx_n_s_send __pyx_mstate_global->__pyx_n_s_send
#define __pyx_n_s_service __pyx_mstate_global->__pyx_n_s_service
#define __pyx_n_s_set_name __pyx_mstate_global->__pyx_n_s_set_name
#define __pyx_n_s_shortest_path __pyx_mstate_global->__pyx_n_s_shortest_path
#define __pyx_n_u_simple __pyx_mstate_global->__pyx_n_u_simple
#define __pyx_n_s_sorted __pyx_mstate_global->__pyx_n_s_sorted
#define __pyx_n_s_sorted_nodes __pyx_mstate_global->__pyx_n_s_sorted_nodes
#define __pyx_n_u_source __pyx_mstate_global->__pyx_n_u_source
#define __pyx_n_s_spec __pyx_mstate_global->__pyx_n_s_spec
#define __pyx_n_s_split __pyx_mstate_global->__pyx_n_s_split
#define __pyx_n_s_sql __pyx_mstate_global->__pyx_n_s_sql
#define __pyx_n_s_sql_lines __pyx_mstate_global->__pyx_n_s_sql_lines
#define __pyx_n_u_src __pyx_mstate_global->__pyx_n_u_src
#define __pyx_n_s_src_name __pyx_mstate_global->__pyx_n_s_src_name
#define __pyx_n_s_start_node __pyx_mstate_global->__pyx_n_s_start_node
#define __pyx_n_s_startswith __pyx_mstate_global->__pyx_n_s_startswith
#define __pyx_n_u_statistics __pyx_mstate_global->__pyx_n_u_statistics
#define __pyx_n_s_str __pyx_mstate_global->__pyx_n_s_str
#define __pyx_n_s_strip __pyx_mstate_global->__pyx_n_s_strip
#define __pyx_n_s_sub __pyx_mstate_global->__pyx_n_s_sub
#define __pyx_n_s_suffix __pyx_mstate_global->__pyx_n_s_suffix
#define __pyx_n_s_super __pyx_mstate_global->__pyx_n_s_super
#define __pyx_n_s_t __pyx_mstate_global->__pyx_n_s_t
#define __pyx_kp_u_table __pyx_mstate_global->__pyx_kp_u_table
#define __pyx_n_s_table_2 __pyx_mstate_global->__pyx_n_s_table_2
#define __pyx_n_u_table_2 __pyx_mstate_global->__pyx_n_u_table_2
#define __pyx_n_s_table_count __pyx_mstate_global->__pyx_n_s_table_count
#define __pyx_n_s_table_data __pyx_mstate_global->__pyx_n_s_table_data
#define __pyx_n_s_table_id __pyx_mstate_global->__pyx_n_s_table_id
#define __pyx_n_s_table_name __pyx_mstate_global->__pyx_n_s_table_name
#define __pyx_n_s_table_names __pyx_mstate_global->__pyx_n_s_table_names
#define __pyx_n_s_table_nodes __pyx_mstate_global->__pyx_n_s_table_nodes
#define __pyx_n_s_tables __pyx_mstate_global->__pyx_n_s_tables
#define __pyx_n_u_tables __pyx_mstate_global->__pyx_n_u_tables
#define __pyx_kp_u_tables_and __pyx_mstate_global->__pyx_kp_u_tables_and
#define __pyx_n_u_tags __pyx_mstate_global->__pyx_n_u_tags
#define __pyx_n_s_target __pyx_mstate_global->__pyx_n_s_target
#define __pyx_n_u_target __pyx_mstate_global->__pyx_n_u_target
#define __pyx_n_s_targets __pyx_mstate_global->__pyx_n_s_targets
#define __pyx_n_s_tbl __pyx_mstate_global->__pyx_n_s_tbl
#define __pyx_n_s_template_query __pyx_mstate_global->__pyx_n_s_template_query
#define __pyx_kp_u_term __pyx_mstate_global->__pyx_kp_u_term
#define __pyx_n_s_term_2 __pyx_mstate_global->__pyx_n_s_term_2
#define __pyx_n_u_term_2 __pyx_mstate_global->__pyx_n_u_term_2
#define __pyx_n_s_term_id __pyx_mstate_global->__pyx_n_s_term_id
#define __pyx_n_u_terminology __pyx_mstate_global->__pyx_n_u_terminology
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_tgt_name __pyx_mstate_global->__pyx_n_s_tgt_name
#define __pyx_n_s_throw __pyx_mstate_global->__pyx_n_s_throw
#define __pyx_n_u_to_table __pyx_mstate_global->__pyx_n_u_to_table
#define __pyx_n_s_to_table_id __pyx_mstate_global->__pyx_n_s_to_table_id
#define __pyx_n_s_type __pyx_mstate_global->__pyx_n_s_type
#define __pyx_n_u_type __pyx_mstate_global->__pyx_n_u_type
#define __pyx_n_s_typing __pyx_mstate_global->__pyx_n_s_typing
#define __pyx_n_s_unique_rules __pyx_mstate_global->__pyx_n_s_unique_rules
#define __pyx_n_u_unknown __pyx_mstate_global->__pyx_n_u_unknown
#define __pyx_n_s_used_aliases __pyx_mstate_global->__pyx_n_s_used_aliases
#define __pyx_n_s_user_prompt __pyx_mstate_global->__pyx_n_s_user_prompt
#define __pyx_kp_u_utf_8 __pyx_mstate_global->__pyx_kp_u_utf_8
#define __pyx_n_s_values __pyx_mstate_global->__pyx_n_s_values
#define __pyx_n_s_verbose __pyx_mstate_global->__pyx_n_s_verbose
#define __pyx_kp_u_version __pyx_mstate_global->__pyx_kp_u_version
#define __pyx_n_s_version_2 __pyx_mstate_global->__pyx_n_s_version_2
#define __pyx_n_s_where_clauses __pyx_mstate_global->__pyx_n_s_where_clauses
#define __pyx_n_s_word __pyx_mstate_global->__pyx_n_s_word
#define __pyx_n_s_x __pyx_mstate_global->__pyx_n_s_x
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_1 __pyx_mstate_global->__pyx_int_1
#define __pyx_int_2 __pyx_mstate_global->__pyx_int_2
#define __pyx_int_3 __pyx_mstate_global->__pyx_int_3
#define __pyx_int_4 __pyx_mstate_global->__pyx_int_4
#define __pyx_int_5 __pyx_mstate_global->__pyx_int_5
#define __pyx_int_neg_1 __pyx_mstate_global->__pyx_int_neg_1
#define __pyx_int_neg_3 __pyx_mstate_global->__pyx_int_neg_3
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__3 __pyx_mstate_global->__pyx_tuple__3
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_slice__11 __pyx_mstate_global->__pyx_slice__11
#define __pyx_tuple__13 __pyx_mstate_global->__pyx_tuple__13
#define __pyx_tuple__14 __pyx_mstate_global->__pyx_tuple__14
#define __pyx_tuple__16 __pyx_mstate_global->__pyx_tuple__16
#define __pyx_tuple__17 __pyx_mstate_global->__pyx_tuple__17
#define __pyx_tuple__18 __pyx_mstate_global->__pyx_tuple__18
#define __pyx_tuple__19 __pyx_mstate_global->__pyx_tuple__19
#define __pyx_tuple__21 __pyx_mstate_global->__pyx_tuple__21
#define __pyx_tuple__22 __pyx_mstate_global->__pyx_tuple__22
#define __pyx_tuple__24 __pyx_mstate_global->__pyx_tuple__24
#define __pyx_tuple__26 __pyx_mstate_global->__pyx_tuple__26
#define __pyx_tuple__30 __pyx_mstate_global->__pyx_tuple__30
#define __pyx_tuple__32 __pyx_mstate_global->__pyx_tuple__32
#define __pyx_tuple__33 __pyx_mstate_global->__pyx_tuple__33
#define __pyx_tuple__34 __pyx_mstate_global->__pyx_tuple__34
#define __pyx_tuple__46 __pyx_mstate_global->__pyx_tuple__46
#define __pyx_tuple__47 __pyx_mstate_global->__pyx_tuple__47
#define __pyx_tuple__49 __pyx_mstate_global->__pyx_tuple__49
#define __pyx_tuple__50 __pyx_mstate_global->__pyx_tuple__50
#define __pyx_tuple__52 __pyx_mstate_global->__pyx_tuple__52
#define __pyx_tuple__53 __pyx_mstate_global->__pyx_tuple__53
#define __pyx_tuple__56 __pyx_mstate_global->__pyx_tuple__56
#define __pyx_tuple__58 __pyx_mstate_global->__pyx_tuple__58
#define __pyx_tuple__59 __pyx_mstate_global->__pyx_tuple__59
#define __pyx_tuple__61 __pyx_mstate_global->__pyx_tuple__61
#define __pyx_tuple__62 __pyx_mstate_global->__pyx_tuple__62
#define __pyx_tuple__64 __pyx_mstate_global->__pyx_tuple__64
#define __pyx_tuple__65 __pyx_mstate_global->__pyx_tuple__65
#define __pyx_tuple__67 __pyx_mstate_global->__pyx_tuple__67
#define __pyx_tuple__69 __pyx_mstate_global->__pyx_tuple__69
#define __pyx_tuple__71 __pyx_mstate_global->__pyx_tuple__71
#define __pyx_tuple__73 __pyx_mstate_global->__pyx_tuple__73
#define __pyx_tuple__75 __pyx_mstate_global->__pyx_tuple__75
#define __pyx_tuple__77 __pyx_mstate_global->__pyx_tuple__77
#define __pyx_tuple__78 __pyx_mstate_global->__pyx_tuple__78
#define __pyx_tuple__80 __pyx_mstate_global->__pyx_tuple__80
#define __pyx_tuple__81 __pyx_mstate_global->__pyx_tuple__81
#define __pyx_tuple__83 __pyx_mstate_global->__pyx_tuple__83
#define __pyx_tuple__85 __pyx_mstate_global->__pyx_tuple__85
#define __pyx_tuple__86 __pyx_mstate_global->__pyx_tuple__86
#define __pyx_tuple__87 __pyx_mstate_global->__pyx_tuple__87
#define __pyx_tuple__88 __pyx_mstate_global->__pyx_tuple__88
#define __pyx_tuple__89 __pyx_mstate_global->__pyx_tuple__89
#define __pyx_tuple__90 __pyx_mstate_global->__pyx_tuple__90
#define __pyx_codeobj__25 __pyx_mstate_global->__pyx_codeobj__25
#define __pyx_codeobj__51 __pyx_mstate_global->__pyx_codeobj__51
#define __pyx_codeobj__54 __pyx_mstate_global->__pyx_codeobj__54
#define __pyx_codeobj__57 __pyx_mstate_global->__pyx_codeobj__57
#define __pyx_codeobj__60 __pyx_mstate_global->__pyx_codeobj__60
#define __pyx_codeobj__63 __pyx_mstate_global->__pyx_codeobj__63
#define __pyx_codeobj__66 __pyx_mstate_global->__pyx_codeobj__66
#define __pyx_codeobj__68 __pyx_mstate_global->__pyx_codeobj__68
#define __pyx_codeobj__70 __pyx_mstate_global->__pyx_codeobj__70
#define __pyx_codeobj__72 __pyx_mstate_global->__pyx_codeobj__72
#define __pyx_codeobj__74 __pyx_mstate_global->__pyx_codeobj__74
#define __pyx_codeobj__76 __pyx_mstate_global->__pyx_codeobj__76
#define __pyx_codeobj__79 __pyx_mstate_global->__pyx_codeobj__79
#define __pyx_codeobj__82 __pyx_mstate_global->__pyx_codeobj__82
#define __pyx_codeobj__84 __pyx_mstate_global->__pyx_codeobj__84
/* #### Code section: module_code ### */

/* "sQQideR/DatabaseContext.pyx":27
 *     """
 * 
 *     def __init__(self, metadata_file: str = None, verbose: bool = False, ai_base=None):             # <<<<<<<<<<<<<<
 *         """
 *         Initialize the service with optional metadata file.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService___init__, "\n        Initialize the service with optional metadata file.\n        \n        Args:\n            metadata_file: Path to the JSON metadata file\n            verbose: Whether to print verbose output during operations\n            ai_base: Optional AIBase instance for AI-enhanced functionality\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_1__init__ = {"__init__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService___init__};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_metadata_file = 0;
  PyObject *__pyx_v_verbose = 0;
  PyObject *__pyx_v_ai_base = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_metadata_file,&__pyx_n_s_verbose,&__pyx_n_s_ai_base,0};
    values[1] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)((PyObject *)Py_False)));
    values[3] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 27, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_metadata_file);
          if (value) { values[1] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_verbose);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_ai_base);
          if (value) { values[3] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 27, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 27, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_metadata_file = ((PyObject*)values[1]);
    __pyx_v_verbose = values[2];
    __pyx_v_ai_base = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, __pyx_nargs); __PYX_ERR(0, 27, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_metadata_file), (&PyUnicode_Type), 1, "metadata_file", 1))) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService___init__(__pyx_self, __pyx_v_self, __pyx_v_metadata_file, __pyx_v_verbose, __pyx_v_ai_base);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_metadata_file, PyObject *__pyx_v_verbose, PyObject *__pyx_v_ai_base) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "sQQideR/DatabaseContext.pyx":36
 *             ai_base: Optional AIBase instance for AI-enhanced functionality
 *         """
 *         self.graph = None             # <<<<<<<<<<<<<<
 *         self.verbose = verbose
 *         self.metadata = None
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_graph, Py_None) < 0) __PYX_ERR(0, 36, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":37
 *         """
 *         self.graph = None
 *         self.verbose = verbose             # <<<<<<<<<<<<<<
 *         self.metadata = None
 *         self.ai_base = ai_base
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_verbose, __pyx_v_verbose) < 0) __PYX_ERR(0, 37, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":38
 *         self.graph = None
 *         self.verbose = verbose
 *         self.metadata = None             # <<<<<<<<<<<<<<
 *         self.ai_base = ai_base
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_metadata, Py_None) < 0) __PYX_ERR(0, 38, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":39
 *         self.verbose = verbose
 *         self.metadata = None
 *         self.ai_base = ai_base             # <<<<<<<<<<<<<<
 * 
 *         if metadata_file:
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_ai_base, __pyx_v_ai_base) < 0) __PYX_ERR(0, 39, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":41
 *         self.ai_base = ai_base
 * 
 *         if metadata_file:             # <<<<<<<<<<<<<<
 *             self.load_metadata(metadata_file)
 * 
 */
  __pyx_t_1 = (__pyx_v_metadata_file != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_metadata_file) != 0);
  if (__pyx_t_1) {

    /* "sQQideR/DatabaseContext.pyx":42
 * 
 *         if metadata_file:
 *             self.load_metadata(metadata_file)             # <<<<<<<<<<<<<<
 * 
 *     def load_metadata(self, filename: str) -> Dict:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_load_metadata); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_metadata_file};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "sQQideR/DatabaseContext.pyx":41
 *         self.ai_base = ai_base
 * 
 *         if metadata_file:             # <<<<<<<<<<<<<<
 *             self.load_metadata(metadata_file)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":27
 *     """
 * 
 *     def __init__(self, metadata_file: str = None, verbose: bool = False, ai_base=None):             # <<<<<<<<<<<<<<
 *         """
 *         Initialize the service with optional metadata file.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":44
 *             self.load_metadata(metadata_file)
 * 
 *     def load_metadata(self, filename: str) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Load metadata from a JSON file.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_3load_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_2load_metadata, "\n        Load metadata from a JSON file.\n        \n        Args:\n            filename: Path to the JSON metadata file\n            \n        Returns:\n            The loaded metadata as a dictionary\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_3load_metadata = {"load_metadata", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_3load_metadata, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_2load_metadata};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_3load_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_filename = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load_metadata (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_filename,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 44, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_filename)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 44, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("load_metadata", 1, 2, 2, 1); __PYX_ERR(0, 44, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "load_metadata") < 0)) __PYX_ERR(0, 44, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_filename = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("load_metadata", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 44, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.load_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_filename), (&PyUnicode_Type), 0, "filename", 1))) __PYX_ERR(0, 44, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_2load_metadata(__pyx_self, __pyx_v_self, __pyx_v_filename);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_2load_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_filename) {
  PyObject *__pyx_v_f = NULL;
  PyObject *__pyx_v_metadata_sections = NULL;
  Py_ssize_t __pyx_v_table_count;
  Py_ssize_t __pyx_v_relationship_count;
  PyObject *__pyx_v_required_sections = NULL;
  PyObject *__pyx_v_missing = NULL;
  PyObject *__pyx_v_cleaned_at = NULL;
  PyObject *__pyx_v_version = NULL;
  PyObject *__pyx_7genexpr__pyx_v_s = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  Py_ssize_t __pyx_t_13;
  Py_UCS4 __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load_metadata", 1);

  /* "sQQideR/DatabaseContext.pyx":54
 *             The loaded metadata as a dictionary
 *         """
 *         if not os.path.exists(filename):             # <<<<<<<<<<<<<<
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_os); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_path); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_exists); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_filename};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "sQQideR/DatabaseContext.pyx":55
 *         """
 *         if not os.path.exists(filename):
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")             # <<<<<<<<<<<<<<
 * 
 *         with open(filename, 'r', encoding='utf-8') as f:  # Explicitly specify UTF-8 encoding
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FileNotFoundError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Metadata_file_not_found, __pyx_v_filename); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 55, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":54
 *             The loaded metadata as a dictionary
 *         """
 *         if not os.path.exists(filename):             # <<<<<<<<<<<<<<
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":57
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")
 * 
 *         with open(filename, 'r', encoding='utf-8') as f:  # Explicitly specify UTF-8 encoding             # <<<<<<<<<<<<<<
 *             self.metadata = json.load(f)
 * 
 */
  /*with:*/ {
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_filename);
    __Pyx_GIVEREF(__pyx_v_filename);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_filename)) __PYX_ERR(0, 57, __pyx_L1_error);
    __Pyx_INCREF(__pyx_n_u_r);
    __Pyx_GIVEREF(__pyx_n_u_r);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_n_u_r)) __PYX_ERR(0, 57, __pyx_L1_error);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_encoding, __pyx_kp_u_utf_8) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_open, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_7 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_exit); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 57, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_t_3 = __pyx_t_2;
    __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {
          __pyx_v_f = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":58
 * 
 *         with open(filename, 'r', encoding='utf-8') as f:  # Explicitly specify UTF-8 encoding
 *             self.metadata = json.load(f)             # <<<<<<<<<<<<<<
 * 
 *         if self.verbose:
 */
          __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_json); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_load); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = NULL;
          __pyx_t_4 = 0;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_2, function);
              __pyx_t_4 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_f};
            __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 58, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          }
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_metadata, __pyx_t_3) < 0) __PYX_ERR(0, 58, __pyx_L8_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":57
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")
 * 
 *         with open(filename, 'r', encoding='utf-8') as f:  # Explicitly specify UTF-8 encoding             # <<<<<<<<<<<<<<
 *             self.metadata = json.load(f)
 * 
 */
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L13_try_end;
        __pyx_L8_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.load_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1) < 0) __PYX_ERR(0, 57, __pyx_L10_except_error)
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_1);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 57, __pyx_L10_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 57, __pyx_L10_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_6 < 0) __PYX_ERR(0, 57, __pyx_L10_except_error)
          __pyx_t_5 = (!__pyx_t_6);
          if (unlikely(__pyx_t_5)) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_1);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_1 = 0; 
            __PYX_ERR(0, 57, __pyx_L10_except_error)
          }
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L9_exception_handled;
        }
        __pyx_L10_except_error:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L9_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        __pyx_L13_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_7) {
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple_, NULL);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 57, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        goto __pyx_L7;
      }
      __pyx_L7:;
    }
    goto __pyx_L17;
    __pyx_L4_error:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "sQQideR/DatabaseContext.pyx":60
 *             self.metadata = json.load(f)
 * 
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Loaded metadata from {filename}")
 *             metadata_sections = list(self.metadata.keys())
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "sQQideR/DatabaseContext.pyx":61
 * 
 *         if self.verbose:
 *             print(f"Loaded metadata from {filename}")             # <<<<<<<<<<<<<<
 *             metadata_sections = list(self.metadata.keys())
 *             table_count = len(self.metadata.get("tables", {}))
 */
    __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Loaded_metadata_from, __pyx_v_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "sQQideR/DatabaseContext.pyx":62
 *         if self.verbose:
 *             print(f"Loaded metadata from {filename}")
 *             metadata_sections = list(self.metadata.keys())             # <<<<<<<<<<<<<<
 *             table_count = len(self.metadata.get("tables", {}))
 *             relationship_count = len(self.metadata.get("relationships", []))
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_keys); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 62, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 62, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_t_3 = __Pyx_PySequence_ListKeepNew(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 62, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_metadata_sections = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":63
 *             print(f"Loaded metadata from {filename}")
 *             metadata_sections = list(self.metadata.keys())
 *             table_count = len(self.metadata.get("tables", {}))             # <<<<<<<<<<<<<<
 *             relationship_count = len(self.metadata.get("relationships", []))
 *             print(f"  - Found {table_count} tables and {relationship_count} relationships")
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_n_u_tables, __pyx_t_2};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_4, 2+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 63, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_t_13 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 63, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_table_count = __pyx_t_13;

    /* "sQQideR/DatabaseContext.pyx":64
 *             metadata_sections = list(self.metadata.keys())
 *             table_count = len(self.metadata.get("tables", {}))
 *             relationship_count = len(self.metadata.get("relationships", []))             # <<<<<<<<<<<<<<
 *             print(f"  - Found {table_count} tables and {relationship_count} relationships")
 *             print(f"  - Metadata contains sections: {', '.join(metadata_sections)}")
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_n_u_relationships, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 2+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 64, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_t_13 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_relationship_count = __pyx_t_13;

    /* "sQQideR/DatabaseContext.pyx":65
 *             table_count = len(self.metadata.get("tables", {}))
 *             relationship_count = len(self.metadata.get("relationships", []))
 *             print(f"  - Found {table_count} tables and {relationship_count} relationships")             # <<<<<<<<<<<<<<
 *             print(f"  - Metadata contains sections: {', '.join(metadata_sections)}")
 * 
 */
    __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_13 = 0;
    __pyx_t_14 = 127;
    __Pyx_INCREF(__pyx_kp_u_Found);
    __pyx_t_13 += 10;
    __Pyx_GIVEREF(__pyx_kp_u_Found);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Found);
    __pyx_t_2 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_table_count, 0, ' ', 'd'); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_kp_u_tables_and);
    __pyx_t_13 += 12;
    __Pyx_GIVEREF(__pyx_kp_u_tables_and);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_tables_and);
    __pyx_t_2 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_relationship_count, 0, ' ', 'd'); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_kp_u_relationships_2);
    __pyx_t_13 += 14;
    __Pyx_GIVEREF(__pyx_kp_u_relationships_2);
    PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_kp_u_relationships_2);
    __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_3, 5, __pyx_t_13, __pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":66
 *             relationship_count = len(self.metadata.get("relationships", []))
 *             print(f"  - Found {table_count} tables and {relationship_count} relationships")
 *             print(f"  - Metadata contains sections: {', '.join(metadata_sections)}")             # <<<<<<<<<<<<<<
 * 
 *             # Check for required sections
 */
    __pyx_t_3 = PyUnicode_Join(__pyx_kp_u__2, __pyx_v_metadata_sections); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Metadata_contains_sections, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":69
 * 
 *             # Check for required sections
 *             required_sections = ["tables", "relationships"]             # <<<<<<<<<<<<<<
 *             missing = [s for s in required_sections if s not in metadata_sections]
 *             if missing:
 */
    __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_n_u_tables);
    __Pyx_GIVEREF(__pyx_n_u_tables);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_tables)) __PYX_ERR(0, 69, __pyx_L1_error);
    __Pyx_INCREF(__pyx_n_u_relationships);
    __Pyx_GIVEREF(__pyx_n_u_relationships);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_u_relationships)) __PYX_ERR(0, 69, __pyx_L1_error);
    __pyx_v_required_sections = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":70
 *             # Check for required sections
 *             required_sections = ["tables", "relationships"]
 *             missing = [s for s in required_sections if s not in metadata_sections]             # <<<<<<<<<<<<<<
 *             if missing:
 *                 print(f"  - WARNING: Missing required sections: {', '.join(missing)}")
 */
    { /* enter inner scope */
      __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 70, __pyx_L21_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = __pyx_v_required_sections; __Pyx_INCREF(__pyx_t_2);
      __pyx_t_13 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 70, __pyx_L21_error)
          #endif
          if (__pyx_t_13 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_13); __Pyx_INCREF(__pyx_t_1); __pyx_t_13++; if (unlikely((0 < 0))) __PYX_ERR(0, 70, __pyx_L21_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L21_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_s, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_7genexpr__pyx_v_s, __pyx_v_metadata_sections, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 70, __pyx_L21_error)
        if (__pyx_t_5) {
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_7genexpr__pyx_v_s))) __PYX_ERR(0, 70, __pyx_L21_error)
        }
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_s); __pyx_7genexpr__pyx_v_s = 0;
      goto __pyx_L26_exit_scope;
      __pyx_L21_error:;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_s); __pyx_7genexpr__pyx_v_s = 0;
      goto __pyx_L1_error;
      __pyx_L26_exit_scope:;
    } /* exit inner scope */
    __pyx_v_missing = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":71
 *             required_sections = ["tables", "relationships"]
 *             missing = [s for s in required_sections if s not in metadata_sections]
 *             if missing:             # <<<<<<<<<<<<<<
 *                 print(f"  - WARNING: Missing required sections: {', '.join(missing)}")
 * 
 */
    __pyx_t_5 = (PyList_GET_SIZE(__pyx_v_missing) != 0);
    if (__pyx_t_5) {

      /* "sQQideR/DatabaseContext.pyx":72
 *             missing = [s for s in required_sections if s not in metadata_sections]
 *             if missing:
 *                 print(f"  - WARNING: Missing required sections: {', '.join(missing)}")             # <<<<<<<<<<<<<<
 * 
 *             # Additional info about enriched sections
 */
      __pyx_t_3 = PyUnicode_Join(__pyx_kp_u__2, __pyx_v_missing); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_WARNING_Missing_required_sectio, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "sQQideR/DatabaseContext.pyx":71
 *             required_sections = ["tables", "relationships"]
 *             missing = [s for s in required_sections if s not in metadata_sections]
 *             if missing:             # <<<<<<<<<<<<<<
 *                 print(f"  - WARNING: Missing required sections: {', '.join(missing)}")
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":75
 * 
 *             # Additional info about enriched sections
 *             if "metadata_cleaning" in metadata_sections:             # <<<<<<<<<<<<<<
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")
 */
    __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_metadata_cleaning, __pyx_v_metadata_sections, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 75, __pyx_L1_error)
    if (__pyx_t_5) {

      /* "sQQideR/DatabaseContext.pyx":76
 *             # Additional info about enriched sections
 *             if "metadata_cleaning" in metadata_sections:
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")             # <<<<<<<<<<<<<<
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_metadata_cleaning); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_cleaned_at = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":77
 *             if "metadata_cleaning" in metadata_sections:
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")             # <<<<<<<<<<<<<<
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_metadata_cleaning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_version = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "sQQideR/DatabaseContext.pyx":78
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")             # <<<<<<<<<<<<<<
 * 
 *         return self.metadata
 */
      __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_13 = 0;
      __pyx_t_14 = 127;
      __Pyx_INCREF(__pyx_kp_u_Metadata_was_cleaned_at);
      __pyx_t_13 += 28;
      __Pyx_GIVEREF(__pyx_kp_u_Metadata_was_cleaned_at);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Metadata_was_cleaned_at);
      __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_cleaned_at, __pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_14 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) > __pyx_t_14) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) : __pyx_t_14;
      __pyx_t_13 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_kp_u_version);
      __pyx_t_13 += 10;
      __Pyx_GIVEREF(__pyx_kp_u_version);
      PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_version);
      __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_version, __pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_14 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) > __pyx_t_14) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) : __pyx_t_14;
      __pyx_t_13 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_kp_u__5);
      __pyx_t_13 += 1;
      __Pyx_GIVEREF(__pyx_kp_u__5);
      PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_kp_u__5);
      __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_3, 5, __pyx_t_13, __pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "sQQideR/DatabaseContext.pyx":75
 * 
 *             # Additional info about enriched sections
 *             if "metadata_cleaning" in metadata_sections:             # <<<<<<<<<<<<<<
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")
 */
    }

    /* "sQQideR/DatabaseContext.pyx":60
 *             self.metadata = json.load(f)
 * 
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Loaded metadata from {filename}")
 *             metadata_sections = list(self.metadata.keys())
 */
  }

  /* "sQQideR/DatabaseContext.pyx":80
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")
 * 
 *         return self.metadata             # <<<<<<<<<<<<<<
 * 
 *     # ...existing code...
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(PyDict_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_3))) __PYX_ERR(0, 80, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":44
 *             self.load_metadata(metadata_file)
 * 
 *     def load_metadata(self, filename: str) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Load metadata from a JSON file.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.load_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_metadata_sections);
  __Pyx_XDECREF(__pyx_v_required_sections);
  __Pyx_XDECREF(__pyx_v_missing);
  __Pyx_XDECREF(__pyx_v_cleaned_at);
  __Pyx_XDECREF(__pyx_v_version);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_s);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":84
 *     # ...existing code...
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,             # <<<<<<<<<<<<<<
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:
 *         """
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_5process_query_with_chat_history(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_4process_query_with_chat_history, "\n        Process a natural language query with chat history using AI enhancement and return context information.\n        \n        Args:\n            query_text: Natural language query string\n            chat_history: Optional list of previous chat messages\n            previous_sql: Optional previous SQL query that was executed\n            output_format: Output format, 'json' or 'markdown'\n            \n        Returns:\n            Dictionary with context information\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_5process_query_with_chat_history = {"process_query_with_chat_history", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_5process_query_with_chat_history, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_4process_query_with_chat_history};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_5process_query_with_chat_history(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_query_text = 0;
  PyObject *__pyx_v_chat_history = 0;
  PyObject *__pyx_v_previous_sql = 0;
  PyObject *__pyx_v_output_format = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("process_query_with_chat_history (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_query_text,&__pyx_n_s_chat_history,&__pyx_n_s_previous_sql,&__pyx_n_s_output_format,0};
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));

    /* "sQQideR/DatabaseContext.pyx":85
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query with chat history using AI enhancement and return context information.
 */
    values[3] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    values[4] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)((PyObject*)__pyx_n_u_json)));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 84, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_query_text)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 84, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("process_query_with_chat_history", 0, 2, 5, 1); __PYX_ERR(0, 84, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_chat_history);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 84, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_previous_sql);
          if (value) { values[3] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 84, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_output_format);
          if (value) { values[4] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 84, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "process_query_with_chat_history") < 0)) __PYX_ERR(0, 84, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_query_text = ((PyObject*)values[1]);
    __pyx_v_chat_history = ((PyObject*)values[2]);
    __pyx_v_previous_sql = ((PyObject*)values[3]);
    __pyx_v_output_format = ((PyObject*)values[4]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("process_query_with_chat_history", 0, 2, 5, __pyx_nargs); __PYX_ERR(0, 84, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_chat_history", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query_text), (&PyUnicode_Type), 0, "query_text", 1))) __PYX_ERR(0, 84, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_chat_history), (&PyList_Type), 1, "chat_history", 1))) __PYX_ERR(0, 84, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_previous_sql), (&PyUnicode_Type), 1, "previous_sql", 1))) __PYX_ERR(0, 85, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_output_format), (&PyUnicode_Type), 0, "output_format", 1))) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_4process_query_with_chat_history(__pyx_self, __pyx_v_self, __pyx_v_query_text, __pyx_v_chat_history, __pyx_v_previous_sql, __pyx_v_output_format);

  /* "sQQideR/DatabaseContext.pyx":84
 *     # ...existing code...
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,             # <<<<<<<<<<<<<<
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:
 *         """
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_4process_query_with_chat_history(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_chat_history, PyObject *__pyx_v_previous_sql, PyObject *__pyx_v_output_format) {
  PyObject *__pyx_v_enhanced_query = NULL;
  PyObject *__pyx_v_context_prompt = NULL;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_role = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_clean_response = NULL;
  PyObject *__pyx_v_first_newline = NULL;
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_keywords = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_matches = NULL;
  PyObject *__pyx_v_ai_context = NULL;
  PyObject *__pyx_v_tables = NULL;
  PyObject *__pyx_v_columns = NULL;
  PyObject *__pyx_v_concepts = NULL;
  PyObject *__pyx_v_node_id = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_domain = NULL;
  PyObject *__pyx_v_domain_results = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_concept = NULL;
  PyObject *__pyx_v_neighbor = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_table_id = NULL;
  PyObject *__pyx_v_all_tables = NULL;
  PyObject *__pyx_v_all_rules = NULL;
  PyObject *__pyx_v_node_rules = NULL;
  PyObject *__pyx_v_unique_rules = NULL;
  PyObject *__pyx_v_rule_id = NULL;
  PyObject *__pyx_v_rule_desc = NULL;
  PyObject *__pyx_v_filtered_rules = NULL;
  PyObject *__pyx_v_sql = NULL;
  PyObject *__pyx_v_output = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_word = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_UCS4 __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  unsigned int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  int __pyx_t_20;
  char const *__pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  char const *__pyx_t_28;
  PyObject *(*__pyx_t_29)(PyObject *);
  PyObject *(*__pyx_t_30)(PyObject *);
  int __pyx_t_31;
  PyObject *(*__pyx_t_32)(PyObject *);
  int __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_query_with_chat_history", 1);

  /* "sQQideR/DatabaseContext.pyx":98
 *             Dictionary with context information
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":99
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         # Get AI-enhanced query information
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 99, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 99, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":98
 *             Dictionary with context information
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":102
 * 
 *         # Get AI-enhanced query information
 *         enhanced_query = None             # <<<<<<<<<<<<<<
 *         if self.ai_base:
 *             try:
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_enhanced_query = Py_None;

  /* "sQQideR/DatabaseContext.pyx":103
 *         # Get AI-enhanced query information
 *         enhanced_query = None
 *         if self.ai_base:             # <<<<<<<<<<<<<<
 *             try:
 *                 # Build prompt that includes chat history if available
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":104
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Build prompt that includes chat history if available
 *                 context_prompt = f"""
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "sQQideR/DatabaseContext.pyx":106
 *             try:
 *                 # Build prompt that includes chat history if available
 *                 context_prompt = f"""             # <<<<<<<<<<<<<<
 *                 You are an expert in project management and database systems. Analyze this question:
 * 
 */
        __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = 0;
        __pyx_t_7 = 127;
        __Pyx_INCREF(__pyx_kp_u_You_are_an_expert_in_project_ma);
        __pyx_t_6 += 136;
        __Pyx_GIVEREF(__pyx_kp_u_You_are_an_expert_in_project_ma);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_You_are_an_expert_in_project_ma);
        __Pyx_INCREF(__pyx_v_query_text);
        __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_query_text) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_query_text) : __pyx_t_7;
        __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_v_query_text);
        __Pyx_GIVEREF(__pyx_v_query_text);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_query_text);
        __Pyx_INCREF(__pyx_kp_u__7);
        __pyx_t_6 += 18;
        __Pyx_GIVEREF(__pyx_kp_u__7);
        PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__7);
        __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 106, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v_context_prompt = ((PyObject*)__pyx_t_8);
        __pyx_t_8 = 0;

        /* "sQQideR/DatabaseContext.pyx":113
 * 
 *                 # Add chat history context if available
 *                 if chat_history and len(chat_history) > 0:             # <<<<<<<<<<<<<<
 *                     context_prompt += "\n\nConsider this conversation history:\n"
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context
 */
        __pyx_t_9 = (__pyx_v_chat_history != Py_None)&&(PyList_GET_SIZE(__pyx_v_chat_history) != 0);
        if (__pyx_t_9) {
        } else {
          __pyx_t_2 = __pyx_t_9;
          goto __pyx_L12_bool_binop_done;
        }
        if (unlikely(__pyx_v_chat_history == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 113, __pyx_L5_error)
        }
        __pyx_t_6 = __Pyx_PyList_GET_SIZE(__pyx_v_chat_history); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 113, __pyx_L5_error)
        __pyx_t_9 = (__pyx_t_6 > 0);
        __pyx_t_2 = __pyx_t_9;
        __pyx_L12_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":114
 *                 # Add chat history context if available
 *                 if chat_history and len(chat_history) > 0:
 *                     context_prompt += "\n\nConsider this conversation history:\n"             # <<<<<<<<<<<<<<
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context
 *                         role = "User" if msg.is_user else "Assistant"
 */
          __pyx_t_8 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_context_prompt, __pyx_kp_u_Consider_this_conversation_hist); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 114, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF_SET(__pyx_v_context_prompt, ((PyObject*)__pyx_t_8));
          __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":115
 *                 if chat_history and len(chat_history) > 0:
 *                     context_prompt += "\n\nConsider this conversation history:\n"
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context             # <<<<<<<<<<<<<<
 *                         role = "User" if msg.is_user else "Assistant"
 *                         context_prompt += f"{role}: {msg.content}\n"
 */
          if (unlikely(__pyx_v_chat_history == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 115, __pyx_L5_error)
          }
          __pyx_t_8 = __Pyx_PyList_GetSlice(__pyx_v_chat_history, -5L, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 115, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_1 = __pyx_t_8; __Pyx_INCREF(__pyx_t_1);
          __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          for (;;) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 115, __pyx_L5_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 115, __pyx_L5_error)
            #else
            __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 115, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
            __Pyx_XDECREF_SET(__pyx_v_msg, __pyx_t_8);
            __pyx_t_8 = 0;

            /* "sQQideR/DatabaseContext.pyx":116
 *                     context_prompt += "\n\nConsider this conversation history:\n"
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context
 *                         role = "User" if msg.is_user else "Assistant"             # <<<<<<<<<<<<<<
 *                         context_prompt += f"{role}: {msg.content}\n"
 * 
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_msg, __pyx_n_s_is_user); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 116, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 116, __pyx_L5_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (__pyx_t_2) {
              __Pyx_INCREF(__pyx_n_u_User);
              __pyx_t_8 = __pyx_n_u_User;
            } else {
              __Pyx_INCREF(__pyx_n_u_Assistant);
              __pyx_t_8 = __pyx_n_u_Assistant;
            }
            __Pyx_XDECREF_SET(__pyx_v_role, ((PyObject*)__pyx_t_8));
            __pyx_t_8 = 0;

            /* "sQQideR/DatabaseContext.pyx":117
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context
 *                         role = "User" if msg.is_user else "Assistant"
 *                         context_prompt += f"{role}: {msg.content}\n"             # <<<<<<<<<<<<<<
 * 
 *                 # Add previous SQL context if available
 */
            __pyx_t_8 = PyTuple_New(4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_11 = 0;
            __pyx_t_7 = 127;
            __pyx_t_10 = __Pyx_PyUnicode_Unicode(__pyx_v_role); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_7;
            __pyx_t_11 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
            __Pyx_GIVEREF(__pyx_t_10);
            PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_10);
            __pyx_t_10 = 0;
            __Pyx_INCREF(__pyx_kp_u__8);
            __pyx_t_11 += 2;
            __Pyx_GIVEREF(__pyx_kp_u__8);
            PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_kp_u__8);
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_msg, __pyx_n_s_content); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_12 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_12) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_12) : __pyx_t_7;
            __pyx_t_11 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_12);
            __Pyx_GIVEREF(__pyx_t_12);
            PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_12);
            __pyx_t_12 = 0;
            __Pyx_INCREF(__pyx_kp_u__9);
            __pyx_t_11 += 1;
            __Pyx_GIVEREF(__pyx_kp_u__9);
            PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_kp_u__9);
            __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_8, 4, __pyx_t_11, __pyx_t_7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __pyx_t_8 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_context_prompt, __pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 117, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF_SET(__pyx_v_context_prompt, ((PyObject*)__pyx_t_8));
            __pyx_t_8 = 0;

            /* "sQQideR/DatabaseContext.pyx":115
 *                 if chat_history and len(chat_history) > 0:
 *                     context_prompt += "\n\nConsider this conversation history:\n"
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context             # <<<<<<<<<<<<<<
 *                         role = "User" if msg.is_user else "Assistant"
 *                         context_prompt += f"{role}: {msg.content}\n"
 */
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "sQQideR/DatabaseContext.pyx":113
 * 
 *                 # Add chat history context if available
 *                 if chat_history and len(chat_history) > 0:             # <<<<<<<<<<<<<<
 *                     context_prompt += "\n\nConsider this conversation history:\n"
 *                     for msg in chat_history[-5:]:  # Use last 5 messages for context
 */
        }

        /* "sQQideR/DatabaseContext.pyx":120
 * 
 *                 # Add previous SQL context if available
 *                 if previous_sql:             # <<<<<<<<<<<<<<
 *                     context_prompt += f"\n\nThe previous SQL query was:\n{previous_sql}\n"
 * 
 */
        __pyx_t_2 = (__pyx_v_previous_sql != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_previous_sql) != 0);
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":121
 *                 # Add previous SQL context if available
 *                 if previous_sql:
 *                     context_prompt += f"\n\nThe previous SQL query was:\n{previous_sql}\n"             # <<<<<<<<<<<<<<
 * 
 *                 context_prompt += """
 */
          __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_6 = 0;
          __pyx_t_7 = 127;
          __Pyx_INCREF(__pyx_kp_u_The_previous_SQL_query_was);
          __pyx_t_6 += 30;
          __Pyx_GIVEREF(__pyx_kp_u_The_previous_SQL_query_was);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_The_previous_SQL_query_was);
          __pyx_t_8 = __Pyx_PyUnicode_Unicode(__pyx_v_previous_sql); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 121, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
          __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_8);
          __pyx_t_8 = 0;
          __Pyx_INCREF(__pyx_kp_u__9);
          __pyx_t_6 += 1;
          __Pyx_GIVEREF(__pyx_kp_u__9);
          PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__9);
          __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 121, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_context_prompt, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF_SET(__pyx_v_context_prompt, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "sQQideR/DatabaseContext.pyx":120
 * 
 *                 # Add previous SQL context if available
 *                 if previous_sql:             # <<<<<<<<<<<<<<
 *                     context_prompt += f"\n\nThe previous SQL query was:\n{previous_sql}\n"
 * 
 */
        }

        /* "sQQideR/DatabaseContext.pyx":123
 *                     context_prompt += f"\n\nThe previous SQL query was:\n{previous_sql}\n"
 * 
 *                 context_prompt += """             # <<<<<<<<<<<<<<
 *                 1. Extract the key business domains this relates to (from: scheduling, resource management, critical path, risk assessment, etc.)
 *                 2. Extract important keywords for database searching (specific terms, metrics, or concepts mentioned)
 */
        __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_context_prompt, __pyx_kp_u_1_Extract_the_key_business_doma); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_context_prompt, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":141
 * 
 *                 # Use the synchronous run method
 *                 response = self.ai_base.run(context_prompt)             # <<<<<<<<<<<<<<
 * 
 *                 # Clean the response in case it contains markdown formatting
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_run); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 141, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_context_prompt};
          __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __pyx_v_response = __pyx_t_1;
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":144
 * 
 *                 # Clean the response in case it contains markdown formatting
 *                 clean_response = response.strip()             # <<<<<<<<<<<<<<
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):
 */
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_response, __pyx_n_s_strip); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 144, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_8 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
          __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 0+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __pyx_v_clean_response = __pyx_t_1;
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":146
 *                 clean_response = response.strip()
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):             # <<<<<<<<<<<<<<
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 */
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_startswith); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 146, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_8 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u__10};
          __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 146, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":147
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")             # <<<<<<<<<<<<<<
 *                     if first_newline != -1:
 *                         clean_response = clean_response[first_newline:].strip()
 */
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_find); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 147, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_8 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_12, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u__9};
            __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          }
          __pyx_v_first_newline = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "sQQideR/DatabaseContext.pyx":148
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:             # <<<<<<<<<<<<<<
 *                         clean_response = clean_response[first_newline:].strip()
 *                 # Remove any trailing markdown code block identifiers
 */
          __pyx_t_2 = (__Pyx_PyInt_BoolNeObjC(__pyx_v_first_newline, __pyx_int_neg_1, -1L, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 148, __pyx_L5_error)
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":149
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 *                         clean_response = clean_response[first_newline:].strip()             # <<<<<<<<<<<<<<
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):
 */
            __pyx_t_12 = __Pyx_PyObject_GetSlice(__pyx_v_clean_response, 0, 0, &__pyx_v_first_newline, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 149, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_strip); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 149, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            __pyx_t_13 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_8))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_8);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_8, function);
                __pyx_t_13 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_12, NULL};
              __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 0+__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            }
            __Pyx_DECREF_SET(__pyx_v_clean_response, __pyx_t_1);
            __pyx_t_1 = 0;

            /* "sQQideR/DatabaseContext.pyx":148
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:             # <<<<<<<<<<<<<<
 *                         clean_response = clean_response[first_newline:].strip()
 *                 # Remove any trailing markdown code block identifiers
 */
          }

          /* "sQQideR/DatabaseContext.pyx":146
 *                 clean_response = response.strip()
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):             # <<<<<<<<<<<<<<
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 */
        }

        /* "sQQideR/DatabaseContext.pyx":151
 *                         clean_response = clean_response[first_newline:].strip()
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):             # <<<<<<<<<<<<<<
 *                     clean_response = clean_response[:-3].strip()
 * 
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_endswith); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 151, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_12 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u__10};
          __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 151, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":152
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):
 *                     clean_response = clean_response[:-3].strip()             # <<<<<<<<<<<<<<
 * 
 *                 try:
 */
          __pyx_t_8 = __Pyx_PyObject_GetSlice(__pyx_v_clean_response, 0, -3L, NULL, NULL, &__pyx_slice__11, 0, 1, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 152, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_strip); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 152, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_12, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
            __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 0+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          }
          __Pyx_DECREF_SET(__pyx_v_clean_response, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "sQQideR/DatabaseContext.pyx":151
 *                         clean_response = clean_response[first_newline:].strip()
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):             # <<<<<<<<<<<<<<
 *                     clean_response = clean_response[:-3].strip()
 * 
 */
        }

        /* "sQQideR/DatabaseContext.pyx":154
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          /*try:*/ {

            /* "sQQideR/DatabaseContext.pyx":156
 *                 try:
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)             # <<<<<<<<<<<<<<
 * 
 *                     if self.verbose:
 */
            __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_json); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 156, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_loads); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 156, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_12 = NULL;
            __pyx_t_13 = 0;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_8))) {
              __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_8);
              if (likely(__pyx_t_12)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_8, function);
                __pyx_t_13 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_clean_response};
              __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            }
            __pyx_v_parsed = __pyx_t_1;
            __pyx_t_1 = 0;

            /* "sQQideR/DatabaseContext.pyx":158
 *                     parsed = json.loads(clean_response)
 * 
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 158, __pyx_L21_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (__pyx_t_2) {

              /* "sQQideR/DatabaseContext.pyx":159
 * 
 *                     if self.verbose:
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")             # <<<<<<<<<<<<<<
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 */
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 159, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_enhanced_query, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 159, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "sQQideR/DatabaseContext.pyx":160
 *                     if self.verbose:
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")             # <<<<<<<<<<<<<<
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 *                         if 'history_context' in parsed:
 */
              __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 160, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __pyx_t_12 = PyList_New(0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 160, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_12);
              __pyx_t_10 = NULL;
              __pyx_t_13 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_8))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
                if (likely(__pyx_t_10)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_8, function);
                  __pyx_t_13 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[3] = {__pyx_t_10, __pyx_n_u_keywords, __pyx_t_12};
                __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              }
              __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 160, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_extracted_keywords, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 160, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

              /* "sQQideR/DatabaseContext.pyx":161
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")             # <<<<<<<<<<<<<<
 *                         if 'history_context' in parsed:
 *                             print(f"AI history context: {parsed.get('history_context', '')}")
 */
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_12 = PyList_New(0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 161, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_12);
              __pyx_t_10 = NULL;
              __pyx_t_13 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_1))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
                if (likely(__pyx_t_10)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_1, function);
                  __pyx_t_13 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[3] = {__pyx_t_10, __pyx_n_u_business_domains, __pyx_t_12};
                __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 161, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              }
              __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_identified_domains, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 161, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "sQQideR/DatabaseContext.pyx":162
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 *                         if 'history_context' in parsed:             # <<<<<<<<<<<<<<
 *                             print(f"AI history context: {parsed.get('history_context', '')}")
 * 
 */
              __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_history_context, __pyx_v_parsed, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 162, __pyx_L21_error)
              if (__pyx_t_2) {

                /* "sQQideR/DatabaseContext.pyx":163
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 *                         if 'history_context' in parsed:
 *                             print(f"AI history context: {parsed.get('history_context', '')}")             # <<<<<<<<<<<<<<
 * 
 *                     enhanced_query = SQLPrompt(
 */
                __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_1);
                __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 163, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_history_context, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 163, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L21_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

                /* "sQQideR/DatabaseContext.pyx":162
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 *                         if 'history_context' in parsed:             # <<<<<<<<<<<<<<
 *                             print(f"AI history context: {parsed.get('history_context', '')}")
 * 
 */
              }

              /* "sQQideR/DatabaseContext.pyx":158
 *                     parsed = json.loads(clean_response)
 * 
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 */
            }

            /* "sQQideR/DatabaseContext.pyx":165
 *                             print(f"AI history context: {parsed.get('history_context', '')}")
 * 
 *                     enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 */
            __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SQLPrompt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_1);

            /* "sQQideR/DatabaseContext.pyx":166
 * 
 *                     enhanced_query = SQLPrompt(
 *                         user_prompt=query_text,             # <<<<<<<<<<<<<<
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),
 */
            __pyx_t_8 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 166, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_8);
            if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_user_prompt, __pyx_v_query_text) < 0) __PYX_ERR(0, 166, __pyx_L21_error)

            /* "sQQideR/DatabaseContext.pyx":167
 *                     enhanced_query = SQLPrompt(
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),             # <<<<<<<<<<<<<<
 *                         buisness_domains=parsed.get("business_domains", []),
 *                         keywords=parsed.get("keywords", [])
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 167, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_17 = NULL;
            __pyx_t_13 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_17)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_17);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
                __pyx_t_13 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_17, __pyx_n_u_enhanced_query, __pyx_v_query_text};
              __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 167, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_12);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_ai_context, __pyx_t_12) < 0) __PYX_ERR(0, 166, __pyx_L21_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "sQQideR/DatabaseContext.pyx":168
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),             # <<<<<<<<<<<<<<
 *                         keywords=parsed.get("keywords", [])
 *                     )
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 168, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_17 = PyList_New(0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 168, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_18 = NULL;
            __pyx_t_13 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_18)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_18);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
                __pyx_t_13 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_n_u_business_domains, __pyx_t_17};
              __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 168, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_12);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_buisness_domains, __pyx_t_12) < 0) __PYX_ERR(0, 166, __pyx_L21_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "sQQideR/DatabaseContext.pyx":169
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),
 *                         keywords=parsed.get("keywords", [])             # <<<<<<<<<<<<<<
 *                     )
 *                 except json.JSONDecodeError as e:
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 169, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_17 = PyList_New(0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 169, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_18 = NULL;
            __pyx_t_13 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_18)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_18);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
                __pyx_t_13 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_n_u_keywords, __pyx_t_17};
              __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
              __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 169, __pyx_L21_error)
              __Pyx_GOTREF(__pyx_t_12);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_keywords, __pyx_t_12) < 0) __PYX_ERR(0, 166, __pyx_L21_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "sQQideR/DatabaseContext.pyx":165
 *                             print(f"AI history context: {parsed.get('history_context', '')}")
 * 
 *                     enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 */
            __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 165, __pyx_L21_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF_SET(__pyx_v_enhanced_query, __pyx_t_12);
            __pyx_t_12 = 0;

            /* "sQQideR/DatabaseContext.pyx":154
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
          }
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L26_try_end;
          __pyx_L21_error:;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":171
 *                         keywords=parsed.get("keywords", [])
 *                     )
 *                 except json.JSONDecodeError as e:             # <<<<<<<<<<<<<<
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 */
          __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_8, &__pyx_t_1);
          __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_json); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 171, __pyx_L23_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_JSONDecodeError); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 171, __pyx_L23_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_19 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_12, __pyx_t_17);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_ErrRestore(__pyx_t_12, __pyx_t_8, __pyx_t_1);
          __pyx_t_12 = 0; __pyx_t_8 = 0; __pyx_t_1 = 0;
          if (__pyx_t_19) {
            __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_chat_history", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_12) < 0) __PYX_ERR(0, 171, __pyx_L23_except_error)
            __Pyx_XGOTREF(__pyx_t_1);
            __Pyx_XGOTREF(__pyx_t_8);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_8);
            __pyx_v_e = __pyx_t_8;
            /*try:*/ {

              /* "sQQideR/DatabaseContext.pyx":172
 *                     )
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 */
              __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 172, __pyx_L34_error)
              __Pyx_GOTREF(__pyx_t_17);
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_17); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 172, __pyx_L34_error)
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              if (__pyx_t_2) {

                /* "sQQideR/DatabaseContext.pyx":173
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")             # <<<<<<<<<<<<<<
 *                         print(f"Raw response: {response}")
 *                         print(f"Cleaned response: {clean_response}")
 */
                __pyx_t_17 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_empty_unicode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 173, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Error_parsing_AI_response, __pyx_t_17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_10);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                __pyx_t_17 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_10); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 173, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

                /* "sQQideR/DatabaseContext.pyx":174
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")             # <<<<<<<<<<<<<<
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 */
                __pyx_t_17 = __Pyx_PyObject_FormatSimple(__pyx_v_response, __pyx_empty_unicode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 174, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Raw_response, __pyx_t_17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 174, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_10);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                __pyx_t_17 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_10); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 174, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

                /* "sQQideR/DatabaseContext.pyx":175
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 *                         print(f"Cleaned response: {clean_response}")             # <<<<<<<<<<<<<<
 *                     # Fallback handled below
 *             except Exception as e:
 */
                __pyx_t_17 = __Pyx_PyObject_FormatSimple(__pyx_v_clean_response, __pyx_empty_unicode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 175, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Cleaned_response, __pyx_t_17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 175, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_10);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                __pyx_t_17 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_10); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 175, __pyx_L34_error)
                __Pyx_GOTREF(__pyx_t_17);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

                /* "sQQideR/DatabaseContext.pyx":172
 *                     )
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 */
              }
            }

            /* "sQQideR/DatabaseContext.pyx":171
 *                         keywords=parsed.get("keywords", [])
 *                     )
 *                 except json.JSONDecodeError as e:             # <<<<<<<<<<<<<<
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 */
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                goto __pyx_L35;
              }
              __pyx_L34_error:;
              /*exception exit:*/{
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0; __pyx_t_27 = 0;
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_25, &__pyx_t_26, &__pyx_t_27);
                if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24) < 0)) __Pyx_ErrFetch(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
                __Pyx_XGOTREF(__pyx_t_22);
                __Pyx_XGOTREF(__pyx_t_23);
                __Pyx_XGOTREF(__pyx_t_24);
                __Pyx_XGOTREF(__pyx_t_25);
                __Pyx_XGOTREF(__pyx_t_26);
                __Pyx_XGOTREF(__pyx_t_27);
                __pyx_t_19 = __pyx_lineno; __pyx_t_20 = __pyx_clineno; __pyx_t_21 = __pyx_filename;
                {
                  __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                }
                if (PY_MAJOR_VERSION >= 3) {
                  __Pyx_XGIVEREF(__pyx_t_25);
                  __Pyx_XGIVEREF(__pyx_t_26);
                  __Pyx_XGIVEREF(__pyx_t_27);
                  __Pyx_ExceptionReset(__pyx_t_25, __pyx_t_26, __pyx_t_27);
                }
                __Pyx_XGIVEREF(__pyx_t_22);
                __Pyx_XGIVEREF(__pyx_t_23);
                __Pyx_XGIVEREF(__pyx_t_24);
                __Pyx_ErrRestore(__pyx_t_22, __pyx_t_23, __pyx_t_24);
                __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0; __pyx_t_27 = 0;
                __pyx_lineno = __pyx_t_19; __pyx_clineno = __pyx_t_20; __pyx_filename = __pyx_t_21;
                goto __pyx_L23_except_error;
              }
              __pyx_L35:;
            }
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            goto __pyx_L22_exception_handled;
          }
          goto __pyx_L23_except_error;

          /* "sQQideR/DatabaseContext.pyx":154
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
          __pyx_L23_except_error:;
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L5_error;
          __pyx_L22_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          __pyx_L26_try_end:;
        }

        /* "sQQideR/DatabaseContext.pyx":104
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Build prompt that includes chat history if available
 *                 context_prompt = f"""
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L10_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":177
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")
 */
      __pyx_t_20 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
      if (__pyx_t_20) {
        __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_chat_history", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_12, &__pyx_t_8, &__pyx_t_1) < 0) __PYX_ERR(0, 177, __pyx_L7_except_error)
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_v_e, __pyx_t_8);
        /*try:*/ {

          /* "sQQideR/DatabaseContext.pyx":178
 *                     # Fallback handled below
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"Error enhancing query with AI: {e}")
 *                 # Fallback handled below
 */
          __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 178, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_17); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 178, __pyx_L46_error)
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":179
 *             except Exception as e:
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")             # <<<<<<<<<<<<<<
 *                 # Fallback handled below
 * 
 */
            __pyx_t_17 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_empty_unicode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 179, __pyx_L46_error)
            __Pyx_GOTREF(__pyx_t_17);
            __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Error_enhancing_query_with_AI, __pyx_t_17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 179, __pyx_L46_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __pyx_t_17 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_10); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 179, __pyx_L46_error)
            __Pyx_GOTREF(__pyx_t_17);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

            /* "sQQideR/DatabaseContext.pyx":178
 *                     # Fallback handled below
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"Error enhancing query with AI: {e}")
 *                 # Fallback handled below
 */
          }
        }

        /* "sQQideR/DatabaseContext.pyx":177
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")
 */
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            goto __pyx_L47;
          }
          __pyx_L46_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_16 = 0; __pyx_t_15 = 0; __pyx_t_14 = 0; __pyx_t_27 = 0; __pyx_t_26 = 0; __pyx_t_25 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_27, &__pyx_t_26, &__pyx_t_25);
            if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_15, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_15, &__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_27);
            __Pyx_XGOTREF(__pyx_t_26);
            __Pyx_XGOTREF(__pyx_t_25);
            __pyx_t_20 = __pyx_lineno; __pyx_t_19 = __pyx_clineno; __pyx_t_28 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            }
            if (PY_MAJOR_VERSION >= 3) {
              __Pyx_XGIVEREF(__pyx_t_27);
              __Pyx_XGIVEREF(__pyx_t_26);
              __Pyx_XGIVEREF(__pyx_t_25);
              __Pyx_ExceptionReset(__pyx_t_27, __pyx_t_26, __pyx_t_25);
            }
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_ErrRestore(__pyx_t_16, __pyx_t_15, __pyx_t_14);
            __pyx_t_16 = 0; __pyx_t_15 = 0; __pyx_t_14 = 0; __pyx_t_27 = 0; __pyx_t_26 = 0; __pyx_t_25 = 0;
            __pyx_lineno = __pyx_t_20; __pyx_clineno = __pyx_t_19; __pyx_filename = __pyx_t_28;
            goto __pyx_L7_except_error;
          }
          __pyx_L47:;
        }
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L6_exception_handled;
      }
      goto __pyx_L7_except_error;

      /* "sQQideR/DatabaseContext.pyx":104
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Build prompt that includes chat history if available
 *                 context_prompt = f"""
 */
      __pyx_L7_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L6_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      __pyx_L10_try_end:;
    }

    /* "sQQideR/DatabaseContext.pyx":103
 *         # Get AI-enhanced query information
 *         enhanced_query = None
 *         if self.ai_base:             # <<<<<<<<<<<<<<
 *             try:
 *                 # Build prompt that includes chat history if available
 */
  }

  /* "sQQideR/DatabaseContext.pyx":183
 * 
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:             # <<<<<<<<<<<<<<
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 */
  __pyx_t_2 = (__pyx_v_enhanced_query == Py_None);
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":184
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]             # <<<<<<<<<<<<<<
 *             if self.verbose:
 *                 print(f"Using basic keyword extraction: {keywords}")
 */
    { /* enter inner scope */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L56_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = PyUnicode_Split(__pyx_v_query_text, ((PyObject *)NULL), -1L); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 184, __pyx_L56_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_12 = __pyx_t_8; __Pyx_INCREF(__pyx_t_12);
      __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_12);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 184, __pyx_L56_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 184, __pyx_L56_error)
        #else
        __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_12, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 184, __pyx_L56_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
        __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_word, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_11 = PyObject_Length(__pyx_8genexpr1__pyx_v_word); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 184, __pyx_L56_error)
        __pyx_t_2 = (__pyx_t_11 > 3);
        if (__pyx_t_2) {
          __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr1__pyx_v_word, __pyx_n_s_lower); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 184, __pyx_L56_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_10 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_17))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_17);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_17, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_10, NULL};
            __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 0+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 184, __pyx_L56_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          }
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_8))) __PYX_ERR(0, 184, __pyx_L56_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_word); __pyx_8genexpr1__pyx_v_word = 0;
      goto __pyx_L61_exit_scope;
      __pyx_L56_error:;
      __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_word); __pyx_8genexpr1__pyx_v_word = 0;
      goto __pyx_L1_error;
      __pyx_L61_exit_scope:;
    } /* exit inner scope */
    __pyx_v_keywords = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":185
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":186
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 *                 print(f"Using basic keyword extraction: {keywords}")             # <<<<<<<<<<<<<<
 * 
 *             enhanced_query = SQLPrompt(
 */
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_keywords, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_12 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Using_basic_keyword_extraction, __pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":185
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":188
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 *             enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SQLPrompt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "sQQideR/DatabaseContext.pyx":189
 * 
 *             enhanced_query = SQLPrompt(
 *                 user_prompt=query_text,             # <<<<<<<<<<<<<<
 *                 ai_context=query_text,
 *                 buisness_domains=[],
 */
    __pyx_t_12 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (PyDict_SetItem(__pyx_t_12, __pyx_n_s_user_prompt, __pyx_v_query_text) < 0) __PYX_ERR(0, 189, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":190
 *             enhanced_query = SQLPrompt(
 *                 user_prompt=query_text,
 *                 ai_context=query_text,             # <<<<<<<<<<<<<<
 *                 buisness_domains=[],
 *                 keywords=keywords
 */
    if (PyDict_SetItem(__pyx_t_12, __pyx_n_s_ai_context, __pyx_v_query_text) < 0) __PYX_ERR(0, 189, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":191
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 *                 buisness_domains=[],             # <<<<<<<<<<<<<<
 *                 keywords=keywords
 *             )
 */
    __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (PyDict_SetItem(__pyx_t_12, __pyx_n_s_buisness_domains, __pyx_t_8) < 0) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":192
 *                 ai_context=query_text,
 *                 buisness_domains=[],
 *                 keywords=keywords             # <<<<<<<<<<<<<<
 *             )
 * 
 */
    if (PyDict_SetItem(__pyx_t_12, __pyx_n_s_keywords, __pyx_v_keywords) < 0) __PYX_ERR(0, 189, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":188
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 *             enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF_SET(__pyx_v_enhanced_query, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":183
 * 
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:             # <<<<<<<<<<<<<<
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":197
 *         # Continue with the regular processing using the enhanced query
 *         # Use the enhanced query for keyword search
 *         results, matches = self.search_by_keywords(enhanced_query.keywords)             # <<<<<<<<<<<<<<
 * 
 *         # Store the original query information
 */
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_search_by_keywords); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_keywords); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_17 = NULL;
  __pyx_t_13 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_17)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_17);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
      __pyx_t_13 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_t_1};
    __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
    __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
    PyObject* sequence = __pyx_t_8;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 197, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_12);
    __Pyx_INCREF(__pyx_t_1);
    #else
    __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_17 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_17);
    index = 0; __pyx_t_12 = __pyx_t_29(__pyx_t_17); if (unlikely(!__pyx_t_12)) goto __pyx_L63_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_12);
    index = 1; __pyx_t_1 = __pyx_t_29(__pyx_t_17); if (unlikely(!__pyx_t_1)) goto __pyx_L63_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_17), 2) < 0) __PYX_ERR(0, 197, __pyx_L1_error)
    __pyx_t_29 = NULL;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    goto __pyx_L64_unpacking_done;
    __pyx_L63_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __pyx_t_29 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 197, __pyx_L1_error)
    __pyx_L64_unpacking_done:;
  }
  __pyx_v_results = __pyx_t_12;
  __pyx_t_12 = 0;
  __pyx_v_matches = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":200
 * 
 *         # Store the original query information
 *         ai_context = enhanced_query.ai_context if enhanced_query.ai_context else query_text             # <<<<<<<<<<<<<<
 * 
 *         # Group results by type
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __pyx_t_1;
    __pyx_t_1 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_query_text);
    __pyx_t_8 = __pyx_v_query_text;
  }
  __pyx_v_ai_context = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":203
 * 
 *         # Group results by type
 *         tables = []             # <<<<<<<<<<<<<<
 *         columns = []
 *         concepts = []
 */
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_tables = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":204
 *         # Group results by type
 *         tables = []
 *         columns = []             # <<<<<<<<<<<<<<
 *         concepts = []
 *         for node_id, data in results:
 */
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_columns = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":205
 *         tables = []
 *         columns = []
 *         concepts = []             # <<<<<<<<<<<<<<
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):
 */
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_concepts = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":206
 *         columns = []
 *         concepts = []
 *         for node_id, data in results:             # <<<<<<<<<<<<<<
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 */
  if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
    __pyx_t_8 = __pyx_v_results; __Pyx_INCREF(__pyx_t_8);
    __pyx_t_6 = 0;
    __pyx_t_30 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_30 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 206, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_30)) {
      if (likely(PyList_CheckExact(__pyx_t_8))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 206, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 206, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 206, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 206, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_30(__pyx_t_8);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 206, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 206, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_17);
      #else
      __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_10 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_10);
      index = 0; __pyx_t_12 = __pyx_t_29(__pyx_t_10); if (unlikely(!__pyx_t_12)) goto __pyx_L67_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      index = 1; __pyx_t_17 = __pyx_t_29(__pyx_t_10); if (unlikely(!__pyx_t_17)) goto __pyx_L67_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_17);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_10), 2) < 0) __PYX_ERR(0, 206, __pyx_L1_error)
      __pyx_t_29 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L68_unpacking_done;
      __pyx_L67_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_29 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 206, __pyx_L1_error)
      __pyx_L68_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_12);
    __pyx_t_12 = 0;
    __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_17);
    __pyx_t_17 = 0;

    /* "sQQideR/DatabaseContext.pyx":207
 *         concepts = []
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 */
    __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_12 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_17))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_17);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_17, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u_table};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":208
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)             # <<<<<<<<<<<<<<
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 */
      __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 208, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":207
 *         concepts = []
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 */
      goto __pyx_L69;
    }

    /* "sQQideR/DatabaseContext.pyx":209
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 */
    __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_12 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_17))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_17);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_17, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u_column};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 209, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":210
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)             # <<<<<<<<<<<<<<
 *             elif node_id.startswith("concept:"):
 *                 concepts.append(node_id)
 */
      __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_columns, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 210, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":209
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 */
      goto __pyx_L69;
    }

    /* "sQQideR/DatabaseContext.pyx":211
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):             # <<<<<<<<<<<<<<
 *                 concepts.append(node_id)
 * 
 */
    __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_12 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_17))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_17);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_17, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u_concept};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":212
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 *                 concepts.append(node_id)             # <<<<<<<<<<<<<<
 * 
 *         # Look for domain-specific matches based on business domains
 */
      __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_concepts, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 212, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":211
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):             # <<<<<<<<<<<<<<
 *                 concepts.append(node_id)
 * 
 */
    }
    __pyx_L69:;

    /* "sQQideR/DatabaseContext.pyx":206
 *         columns = []
 *         concepts = []
 *         for node_id, data in results:             # <<<<<<<<<<<<<<
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":215
 * 
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":216
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (likely(PyList_CheckExact(__pyx_t_8)) || PyTuple_CheckExact(__pyx_t_8)) {
      __pyx_t_1 = __pyx_t_8; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = 0;
      __pyx_t_30 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_30 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 216, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    for (;;) {
      if (likely(!__pyx_t_30)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 216, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 216, __pyx_L1_error)
          #else
          __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 216, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 216, __pyx_L1_error)
          #else
          __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          #endif
        }
      } else {
        __pyx_t_8 = __pyx_t_30(__pyx_t_1);
        if (unlikely(!__pyx_t_8)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 216, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_XDECREF_SET(__pyx_v_domain, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":217
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])             # <<<<<<<<<<<<<<
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:
 */
      __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_search_by_keywords); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 217, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      __pyx_t_12 = PyList_New(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 217, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_INCREF(__pyx_v_domain);
      __Pyx_GIVEREF(__pyx_v_domain);
      if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 0, __pyx_v_domain)) __PYX_ERR(0, 217, __pyx_L1_error);
      __pyx_t_10 = NULL;
      __pyx_t_13 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_17))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_17);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_17, function);
          __pyx_t_13 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_12};
        __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 217, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
        PyObject* sequence = __pyx_t_8;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 217, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_17 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_12);
        #else
        __pyx_t_17 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 217, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_12 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 217, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        #endif
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_10 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 217, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_10);
        index = 0; __pyx_t_17 = __pyx_t_29(__pyx_t_10); if (unlikely(!__pyx_t_17)) goto __pyx_L74_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        index = 1; __pyx_t_12 = __pyx_t_29(__pyx_t_10); if (unlikely(!__pyx_t_12)) goto __pyx_L74_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_10), 2) < 0) __PYX_ERR(0, 217, __pyx_L1_error)
        __pyx_t_29 = NULL;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        goto __pyx_L75_unpacking_done;
        __pyx_L74_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_29 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 217, __pyx_L1_error)
        __pyx_L75_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_domain_results, __pyx_t_17);
      __pyx_t_17 = 0;
      __Pyx_XDECREF_SET(__pyx_v__, __pyx_t_12);
      __pyx_t_12 = 0;

      /* "sQQideR/DatabaseContext.pyx":218
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 */
      if (likely(PyList_CheckExact(__pyx_v_domain_results)) || PyTuple_CheckExact(__pyx_v_domain_results)) {
        __pyx_t_8 = __pyx_v_domain_results; __Pyx_INCREF(__pyx_t_8);
        __pyx_t_11 = 0;
        __pyx_t_32 = NULL;
      } else {
        __pyx_t_11 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_v_domain_results); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 218, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_32 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 218, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_32)) {
          if (likely(PyList_CheckExact(__pyx_t_8))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 218, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_12 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_11); __Pyx_INCREF(__pyx_t_12); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 218, __pyx_L1_error)
            #else
            __pyx_t_12 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 218, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 218, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_12 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_11); __Pyx_INCREF(__pyx_t_12); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 218, __pyx_L1_error)
            #else
            __pyx_t_12 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 218, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            #endif
          }
        } else {
          __pyx_t_12 = __pyx_t_32(__pyx_t_8);
          if (unlikely(!__pyx_t_12)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 218, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_12);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_12))) || (PyList_CheckExact(__pyx_t_12))) {
          PyObject* sequence = __pyx_t_12;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 218, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_17 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_17 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_10);
          #else
          __pyx_t_17 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 218, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 218, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_18 = PyObject_GetIter(__pyx_t_12); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 218, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_18);
          index = 0; __pyx_t_17 = __pyx_t_29(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L78_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_17);
          index = 1; __pyx_t_10 = __pyx_t_29(__pyx_t_18); if (unlikely(!__pyx_t_10)) goto __pyx_L78_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_10);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_18), 2) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
          __pyx_t_29 = NULL;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          goto __pyx_L79_unpacking_done;
          __pyx_L78_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          __pyx_t_29 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 218, __pyx_L1_error)
          __pyx_L79_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_17);
        __pyx_t_17 = 0;
        __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "sQQideR/DatabaseContext.pyx":219
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:             # <<<<<<<<<<<<<<
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_17 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_17)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_17);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_kp_u_table};
          __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 219, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (__pyx_t_9) {
        } else {
          __pyx_t_2 = __pyx_t_9;
          goto __pyx_L81_bool_binop_done;
        }
        __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_tables, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 219, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_9;
        __pyx_L81_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":220
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)             # <<<<<<<<<<<<<<
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 */
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 220, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":219
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:             # <<<<<<<<<<<<<<
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 */
          goto __pyx_L80;
        }

        /* "sQQideR/DatabaseContext.pyx":221
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:             # <<<<<<<<<<<<<<
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 221, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_17 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_17)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_17);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_kp_u_column};
          __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 221, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 221, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (__pyx_t_9) {
        } else {
          __pyx_t_2 = __pyx_t_9;
          goto __pyx_L83_bool_binop_done;
        }
        __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_columns, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 221, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_9;
        __pyx_L83_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":222
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)             # <<<<<<<<<<<<<<
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 *                         concepts.append(node_id)
 */
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_columns, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 222, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":221
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:             # <<<<<<<<<<<<<<
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 */
          goto __pyx_L80;
        }

        /* "sQQideR/DatabaseContext.pyx":223
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:             # <<<<<<<<<<<<<<
 *                         concepts.append(node_id)
 * 
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 223, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_17 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_17)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_17);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_kp_u_concept};
          __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 223, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 223, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (__pyx_t_9) {
        } else {
          __pyx_t_2 = __pyx_t_9;
          goto __pyx_L85_bool_binop_done;
        }
        __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_concepts, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 223, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_9;
        __pyx_L85_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":224
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 *                         concepts.append(node_id)             # <<<<<<<<<<<<<<
 * 
 *         # If we found concepts but no tables, find related tables
 */
          __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_concepts, __pyx_v_node_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 224, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":223
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:             # <<<<<<<<<<<<<<
 *                         concepts.append(node_id)
 * 
 */
        }
        __pyx_L80:;

        /* "sQQideR/DatabaseContext.pyx":218
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 */
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":216
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":215
 * 
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 */
  }

  /* "sQQideR/DatabaseContext.pyx":227
 * 
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:             # <<<<<<<<<<<<<<
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 */
  __pyx_t_9 = (PyList_GET_SIZE(__pyx_v_concepts) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_2 = __pyx_t_9;
    goto __pyx_L90_bool_binop_done;
  }
  __pyx_t_9 = (PyList_GET_SIZE(__pyx_v_tables) != 0);
  __pyx_t_33 = (!__pyx_t_9);
  __pyx_t_2 = __pyx_t_33;
  __pyx_L90_bool_binop_done:;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":228
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:
 *             for concept in concepts:             # <<<<<<<<<<<<<<
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 */
    __pyx_t_1 = __pyx_v_concepts; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 228, __pyx_L1_error)
        #endif
        if (__pyx_t_6 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 228, __pyx_L1_error)
      #else
      __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 228, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_concept, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":229
 *         if concepts and not tables:
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_nodes); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_data, Py_True) < 0) __PYX_ERR(0, 229, __pyx_L1_error)
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
        __pyx_t_8 = __pyx_t_10; __Pyx_INCREF(__pyx_t_8);
        __pyx_t_11 = 0;
        __pyx_t_30 = NULL;
      } else {
        __pyx_t_11 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 229, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_30 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 229, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      for (;;) {
        if (likely(!__pyx_t_30)) {
          if (likely(PyList_CheckExact(__pyx_t_8))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 229, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_10 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_11); __Pyx_INCREF(__pyx_t_10); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 229, __pyx_L1_error)
            #else
            __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 229, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 229, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_11); __Pyx_INCREF(__pyx_t_10); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 229, __pyx_L1_error)
            #else
            __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 229, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            #endif
          }
        } else {
          __pyx_t_10 = __pyx_t_30(__pyx_t_8);
          if (unlikely(!__pyx_t_10)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 229, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_10);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_10))) || (PyList_CheckExact(__pyx_t_10))) {
          PyObject* sequence = __pyx_t_10;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 229, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_17);
          #else
          __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          #endif
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_18 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_18);
          index = 0; __pyx_t_12 = __pyx_t_29(__pyx_t_18); if (unlikely(!__pyx_t_12)) goto __pyx_L96_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_12);
          index = 1; __pyx_t_17 = __pyx_t_29(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L96_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_17);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_18), 2) < 0) __PYX_ERR(0, 229, __pyx_L1_error)
          __pyx_t_29 = NULL;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          goto __pyx_L97_unpacking_done;
          __pyx_L96_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          __pyx_t_29 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 229, __pyx_L1_error)
          __pyx_L97_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_12);
        __pyx_t_12 = 0;
        __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_17);
        __pyx_t_17 = 0;

        /* "sQQideR/DatabaseContext.pyx":230
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 */
        __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 230, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_12 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_17))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_17);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_17, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u_column};
          __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 230, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 230, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":231
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 */
          __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_n_s_neighbors); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          __pyx_t_17 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_12);
            if (likely(__pyx_t_17)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_17);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_12, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_v_node_id};
            __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 231, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          }
          if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
            __pyx_t_12 = __pyx_t_10; __Pyx_INCREF(__pyx_t_12);
            __pyx_t_34 = 0;
            __pyx_t_32 = NULL;
          } else {
            __pyx_t_34 = -1; __pyx_t_12 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 231, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_32 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_12); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 231, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          for (;;) {
            if (likely(!__pyx_t_32)) {
              if (likely(PyList_CheckExact(__pyx_t_12))) {
                {
                  Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_12);
                  #if !CYTHON_ASSUME_SAFE_MACROS
                  if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
                  #endif
                  if (__pyx_t_34 >= __pyx_temp) break;
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_10 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_34); __Pyx_INCREF(__pyx_t_10); __pyx_t_34++; if (unlikely((0 < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
                #else
                __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_12, __pyx_t_34); __pyx_t_34++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 231, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                #endif
              } else {
                {
                  Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_12);
                  #if !CYTHON_ASSUME_SAFE_MACROS
                  if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
                  #endif
                  if (__pyx_t_34 >= __pyx_temp) break;
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_12, __pyx_t_34); __Pyx_INCREF(__pyx_t_10); __pyx_t_34++; if (unlikely((0 < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
                #else
                __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_12, __pyx_t_34); __pyx_t_34++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 231, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                #endif
              }
            } else {
              __pyx_t_10 = __pyx_t_32(__pyx_t_12);
              if (unlikely(!__pyx_t_10)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 231, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_10);
            }
            __Pyx_XDECREF_SET(__pyx_v_neighbor, __pyx_t_10);
            __pyx_t_10 = 0;

            /* "sQQideR/DatabaseContext.pyx":232
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:             # <<<<<<<<<<<<<<
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 */
            __pyx_t_10 = PyObject_RichCompare(__pyx_v_neighbor, __pyx_v_concept, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 232, __pyx_L1_error)
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 232, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (__pyx_t_2) {

              /* "sQQideR/DatabaseContext.pyx":233
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")             # <<<<<<<<<<<<<<
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:
 */
              __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_split); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 233, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              __pyx_t_18 = NULL;
              __pyx_t_13 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_17))) {
                __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_17);
                if (likely(__pyx_t_18)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
                  __Pyx_INCREF(__pyx_t_18);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_17, function);
                  __pyx_t_13 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_18, __pyx_kp_u__15};
                __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_17, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
                __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 233, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_10);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              }
              __pyx_t_17 = __Pyx_GetItemInt(__pyx_t_10, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 233, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_n_s_replace); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 233, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 233, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_17);
              __pyx_t_17 = 0;

              /* "sQQideR/DatabaseContext.pyx":234
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"             # <<<<<<<<<<<<<<
 *                                 if table_id not in tables:
 *                                     tables.append(table_id)
 */
              __pyx_t_17 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 234, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_17);
              __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_17); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 234, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              __Pyx_XDECREF_SET(__pyx_v_table_id, ((PyObject*)__pyx_t_10));
              __pyx_t_10 = 0;

              /* "sQQideR/DatabaseContext.pyx":235
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:             # <<<<<<<<<<<<<<
 *                                     tables.append(table_id)
 * 
 */
              __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_table_id, __pyx_v_tables, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 235, __pyx_L1_error)
              if (__pyx_t_2) {

                /* "sQQideR/DatabaseContext.pyx":236
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:
 *                                     tables.append(table_id)             # <<<<<<<<<<<<<<
 * 
 *         # Store all tables before filtering for relationship detection
 */
                __pyx_t_31 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_table_id); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 236, __pyx_L1_error)

                /* "sQQideR/DatabaseContext.pyx":235
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:             # <<<<<<<<<<<<<<
 *                                     tables.append(table_id)
 * 
 */
              }

              /* "sQQideR/DatabaseContext.pyx":232
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:             # <<<<<<<<<<<<<<
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 */
            }

            /* "sQQideR/DatabaseContext.pyx":231
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 */
          }
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

          /* "sQQideR/DatabaseContext.pyx":230
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 */
        }

        /* "sQQideR/DatabaseContext.pyx":229
 *         if concepts and not tables:
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 */
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":228
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:
 *             for concept in concepts:             # <<<<<<<<<<<<<<
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":227
 * 
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:             # <<<<<<<<<<<<<<
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 */
  }

  /* "sQQideR/DatabaseContext.pyx":239
 * 
 *         # Store all tables before filtering for relationship detection
 *         all_tables = tables.copy()             # <<<<<<<<<<<<<<
 * 
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 */
  __pyx_t_1 = __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyList_Type_copy, __pyx_v_tables); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_all_tables = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":242
 * 
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 *         tables = tables[:2]             # <<<<<<<<<<<<<<
 *         columns = columns[:2]
 *         concepts = concepts[:2]
 */
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_tables, 0, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_tables, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":243
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 *         tables = tables[:2]
 *         columns = columns[:2]             # <<<<<<<<<<<<<<
 *         concepts = concepts[:2]
 * 
 */
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_columns, 0, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_columns, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":244
 *         tables = tables[:2]
 *         columns = columns[:2]
 *         concepts = concepts[:2]             # <<<<<<<<<<<<<<
 * 
 *         # Get related business rules only from tables, columns, and concepts
 */
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_concepts, 0, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_concepts, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":247
 * 
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []             # <<<<<<<<<<<<<<
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_all_rules = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":248
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []
 *         for node_id in tables + columns + concepts:             # <<<<<<<<<<<<<<
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)
 */
  __pyx_t_1 = PyNumber_Add(__pyx_v_tables, __pyx_v_columns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = PyNumber_Add(__pyx_t_1, __pyx_v_concepts); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_8; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 248, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 248, __pyx_L1_error)
    #else
    __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 248, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":249
 *         all_rules = []
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)             # <<<<<<<<<<<<<<
 *             all_rules.extend(node_rules)
 * 
 */
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_get_related_rules); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_10 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_12, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_node_id};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_12, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_rules, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":250
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)             # <<<<<<<<<<<<<<
 * 
 *         # Deduplicate and limit to 2 business rules
 */
    __pyx_t_31 = __Pyx_PyList_Extend(__pyx_v_all_rules, __pyx_v_node_rules); if (unlikely(__pyx_t_31 == ((int)-1))) __PYX_ERR(0, 250, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":248
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []
 *         for node_id in tables + columns + concepts:             # <<<<<<<<<<<<<<
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":253
 * 
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}             # <<<<<<<<<<<<<<
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_unique_rules = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":254
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:             # <<<<<<<<<<<<<<
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]
 */
  __pyx_t_1 = __pyx_v_all_rules; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 254, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 254, __pyx_L1_error)
    #else
    __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    #endif
    if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
      PyObject* sequence = __pyx_t_8;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 254, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_10);
      #else
      __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      #endif
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_17 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_17);
      index = 0; __pyx_t_12 = __pyx_t_29(__pyx_t_17); if (unlikely(!__pyx_t_12)) goto __pyx_L111_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      index = 1; __pyx_t_10 = __pyx_t_29(__pyx_t_17); if (unlikely(!__pyx_t_10)) goto __pyx_L111_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_29(__pyx_t_17), 2) < 0) __PYX_ERR(0, 254, __pyx_L1_error)
      __pyx_t_29 = NULL;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      goto __pyx_L112_unpacking_done;
      __pyx_L111_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_29 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 254, __pyx_L1_error)
      __pyx_L112_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_rule_id, __pyx_t_12);
    __pyx_t_12 = 0;
    __Pyx_XDECREF_SET(__pyx_v_rule_desc, __pyx_t_10);
    __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":255
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc             # <<<<<<<<<<<<<<
 *         filtered_rules = list(unique_rules.keys())[:2]
 * 
 */
    if (unlikely((PyDict_SetItem(__pyx_v_unique_rules, __pyx_v_rule_id, __pyx_v_rule_desc) < 0))) __PYX_ERR(0, 255, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":254
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:             # <<<<<<<<<<<<<<
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":256
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]             # <<<<<<<<<<<<<<
 * 
 *         # Generate SQL if we have tables
 */
  __pyx_t_1 = __Pyx_PyDict_Keys(__pyx_v_unique_rules); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PySequence_ListKeepNew(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_t_8, 0, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_filtered_rules = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":259
 * 
 *         # Generate SQL if we have tables
 *         sql = None             # <<<<<<<<<<<<<<
 *         if tables:
 *             sql = self.generate_sql_from_nl(ai_context)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_sql = Py_None;

  /* "sQQideR/DatabaseContext.pyx":260
 *         # Generate SQL if we have tables
 *         sql = None
 *         if tables:             # <<<<<<<<<<<<<<
 *             sql = self.generate_sql_from_nl(ai_context)
 * 
 */
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_tables) != 0);
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":261
 *         sql = None
 *         if tables:
 *             sql = self.generate_sql_from_nl(ai_context)             # <<<<<<<<<<<<<<
 * 
 *         # Generate output with the filtered lists and all_tables for relationship detection
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_sql_from_nl); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_ai_context};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_sql, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":260
 *         # Generate SQL if we have tables
 *         sql = None
 *         if tables:             # <<<<<<<<<<<<<<
 *             sql = self.generate_sql_from_nl(ai_context)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":264
 * 
 *         # Generate output with the filtered lists and all_tables for relationship detection
 *         output = self.generate_output(             # <<<<<<<<<<<<<<
 *             query_text, results, matches, tables, columns, concepts,
 *             filtered_rules, sql, all_tables
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_output); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);

  /* "sQQideR/DatabaseContext.pyx":266
 *         output = self.generate_output(
 *             query_text, results, matches, tables, columns, concepts,
 *             filtered_rules, sql, all_tables             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_10 = NULL;
  __pyx_t_13 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[10] = {__pyx_t_10, __pyx_v_query_text, __pyx_v_results, __pyx_v_matches, __pyx_v_tables, __pyx_v_columns, __pyx_v_concepts, __pyx_v_filtered_rules, __pyx_v_sql, __pyx_v_all_tables};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 9+__pyx_t_13);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __pyx_v_output = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":271
 *         # Add the AI enhanced query information to the output
 *         output["ai_enhanced"] = {
 *             "keywords": enhanced_query.keywords,             # <<<<<<<<<<<<<<
 *             "business_domains": enhanced_query.buisness_domains,
 *             "enhanced_query": enhanced_query.ai_context
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_keywords); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_keywords, __pyx_t_8) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":272
 *         output["ai_enhanced"] = {
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,             # <<<<<<<<<<<<<<
 *             "enhanced_query": enhanced_query.ai_context
 *         }
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_business_domains, __pyx_t_8) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":273
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,
 *             "enhanced_query": enhanced_query.ai_context             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_enhanced_query, __pyx_t_8) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":270
 * 
 *         # Add the AI enhanced query information to the output
 *         output["ai_enhanced"] = {             # <<<<<<<<<<<<<<
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,
 */
  if (unlikely((PyObject_SetItem(__pyx_v_output, __pyx_n_u_ai_enhanced, __pyx_t_1) < 0))) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":277
 * 
 *         # Add chat history context if it was generated
 *         if hasattr(enhanced_query, 'history_context') and enhanced_query.history_context:             # <<<<<<<<<<<<<<
 *             output["chat_history_context"] = enhanced_query.history_context
 * 
 */
  __pyx_t_33 = __Pyx_HasAttr(__pyx_v_enhanced_query, __pyx_n_u_history_context); if (unlikely(__pyx_t_33 == ((int)-1))) __PYX_ERR(0, 277, __pyx_L1_error)
  if (__pyx_t_33) {
  } else {
    __pyx_t_2 = __pyx_t_33;
    goto __pyx_L116_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_history_context); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_33 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_33 < 0))) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_33;
  __pyx_L116_bool_binop_done:;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":278
 *         # Add chat history context if it was generated
 *         if hasattr(enhanced_query, 'history_context') and enhanced_query.history_context:
 *             output["chat_history_context"] = enhanced_query.history_context             # <<<<<<<<<<<<<<
 * 
 *         if output_format == "markdown":
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_history_context); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely((PyObject_SetItem(__pyx_v_output, __pyx_n_u_chat_history_context, __pyx_t_1) < 0))) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":277
 * 
 *         # Add chat history context if it was generated
 *         if hasattr(enhanced_query, 'history_context') and enhanced_query.history_context:             # <<<<<<<<<<<<<<
 *             output["chat_history_context"] = enhanced_query.history_context
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":280
 *             output["chat_history_context"] = enhanced_query.history_context
 * 
 *         if output_format == "markdown":             # <<<<<<<<<<<<<<
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_output_format, __pyx_n_u_markdown, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 280, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":281
 * 
 *         if output_format == "markdown":
 *             output["markdown"] = self.format_as_markdown(output)             # <<<<<<<<<<<<<<
 * 
 *         return output
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_format_as_markdown); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    __pyx_t_13 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_13 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_output};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 1+__pyx_t_13);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    if (unlikely((PyObject_SetItem(__pyx_v_output, __pyx_n_u_markdown, __pyx_t_1) < 0))) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":280
 *             output["chat_history_context"] = enhanced_query.history_context
 * 
 *         if output_format == "markdown":             # <<<<<<<<<<<<<<
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":283
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 *         return output             # <<<<<<<<<<<<<<
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",
 */
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(PyDict_CheckExact(__pyx_v_output))||((__pyx_v_output) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_v_output))) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_INCREF(__pyx_v_output);
  __pyx_r = ((PyObject*)__pyx_v_output);
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":84
 *     # ...existing code...
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,             # <<<<<<<<<<<<<<
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:
 *         """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_chat_history", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_enhanced_query);
  __Pyx_XDECREF(__pyx_v_context_prompt);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_role);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_clean_response);
  __Pyx_XDECREF(__pyx_v_first_newline);
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_keywords);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_matches);
  __Pyx_XDECREF(__pyx_v_ai_context);
  __Pyx_XDECREF(__pyx_v_tables);
  __Pyx_XDECREF(__pyx_v_columns);
  __Pyx_XDECREF(__pyx_v_concepts);
  __Pyx_XDECREF(__pyx_v_node_id);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_domain);
  __Pyx_XDECREF(__pyx_v_domain_results);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_concept);
  __Pyx_XDECREF(__pyx_v_neighbor);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_table_id);
  __Pyx_XDECREF(__pyx_v_all_tables);
  __Pyx_XDECREF(__pyx_v_all_rules);
  __Pyx_XDECREF(__pyx_v_node_rules);
  __Pyx_XDECREF(__pyx_v_unique_rules);
  __Pyx_XDECREF(__pyx_v_rule_id);
  __Pyx_XDECREF(__pyx_v_rule_desc);
  __Pyx_XDECREF(__pyx_v_filtered_rules);
  __Pyx_XDECREF(__pyx_v_sql);
  __Pyx_XDECREF(__pyx_v_output);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_word);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":285
 *         return output
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",             # <<<<<<<<<<<<<<
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:
 *         """
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_7process_query(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_6process_query, "\n        Process a natural language query and return context information.\n        \n        Now supports optional chat history and previous SQL.\n        \n        Args:\n            query_text: Natural language query string\n            output_format: Output format, 'json' or 'markdown'\n            chat_history: Optional list of previous chat messages\n            previous_sql: Optional previous SQL query\n            \n        Returns:\n            Dictionary with context information\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_7process_query = {"process_query", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_7process_query, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_6process_query};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_7process_query(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_query_text = 0;
  PyObject *__pyx_v_output_format = 0;
  PyObject *__pyx_v_chat_history = 0;
  PyObject *__pyx_v_previous_sql = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("process_query (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_query_text,&__pyx_n_s_output_format,&__pyx_n_s_chat_history,&__pyx_n_s_previous_sql,0};
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)((PyObject*)__pyx_n_u_json)));

    /* "sQQideR/DatabaseContext.pyx":286
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query and return context information.
 */
    values[3] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    values[4] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_query_text)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("process_query", 0, 2, 5, 1); __PYX_ERR(0, 285, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_output_format);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_chat_history);
          if (value) { values[3] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_previous_sql);
          if (value) { values[4] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 285, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "process_query") < 0)) __PYX_ERR(0, 285, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_query_text = ((PyObject*)values[1]);
    __pyx_v_output_format = ((PyObject*)values[2]);
    __pyx_v_chat_history = ((PyObject*)values[3]);
    __pyx_v_previous_sql = ((PyObject*)values[4]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("process_query", 0, 2, 5, __pyx_nargs); __PYX_ERR(0, 285, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query_text), (&PyUnicode_Type), 0, "query_text", 1))) __PYX_ERR(0, 285, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_output_format), (&PyUnicode_Type), 0, "output_format", 1))) __PYX_ERR(0, 285, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_chat_history), (&PyList_Type), 1, "chat_history", 1))) __PYX_ERR(0, 286, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_previous_sql), (&PyUnicode_Type), 1, "previous_sql", 1))) __PYX_ERR(0, 286, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_6process_query(__pyx_self, __pyx_v_self, __pyx_v_query_text, __pyx_v_output_format, __pyx_v_chat_history, __pyx_v_previous_sql);

  /* "sQQideR/DatabaseContext.pyx":285
 *         return output
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",             # <<<<<<<<<<<<<<
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:
 *         """
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_6process_query(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_output_format, PyObject *__pyx_v_chat_history, PyObject *__pyx_v_previous_sql) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_query", 1);

  /* "sQQideR/DatabaseContext.pyx":301
 *             Dictionary with context information
 *         """
 *         if chat_history or previous_sql:             # <<<<<<<<<<<<<<
 *             return self.process_query_with_chat_history(query_text, chat_history, previous_sql, output_format)
 *         else:
 */
  __pyx_t_2 = (__pyx_v_chat_history != Py_None)&&(PyList_GET_SIZE(__pyx_v_chat_history) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_previous_sql != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_previous_sql) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sQQideR/DatabaseContext.pyx":302
 *         """
 *         if chat_history or previous_sql:
 *             return self.process_query_with_chat_history(query_text, chat_history, previous_sql, output_format)             # <<<<<<<<<<<<<<
 *         else:
 *             return self.process_query_with_ai(query_text, output_format)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_process_query_with_chat_history); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_5, __pyx_v_query_text, __pyx_v_chat_history, __pyx_v_previous_sql, __pyx_v_output_format};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 4+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    if (!(likely(PyDict_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_3))) __PYX_ERR(0, 302, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "sQQideR/DatabaseContext.pyx":301
 *             Dictionary with context information
 *         """
 *         if chat_history or previous_sql:             # <<<<<<<<<<<<<<
 *             return self.process_query_with_chat_history(query_text, chat_history, previous_sql, output_format)
 *         else:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":304
 *             return self.process_query_with_chat_history(query_text, chat_history, previous_sql, output_format)
 *         else:
 *             return self.process_query_with_ai(query_text, output_format)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_process_query_with_ai); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_v_query_text, __pyx_v_output_format};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    if (!(likely(PyDict_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_3))) __PYX_ERR(0, 304, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "sQQideR/DatabaseContext.pyx":285
 *         return output
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",             # <<<<<<<<<<<<<<
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:
 *         """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":307
 * 
 * 
 *     def build_graph(self, metadata: Dict = None) -> nx.DiGraph:             # <<<<<<<<<<<<<<
 *         """
 *         Build a graph from the metadata.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_9build_graph(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_8build_graph, "\n        Build a graph from the metadata.\n        \n        Args:\n            metadata: Optional metadata dictionary (uses self.metadata if not provided)\n            \n        Returns:\n            The built NetworkX directed graph\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_9build_graph = {"build_graph", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_9build_graph, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_8build_graph};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_9build_graph(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("build_graph (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_metadata,0};
    values[1] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_metadata);
          if (value) { values[1] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "build_graph") < 0)) __PYX_ERR(0, 307, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_metadata = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("build_graph", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 307, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.build_graph", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_metadata), (&PyDict_Type), 1, "metadata", 1))) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_8build_graph(__pyx_self, __pyx_v_self, __pyx_v_metadata);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_8build_graph(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_v_G = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_table_data = NULL;
  PyObject *__pyx_v_node_id = NULL;
  PyObject *__pyx_v_col_name = NULL;
  PyObject *__pyx_v_col_data = NULL;
  PyObject *__pyx_v_col_node_id = NULL;
  PyObject *__pyx_v_node_props = NULL;
  PyObject *__pyx_v_rule = NULL;
  PyObject *__pyx_v_rule_id = NULL;
  PyObject *__pyx_v_rule_data = NULL;
  PyObject *__pyx_v_concept = NULL;
  PyObject *__pyx_v_concept_id = NULL;
  PyObject *__pyx_v_concept_data = NULL;
  PyObject *__pyx_v_rel = NULL;
  PyObject *__pyx_v_from_table_id = NULL;
  PyObject *__pyx_v_to_table_id = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_example = NULL;
  PyObject *__pyx_v_example_id = NULL;
  PyObject *__pyx_v_keyword = NULL;
  PyObject *__pyx_v_targets = NULL;
  PyObject *__pyx_v_keyword_id = NULL;
  PyObject *__pyx_v_target = NULL;
  PyObject *__pyx_v_term = NULL;
  PyObject *__pyx_v_definition = NULL;
  PyObject *__pyx_v_term_id = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_UCS4 __pyx_t_17;
  PyObject *(*__pyx_t_18)(PyObject *);
  int __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *(*__pyx_t_23)(PyObject *);
  int __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build_graph", 0);
  __Pyx_INCREF(__pyx_v_metadata);

  /* "sQQideR/DatabaseContext.pyx":317
 *             The built NetworkX directed graph
 *         """
 *         if metadata is None:             # <<<<<<<<<<<<<<
 *             if self.metadata is None:
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")
 */
  __pyx_t_1 = (__pyx_v_metadata == ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "sQQideR/DatabaseContext.pyx":318
 *         """
 *         if metadata is None:
 *             if self.metadata is None:             # <<<<<<<<<<<<<<
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")
 *             metadata = self.metadata
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = (__pyx_t_2 == Py_None);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(__pyx_t_1)) {

      /* "sQQideR/DatabaseContext.pyx":319
 *         if metadata is None:
 *             if self.metadata is None:
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")             # <<<<<<<<<<<<<<
 *             metadata = self.metadata
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 319, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":318
 *         """
 *         if metadata is None:
 *             if self.metadata is None:             # <<<<<<<<<<<<<<
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")
 *             metadata = self.metadata
 */
    }

    /* "sQQideR/DatabaseContext.pyx":320
 *             if self.metadata is None:
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")
 *             metadata = self.metadata             # <<<<<<<<<<<<<<
 * 
 *         # Create a new directed graph
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 320, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(PyDict_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_2))) __PYX_ERR(0, 320, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_metadata, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "sQQideR/DatabaseContext.pyx":317
 *             The built NetworkX directed graph
 *         """
 *         if metadata is None:             # <<<<<<<<<<<<<<
 *             if self.metadata is None:
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")
 */
  }

  /* "sQQideR/DatabaseContext.pyx":323
 * 
 *         # Create a new directed graph
 *         G = nx.DiGraph()             # <<<<<<<<<<<<<<
 * 
 *         # Add tables as nodes
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_nx); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_DiGraph); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_v_G = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":326
 * 
 *         # Add tables as nodes
 *         for table_name, table_data in metadata["tables"].items():             # <<<<<<<<<<<<<<
 *             node_id = f"table:{table_name}"
 *             G.add_node(node_id,
 */
  __pyx_t_6 = 0;
  if (unlikely(__pyx_v_metadata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 326, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_tables); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_t_4 == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 326, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_dict_iterator(__pyx_t_4, 0, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_XDECREF(__pyx_t_2);
  __pyx_t_2 = __pyx_t_3;
  __pyx_t_3 = 0;
  while (1) {
    __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_7, &__pyx_t_6, &__pyx_t_3, &__pyx_t_4, NULL, __pyx_t_8);
    if (unlikely(__pyx_t_9 == 0)) break;
    if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_table_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":327
 *         # Add tables as nodes
 *         for table_name, table_data in metadata["tables"].items():
 *             node_id = f"table:{table_name}"             # <<<<<<<<<<<<<<
 *             G.add_node(node_id,
 *                        type="table",
 */
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_node_id, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":328
 *         for table_name, table_data in metadata["tables"].items():
 *             node_id = f"table:{table_name}"
 *             G.add_node(node_id,             # <<<<<<<<<<<<<<
 *                        type="table",
 *                        name=table_name,
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_node_id);
    __Pyx_GIVEREF(__pyx_v_node_id);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_node_id)) __PYX_ERR(0, 328, __pyx_L1_error);

    /* "sQQideR/DatabaseContext.pyx":329
 *             node_id = f"table:{table_name}"
 *             G.add_node(node_id,
 *                        type="table",             # <<<<<<<<<<<<<<
 *                        name=table_name,
 *                        description=table_data["description"],
 */
    __pyx_t_10 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_type, __pyx_n_u_table_2) < 0) __PYX_ERR(0, 329, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":330
 *             G.add_node(node_id,
 *                        type="table",
 *                        name=table_name,             # <<<<<<<<<<<<<<
 *                        description=table_data["description"],
 *                        domain=table_data["domain"],
 */
    if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_name, __pyx_v_table_name) < 0) __PYX_ERR(0, 329, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":331
 *                        type="table",
 *                        name=table_name,
 *                        description=table_data["description"],             # <<<<<<<<<<<<<<
 *                        domain=table_data["domain"],
 *                        row_count=table_data.get("row_count", 0))
 */
    __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table_data, __pyx_n_u_description); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_description, __pyx_t_11) < 0) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "sQQideR/DatabaseContext.pyx":332
 *                        name=table_name,
 *                        description=table_data["description"],
 *                        domain=table_data["domain"],             # <<<<<<<<<<<<<<
 *                        row_count=table_data.get("row_count", 0))
 * 
 */
    __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table_data, __pyx_n_u_domain); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_domain, __pyx_t_11) < 0) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "sQQideR/DatabaseContext.pyx":333
 *                        description=table_data["description"],
 *                        domain=table_data["domain"],
 *                        row_count=table_data.get("row_count", 0))             # <<<<<<<<<<<<<<
 * 
 *             # Add columns as nodes and connect to tables
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_table_data, __pyx_n_s_get); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_row_count, __pyx_t_12) < 0) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":328
 *         for table_name, table_data in metadata["tables"].items():
 *             node_id = f"table:{table_name}"
 *             G.add_node(node_id,             # <<<<<<<<<<<<<<
 *                        type="table",
 *                        name=table_name,
 */
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":336
 * 
 *             # Add columns as nodes and connect to tables
 *             for col_name, col_data in table_data["columns"].items():             # <<<<<<<<<<<<<<
 *                 col_node_id = f"column:{table_name}.{col_name}"
 *                 node_props = {
 */
    __pyx_t_13 = 0;
    __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table_data, __pyx_n_u_columns); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__pyx_t_10 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 336, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_dict_iterator(__pyx_t_10, 0, __pyx_n_s_items, (&__pyx_t_14), (&__pyx_t_9)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_12);
    __pyx_t_12 = __pyx_t_4;
    __pyx_t_4 = 0;
    while (1) {
      __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_12, __pyx_t_14, &__pyx_t_13, &__pyx_t_4, &__pyx_t_10, NULL, __pyx_t_9);
      if (unlikely(__pyx_t_15 == 0)) break;
      if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(0, 336, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_col_name, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_v_col_data, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":337
 *             # Add columns as nodes and connect to tables
 *             for col_name, col_data in table_data["columns"].items():
 *                 col_node_id = f"column:{table_name}.{col_name}"             # <<<<<<<<<<<<<<
 *                 node_props = {
 *                     "type": "column",
 */
      __pyx_t_10 = PyTuple_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_16 = 0;
      __pyx_t_17 = 127;
      __Pyx_INCREF(__pyx_kp_u_column);
      __pyx_t_16 += 7;
      __Pyx_GIVEREF(__pyx_kp_u_column);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_kp_u_column);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_17 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_17) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_17;
      __pyx_t_16 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__15);
      __pyx_t_16 += 1;
      __Pyx_GIVEREF(__pyx_kp_u__15);
      PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_kp_u__15);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_col_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_17 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_17) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_17;
      __pyx_t_16 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, __pyx_t_16, __pyx_t_17); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_col_node_id, ((PyObject*)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":339
 *                 col_node_id = f"column:{table_name}.{col_name}"
 *                 node_props = {
 *                     "type": "column",             # <<<<<<<<<<<<<<
 *                     "name": col_name,
 *                     "description": col_data["description"],
 */
      __pyx_t_4 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_type, __pyx_n_u_column_2) < 0) __PYX_ERR(0, 339, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":340
 *                 node_props = {
 *                     "type": "column",
 *                     "name": col_name,             # <<<<<<<<<<<<<<
 *                     "description": col_data["description"],
 *                     "data_type": col_data["data_type"],
 */
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_name, __pyx_v_col_name) < 0) __PYX_ERR(0, 339, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":341
 *                     "type": "column",
 *                     "name": col_name,
 *                     "description": col_data["description"],             # <<<<<<<<<<<<<<
 *                     "data_type": col_data["data_type"],
 *                     "indexed": col_data.get("indexed", False)
 */
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_description); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_description, __pyx_t_10) < 0) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":342
 *                     "name": col_name,
 *                     "description": col_data["description"],
 *                     "data_type": col_data["data_type"],             # <<<<<<<<<<<<<<
 *                     "indexed": col_data.get("indexed", False)
 *                 }
 */
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_data_type); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_data_type, __pyx_t_10) < 0) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":343
 *                     "description": col_data["description"],
 *                     "data_type": col_data["data_type"],
 *                     "indexed": col_data.get("indexed", False)             # <<<<<<<<<<<<<<
 *                 }
 *                 # Add statistics if available
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_col_data, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_indexed, __pyx_t_3) < 0) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_node_props, ((PyObject*)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":346
 *                 }
 *                 # Add statistics if available
 *                 if "statistics" in col_data:             # <<<<<<<<<<<<<<
 *                     node_props["statistics"] = col_data["statistics"]
 *                 G.add_node(col_node_id, **node_props)
 */
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_statistics, __pyx_v_col_data, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 346, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "sQQideR/DatabaseContext.pyx":347
 *                 # Add statistics if available
 *                 if "statistics" in col_data:
 *                     node_props["statistics"] = col_data["statistics"]             # <<<<<<<<<<<<<<
 *                 G.add_node(col_node_id, **node_props)
 *                 G.add_edge(node_id, col_node_id, relationship="has_column")
 */
        __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_statistics); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 347, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (unlikely((PyDict_SetItem(__pyx_v_node_props, __pyx_n_u_statistics, __pyx_t_4) < 0))) __PYX_ERR(0, 347, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "sQQideR/DatabaseContext.pyx":346
 *                 }
 *                 # Add statistics if available
 *                 if "statistics" in col_data:             # <<<<<<<<<<<<<<
 *                     node_props["statistics"] = col_data["statistics"]
 *                 G.add_node(col_node_id, **node_props)
 */
      }

      /* "sQQideR/DatabaseContext.pyx":348
 *                 if "statistics" in col_data:
 *                     node_props["statistics"] = col_data["statistics"]
 *                 G.add_node(col_node_id, **node_props)             # <<<<<<<<<<<<<<
 *                 G.add_edge(node_id, col_node_id, relationship="has_column")
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 348, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 348, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_v_col_node_id);
      __Pyx_GIVEREF(__pyx_v_col_node_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_col_node_id)) __PYX_ERR(0, 348, __pyx_L1_error);
      __pyx_t_10 = PyDict_Copy(__pyx_v_node_props); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 348, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 348, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":349
 *                     node_props["statistics"] = col_data["statistics"]
 *                 G.add_node(col_node_id, **node_props)
 *                 G.add_edge(node_id, col_node_id, relationship="has_column")             # <<<<<<<<<<<<<<
 * 
 *                 # Add business rules connected to this column
 */
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_v_node_id);
      __Pyx_GIVEREF(__pyx_v_node_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_node_id)) __PYX_ERR(0, 349, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_col_node_id);
      __Pyx_GIVEREF(__pyx_v_col_node_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_v_col_node_id)) __PYX_ERR(0, 349, __pyx_L1_error);
      __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_relationship, __pyx_n_u_has_column) < 0) __PYX_ERR(0, 349, __pyx_L1_error)
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_10, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":352
 * 
 *                 # Add business rules connected to this column
 *                 if "business_rules" in col_data:             # <<<<<<<<<<<<<<
 *                     for rule in col_data["business_rules"]:
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 */
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_business_rules, __pyx_v_col_data, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 352, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "sQQideR/DatabaseContext.pyx":353
 *                 # Add business rules connected to this column
 *                 if "business_rules" in col_data:
 *                     for rule in col_data["business_rules"]:             # <<<<<<<<<<<<<<
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):
 */
        __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_business_rules); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 353, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
          __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3);
          __pyx_t_16 = 0;
          __pyx_t_18 = NULL;
        } else {
          __pyx_t_16 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 353, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        for (;;) {
          if (likely(!__pyx_t_18)) {
            if (likely(PyList_CheckExact(__pyx_t_3))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 353, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_16); __Pyx_INCREF(__pyx_t_4); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 353, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 353, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 353, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_16); __Pyx_INCREF(__pyx_t_4); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 353, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 353, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            }
          } else {
            __pyx_t_4 = __pyx_t_18(__pyx_t_3);
            if (unlikely(!__pyx_t_4)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 353, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_XDECREF_SET(__pyx_v_rule, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":354
 *                 if "business_rules" in col_data:
 *                     for rule in col_data["business_rules"]:
 *                         rule_id = f"rule:{rule.split(':')[0]}"             # <<<<<<<<<<<<<<
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_n_s_split); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 354, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_kp_u__20};
            __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 354, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          }
          __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 354, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 354, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_kp_u_rule, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 354, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF_SET(__pyx_v_rule_id, ((PyObject*)__pyx_t_10));
          __pyx_t_10 = 0;

          /* "sQQideR/DatabaseContext.pyx":355
 *                     for rule in col_data["business_rules"]:
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):             # <<<<<<<<<<<<<<
 *                             G.add_node(rule_id,
 *                                        type="rule",
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_has_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_rule_id};
            __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 355, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          }
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_19 = (!__pyx_t_1);
          if (__pyx_t_19) {

            /* "sQQideR/DatabaseContext.pyx":356
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,             # <<<<<<<<<<<<<<
 *                                        type="rule",
 *                                        name=rule.split(':')[0],
 */
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 356, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 356, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_v_rule_id);
            __Pyx_GIVEREF(__pyx_v_rule_id);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_rule_id)) __PYX_ERR(0, 356, __pyx_L1_error);

            /* "sQQideR/DatabaseContext.pyx":357
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,
 *                                        type="rule",             # <<<<<<<<<<<<<<
 *                                        name=rule.split(':')[0],
 *                                        description=rule)
 */
            __pyx_t_11 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 357, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_type, __pyx_n_u_rule_2) < 0) __PYX_ERR(0, 357, __pyx_L1_error)

            /* "sQQideR/DatabaseContext.pyx":358
 *                             G.add_node(rule_id,
 *                                        type="rule",
 *                                        name=rule.split(':')[0],             # <<<<<<<<<<<<<<
 *                                        description=rule)
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")
 */
            __pyx_t_21 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_n_s_split); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 358, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            __pyx_t_22 = NULL;
            __pyx_t_5 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_21))) {
              __pyx_t_22 = PyMethod_GET_SELF(__pyx_t_21);
              if (likely(__pyx_t_22)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_21);
                __Pyx_INCREF(__pyx_t_22);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_21, function);
                __pyx_t_5 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_22, __pyx_kp_u__20};
              __pyx_t_20 = __Pyx_PyObject_FastCall(__pyx_t_21, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
              __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
              if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 358, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_20);
              __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
            }
            __pyx_t_21 = __Pyx_GetItemInt(__pyx_t_20, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 358, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_name, __pyx_t_21) < 0) __PYX_ERR(0, 357, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

            /* "sQQideR/DatabaseContext.pyx":359
 *                                        type="rule",
 *                                        name=rule.split(':')[0],
 *                                        description=rule)             # <<<<<<<<<<<<<<
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")
 * 
 */
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_description, __pyx_v_rule) < 0) __PYX_ERR(0, 357, __pyx_L1_error)

            /* "sQQideR/DatabaseContext.pyx":356
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,             # <<<<<<<<<<<<<<
 *                                        type="rule",
 *                                        name=rule.split(':')[0],
 */
            __pyx_t_21 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_4, __pyx_t_11); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 356, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

            /* "sQQideR/DatabaseContext.pyx":355
 *                     for rule in col_data["business_rules"]:
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):             # <<<<<<<<<<<<<<
 *                             G.add_node(rule_id,
 *                                        type="rule",
 */
          }

          /* "sQQideR/DatabaseContext.pyx":360
 *                                        name=rule.split(':')[0],
 *                                        description=rule)
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")             # <<<<<<<<<<<<<<
 * 
 *                 # Add related business rules if available
 */
          __pyx_t_21 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_21);
          __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_v_col_node_id);
          __Pyx_GIVEREF(__pyx_v_col_node_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_col_node_id)) __PYX_ERR(0, 360, __pyx_L1_error);
          __Pyx_INCREF(__pyx_v_rule_id);
          __Pyx_GIVEREF(__pyx_v_rule_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_rule_id)) __PYX_ERR(0, 360, __pyx_L1_error);
          __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_relationship, __pyx_n_u_governed_by) < 0) __PYX_ERR(0, 360, __pyx_L1_error)
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_21, __pyx_t_11, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "sQQideR/DatabaseContext.pyx":353
 *                 # Add business rules connected to this column
 *                 if "business_rules" in col_data:
 *                     for rule in col_data["business_rules"]:             # <<<<<<<<<<<<<<
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 *                         if not G.has_node(rule_id):
 */
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":352
 * 
 *                 # Add business rules connected to this column
 *                 if "business_rules" in col_data:             # <<<<<<<<<<<<<<
 *                     for rule in col_data["business_rules"]:
 *                         rule_id = f"rule:{rule.split(':')[0]}"
 */
      }

      /* "sQQideR/DatabaseContext.pyx":363
 * 
 *                 # Add related business rules if available
 *                 if "related_business_rules" in col_data:             # <<<<<<<<<<<<<<
 *                     for rule_data in col_data["related_business_rules"]:
 *                         rule_id = f"rule:{rule_data['name']}"
 */
      __pyx_t_19 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_related_business_rules, __pyx_v_col_data, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 363, __pyx_L1_error)
      if (__pyx_t_19) {

        /* "sQQideR/DatabaseContext.pyx":364
 *                 # Add related business rules if available
 *                 if "related_business_rules" in col_data:
 *                     for rule_data in col_data["related_business_rules"]:             # <<<<<<<<<<<<<<
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):
 */
        __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_related_business_rules); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
          __pyx_t_10 = __pyx_t_3; __Pyx_INCREF(__pyx_t_10);
          __pyx_t_16 = 0;
          __pyx_t_18 = NULL;
        } else {
          __pyx_t_16 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 364, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_10); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 364, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        for (;;) {
          if (likely(!__pyx_t_18)) {
            if (likely(PyList_CheckExact(__pyx_t_10))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_10);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 364, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_16); __Pyx_INCREF(__pyx_t_3); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 364, __pyx_L1_error)
              #else
              __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_10, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_10);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 364, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_16); __Pyx_INCREF(__pyx_t_3); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 364, __pyx_L1_error)
              #else
              __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_10, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            }
          } else {
            __pyx_t_3 = __pyx_t_18(__pyx_t_10);
            if (unlikely(!__pyx_t_3)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 364, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_3);
          }
          __Pyx_XDECREF_SET(__pyx_v_rule_data, __pyx_t_3);
          __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":365
 *                 if "related_business_rules" in col_data:
 *                     for rule_data in col_data["related_business_rules"]:
 *                         rule_id = f"rule:{rule_data['name']}"             # <<<<<<<<<<<<<<
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,
 */
          __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rule_data, __pyx_n_u_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_3, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_kp_u_rule, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF_SET(__pyx_v_rule_id, ((PyObject*)__pyx_t_3));
          __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":366
 *                     for rule_data in col_data["related_business_rules"]:
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):             # <<<<<<<<<<<<<<
 *                             G.add_node(rule_id,
 *                                        type="rule",
 */
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_has_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 366, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_rule_id};
            __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          }
          __pyx_t_19 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 366, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_1 = (!__pyx_t_19);
          if (__pyx_t_1) {

            /* "sQQideR/DatabaseContext.pyx":367
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,             # <<<<<<<<<<<<<<
 *                                        type="rule",
 *                                        name=rule_data['name'],
 */
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 367, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_v_rule_id);
            __Pyx_GIVEREF(__pyx_v_rule_id);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_rule_id)) __PYX_ERR(0, 367, __pyx_L1_error);

            /* "sQQideR/DatabaseContext.pyx":368
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,
 *                                        type="rule",             # <<<<<<<<<<<<<<
 *                                        name=rule_data['name'],
 *                                        description=rule_data['description'])
 */
            __pyx_t_11 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 368, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_type, __pyx_n_u_rule_2) < 0) __PYX_ERR(0, 368, __pyx_L1_error)

            /* "sQQideR/DatabaseContext.pyx":369
 *                             G.add_node(rule_id,
 *                                        type="rule",
 *                                        name=rule_data['name'],             # <<<<<<<<<<<<<<
 *                                        description=rule_data['description'])
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")
 */
            __pyx_t_21 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rule_data, __pyx_n_u_name); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 369, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_name, __pyx_t_21) < 0) __PYX_ERR(0, 368, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

            /* "sQQideR/DatabaseContext.pyx":370
 *                                        type="rule",
 *                                        name=rule_data['name'],
 *                                        description=rule_data['description'])             # <<<<<<<<<<<<<<
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")
 * 
 */
            __pyx_t_21 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rule_data, __pyx_n_u_description); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 370, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_description, __pyx_t_21) < 0) __PYX_ERR(0, 368, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

            /* "sQQideR/DatabaseContext.pyx":367
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):
 *                             G.add_node(rule_id,             # <<<<<<<<<<<<<<
 *                                        type="rule",
 *                                        name=rule_data['name'],
 */
            __pyx_t_21 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_11); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 367, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;

            /* "sQQideR/DatabaseContext.pyx":366
 *                     for rule_data in col_data["related_business_rules"]:
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):             # <<<<<<<<<<<<<<
 *                             G.add_node(rule_id,
 *                                        type="rule",
 */
          }

          /* "sQQideR/DatabaseContext.pyx":371
 *                                        name=rule_data['name'],
 *                                        description=rule_data['description'])
 *                         G.add_edge(col_node_id, rule_id, relationship="governed_by")             # <<<<<<<<<<<<<<
 * 
 *                 # Add concepts related to this column
 */
          __pyx_t_21 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 371, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_21);
          __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 371, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_v_col_node_id);
          __Pyx_GIVEREF(__pyx_v_col_node_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_col_node_id)) __PYX_ERR(0, 371, __pyx_L1_error);
          __Pyx_INCREF(__pyx_v_rule_id);
          __Pyx_GIVEREF(__pyx_v_rule_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_rule_id)) __PYX_ERR(0, 371, __pyx_L1_error);
          __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 371, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_relationship, __pyx_n_u_governed_by) < 0) __PYX_ERR(0, 371, __pyx_L1_error)
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_21, __pyx_t_11, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 371, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":364
 *                 # Add related business rules if available
 *                 if "related_business_rules" in col_data:
 *                     for rule_data in col_data["related_business_rules"]:             # <<<<<<<<<<<<<<
 *                         rule_id = f"rule:{rule_data['name']}"
 *                         if not G.has_node(rule_id):
 */
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "sQQideR/DatabaseContext.pyx":363
 * 
 *                 # Add related business rules if available
 *                 if "related_business_rules" in col_data:             # <<<<<<<<<<<<<<
 *                     for rule_data in col_data["related_business_rules"]:
 *                         rule_id = f"rule:{rule_data['name']}"
 */
      }

      /* "sQQideR/DatabaseContext.pyx":374
 * 
 *                 # Add concepts related to this column
 *                 if "concepts" in col_data:             # <<<<<<<<<<<<<<
 *                     for concept in col_data["concepts"]:
 *                         concept_id = f"concept:{concept}"
 */
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_concepts, __pyx_v_col_data, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 374, __pyx_L1_error)
      if (__pyx_t_1) {

        /* "sQQideR/DatabaseContext.pyx":375
 *                 # Add concepts related to this column
 *                 if "concepts" in col_data:
 *                     for concept in col_data["concepts"]:             # <<<<<<<<<<<<<<
 *                         concept_id = f"concept:{concept}"
 *                         if not G.has_node(concept_id):
 */
        __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_col_data, __pyx_n_u_concepts); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 375, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
          __pyx_t_3 = __pyx_t_10; __Pyx_INCREF(__pyx_t_3);
          __pyx_t_16 = 0;
          __pyx_t_18 = NULL;
        } else {
          __pyx_t_16 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 375, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 375, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        for (;;) {
          if (likely(!__pyx_t_18)) {
            if (likely(PyList_CheckExact(__pyx_t_3))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 375, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_10 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_16); __Pyx_INCREF(__pyx_t_10); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 375, __pyx_L1_error)
              #else
              __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 375, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              #endif
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 375, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_16); __Pyx_INCREF(__pyx_t_10); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 375, __pyx_L1_error)
              #else
              __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 375, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_10);
              #endif
            }
          } else {
            __pyx_t_10 = __pyx_t_18(__pyx_t_3);
            if (unlikely(!__pyx_t_10)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 375, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_10);
          }
          __Pyx_XDECREF_SET(__pyx_v_concept, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "sQQideR/DatabaseContext.pyx":376
 *                 if "concepts" in col_data:
 *                     for concept in col_data["concepts"]:
 *                         concept_id = f"concept:{concept}"             # <<<<<<<<<<<<<<
 *                         if not G.has_node(concept_id):
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 */
          __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_v_concept, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 376, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_kp_u_concept, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 376, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF_SET(__pyx_v_concept_id, ((PyObject*)__pyx_t_4));
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":377
 *                     for concept in col_data["concepts"]:
 *                         concept_id = f"concept:{concept}"
 *                         if not G.has_node(concept_id):             # <<<<<<<<<<<<<<
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 *                             G.add_node(concept_id,
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_has_node); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_concept_id};
            __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 377, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          }
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_19 = (!__pyx_t_1);
          if (__pyx_t_19) {

            /* "sQQideR/DatabaseContext.pyx":378
 *                         concept_id = f"concept:{concept}"
 *                         if not G.has_node(concept_id):
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})             # <<<<<<<<<<<<<<
 *                             G.add_node(concept_id,
 *                                        type="concept",
 */
            if (unlikely(__pyx_v_metadata == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
              __PYX_ERR(0, 378, __pyx_L1_error)
            }
            __pyx_t_10 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 378, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyDict_GetItemDefault(__pyx_v_metadata, __pyx_n_u_business_concepts, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 378, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 378, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __pyx_t_11 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 378, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_21 = NULL;
            __pyx_t_5 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_10))) {
              __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_10);
              if (likely(__pyx_t_21)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_21);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_10, function);
                __pyx_t_5 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_21, __pyx_v_concept, __pyx_t_11};
              __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_5, 2+__pyx_t_5);
              __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 378, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            __Pyx_XDECREF_SET(__pyx_v_concept_data, __pyx_t_4);
            __pyx_t_4 = 0;

            /* "sQQideR/DatabaseContext.pyx":379
 *                         if not G.has_node(concept_id):
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 *                             G.add_node(concept_id,             # <<<<<<<<<<<<<<
 *                                        type="concept",
 *                                        name=concept,
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 379, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 379, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_INCREF(__pyx_v_concept_id);
            __Pyx_GIVEREF(__pyx_v_concept_id);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_concept_id)) __PYX_ERR(0, 379, __pyx_L1_error);

            /* "sQQideR/DatabaseContext.pyx":380
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 *                             G.add_node(concept_id,
 *                                        type="concept",             # <<<<<<<<<<<<<<
 *                                        name=concept,
 *                                        description=concept_data.get("description", ""))
 */
            __pyx_t_11 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 380, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_11);
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_type, __pyx_n_u_concept_2) < 0) __PYX_ERR(0, 380, __pyx_L1_error)

            /* "sQQideR/DatabaseContext.pyx":381
 *                             G.add_node(concept_id,
 *                                        type="concept",
 *                                        name=concept,             # <<<<<<<<<<<<<<
 *                                        description=concept_data.get("description", ""))
 *                         G.add_edge(col_node_id, concept_id, relationship="implements")
 */
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_name, __pyx_v_concept) < 0) __PYX_ERR(0, 380, __pyx_L1_error)

            /* "sQQideR/DatabaseContext.pyx":382
 *                                        type="concept",
 *                                        name=concept,
 *                                        description=concept_data.get("description", ""))             # <<<<<<<<<<<<<<
 *                         G.add_edge(col_node_id, concept_id, relationship="implements")
 * 
 */
            __pyx_t_21 = __Pyx_PyObject_GetAttrStr(__pyx_v_concept_data, __pyx_n_s_get); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 382, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_21);
            __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_21, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 382, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_20);
            __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
            if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_description, __pyx_t_20) < 0) __PYX_ERR(0, 380, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

            /* "sQQideR/DatabaseContext.pyx":379
 *                         if not G.has_node(concept_id):
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 *                             G.add_node(concept_id,             # <<<<<<<<<<<<<<
 *                                        type="concept",
 *                                        name=concept,
 */
            __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 379, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_20);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

            /* "sQQideR/DatabaseContext.pyx":377
 *                     for concept in col_data["concepts"]:
 *                         concept_id = f"concept:{concept}"
 *                         if not G.has_node(concept_id):             # <<<<<<<<<<<<<<
 *                             concept_data = metadata.get("business_concepts", {}).get(concept, {})
 *                             G.add_node(concept_id,
 */
          }

          /* "sQQideR/DatabaseContext.pyx":383
 *                                        name=concept,
 *                                        description=concept_data.get("description", ""))
 *                         G.add_edge(col_node_id, concept_id, relationship="implements")             # <<<<<<<<<<<<<<
 * 
 *         # Add table relationships
 */
          __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_v_col_node_id);
          __Pyx_GIVEREF(__pyx_v_col_node_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_col_node_id)) __PYX_ERR(0, 383, __pyx_L1_error);
          __Pyx_INCREF(__pyx_v_concept_id);
          __Pyx_GIVEREF(__pyx_v_concept_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_v_concept_id)) __PYX_ERR(0, 383, __pyx_L1_error);
          __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_relationship, __pyx_n_u_implements) < 0) __PYX_ERR(0, 383, __pyx_L1_error)
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_20, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":375
 *                 # Add concepts related to this column
 *                 if "concepts" in col_data:
 *                     for concept in col_data["concepts"]:             # <<<<<<<<<<<<<<
 *                         concept_id = f"concept:{concept}"
 *                         if not G.has_node(concept_id):
 */
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":374
 * 
 *                 # Add concepts related to this column
 *                 if "concepts" in col_data:             # <<<<<<<<<<<<<<
 *                     for concept in col_data["concepts"]:
 *                         concept_id = f"concept:{concept}"
 */
      }
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":386
 * 
 *         # Add table relationships
 *         if "relationships" in metadata:             # <<<<<<<<<<<<<<
 *             for rel in metadata["relationships"]:
 *                 from_table_id = f"table:{rel['from_table']}"
 */
  if (unlikely(__pyx_v_metadata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 386, __pyx_L1_error)
  }
  __pyx_t_19 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_relationships, __pyx_v_metadata, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 386, __pyx_L1_error)
  if (__pyx_t_19) {

    /* "sQQideR/DatabaseContext.pyx":387
 *         # Add table relationships
 *         if "relationships" in metadata:
 *             for rel in metadata["relationships"]:             # <<<<<<<<<<<<<<
 *                 from_table_id = f"table:{rel['from_table']}"
 *                 to_table_id = f"table:{rel['to_table']}"
 */
    if (unlikely(__pyx_v_metadata == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 387, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_relationships); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_12 = __pyx_t_2; __Pyx_INCREF(__pyx_t_12);
      __pyx_t_7 = 0;
      __pyx_t_18 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 387, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_12); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 387, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_18)) {
        if (likely(PyList_CheckExact(__pyx_t_12))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_12);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 387, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely((0 < 0))) __PYX_ERR(0, 387, __pyx_L1_error)
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_12, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_12);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 387, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_12, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely((0 < 0))) __PYX_ERR(0, 387, __pyx_L1_error)
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_12, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_18(__pyx_t_12);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 387, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_rel, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":388
 *         if "relationships" in metadata:
 *             for rel in metadata["relationships"]:
 *                 from_table_id = f"table:{rel['from_table']}"             # <<<<<<<<<<<<<<
 *                 to_table_id = f"table:{rel['to_table']}"
 *                 G.add_edge(from_table_id, to_table_id,
 */
      __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_from_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_t_2, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 388, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_from_table_id, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":389
 *             for rel in metadata["relationships"]:
 *                 from_table_id = f"table:{rel['from_table']}"
 *                 to_table_id = f"table:{rel['to_table']}"             # <<<<<<<<<<<<<<
 *                 G.add_edge(from_table_id, to_table_id,
 *                            relationship="related_to",
 */
      __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_to_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_t_2, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_to_table_id, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":390
 *                 from_table_id = f"table:{rel['from_table']}"
 *                 to_table_id = f"table:{rel['to_table']}"
 *                 G.add_edge(from_table_id, to_table_id,             # <<<<<<<<<<<<<<
 *                            relationship="related_to",
 *                            join_condition=rel["join_condition"],
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_v_from_table_id);
      __Pyx_GIVEREF(__pyx_v_from_table_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_from_table_id)) __PYX_ERR(0, 390, __pyx_L1_error);
      __Pyx_INCREF(__pyx_v_to_table_id);
      __Pyx_GIVEREF(__pyx_v_to_table_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_to_table_id)) __PYX_ERR(0, 390, __pyx_L1_error);

      /* "sQQideR/DatabaseContext.pyx":391
 *                 to_table_id = f"table:{rel['to_table']}"
 *                 G.add_edge(from_table_id, to_table_id,
 *                            relationship="related_to",             # <<<<<<<<<<<<<<
 *                            join_condition=rel["join_condition"],
 *                            description=rel["description"])
 */
      __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_relationship, __pyx_n_u_related_to) < 0) __PYX_ERR(0, 391, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":392
 *                 G.add_edge(from_table_id, to_table_id,
 *                            relationship="related_to",
 *                            join_condition=rel["join_condition"],             # <<<<<<<<<<<<<<
 *                            description=rel["description"])
 * 
 */
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_join_condition); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_join_condition, __pyx_t_10) < 0) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":393
 *                            relationship="related_to",
 *                            join_condition=rel["join_condition"],
 *                            description=rel["description"])             # <<<<<<<<<<<<<<
 * 
 *         # Add query examples as nodes if available
 */
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_description); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_description, __pyx_t_10) < 0) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":390
 *                 from_table_id = f"table:{rel['from_table']}"
 *                 to_table_id = f"table:{rel['to_table']}"
 *                 G.add_edge(from_table_id, to_table_id,             # <<<<<<<<<<<<<<
 *                            relationship="related_to",
 *                            join_condition=rel["join_condition"],
 */
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":387
 *         # Add table relationships
 *         if "relationships" in metadata:
 *             for rel in metadata["relationships"]:             # <<<<<<<<<<<<<<
 *                 from_table_id = f"table:{rel['from_table']}"
 *                 to_table_id = f"table:{rel['to_table']}"
 */
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":386
 * 
 *         # Add table relationships
 *         if "relationships" in metadata:             # <<<<<<<<<<<<<<
 *             for rel in metadata["relationships"]:
 *                 from_table_id = f"table:{rel['from_table']}"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":396
 * 
 *         # Add query examples as nodes if available
 *         if "query_examples" in metadata:             # <<<<<<<<<<<<<<
 *             for i, example in enumerate(metadata["query_examples"]):
 *                 example_id = f"example:{i}"
 */
  if (unlikely(__pyx_v_metadata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 396, __pyx_L1_error)
  }
  __pyx_t_19 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_query_examples, __pyx_v_metadata, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 396, __pyx_L1_error)
  if (__pyx_t_19) {

    /* "sQQideR/DatabaseContext.pyx":397
 *         # Add query examples as nodes if available
 *         if "query_examples" in metadata:
 *             for i, example in enumerate(metadata["query_examples"]):             # <<<<<<<<<<<<<<
 *                 example_id = f"example:{i}"
 *                 G.add_node(example_id,
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_12 = __pyx_int_0;
    if (unlikely(__pyx_v_metadata == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 397, __pyx_L1_error)
    }
    __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_query_examples); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
      __pyx_t_4 = __pyx_t_10; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_7 = 0;
      __pyx_t_18 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 397, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    for (;;) {
      if (likely(!__pyx_t_18)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 397, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_10); __pyx_t_7++; if (unlikely((0 < 0))) __PYX_ERR(0, 397, __pyx_L1_error)
          #else
          __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 397, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_10); __pyx_t_7++; if (unlikely((0 < 0))) __PYX_ERR(0, 397, __pyx_L1_error)
          #else
          __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        }
      } else {
        __pyx_t_10 = __pyx_t_18(__pyx_t_4);
        if (unlikely(!__pyx_t_10)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 397, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_example, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_12);
      __pyx_t_10 = __Pyx_PyInt_AddObjC(__pyx_t_12, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_12);
      __pyx_t_12 = __pyx_t_10;
      __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":398
 *         if "query_examples" in metadata:
 *             for i, example in enumerate(metadata["query_examples"]):
 *                 example_id = f"example:{i}"             # <<<<<<<<<<<<<<
 *                 G.add_node(example_id,
 *                            type="example",
 */
      __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_v_i, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 398, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_kp_u_example, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 398, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_example_id, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "sQQideR/DatabaseContext.pyx":399
 *             for i, example in enumerate(metadata["query_examples"]):
 *                 example_id = f"example:{i}"
 *                 G.add_node(example_id,             # <<<<<<<<<<<<<<
 *                            type="example",
 *                            query=example["query"],
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_v_example_id);
      __Pyx_GIVEREF(__pyx_v_example_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_example_id)) __PYX_ERR(0, 399, __pyx_L1_error);

      /* "sQQideR/DatabaseContext.pyx":400
 *                 example_id = f"example:{i}"
 *                 G.add_node(example_id,
 *                            type="example",             # <<<<<<<<<<<<<<
 *                            query=example["query"],
 *                            description=example["description"],
 */
      __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_type, __pyx_n_u_example_2) < 0) __PYX_ERR(0, 400, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":401
 *                 G.add_node(example_id,
 *                            type="example",
 *                            query=example["query"],             # <<<<<<<<<<<<<<
 *                            description=example["description"],
 *                            complexity=example.get("complexity", "simple"))
 */
      __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_v_example, __pyx_n_u_query); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_query, __pyx_t_11) < 0) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":402
 *                            type="example",
 *                            query=example["query"],
 *                            description=example["description"],             # <<<<<<<<<<<<<<
 *                            complexity=example.get("complexity", "simple"))
 *                 # Connect examples to related tables
 */
      __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_v_example, __pyx_n_u_description); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_description, __pyx_t_11) < 0) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":403
 *                            query=example["query"],
 *                            description=example["description"],
 *                            complexity=example.get("complexity", "simple"))             # <<<<<<<<<<<<<<
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:
 */
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_example, __pyx_n_s_get); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_complexity, __pyx_t_20) < 0) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

      /* "sQQideR/DatabaseContext.pyx":399
 *             for i, example in enumerate(metadata["query_examples"]):
 *                 example_id = f"example:{i}"
 *                 G.add_node(example_id,             # <<<<<<<<<<<<<<
 *                            type="example",
 *                            query=example["query"],
 */
      __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_10, __pyx_t_2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

      /* "sQQideR/DatabaseContext.pyx":405
 *                            complexity=example.get("complexity", "simple"))
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:             # <<<<<<<<<<<<<<
 *                     if table_name.lower() in example["query"].lower():
 *                         G.add_edge(example_id, f"table:{table_name}", relationship="references")
 */
      if (unlikely(__pyx_v_metadata == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 405, __pyx_L1_error)
      }
      __pyx_t_20 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_tables); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 405, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
      if (likely(PyList_CheckExact(__pyx_t_20)) || PyTuple_CheckExact(__pyx_t_20)) {
        __pyx_t_2 = __pyx_t_20; __Pyx_INCREF(__pyx_t_2);
        __pyx_t_6 = 0;
        __pyx_t_23 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_20); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_23 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 405, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      for (;;) {
        if (likely(!__pyx_t_23)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_20 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_20); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
            #else
            __pyx_t_20 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 405, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_20);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_20 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_20); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
            #else
            __pyx_t_20 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 405, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_20);
            #endif
          }
        } else {
          __pyx_t_20 = __pyx_t_23(__pyx_t_2);
          if (unlikely(!__pyx_t_20)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 405, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_20);
        }
        __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_20);
        __pyx_t_20 = 0;

        /* "sQQideR/DatabaseContext.pyx":406
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:
 *                     if table_name.lower() in example["query"].lower():             # <<<<<<<<<<<<<<
 *                         G.add_edge(example_id, f"table:{table_name}", relationship="references")
 * 
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_table_name, __pyx_n_s_lower); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_3 = NULL;
        __pyx_t_5 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_5 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_20 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_example, __pyx_n_u_query); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_lower); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = NULL;
        __pyx_t_5 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_5 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __pyx_t_19 = (__Pyx_PySequence_ContainsTF(__pyx_t_20, __pyx_t_10, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (__pyx_t_19) {

          /* "sQQideR/DatabaseContext.pyx":407
 *                 for table_name in metadata["tables"]:
 *                     if table_name.lower() in example["query"].lower():
 *                         G.add_edge(example_id, f"table:{table_name}", relationship="references")             # <<<<<<<<<<<<<<
 * 
 *         # Add keyword mappings to nodes
 */
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_20 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __pyx_t_11 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_20); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          __pyx_t_20 = PyTuple_New(2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __Pyx_INCREF(__pyx_v_example_id);
          __Pyx_GIVEREF(__pyx_v_example_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_v_example_id)) __PYX_ERR(0, 407, __pyx_L1_error);
          __Pyx_GIVEREF(__pyx_t_11);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_20, 1, __pyx_t_11)) __PYX_ERR(0, 407, __pyx_L1_error);
          __pyx_t_11 = 0;
          __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_relationship, __pyx_n_u_references) < 0) __PYX_ERR(0, 407, __pyx_L1_error)
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_20, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":406
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:
 *                     if table_name.lower() in example["query"].lower():             # <<<<<<<<<<<<<<
 *                         G.add_edge(example_id, f"table:{table_name}", relationship="references")
 * 
 */
        }

        /* "sQQideR/DatabaseContext.pyx":405
 *                            complexity=example.get("complexity", "simple"))
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:             # <<<<<<<<<<<<<<
 *                     if table_name.lower() in example["query"].lower():
 *                         G.add_edge(example_id, f"table:{table_name}", relationship="references")
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":397
 *         # Add query examples as nodes if available
 *         if "query_examples" in metadata:
 *             for i, example in enumerate(metadata["query_examples"]):             # <<<<<<<<<<<<<<
 *                 example_id = f"example:{i}"
 *                 G.add_node(example_id,
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":396
 * 
 *         # Add query examples as nodes if available
 *         if "query_examples" in metadata:             # <<<<<<<<<<<<<<
 *             for i, example in enumerate(metadata["query_examples"]):
 *                 example_id = f"example:{i}"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":410
 * 
 *         # Add keyword mappings to nodes
 *         if "keywords" in metadata:             # <<<<<<<<<<<<<<
 *             for keyword, targets in metadata["keywords"].items():
 *                 keyword_id = f"keyword:{keyword}"
 */
  if (unlikely(__pyx_v_metadata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 410, __pyx_L1_error)
  }
  __pyx_t_19 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_keywords, __pyx_v_metadata, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 410, __pyx_L1_error)
  if (__pyx_t_19) {

    /* "sQQideR/DatabaseContext.pyx":411
 *         # Add keyword mappings to nodes
 *         if "keywords" in metadata:
 *             for keyword, targets in metadata["keywords"].items():             # <<<<<<<<<<<<<<
 *                 keyword_id = f"keyword:{keyword}"
 *                 G.add_node(keyword_id, type="keyword", name=keyword)
 */
    __pyx_t_7 = 0;
    if (unlikely(__pyx_v_metadata == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 411, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_keywords); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (unlikely(__pyx_t_4 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 411, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_dict_iterator(__pyx_t_4, 0, __pyx_n_s_items, (&__pyx_t_6), (&__pyx_t_8)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_12);
    __pyx_t_12 = __pyx_t_2;
    __pyx_t_2 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_12, __pyx_t_6, &__pyx_t_7, &__pyx_t_2, &__pyx_t_4, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 411, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_keyword, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_targets, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":412
 *         if "keywords" in metadata:
 *             for keyword, targets in metadata["keywords"].items():
 *                 keyword_id = f"keyword:{keyword}"             # <<<<<<<<<<<<<<
 *                 G.add_node(keyword_id, type="keyword", name=keyword)
 *                 for target in targets:
 */
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 412, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_keyword, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_v_keyword_id, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":413
 *             for keyword, targets in metadata["keywords"].items():
 *                 keyword_id = f"keyword:{keyword}"
 *                 G.add_node(keyword_id, type="keyword", name=keyword)             # <<<<<<<<<<<<<<
 *                 for target in targets:
 *                     # Determine if target is a table, column, or concept
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 413, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 413, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_keyword_id);
      __Pyx_GIVEREF(__pyx_v_keyword_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_keyword_id)) __PYX_ERR(0, 413, __pyx_L1_error);
      __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_type, __pyx_n_u_keyword_2) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_name, __pyx_v_keyword) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
      __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 413, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":414
 *                 keyword_id = f"keyword:{keyword}"
 *                 G.add_node(keyword_id, type="keyword", name=keyword)
 *                 for target in targets:             # <<<<<<<<<<<<<<
 *                     # Determine if target is a table, column, or concept
 *                     if "." in target:  # Column
 */
      if (likely(PyList_CheckExact(__pyx_v_targets)) || PyTuple_CheckExact(__pyx_v_targets)) {
        __pyx_t_11 = __pyx_v_targets; __Pyx_INCREF(__pyx_t_11);
        __pyx_t_14 = 0;
        __pyx_t_18 = NULL;
      } else {
        __pyx_t_14 = -1; __pyx_t_11 = PyObject_GetIter(__pyx_v_targets); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 414, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_11); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 414, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_18)) {
          if (likely(PyList_CheckExact(__pyx_t_11))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 414, __pyx_L1_error)
              #endif
              if (__pyx_t_14 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely((0 < 0))) __PYX_ERR(0, 414, __pyx_L1_error)
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_11, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_11);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 414, __pyx_L1_error)
              #endif
              if (__pyx_t_14 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_11, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely((0 < 0))) __PYX_ERR(0, 414, __pyx_L1_error)
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_11, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 414, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          }
        } else {
          __pyx_t_3 = __pyx_t_18(__pyx_t_11);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 414, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_XDECREF_SET(__pyx_v_target, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":416
 *                 for target in targets:
 *                     # Determine if target is a table, column, or concept
 *                     if "." in target:  # Column             # <<<<<<<<<<<<<<
 *                         node_id = f"column:{target}"
 *                     elif target in metadata.get("business_concepts", {}):  # Concept
 */
        __pyx_t_19 = (__Pyx_PySequence_ContainsTF(__pyx_kp_u__15, __pyx_v_target, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 416, __pyx_L1_error)
        if (__pyx_t_19) {

          /* "sQQideR/DatabaseContext.pyx":417
 *                     # Determine if target is a table, column, or concept
 *                     if "." in target:  # Column
 *                         node_id = f"column:{target}"             # <<<<<<<<<<<<<<
 *                     elif target in metadata.get("business_concepts", {}):  # Concept
 *                         node_id = f"concept:{target}"
 */
          __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_target, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 417, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_kp_u_column, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 417, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF_SET(__pyx_v_node_id, ((PyObject*)__pyx_t_4));
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":416
 *                 for target in targets:
 *                     # Determine if target is a table, column, or concept
 *                     if "." in target:  # Column             # <<<<<<<<<<<<<<
 *                         node_id = f"column:{target}"
 *                     elif target in metadata.get("business_concepts", {}):  # Concept
 */
          goto __pyx_L42;
        }

        /* "sQQideR/DatabaseContext.pyx":418
 *                     if "." in target:  # Column
 *                         node_id = f"column:{target}"
 *                     elif target in metadata.get("business_concepts", {}):  # Concept             # <<<<<<<<<<<<<<
 *                         node_id = f"concept:{target}"
 *                     else:  # Assume table
 */
        if (unlikely(__pyx_v_metadata == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
          __PYX_ERR(0, 418, __pyx_L1_error)
        }
        __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 418, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_metadata, __pyx_n_u_business_concepts, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_19 = (__Pyx_PySequence_ContainsTF(__pyx_v_target, __pyx_t_3, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 418, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_19) {

          /* "sQQideR/DatabaseContext.pyx":419
 *                         node_id = f"column:{target}"
 *                     elif target in metadata.get("business_concepts", {}):  # Concept
 *                         node_id = f"concept:{target}"             # <<<<<<<<<<<<<<
 *                     else:  # Assume table
 *                         node_id = f"table:{target}"
 */
          __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_target, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_kp_u_concept, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 419, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF_SET(__pyx_v_node_id, ((PyObject*)__pyx_t_4));
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":418
 *                     if "." in target:  # Column
 *                         node_id = f"column:{target}"
 *                     elif target in metadata.get("business_concepts", {}):  # Concept             # <<<<<<<<<<<<<<
 *                         node_id = f"concept:{target}"
 *                     else:  # Assume table
 */
          goto __pyx_L42;
        }

        /* "sQQideR/DatabaseContext.pyx":421
 *                         node_id = f"concept:{target}"
 *                     else:  # Assume table
 *                         node_id = f"table:{target}"             # <<<<<<<<<<<<<<
 *                     if G.has_node(node_id):
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 */
        /*else*/ {
          __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_target, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 421, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 421, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF_SET(__pyx_v_node_id, ((PyObject*)__pyx_t_3));
          __pyx_t_3 = 0;
        }
        __pyx_L42:;

        /* "sQQideR/DatabaseContext.pyx":422
 *                     else:  # Assume table
 *                         node_id = f"table:{target}"
 *                     if G.has_node(node_id):             # <<<<<<<<<<<<<<
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 *                         if "keywords" not in G.nodes[node_id]:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_has_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = NULL;
        __pyx_t_5 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_5 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_node_id};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 422, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        __pyx_t_19 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 422, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_19) {

          /* "sQQideR/DatabaseContext.pyx":423
 *                         node_id = f"table:{target}"
 *                     if G.has_node(node_id):
 *                         G.add_edge(keyword_id, node_id, relationship="tags")             # <<<<<<<<<<<<<<
 *                         if "keywords" not in G.nodes[node_id]:
 *                             G.nodes[node_id]["keywords"] = []
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 423, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_v_keyword_id);
          __Pyx_GIVEREF(__pyx_v_keyword_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_keyword_id)) __PYX_ERR(0, 423, __pyx_L1_error);
          __Pyx_INCREF(__pyx_v_node_id);
          __Pyx_GIVEREF(__pyx_v_node_id);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_node_id)) __PYX_ERR(0, 423, __pyx_L1_error);
          __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 423, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_relationship, __pyx_n_u_tags) < 0) __PYX_ERR(0, 423, __pyx_L1_error)
          __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 423, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

          /* "sQQideR/DatabaseContext.pyx":424
 *                     if G.has_node(node_id):
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 *                         if "keywords" not in G.nodes[node_id]:             # <<<<<<<<<<<<<<
 *                             G.nodes[node_id]["keywords"] = []
 *                         G.nodes[node_id]["keywords"].append(keyword)
 */
          __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_nodes); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 424, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_20, __pyx_v_node_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 424, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          __pyx_t_19 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_keywords, __pyx_t_2, Py_NE)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 424, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (__pyx_t_19) {

            /* "sQQideR/DatabaseContext.pyx":425
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 *                         if "keywords" not in G.nodes[node_id]:
 *                             G.nodes[node_id]["keywords"] = []             # <<<<<<<<<<<<<<
 *                         G.nodes[node_id]["keywords"].append(keyword)
 * 
 */
            __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_nodes); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 425, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_20);
            __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_20, __pyx_v_node_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 425, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
            if (unlikely((PyObject_SetItem(__pyx_t_4, __pyx_n_u_keywords, __pyx_t_2) < 0))) __PYX_ERR(0, 425, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

            /* "sQQideR/DatabaseContext.pyx":424
 *                     if G.has_node(node_id):
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 *                         if "keywords" not in G.nodes[node_id]:             # <<<<<<<<<<<<<<
 *                             G.nodes[node_id]["keywords"] = []
 *                         G.nodes[node_id]["keywords"].append(keyword)
 */
          }

          /* "sQQideR/DatabaseContext.pyx":426
 *                         if "keywords" not in G.nodes[node_id]:
 *                             G.nodes[node_id]["keywords"] = []
 *                         G.nodes[node_id]["keywords"].append(keyword)             # <<<<<<<<<<<<<<
 * 
 *         # Add terminology nodes
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_nodes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_v_node_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 426, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_keywords); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_24 = __Pyx_PyObject_Append(__pyx_t_2, __pyx_v_keyword); if (unlikely(__pyx_t_24 == ((int)-1))) __PYX_ERR(0, 426, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "sQQideR/DatabaseContext.pyx":422
 *                     else:  # Assume table
 *                         node_id = f"table:{target}"
 *                     if G.has_node(node_id):             # <<<<<<<<<<<<<<
 *                         G.add_edge(keyword_id, node_id, relationship="tags")
 *                         if "keywords" not in G.nodes[node_id]:
 */
        }

        /* "sQQideR/DatabaseContext.pyx":414
 *                 keyword_id = f"keyword:{keyword}"
 *                 G.add_node(keyword_id, type="keyword", name=keyword)
 *                 for target in targets:             # <<<<<<<<<<<<<<
 *                     # Determine if target is a table, column, or concept
 *                     if "." in target:  # Column
 */
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":410
 * 
 *         # Add keyword mappings to nodes
 *         if "keywords" in metadata:             # <<<<<<<<<<<<<<
 *             for keyword, targets in metadata["keywords"].items():
 *                 keyword_id = f"keyword:{keyword}"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":429
 * 
 *         # Add terminology nodes
 *         if "terminology" in metadata:             # <<<<<<<<<<<<<<
 *             for term, definition in metadata["terminology"].items():
 *                 term_id = f"term:{term}"
 */
  if (unlikely(__pyx_v_metadata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 429, __pyx_L1_error)
  }
  __pyx_t_19 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_terminology, __pyx_v_metadata, Py_EQ)); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 429, __pyx_L1_error)
  if (__pyx_t_19) {

    /* "sQQideR/DatabaseContext.pyx":430
 *         # Add terminology nodes
 *         if "terminology" in metadata:
 *             for term, definition in metadata["terminology"].items():             # <<<<<<<<<<<<<<
 *                 term_id = f"term:{term}"
 *                 G.add_node(term_id, type="term", name=term, description=definition)
 */
    __pyx_t_6 = 0;
    if (unlikely(__pyx_v_metadata == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 430, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_terminology); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (unlikely(__pyx_t_11 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 430, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_dict_iterator(__pyx_t_11, 0, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12);
    __pyx_t_12 = __pyx_t_2;
    __pyx_t_2 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_12, __pyx_t_7, &__pyx_t_6, &__pyx_t_2, &__pyx_t_11, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_XDECREF_SET(__pyx_v_term, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_definition, __pyx_t_11);
      __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":431
 *         if "terminology" in metadata:
 *             for term, definition in metadata["terminology"].items():
 *                 term_id = f"term:{term}"             # <<<<<<<<<<<<<<
 *                 G.add_node(term_id, type="term", name=term, description=definition)
 *                 # Connect terms to relevant columns
 */
      __pyx_t_11 = __Pyx_PyObject_FormatSimple(__pyx_v_term, __pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_term, __pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_term_id, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":432
 *             for term, definition in metadata["terminology"].items():
 *                 term_id = f"term:{term}"
 *                 G.add_node(term_id, type="term", name=term, description=definition)             # <<<<<<<<<<<<<<
 *                 # Connect terms to relevant columns
 *                 for table_name, table_data in metadata["tables"].items():
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_node); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_v_term_id);
      __Pyx_GIVEREF(__pyx_v_term_id);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_term_id)) __PYX_ERR(0, 432, __pyx_L1_error);
      __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_type, __pyx_n_u_term_2) < 0) __PYX_ERR(0, 432, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_name, __pyx_v_term) < 0) __PYX_ERR(0, 432, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_description, __pyx_v_definition) < 0) __PYX_ERR(0, 432, __pyx_L1_error)
      __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_11, __pyx_t_4); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

      /* "sQQideR/DatabaseContext.pyx":434
 *                 G.add_node(term_id, type="term", name=term, description=definition)
 *                 # Connect terms to relevant columns
 *                 for table_name, table_data in metadata["tables"].items():             # <<<<<<<<<<<<<<
 *                     for col_name in table_data["columns"]:
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():
 */
      __pyx_t_14 = 0;
      if (unlikely(__pyx_v_metadata == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 434, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_metadata, __pyx_n_u_tables); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(__pyx_t_4 == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 434, __pyx_L1_error)
      }
      __pyx_t_11 = __Pyx_dict_iterator(__pyx_t_4, 0, __pyx_n_s_items, (&__pyx_t_13), (&__pyx_t_9)); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_20);
      __pyx_t_20 = __pyx_t_11;
      __pyx_t_11 = 0;
      while (1) {
        __pyx_t_15 = __Pyx_dict_iter_next(__pyx_t_20, __pyx_t_13, &__pyx_t_14, &__pyx_t_11, &__pyx_t_4, NULL, __pyx_t_9);
        if (unlikely(__pyx_t_15 == 0)) break;
        if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(0, 434, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_11);
        __pyx_t_11 = 0;
        __Pyx_XDECREF_SET(__pyx_v_table_data, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "sQQideR/DatabaseContext.pyx":435
 *                 # Connect terms to relevant columns
 *                 for table_name, table_data in metadata["tables"].items():
 *                     for col_name in table_data["columns"]:             # <<<<<<<<<<<<<<
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")
 */
        __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table_data, __pyx_n_u_columns); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
          __pyx_t_11 = __pyx_t_4; __Pyx_INCREF(__pyx_t_11);
          __pyx_t_16 = 0;
          __pyx_t_18 = NULL;
        } else {
          __pyx_t_16 = -1; __pyx_t_11 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 435, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_18 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_11); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 435, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        for (;;) {
          if (likely(!__pyx_t_18)) {
            if (likely(PyList_CheckExact(__pyx_t_11))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 435, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_16); __Pyx_INCREF(__pyx_t_4); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 435, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_11, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_11);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 435, __pyx_L1_error)
                #endif
                if (__pyx_t_16 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_11, __pyx_t_16); __Pyx_INCREF(__pyx_t_4); __pyx_t_16++; if (unlikely((0 < 0))) __PYX_ERR(0, 435, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_11, __pyx_t_16); __pyx_t_16++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            }
          } else {
            __pyx_t_4 = __pyx_t_18(__pyx_t_11);
            if (unlikely(!__pyx_t_4)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 435, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_XDECREF_SET(__pyx_v_col_name, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":436
 *                 for table_name, table_data in metadata["tables"].items():
 *                     for col_name in table_data["columns"]:
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():             # <<<<<<<<<<<<<<
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")
 * 
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_term, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
            if (likely(__pyx_t_3)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_2, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
            __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 436, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          }
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_col_name, __pyx_n_s_lower); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_10 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_10, NULL};
            __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          }
          __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (!__pyx_t_1) {
          } else {
            __pyx_t_19 = __pyx_t_1;
            goto __pyx_L54_bool_binop_done;
          }
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_term, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_2, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
            __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          }
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_col_name, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_4, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_10, NULL};
            __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          }
          __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_t_3, __pyx_t_2, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 436, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_19 = __pyx_t_1;
          __pyx_L54_bool_binop_done:;
          if (__pyx_t_19) {

            /* "sQQideR/DatabaseContext.pyx":437
 *                     for col_name in table_data["columns"]:
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")             # <<<<<<<<<<<<<<
 * 
 *         self.graph = G
 */
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_add_edge); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_25 = 0;
            __pyx_t_17 = 127;
            __Pyx_INCREF(__pyx_kp_u_column);
            __pyx_t_25 += 7;
            __Pyx_GIVEREF(__pyx_kp_u_column);
            PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_column);
            __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_17 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_17) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_17;
            __pyx_t_25 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_4);
            PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
            __pyx_t_4 = 0;
            __Pyx_INCREF(__pyx_kp_u__15);
            __pyx_t_25 += 1;
            __Pyx_GIVEREF(__pyx_kp_u__15);
            PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u__15);
            __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_col_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_17 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_17) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_17;
            __pyx_t_25 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_4);
            PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_4);
            __pyx_t_4 = 0;
            __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_3, 4, __pyx_t_25, __pyx_t_17); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_INCREF(__pyx_v_term_id);
            __Pyx_GIVEREF(__pyx_v_term_id);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_term_id)) __PYX_ERR(0, 437, __pyx_L1_error);
            __Pyx_GIVEREF(__pyx_t_4);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error);
            __pyx_t_4 = 0;
            __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_relationship, __pyx_n_u_defines) < 0) __PYX_ERR(0, 437, __pyx_L1_error)
            __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

            /* "sQQideR/DatabaseContext.pyx":436
 *                 for table_name, table_data in metadata["tables"].items():
 *                     for col_name in table_data["columns"]:
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():             # <<<<<<<<<<<<<<
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")
 * 
 */
          }

          /* "sQQideR/DatabaseContext.pyx":435
 *                 # Connect terms to relevant columns
 *                 for table_name, table_data in metadata["tables"].items():
 *                     for col_name in table_data["columns"]:             # <<<<<<<<<<<<<<
 *                         if term.lower() == col_name.lower() or term.lower() in col_name.lower():
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")
 */
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":429
 * 
 *         # Add terminology nodes
 *         if "terminology" in metadata:             # <<<<<<<<<<<<<<
 *             for term, definition in metadata["terminology"].items():
 *                 term_id = f"term:{term}"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":439
 *                             G.add_edge(term_id, f"column:{table_name}.{col_name}", relationship="defines")
 * 
 *         self.graph = G             # <<<<<<<<<<<<<<
 * 
 *         if self.verbose:
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_graph, __pyx_v_G) < 0) __PYX_ERR(0, 439, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":441
 *         self.graph = G
 * 
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Graph built with {len(G.nodes)} nodes and {len(G.edges)} edges")
 * 
 */
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_19 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__pyx_t_19) {

    /* "sQQideR/DatabaseContext.pyx":442
 * 
 *         if self.verbose:
 *             print(f"Graph built with {len(G.nodes)} nodes and {len(G.edges)} edges")             # <<<<<<<<<<<<<<
 * 
 *         return G
 */
    __pyx_t_12 = PyTuple_New(5); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_7 = 0;
    __pyx_t_17 = 127;
    __Pyx_INCREF(__pyx_kp_u_Graph_built_with);
    __pyx_t_7 += 17;
    __Pyx_GIVEREF(__pyx_kp_u_Graph_built_with);
    PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_kp_u_Graph_built_with);
    __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_nodes); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_6 = PyObject_Length(__pyx_t_20); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __pyx_t_20 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_6, 0, ' ', 'd'); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_20);
    __Pyx_GIVEREF(__pyx_t_20);
    PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_20);
    __pyx_t_20 = 0;
    __Pyx_INCREF(__pyx_kp_u_nodes_and);
    __pyx_t_7 += 11;
    __Pyx_GIVEREF(__pyx_kp_u_nodes_and);
    PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_kp_u_nodes_and);
    __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_v_G, __pyx_n_s_edges); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_6 = PyObject_Length(__pyx_t_20); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __pyx_t_20 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_6, 0, ' ', 'd'); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_20);
    __Pyx_GIVEREF(__pyx_t_20);
    PyTuple_SET_ITEM(__pyx_t_12, 3, __pyx_t_20);
    __pyx_t_20 = 0;
    __Pyx_INCREF(__pyx_kp_u_edges_2);
    __pyx_t_7 += 6;
    __Pyx_GIVEREF(__pyx_kp_u_edges_2);
    PyTuple_SET_ITEM(__pyx_t_12, 4, __pyx_kp_u_edges_2);
    __pyx_t_20 = __Pyx_PyUnicode_Join(__pyx_t_12, 5, __pyx_t_7, __pyx_t_17); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_20); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "sQQideR/DatabaseContext.pyx":441
 *         self.graph = G
 * 
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Graph built with {len(G.nodes)} nodes and {len(G.edges)} edges")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":444
 *             print(f"Graph built with {len(G.nodes)} nodes and {len(G.edges)} edges")
 * 
 *         return G             # <<<<<<<<<<<<<<
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_G);
  __pyx_r = __pyx_v_G;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":307
 * 
 * 
 *     def build_graph(self, metadata: Dict = None) -> nx.DiGraph:             # <<<<<<<<<<<<<<
 *         """
 *         Build a graph from the metadata.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.build_graph", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_G);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_table_data);
  __Pyx_XDECREF(__pyx_v_node_id);
  __Pyx_XDECREF(__pyx_v_col_name);
  __Pyx_XDECREF(__pyx_v_col_data);
  __Pyx_XDECREF(__pyx_v_col_node_id);
  __Pyx_XDECREF(__pyx_v_node_props);
  __Pyx_XDECREF(__pyx_v_rule);
  __Pyx_XDECREF(__pyx_v_rule_id);
  __Pyx_XDECREF(__pyx_v_rule_data);
  __Pyx_XDECREF(__pyx_v_concept);
  __Pyx_XDECREF(__pyx_v_concept_id);
  __Pyx_XDECREF(__pyx_v_concept_data);
  __Pyx_XDECREF(__pyx_v_rel);
  __Pyx_XDECREF(__pyx_v_from_table_id);
  __Pyx_XDECREF(__pyx_v_to_table_id);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_example);
  __Pyx_XDECREF(__pyx_v_example_id);
  __Pyx_XDECREF(__pyx_v_keyword);
  __Pyx_XDECREF(__pyx_v_targets);
  __Pyx_XDECREF(__pyx_v_keyword_id);
  __Pyx_XDECREF(__pyx_v_target);
  __Pyx_XDECREF(__pyx_v_term);
  __Pyx_XDECREF(__pyx_v_definition);
  __Pyx_XDECREF(__pyx_v_term_id);
  __Pyx_XDECREF(__pyx_v_metadata);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_11search_by_keywords(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_10search_by_keywords, "\n        Search the graph for nodes matching the given keywords.\n        \n        Args:\n            keywords: List of keywords to search for\n            \n        Returns:\n            Tuple of (results, matches):\n                - results: List of tuples (node_id, node_data)\n                - matches: Dictionary mapping node_ids to match reasons\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_11search_by_keywords = {"search_by_keywords", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_11search_by_keywords, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_10search_by_keywords};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_11search_by_keywords(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_keywords = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("search_by_keywords (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_keywords,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_keywords)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("search_by_keywords", 1, 2, 2, 1); __PYX_ERR(0, 446, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "search_by_keywords") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_keywords = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("search_by_keywords", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 446, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.search_by_keywords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_keywords), (&PyList_Type), 0, "keywords", 1))) __PYX_ERR(0, 446, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_10search_by_keywords(__pyx_self, __pyx_v_self, __pyx_v_keywords);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "sQQideR/DatabaseContext.pyx":497
 *                     match_score = 3
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 */

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_genexpr(PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 497, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *) __pyx_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __pyx_cur_scope->__pyx_genexpr_arg_0 = __pyx_genexpr_arg_0;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_2generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_DatabaseContextService_search_by, __pyx_n_s_sQQideR_DatabaseContext); if (unlikely(!gen)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.search_by_keywords.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 497, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_genexpr_arg_0)) { __Pyx_RaiseUnboundLocalError(".0"); __PYX_ERR(0, 497, __pyx_L1_error) }
  if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_genexpr_arg_0)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_genexpr_arg_0)) {
    __pyx_t_1 = __pyx_cur_scope->__pyx_genexpr_arg_0; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_genexpr_arg_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 497, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 497, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_k);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_k, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_pattern)) { __Pyx_RaiseClosureNameError("pattern"); __PYX_ERR(0, 497, __pyx_L1_error) }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_pattern, __pyx_n_s_search); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_cur_scope->__pyx_v_k};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_8) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_Generator_Replace_StopIteration(0);
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":521
 * 
 *         # Sort nodes by score
 *         sorted_nodes = sorted(node_scores.items(), key=lambda x: x[1], reverse=True)             # <<<<<<<<<<<<<<
 * 
 *         # Add nodes to results in order of score
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_3lambda1(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_3lambda1 = {"lambda1", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_3lambda1, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_3lambda1(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_x = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda1 (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_x)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 521, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "lambda1") < 0)) __PYX_ERR(0, 521, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_x = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda1", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 521, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.search_by_keywords.lambda1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda1(__pyx_self, __pyx_v_x);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda1(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda1", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_x, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.search_by_keywords.lambda1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_10search_by_keywords(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_keywords) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *__pyx_cur_scope;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_matches = NULL;
  PyObject *__pyx_v_normalized_keywords = NULL;
  PyObject *__pyx_v_node_scores = NULL;
  PyObject *__pyx_v_node_id = NULL;
  PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_node_score = NULL;
  PyObject *__pyx_v_node_matches = NULL;
  PyObject *__pyx_v_keyword = NULL;
  int __pyx_v_matched;
  PyObject *__pyx_v_match_reason = NULL;
  PyObject *__pyx_v_match_score = NULL;
  PyObject *__pyx_v_sorted_nodes = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_score = NULL;
  PyObject *__pyx_8genexpr2__pyx_v_k = NULL;
  PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_2generator = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  unsigned int __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *(*__pyx_t_11)(PyObject *);
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  Py_ssize_t __pyx_t_14;
  Py_UCS4 __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("search_by_keywords", 0);
  __pyx_cur_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 446, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }

  /* "sQQideR/DatabaseContext.pyx":458
 *                 - matches: Dictionary mapping node_ids to match reasons
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":459
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         results = []
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 459, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":458
 *                 - matches: Dictionary mapping node_ids to match reasons
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":461
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         results = []             # <<<<<<<<<<<<<<
 *         matches = {}  # Track which keyword matched which node
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":462
 * 
 *         results = []
 *         matches = {}  # Track which keyword matched which node             # <<<<<<<<<<<<<<
 * 
 *         # Normalize keywords
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_matches = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":465
 * 
 *         # Normalize keywords
 *         normalized_keywords = [k.lower().strip() for k in keywords]             # <<<<<<<<<<<<<<
 * 
 *         # Track scores for each node
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __pyx_v_keywords; __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 465, __pyx_L6_error)
        #endif
        if (__pyx_t_4 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_5 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_5); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 465, __pyx_L6_error)
      #else
      __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 465, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
      __Pyx_XDECREF_SET(__pyx_8genexpr2__pyx_v_k, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr2__pyx_v_k, __pyx_n_s_lower); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 465, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 0+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 465, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_strip); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 465, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
        __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 0+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 465, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 465, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_k); __pyx_8genexpr2__pyx_v_k = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_k); __pyx_8genexpr2__pyx_v_k = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_v_normalized_keywords = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":468
 * 
 *         # Track scores for each node
 *         node_scores = {}             # <<<<<<<<<<<<<<
 * 
 *         for node_id, data in self.graph.nodes(data=True):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_node_scores = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":470
 *         node_scores = {}
 * 
 *         for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *             node_score = 0
 *             node_matches = []
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_data, Py_True) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 470, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 470, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_5); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 470, __pyx_L1_error)
        #else
        __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 470, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_5); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 470, __pyx_L1_error)
        #else
        __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_10(__pyx_t_1);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 470, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 470, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 470, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 470, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_11 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
      index = 0; __pyx_t_3 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_7 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_7)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_6), 2) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L14_unpacking_done;
      __pyx_L13_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 470, __pyx_L1_error)
      __pyx_L14_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "sQQideR/DatabaseContext.pyx":471
 * 
 *         for node_id, data in self.graph.nodes(data=True):
 *             node_score = 0             # <<<<<<<<<<<<<<
 *             node_matches = []
 * 
 */
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_XDECREF_SET(__pyx_v_node_score, __pyx_int_0);

    /* "sQQideR/DatabaseContext.pyx":472
 *         for node_id, data in self.graph.nodes(data=True):
 *             node_score = 0
 *             node_matches = []             # <<<<<<<<<<<<<<
 * 
 *             # Check if any keyword matches node properties
 */
    __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_node_matches, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":475
 * 
 *             # Check if any keyword matches node properties
 *             for keyword in normalized_keywords:             # <<<<<<<<<<<<<<
 *                 pattern = re.compile(r'\b' + re.escape(keyword) + r'\b', re.IGNORECASE)
 *                 matched = False
 */
    __pyx_t_5 = __pyx_v_normalized_keywords; __Pyx_INCREF(__pyx_t_5);
    __pyx_t_12 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_5);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 475, __pyx_L1_error)
        #endif
        if (__pyx_t_12 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_7 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 475, __pyx_L1_error)
      #else
      __pyx_t_7 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 475, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_keyword, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "sQQideR/DatabaseContext.pyx":476
 *             # Check if any keyword matches node properties
 *             for keyword in normalized_keywords:
 *                 pattern = re.compile(r'\b' + re.escape(keyword) + r'\b', re.IGNORECASE)             # <<<<<<<<<<<<<<
 *                 matched = False
 *                 match_reason = ""
 */
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_re); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_compile); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_re); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_escape); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_keyword};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_13, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
      __pyx_t_13 = PyNumber_Add(__pyx_kp_u_b, __pyx_t_3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Add(__pyx_t_13, __pyx_kp_u_b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_re); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_IGNORECASE); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_13, __pyx_t_3, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 2+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 476, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_pattern);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_pattern, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;

      /* "sQQideR/DatabaseContext.pyx":477
 *             for keyword in normalized_keywords:
 *                 pattern = re.compile(r'\b' + re.escape(keyword) + r'\b', re.IGNORECASE)
 *                 matched = False             # <<<<<<<<<<<<<<
 *                 match_reason = ""
 *                 match_score = 0
 */
      __pyx_v_matched = 0;

      /* "sQQideR/DatabaseContext.pyx":478
 *                 pattern = re.compile(r'\b' + re.escape(keyword) + r'\b', re.IGNORECASE)
 *                 matched = False
 *                 match_reason = ""             # <<<<<<<<<<<<<<
 *                 match_score = 0
 * 
 */
      __Pyx_INCREF(__pyx_kp_u__12);
      __Pyx_XDECREF_SET(__pyx_v_match_reason, __pyx_kp_u__12);

      /* "sQQideR/DatabaseContext.pyx":479
 *                 matched = False
 *                 match_reason = ""
 *                 match_score = 0             # <<<<<<<<<<<<<<
 * 
 *                 # Match against node id - highest priority
 */
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_XDECREF_SET(__pyx_v_match_score, __pyx_int_0);

      /* "sQQideR/DatabaseContext.pyx":482
 * 
 *                 # Match against node id - highest priority
 *                 if pattern.search(node_id):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"ID contains '{keyword}'"
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_pattern, __pyx_n_s_search); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 482, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_node_id};
        __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 482, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 482, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":483
 *                 # Match against node id - highest priority
 *                 if pattern.search(node_id):
 *                     matched = True             # <<<<<<<<<<<<<<
 *                     match_reason = f"ID contains '{keyword}'"
 *                     match_score = 5
 */
        __pyx_v_matched = 1;

        /* "sQQideR/DatabaseContext.pyx":484
 *                 if pattern.search(node_id):
 *                     matched = True
 *                     match_reason = f"ID contains '{keyword}'"             # <<<<<<<<<<<<<<
 *                     match_score = 5
 *                 # Match against name
 */
        __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 484, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_14 = 0;
        __pyx_t_15 = 127;
        __Pyx_INCREF(__pyx_kp_u_ID_contains);
        __pyx_t_14 += 13;
        __Pyx_GIVEREF(__pyx_kp_u_ID_contains);
        PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_kp_u_ID_contains);
        __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 484, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_15;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_INCREF(__pyx_kp_u__23);
        __pyx_t_14 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__23);
        PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_kp_u__23);
        __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 484, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":485
 *                     matched = True
 *                     match_reason = f"ID contains '{keyword}'"
 *                     match_score = 5             # <<<<<<<<<<<<<<
 *                 # Match against name
 *                 elif 'name' in data and pattern.search(data['name']):
 */
        __Pyx_INCREF(__pyx_int_5);
        __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_5);

        /* "sQQideR/DatabaseContext.pyx":482
 * 
 *                 # Match against node id - highest priority
 *                 if pattern.search(node_id):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"ID contains '{keyword}'"
 */
        goto __pyx_L17;
      }

      /* "sQQideR/DatabaseContext.pyx":487
 *                     match_score = 5
 *                 # Match against name
 *                 elif 'name' in data and pattern.search(data['name']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Name contains '{keyword}'"
 */
      __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_name, __pyx_v_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 487, __pyx_L1_error)
      if (__pyx_t_16) {
      } else {
        __pyx_t_2 = __pyx_t_16;
        goto __pyx_L18_bool_binop_done;
      }
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_pattern, __pyx_n_s_search); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_v_data, __pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_8};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 487, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 487, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_2 = __pyx_t_16;
      __pyx_L18_bool_binop_done:;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":488
 *                 # Match against name
 *                 elif 'name' in data and pattern.search(data['name']):
 *                     matched = True             # <<<<<<<<<<<<<<
 *                     match_reason = f"Name contains '{keyword}'"
 *                     match_score = 4
 */
        __pyx_v_matched = 1;

        /* "sQQideR/DatabaseContext.pyx":489
 *                 elif 'name' in data and pattern.search(data['name']):
 *                     matched = True
 *                     match_reason = f"Name contains '{keyword}'"             # <<<<<<<<<<<<<<
 *                     match_score = 4
 *                 # Match against description
 */
        __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_14 = 0;
        __pyx_t_15 = 127;
        __Pyx_INCREF(__pyx_kp_u_Name_contains);
        __pyx_t_14 += 15;
        __Pyx_GIVEREF(__pyx_kp_u_Name_contains);
        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_kp_u_Name_contains);
        __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) : __pyx_t_15;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7);
        __pyx_t_7 = 0;
        __Pyx_INCREF(__pyx_kp_u__23);
        __pyx_t_14 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__23);
        PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_kp_u__23);
        __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_6, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_7));
        __pyx_t_7 = 0;

        /* "sQQideR/DatabaseContext.pyx":490
 *                     matched = True
 *                     match_reason = f"Name contains '{keyword}'"
 *                     match_score = 4             # <<<<<<<<<<<<<<
 *                 # Match against description
 *                 elif 'description' in data and pattern.search(data['description']):
 */
        __Pyx_INCREF(__pyx_int_4);
        __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_4);

        /* "sQQideR/DatabaseContext.pyx":487
 *                     match_score = 5
 *                 # Match against name
 *                 elif 'name' in data and pattern.search(data['name']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Name contains '{keyword}'"
 */
        goto __pyx_L17;
      }

      /* "sQQideR/DatabaseContext.pyx":492
 *                     match_score = 4
 *                 # Match against description
 *                 elif 'description' in data and pattern.search(data['description']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Description contains '{keyword}'"
 */
      __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_description, __pyx_v_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 492, __pyx_L1_error)
      if (__pyx_t_16) {
      } else {
        __pyx_t_2 = __pyx_t_16;
        goto __pyx_L20_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_pattern, __pyx_n_s_search); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 492, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_v_data, __pyx_n_u_description); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 492, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 492, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 492, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __pyx_t_16;
      __pyx_L20_bool_binop_done:;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":493
 *                 # Match against description
 *                 elif 'description' in data and pattern.search(data['description']):
 *                     matched = True             # <<<<<<<<<<<<<<
 *                     match_reason = f"Description contains '{keyword}'"
 *                     match_score = 3
 */
        __pyx_v_matched = 1;

        /* "sQQideR/DatabaseContext.pyx":494
 *                 elif 'description' in data and pattern.search(data['description']):
 *                     matched = True
 *                     match_reason = f"Description contains '{keyword}'"             # <<<<<<<<<<<<<<
 *                     match_score = 3
 *                 # Match against keywords list
 */
        __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 494, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_14 = 0;
        __pyx_t_15 = 127;
        __Pyx_INCREF(__pyx_kp_u_Description_contains);
        __pyx_t_14 += 22;
        __Pyx_GIVEREF(__pyx_kp_u_Description_contains);
        PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_kp_u_Description_contains);
        __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 494, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_15;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_INCREF(__pyx_kp_u__23);
        __pyx_t_14 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__23);
        PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_kp_u__23);
        __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 494, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":495
 *                     matched = True
 *                     match_reason = f"Description contains '{keyword}'"
 *                     match_score = 3             # <<<<<<<<<<<<<<
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):
 */
        __Pyx_INCREF(__pyx_int_3);
        __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_3);

        /* "sQQideR/DatabaseContext.pyx":492
 *                     match_score = 4
 *                 # Match against description
 *                 elif 'description' in data and pattern.search(data['description']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Description contains '{keyword}'"
 */
        goto __pyx_L17;
      }

      /* "sQQideR/DatabaseContext.pyx":497
 *                     match_score = 3
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 */
      __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_keywords, __pyx_v_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
      if (__pyx_t_16) {
      } else {
        __pyx_t_2 = __pyx_t_16;
        goto __pyx_L22_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_data, __pyx_n_u_keywords); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_genexpr(((PyObject*)__pyx_cur_scope), __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_Generator_Next(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_2 = __pyx_t_16;
      __pyx_L22_bool_binop_done:;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":498
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):
 *                     matched = True             # <<<<<<<<<<<<<<
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 *                     match_score = 4
 */
        __pyx_v_matched = 1;

        /* "sQQideR/DatabaseContext.pyx":499
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"             # <<<<<<<<<<<<<<
 *                     match_score = 4
 *                 # Partial matches with lower scores
 */
        __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_14 = 0;
        __pyx_t_15 = 127;
        __Pyx_INCREF(__pyx_kp_u_Tagged_with_keyword);
        __pyx_t_14 += 21;
        __Pyx_GIVEREF(__pyx_kp_u_Tagged_with_keyword);
        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_kp_u_Tagged_with_keyword);
        __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) : __pyx_t_15;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7);
        __pyx_t_7 = 0;
        __Pyx_INCREF(__pyx_kp_u__23);
        __pyx_t_14 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__23);
        PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_kp_u__23);
        __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_6, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_7));
        __pyx_t_7 = 0;

        /* "sQQideR/DatabaseContext.pyx":500
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 *                     match_score = 4             # <<<<<<<<<<<<<<
 *                 # Partial matches with lower scores
 *                 elif not matched:
 */
        __Pyx_INCREF(__pyx_int_4);
        __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_4);

        /* "sQQideR/DatabaseContext.pyx":497
 *                     match_score = 3
 *                 # Match against keywords list
 *                 elif 'keywords' in data and any(pattern.search(k) for k in data['keywords']):             # <<<<<<<<<<<<<<
 *                     matched = True
 *                     match_reason = f"Tagged with keyword '{keyword}'"
 */
        goto __pyx_L17;
      }

      /* "sQQideR/DatabaseContext.pyx":502
 *                     match_score = 4
 *                 # Partial matches with lower scores
 *                 elif not matched:             # <<<<<<<<<<<<<<
 *                     if 'name' in data and keyword in data['name'].lower():
 *                         matched = True
 */
      __pyx_t_2 = (!__pyx_v_matched);
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":503
 *                 # Partial matches with lower scores
 *                 elif not matched:
 *                     if 'name' in data and keyword in data['name'].lower():             # <<<<<<<<<<<<<<
 *                         matched = True
 *                         match_reason = f"Name partially matches '{keyword}'"
 */
        __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_name, __pyx_v_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 503, __pyx_L1_error)
        if (__pyx_t_16) {
        } else {
          __pyx_t_2 = __pyx_t_16;
          goto __pyx_L25_bool_binop_done;
        }
        __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_data, __pyx_n_u_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_lower); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        __pyx_t_9 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_9 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
          __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_9, 0+__pyx_t_9);
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 503, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_v_keyword, __pyx_t_7, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_2 = __pyx_t_16;
        __pyx_L25_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":504
 *                 elif not matched:
 *                     if 'name' in data and keyword in data['name'].lower():
 *                         matched = True             # <<<<<<<<<<<<<<
 *                         match_reason = f"Name partially matches '{keyword}'"
 *                         match_score = 2
 */
          __pyx_v_matched = 1;

          /* "sQQideR/DatabaseContext.pyx":505
 *                     if 'name' in data and keyword in data['name'].lower():
 *                         matched = True
 *                         match_reason = f"Name partially matches '{keyword}'"             # <<<<<<<<<<<<<<
 *                         match_score = 2
 *                     elif 'description' in data and keyword in data['description'].lower():
 */
          __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_14 = 0;
          __pyx_t_15 = 127;
          __Pyx_INCREF(__pyx_kp_u_Name_partially_matches);
          __pyx_t_14 += 24;
          __Pyx_GIVEREF(__pyx_kp_u_Name_partially_matches);
          PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_kp_u_Name_partially_matches);
          __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_15;
          __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);
          __pyx_t_8 = 0;
          __Pyx_INCREF(__pyx_kp_u__23);
          __pyx_t_14 += 1;
          __Pyx_GIVEREF(__pyx_kp_u__23);
          PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_kp_u__23);
          __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_8));
          __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":506
 *                         matched = True
 *                         match_reason = f"Name partially matches '{keyword}'"
 *                         match_score = 2             # <<<<<<<<<<<<<<
 *                     elif 'description' in data and keyword in data['description'].lower():
 *                         matched = True
 */
          __Pyx_INCREF(__pyx_int_2);
          __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_2);

          /* "sQQideR/DatabaseContext.pyx":503
 *                 # Partial matches with lower scores
 *                 elif not matched:
 *                     if 'name' in data and keyword in data['name'].lower():             # <<<<<<<<<<<<<<
 *                         matched = True
 *                         match_reason = f"Name partially matches '{keyword}'"
 */
          goto __pyx_L24;
        }

        /* "sQQideR/DatabaseContext.pyx":507
 *                         match_reason = f"Name partially matches '{keyword}'"
 *                         match_score = 2
 *                     elif 'description' in data and keyword in data['description'].lower():             # <<<<<<<<<<<<<<
 *                         matched = True
 *                         match_reason = f"Description partially matches '{keyword}'"
 */
        __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_description, __pyx_v_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 507, __pyx_L1_error)
        if (__pyx_t_16) {
        } else {
          __pyx_t_2 = __pyx_t_16;
          goto __pyx_L27_bool_binop_done;
        }
        __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_v_data, __pyx_n_u_description); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 507, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_lower); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 507, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_9 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_9 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
          __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 0+__pyx_t_9);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 507, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_v_keyword, __pyx_t_8, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 507, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_2 = __pyx_t_16;
        __pyx_L27_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":508
 *                         match_score = 2
 *                     elif 'description' in data and keyword in data['description'].lower():
 *                         matched = True             # <<<<<<<<<<<<<<
 *                         match_reason = f"Description partially matches '{keyword}'"
 *                         match_score = 1
 */
          __pyx_v_matched = 1;

          /* "sQQideR/DatabaseContext.pyx":509
 *                     elif 'description' in data and keyword in data['description'].lower():
 *                         matched = True
 *                         match_reason = f"Description partially matches '{keyword}'"             # <<<<<<<<<<<<<<
 *                         match_score = 1
 * 
 */
          __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 509, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_14 = 0;
          __pyx_t_15 = 127;
          __Pyx_INCREF(__pyx_kp_u_Description_partially_matches);
          __pyx_t_14 += 31;
          __Pyx_GIVEREF(__pyx_kp_u_Description_partially_matches);
          PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_kp_u_Description_partially_matches);
          __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_keyword, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 509, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_15 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_15) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_15;
          __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
          __pyx_t_6 = 0;
          __Pyx_INCREF(__pyx_kp_u__23);
          __pyx_t_14 += 1;
          __Pyx_GIVEREF(__pyx_kp_u__23);
          PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_kp_u__23);
          __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_8, 3, __pyx_t_14, __pyx_t_15); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 509, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF_SET(__pyx_v_match_reason, ((PyObject*)__pyx_t_6));
          __pyx_t_6 = 0;

          /* "sQQideR/DatabaseContext.pyx":510
 *                         matched = True
 *                         match_reason = f"Description partially matches '{keyword}'"
 *                         match_score = 1             # <<<<<<<<<<<<<<
 * 
 *                 if matched:
 */
          __Pyx_INCREF(__pyx_int_1);
          __Pyx_DECREF_SET(__pyx_v_match_score, __pyx_int_1);

          /* "sQQideR/DatabaseContext.pyx":507
 *                         match_reason = f"Name partially matches '{keyword}'"
 *                         match_score = 2
 *                     elif 'description' in data and keyword in data['description'].lower():             # <<<<<<<<<<<<<<
 *                         matched = True
 *                         match_reason = f"Description partially matches '{keyword}'"
 */
        }
        __pyx_L24:;

        /* "sQQideR/DatabaseContext.pyx":502
 *                     match_score = 4
 *                 # Partial matches with lower scores
 *                 elif not matched:             # <<<<<<<<<<<<<<
 *                     if 'name' in data and keyword in data['name'].lower():
 *                         matched = True
 */
      }
      __pyx_L17:;

      /* "sQQideR/DatabaseContext.pyx":512
 *                         match_score = 1
 * 
 *                 if matched:             # <<<<<<<<<<<<<<
 *                     node_score += match_score
 *                     node_matches.append(match_reason)
 */
      if (__pyx_v_matched) {

        /* "sQQideR/DatabaseContext.pyx":513
 * 
 *                 if matched:
 *                     node_score += match_score             # <<<<<<<<<<<<<<
 *                     node_matches.append(match_reason)
 * 
 */
        __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_node_score, __pyx_v_match_score); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 513, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF_SET(__pyx_v_node_score, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":514
 *                 if matched:
 *                     node_score += match_score
 *                     node_matches.append(match_reason)             # <<<<<<<<<<<<<<
 * 
 *             if node_score > 0:
 */
        __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_node_matches, __pyx_v_match_reason); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 514, __pyx_L1_error)

        /* "sQQideR/DatabaseContext.pyx":512
 *                         match_score = 1
 * 
 *                 if matched:             # <<<<<<<<<<<<<<
 *                     node_score += match_score
 *                     node_matches.append(match_reason)
 */
      }

      /* "sQQideR/DatabaseContext.pyx":475
 * 
 *             # Check if any keyword matches node properties
 *             for keyword in normalized_keywords:             # <<<<<<<<<<<<<<
 *                 pattern = re.compile(r'\b' + re.escape(keyword) + r'\b', re.IGNORECASE)
 *                 matched = False
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":516
 *                     node_matches.append(match_reason)
 * 
 *             if node_score > 0:             # <<<<<<<<<<<<<<
 *                 node_scores[node_id] = node_score
 *                 matches[node_id] = node_matches
 */
    __pyx_t_5 = PyObject_RichCompare(__pyx_v_node_score, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 516, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":517
 * 
 *             if node_score > 0:
 *                 node_scores[node_id] = node_score             # <<<<<<<<<<<<<<
 *                 matches[node_id] = node_matches
 * 
 */
      if (unlikely((PyDict_SetItem(__pyx_v_node_scores, __pyx_v_node_id, __pyx_v_node_score) < 0))) __PYX_ERR(0, 517, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":518
 *             if node_score > 0:
 *                 node_scores[node_id] = node_score
 *                 matches[node_id] = node_matches             # <<<<<<<<<<<<<<
 * 
 *         # Sort nodes by score
 */
      if (unlikely((PyDict_SetItem(__pyx_v_matches, __pyx_v_node_id, __pyx_v_node_matches) < 0))) __PYX_ERR(0, 518, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":516
 *                     node_matches.append(match_reason)
 * 
 *             if node_score > 0:             # <<<<<<<<<<<<<<
 *                 node_scores[node_id] = node_score
 *                 matches[node_id] = node_matches
 */
    }

    /* "sQQideR/DatabaseContext.pyx":470
 *         node_scores = {}
 * 
 *         for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *             node_score = 0
 *             node_matches = []
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":521
 * 
 *         # Sort nodes by score
 *         sorted_nodes = sorted(node_scores.items(), key=lambda x: x[1], reverse=True)             # <<<<<<<<<<<<<<
 * 
 *         # Add nodes to results in order of score
 */
  __pyx_t_1 = __Pyx_PyDict_Items(__pyx_v_node_scores); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_3lambda1, 0, __pyx_n_s_DatabaseContextService_search_by_2, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_t_6) < 0) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_reverse, Py_True) < 0) __PYX_ERR(0, 521, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_sorted_nodes = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "sQQideR/DatabaseContext.pyx":524
 * 
 *         # Add nodes to results in order of score
 *         for node_id, score in sorted_nodes:             # <<<<<<<<<<<<<<
 *             results.append((node_id, self.graph.nodes[node_id]))
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_sorted_nodes)) || PyTuple_CheckExact(__pyx_v_sorted_nodes)) {
    __pyx_t_6 = __pyx_v_sorted_nodes; __Pyx_INCREF(__pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_sorted_nodes); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 524, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_6))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 524, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 524, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 524, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 524, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_10(__pyx_t_6);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 524, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 524, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 524, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 524, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 524, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_11 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
      index = 0; __pyx_t_5 = __pyx_t_11(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L35_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_8 = __pyx_t_11(__pyx_t_7); if (unlikely(!__pyx_t_8)) goto __pyx_L35_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_7), 2) < 0) __PYX_ERR(0, 524, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L36_unpacking_done;
      __pyx_L35_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 524, __pyx_L1_error)
      __pyx_L36_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_score, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":525
 *         # Add nodes to results in order of score
 *         for node_id, score in sorted_nodes:
 *             results.append((node_id, self.graph.nodes[node_id]))             # <<<<<<<<<<<<<<
 * 
 *         return results, matches
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_v_node_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(__pyx_v_node_id);
    __Pyx_GIVEREF(__pyx_v_node_id);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_node_id)) __PYX_ERR(0, 525, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_1)) __PYX_ERR(0, 525, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_results, __pyx_t_8); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":524
 * 
 *         # Add nodes to results in order of score
 *         for node_id, score in sorted_nodes:             # <<<<<<<<<<<<<<
 *             results.append((node_id, self.graph.nodes[node_id]))
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "sQQideR/DatabaseContext.pyx":527
 *             results.append((node_id, self.graph.nodes[node_id]))
 * 
 *         return results, matches             # <<<<<<<<<<<<<<
 * 
 *     def get_related_rules(self, node_id: str) -> List[Tuple[str, str]]:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_results);
  __Pyx_GIVEREF(__pyx_v_results);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_results)) __PYX_ERR(0, 527, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_matches);
  __Pyx_GIVEREF(__pyx_v_matches);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_matches)) __PYX_ERR(0, 527, __pyx_L1_error);
  __pyx_r = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.search_by_keywords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_matches);
  __Pyx_XDECREF(__pyx_v_normalized_keywords);
  __Pyx_XDECREF(__pyx_v_node_scores);
  __Pyx_XDECREF(__pyx_v_node_id);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_node_score);
  __Pyx_XDECREF(__pyx_v_node_matches);
  __Pyx_XDECREF(__pyx_v_keyword);
  __Pyx_XDECREF(__pyx_v_match_reason);
  __Pyx_XDECREF(__pyx_v_match_score);
  __Pyx_XDECREF(__pyx_v_sorted_nodes);
  __Pyx_XDECREF(__pyx_v_score);
  __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_k);
  __Pyx_XDECREF(__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_18search_by_keywords_2generator);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":529
 *         return results, matches
 * 
 *     def get_related_rules(self, node_id: str) -> List[Tuple[str, str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Get business rules related to a node.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_13get_related_rules(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_12get_related_rules, "\n        Get business rules related to a node.\n        \n        Args:\n            node_id: ID of the node to get rules for\n            \n        Returns:\n            List of tuples (rule_id, rule_description)\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_13get_related_rules = {"get_related_rules", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_13get_related_rules, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_12get_related_rules};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_13get_related_rules(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_node_id = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_related_rules (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_node_id,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 529, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_node_id)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 529, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("get_related_rules", 1, 2, 2, 1); __PYX_ERR(0, 529, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_related_rules") < 0)) __PYX_ERR(0, 529, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_node_id = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_related_rules", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 529, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.get_related_rules", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_node_id), (&PyUnicode_Type), 0, "node_id", 1))) __PYX_ERR(0, 529, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12get_related_rules(__pyx_self, __pyx_v_self, __pyx_v_node_id);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12get_related_rules(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_node_id) {
  PyObject *__pyx_v_rules = NULL;
  PyObject *__pyx_v_neighbor = NULL;
  PyObject *__pyx_v_rule_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_related_rules", 1);

  /* "sQQideR/DatabaseContext.pyx":539
 *             List of tuples (rule_id, rule_description)
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":540
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         rules = []
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 540, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":539
 *             List of tuples (rule_id, rule_description)
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":542
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         rules = []             # <<<<<<<<<<<<<<
 * 
 *         # Direct connections
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rules = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":545
 * 
 *         # Direct connections
 *         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *             if neighbor.startswith("rule:"):
 *                 rules.append((neighbor, self.graph.nodes[neighbor]["description"]))
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_neighbors); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_node_id};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 545, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 545, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 545, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 545, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 545, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_7(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 545, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_neighbor, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":546
 *         # Direct connections
 *         for neighbor in self.graph.neighbors(node_id):
 *             if neighbor.startswith("rule:"):             # <<<<<<<<<<<<<<
 *                 rules.append((neighbor, self.graph.nodes[neighbor]["description"]))
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_neighbor, __pyx_n_s_startswith); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u_rule};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":547
 *         for neighbor in self.graph.neighbors(node_id):
 *             if neighbor.startswith("rule:"):
 *                 rules.append((neighbor, self.graph.nodes[neighbor]["description"]))             # <<<<<<<<<<<<<<
 * 
 *         # For tables, also check rules on columns
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_neighbor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_description); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_v_neighbor);
      __Pyx_GIVEREF(__pyx_v_neighbor);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_neighbor)) __PYX_ERR(0, 547, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3)) __PYX_ERR(0, 547, __pyx_L1_error);
      __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_rules, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":546
 *         # Direct connections
 *         for neighbor in self.graph.neighbors(node_id):
 *             if neighbor.startswith("rule:"):             # <<<<<<<<<<<<<<
 *                 rules.append((neighbor, self.graph.nodes[neighbor]["description"]))
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":545
 * 
 *         # Direct connections
 *         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *             if neighbor.startswith("rule:"):
 *                 rules.append((neighbor, self.graph.nodes[neighbor]["description"]))
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":550
 * 
 *         # For tables, also check rules on columns
 *         if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):
 */
  __pyx_t_2 = __Pyx_PyUnicode_Tailmatch(__pyx_v_node_id, __pyx_kp_u_table, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 550, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":551
 *         # For tables, also check rules on columns
 *         if node_id.startswith("table:"):
 *             for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_neighbors); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_node_id};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_3 = __pyx_t_4; __Pyx_INCREF(__pyx_t_3);
      __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 551, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 551, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 551, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_3);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 551, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 551, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_7(__pyx_t_3);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 551, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_neighbor, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":552
 *         if node_id.startswith("table:"):
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):             # <<<<<<<<<<<<<<
 *                     for rule_node in self.graph.neighbors(neighbor):
 *                         if rule_node.startswith("rule:"):
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_neighbor, __pyx_n_s_startswith); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = NULL;
      __pyx_t_5 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_5 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u_column};
        __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 552, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":553
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):             # <<<<<<<<<<<<<<
 *                         if rule_node.startswith("rule:"):
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_neighbors); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        __pyx_t_5 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_5 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_neighbor};
          __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
          __pyx_t_8 = __pyx_t_4; __Pyx_INCREF(__pyx_t_8);
          __pyx_t_10 = 0;
          __pyx_t_11 = NULL;
        } else {
          __pyx_t_10 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 553, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_11 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 553, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        for (;;) {
          if (likely(!__pyx_t_11)) {
            if (likely(PyList_CheckExact(__pyx_t_8))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 553, __pyx_L1_error)
                #endif
                if (__pyx_t_10 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_10); __Pyx_INCREF(__pyx_t_4); __pyx_t_10++; if (unlikely((0 < 0))) __PYX_ERR(0, 553, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
                #if !CYTHON_ASSUME_SAFE_MACROS
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 553, __pyx_L1_error)
                #endif
                if (__pyx_t_10 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_10); __Pyx_INCREF(__pyx_t_4); __pyx_t_10++; if (unlikely((0 < 0))) __PYX_ERR(0, 553, __pyx_L1_error)
              #else
              __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
            }
          } else {
            __pyx_t_4 = __pyx_t_11(__pyx_t_8);
            if (unlikely(!__pyx_t_4)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 553, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_XDECREF_SET(__pyx_v_rule_node, __pyx_t_4);
          __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":554
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):
 *                         if rule_node.startswith("rule:"):             # <<<<<<<<<<<<<<
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))
 * 
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule_node, __pyx_n_s_startswith); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_12 = NULL;
          __pyx_t_5 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_12)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_12);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
              __pyx_t_5 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_kp_u_rule};
            __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 554, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          }
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 554, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":555
 *                     for rule_node in self.graph.neighbors(neighbor):
 *                         if rule_node.startswith("rule:"):
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))             # <<<<<<<<<<<<<<
 * 
 *         return rules
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_nodes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_rule_node); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_description); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_v_rule_node);
            __Pyx_GIVEREF(__pyx_v_rule_node);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_rule_node)) __PYX_ERR(0, 555, __pyx_L1_error);
            __Pyx_GIVEREF(__pyx_t_1);
            if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error);
            __pyx_t_1 = 0;
            __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_rules, __pyx_t_4); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 555, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "sQQideR/DatabaseContext.pyx":554
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):
 *                         if rule_node.startswith("rule:"):             # <<<<<<<<<<<<<<
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))
 * 
 */
          }

          /* "sQQideR/DatabaseContext.pyx":553
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):             # <<<<<<<<<<<<<<
 *                         if rule_node.startswith("rule:"):
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))
 */
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "sQQideR/DatabaseContext.pyx":552
 *         if node_id.startswith("table:"):
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):             # <<<<<<<<<<<<<<
 *                     for rule_node in self.graph.neighbors(neighbor):
 *                         if rule_node.startswith("rule:"):
 */
      }

      /* "sQQideR/DatabaseContext.pyx":551
 *         # For tables, also check rules on columns
 *         if node_id.startswith("table:"):
 *             for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                 if neighbor.startswith("column:"):
 *                     for rule_node in self.graph.neighbors(neighbor):
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":550
 * 
 *         # For tables, also check rules on columns
 *         if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *             for neighbor in self.graph.neighbors(node_id):
 *                 if neighbor.startswith("column:"):
 */
  }

  /* "sQQideR/DatabaseContext.pyx":557
 *                             rules.append((rule_node, self.graph.nodes[rule_node]["description"]))
 * 
 *         return rules             # <<<<<<<<<<<<<<
 * 
 *     def find_path(self, start_node: str, end_node: str) -> Optional[List[str]]:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rules);
  __pyx_r = __pyx_v_rules;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":529
 *         return results, matches
 * 
 *     def get_related_rules(self, node_id: str) -> List[Tuple[str, str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Get business rules related to a node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.get_related_rules", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rules);
  __Pyx_XDECREF(__pyx_v_neighbor);
  __Pyx_XDECREF(__pyx_v_rule_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":559
 *         return rules
 * 
 *     def find_path(self, start_node: str, end_node: str) -> Optional[List[str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Find a path between two nodes in the graph.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_15find_path(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_14find_path, "\n        Find a path between two nodes in the graph.\n        \n        Args:\n            start_node: Starting node ID\n            end_node: Ending node ID\n            \n        Returns:\n            List of node IDs forming the path, or None if no path exists\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_15find_path = {"find_path", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_15find_path, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_14find_path};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_15find_path(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_start_node = 0;
  PyObject *__pyx_v_end_node = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_path (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_start_node,&__pyx_n_s_end_node,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 559, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_start_node)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 559, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("find_path", 1, 3, 3, 1); __PYX_ERR(0, 559, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_end_node)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 559, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("find_path", 1, 3, 3, 2); __PYX_ERR(0, 559, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "find_path") < 0)) __PYX_ERR(0, 559, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_start_node = ((PyObject*)values[1]);
    __pyx_v_end_node = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_path", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 559, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.find_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start_node), (&PyUnicode_Type), 0, "start_node", 1))) __PYX_ERR(0, 559, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_end_node), (&PyUnicode_Type), 0, "end_node", 1))) __PYX_ERR(0, 559, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_14find_path(__pyx_self, __pyx_v_self, __pyx_v_start_node, __pyx_v_end_node);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_14find_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_start_node, PyObject *__pyx_v_end_node) {
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  unsigned int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_path", 1);

  /* "sQQideR/DatabaseContext.pyx":570
 *             List of node IDs forming the path, or None if no path exists
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 570, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":571
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 571, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":570
 *             List of node IDs forming the path, or None if no path exists
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":573
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             path = nx.shortest_path(self.graph, start_node, end_node)
 *             return path
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "sQQideR/DatabaseContext.pyx":574
 * 
 *         try:
 *             path = nx.shortest_path(self.graph, start_node, end_node)             # <<<<<<<<<<<<<<
 *             return path
 *         except nx.NetworkXNoPath:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_nx); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 574, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_shortest_path); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 574, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 574, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[4] = {__pyx_t_8, __pyx_t_6, __pyx_v_start_node, __pyx_v_end_node};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 3+__pyx_t_9);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __pyx_v_path = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":575
 *         try:
 *             path = nx.shortest_path(self.graph, start_node, end_node)
 *             return path             # <<<<<<<<<<<<<<
 *         except nx.NetworkXNoPath:
 *             return None
 */
      __Pyx_XDECREF(__pyx_r);
      if (!(likely(PyList_CheckExact(__pyx_v_path))||((__pyx_v_path) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_v_path))) __PYX_ERR(0, 575, __pyx_L4_error)
      __Pyx_INCREF(__pyx_v_path);
      __pyx_r = ((PyObject*)__pyx_v_path);
      goto __pyx_L8_try_return;

      /* "sQQideR/DatabaseContext.pyx":573
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             path = nx.shortest_path(self.graph, start_node, end_node)
 *             return path
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":576
 *             path = nx.shortest_path(self.graph, start_node, end_node)
 *             return path
 *         except nx.NetworkXNoPath:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_7, &__pyx_t_6);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_nx); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 576, __pyx_L6_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_NetworkXNoPath); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 576, __pyx_L6_except_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_7, __pyx_t_6);
    __pyx_t_1 = 0; __pyx_t_7 = 0; __pyx_t_6 = 0;
    if (__pyx_t_11) {
      __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.find_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_1) < 0) __PYX_ERR(0, 576, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_1);

      /* "sQQideR/DatabaseContext.pyx":577
 *             return path
 *         except nx.NetworkXNoPath:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L7_except_return;
    }
    goto __pyx_L6_except_error;

    /* "sQQideR/DatabaseContext.pyx":573
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             path = nx.shortest_path(self.graph, start_node, end_node)
 *             return path
 */
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
  }

  /* "sQQideR/DatabaseContext.pyx":559
 *         return rules
 * 
 *     def find_path(self, start_node: str, end_node: str) -> Optional[List[str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Find a path between two nodes in the graph.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.find_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_17generate_sql(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_16generate_sql, "\n        Generate an improved SQL query based on selected tables and conditions.\n        \n        Args:\n            table_nodes: List of table node IDs.\n            condition_nodes: Optional list of condition node IDs (typically columns).\n            \n        Returns:\n            Generated SQL query string.\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_17generate_sql = {"generate_sql", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_17generate_sql, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_16generate_sql};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_17generate_sql(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_table_nodes = 0;
  PyObject *__pyx_v_condition_nodes = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate_sql (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_table_nodes,&__pyx_n_s_condition_nodes,0};
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 579, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_table_nodes)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 579, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_sql", 0, 2, 3, 1); __PYX_ERR(0, 579, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_condition_nodes);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 579, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "generate_sql") < 0)) __PYX_ERR(0, 579, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_table_nodes = ((PyObject*)values[1]);
    __pyx_v_condition_nodes = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("generate_sql", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 579, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_table_nodes), (&PyList_Type), 0, "table_nodes", 1))) __PYX_ERR(0, 579, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_condition_nodes), (&PyList_Type), 1, "condition_nodes", 1))) __PYX_ERR(0, 579, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_16generate_sql(__pyx_self, __pyx_v_self, __pyx_v_table_nodes, __pyx_v_condition_nodes);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":601
 *         used_aliases = set()
 * 
 *         def get_alias(name):             # <<<<<<<<<<<<<<
 *             base = name[0].lower()
 *             alias = base
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_1get_alias(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_1get_alias = {"get_alias", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_1get_alias, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_1get_alias(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_alias (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_name)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_alias") < 0)) __PYX_ERR(0, 601, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_alias", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 601, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql.get_alias", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_get_alias(__pyx_self, __pyx_v_name);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_get_alias(PyObject *__pyx_self, PyObject *__pyx_v_name) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *__pyx_cur_scope;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *__pyx_outer_scope;
  PyObject *__pyx_v_base = NULL;
  PyObject *__pyx_v_alias = NULL;
  PyObject *__pyx_v_suffix = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_alias", 1);
  __pyx_outer_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "sQQideR/DatabaseContext.pyx":602
 * 
 *         def get_alias(name):
 *             base = name[0].lower()             # <<<<<<<<<<<<<<
 *             alias = base
 *             suffix = 1
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_name, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v_base = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":603
 *         def get_alias(name):
 *             base = name[0].lower()
 *             alias = base             # <<<<<<<<<<<<<<
 *             suffix = 1
 *             while alias in used_aliases:
 */
  __Pyx_INCREF(__pyx_v_base);
  __pyx_v_alias = __pyx_v_base;

  /* "sQQideR/DatabaseContext.pyx":604
 *             base = name[0].lower()
 *             alias = base
 *             suffix = 1             # <<<<<<<<<<<<<<
 *             while alias in used_aliases:
 *                 alias = f"{base}{suffix}"
 */
  __Pyx_INCREF(__pyx_int_1);
  __pyx_v_suffix = __pyx_int_1;

  /* "sQQideR/DatabaseContext.pyx":605
 *             alias = base
 *             suffix = 1
 *             while alias in used_aliases:             # <<<<<<<<<<<<<<
 *                 alias = f"{base}{suffix}"
 *                 suffix += 1
 */
  while (1) {
    if (unlikely(!__pyx_cur_scope->__pyx_v_used_aliases)) { __Pyx_RaiseClosureNameError("used_aliases"); __PYX_ERR(0, 605, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_used_aliases == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 605, __pyx_L1_error)
    }
    __pyx_t_5 = (__Pyx_PySet_ContainsTF(__pyx_v_alias, __pyx_cur_scope->__pyx_v_used_aliases, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 605, __pyx_L1_error)
    if (!__pyx_t_5) break;

    /* "sQQideR/DatabaseContext.pyx":606
 *             suffix = 1
 *             while alias in used_aliases:
 *                 alias = f"{base}{suffix}"             # <<<<<<<<<<<<<<
 *                 suffix += 1
 *             used_aliases.add(alias)
 */
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_base, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_suffix, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_alias, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "sQQideR/DatabaseContext.pyx":607
 *             while alias in used_aliases:
 *                 alias = f"{base}{suffix}"
 *                 suffix += 1             # <<<<<<<<<<<<<<
 *             used_aliases.add(alias)
 *             return alias
 */
    __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_suffix, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_suffix, __pyx_t_2);
    __pyx_t_2 = 0;
  }

  /* "sQQideR/DatabaseContext.pyx":608
 *                 alias = f"{base}{suffix}"
 *                 suffix += 1
 *             used_aliases.add(alias)             # <<<<<<<<<<<<<<
 *             return alias
 * 
 */
  if (unlikely(!__pyx_cur_scope->__pyx_v_used_aliases)) { __Pyx_RaiseClosureNameError("used_aliases"); __PYX_ERR(0, 608, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_used_aliases == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
    __PYX_ERR(0, 608, __pyx_L1_error)
  }
  __pyx_t_6 = PySet_Add(__pyx_cur_scope->__pyx_v_used_aliases, __pyx_v_alias); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 608, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":609
 *                 suffix += 1
 *             used_aliases.add(alias)
 *             return alias             # <<<<<<<<<<<<<<
 * 
 *         for t in table_nodes:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_alias);
  __pyx_r = __pyx_v_alias;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":601
 *         used_aliases = set()
 * 
 *         def get_alias(name):             # <<<<<<<<<<<<<<
 *             base = name[0].lower()
 *             alias = base
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql.get_alias", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_base);
  __Pyx_XDECREF(__pyx_v_alias);
  __Pyx_XDECREF(__pyx_v_suffix);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_16generate_sql(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_table_nodes, PyObject *__pyx_v_condition_nodes) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *__pyx_cur_scope;
  PyObject *__pyx_v_from_tables = NULL;
  PyObject *__pyx_v_aliases = NULL;
  PyObject *__pyx_v_get_alias = 0;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_join_clauses = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_current_table = NULL;
  int __pyx_v_join_found;
  PyObject *__pyx_v_prev = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_prev_table = NULL;
  PyObject *__pyx_v_edge_data = NULL;
  PyObject *__pyx_v_join_condition = NULL;
  PyObject *__pyx_v_tbl = NULL;
  PyObject *__pyx_v_alias = NULL;
  PyObject *__pyx_v_where_clauses = NULL;
  PyObject *__pyx_v_node = NULL;
  PyObject *__pyx_v_node_data = NULL;
  PyObject *__pyx_v_col_name = NULL;
  PyObject *__pyx_v_sql_lines = NULL;
  PyObject *__pyx_v_select_clause = NULL;
  PyObject *__pyx_v_first_table = NULL;
  PyObject *__pyx_v_sql = NULL;
  PyObject *__pyx_8genexpr4__pyx_v_alias = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  unsigned int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  Py_UCS4 __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("generate_sql", 0);
  __pyx_cur_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 579, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }

  /* "sQQideR/DatabaseContext.pyx":590
 *             Generated SQL query string.
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":591
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         if not table_nodes:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 591, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":590
 *             Generated SQL query string.
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":593
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         if not table_nodes:             # <<<<<<<<<<<<<<
 *             return "No tables selected for query"
 * 
 */
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_table_nodes) != 0);
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "sQQideR/DatabaseContext.pyx":594
 * 
 *         if not table_nodes:
 *             return "No tables selected for query"             # <<<<<<<<<<<<<<
 * 
 *         # Build alias mapping ensuring unique aliases
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_No_tables_selected_for_query);
    __pyx_r = __pyx_kp_u_No_tables_selected_for_query;
    goto __pyx_L0;

    /* "sQQideR/DatabaseContext.pyx":593
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 *         if not table_nodes:             # <<<<<<<<<<<<<<
 *             return "No tables selected for query"
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":597
 * 
 *         # Build alias mapping ensuring unique aliases
 *         from_tables = []             # <<<<<<<<<<<<<<
 *         aliases = {}
 *         used_aliases = set()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_from_tables = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":598
 *         # Build alias mapping ensuring unique aliases
 *         from_tables = []
 *         aliases = {}             # <<<<<<<<<<<<<<
 *         used_aliases = set()
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_aliases = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":599
 *         from_tables = []
 *         aliases = {}
 *         used_aliases = set()             # <<<<<<<<<<<<<<
 * 
 *         def get_alias(name):
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_used_aliases = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":601
 *         used_aliases = set()
 * 
 *         def get_alias(name):             # <<<<<<<<<<<<<<
 *             base = name[0].lower()
 *             alias = base
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_1get_alias, 0, __pyx_n_s_DatabaseContextService_generate, ((PyObject*)__pyx_cur_scope), __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__25)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_get_alias = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":611
 *             return alias
 * 
 *         for t in table_nodes:             # <<<<<<<<<<<<<<
 *             table_name = t.replace("table:", "")
 *             from_tables.append(table_name)
 */
  __pyx_t_1 = __pyx_v_table_nodes; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 611, __pyx_L1_error)
      #endif
      if (__pyx_t_4 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_5); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 611, __pyx_L1_error)
    #else
    __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":612
 * 
 *         for t in table_nodes:
 *             table_name = t.replace("table:", "")             # <<<<<<<<<<<<<<
 *             from_tables.append(table_name)
 *             aliases[table_name] = get_alias(table_name)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_t, __pyx_n_s_replace); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 612, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 612, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "sQQideR/DatabaseContext.pyx":613
 *         for t in table_nodes:
 *             table_name = t.replace("table:", "")
 *             from_tables.append(table_name)             # <<<<<<<<<<<<<<
 *             aliases[table_name] = get_alias(table_name)
 * 
 */
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_from_tables, __pyx_v_table_name); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 613, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":614
 *             table_name = t.replace("table:", "")
 *             from_tables.append(table_name)
 *             aliases[table_name] = get_alias(table_name)             # <<<<<<<<<<<<<<
 * 
 *         join_clauses = []
 */
    __pyx_t_6 = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_12generate_sql_get_alias(__pyx_v_get_alias, __pyx_v_table_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely((PyDict_SetItem(__pyx_v_aliases, __pyx_v_table_name, __pyx_t_6) < 0))) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "sQQideR/DatabaseContext.pyx":611
 *             return alias
 * 
 *         for t in table_nodes:             # <<<<<<<<<<<<<<
 *             table_name = t.replace("table:", "")
 *             from_tables.append(table_name)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":616
 *             aliases[table_name] = get_alias(table_name)
 * 
 *         join_clauses = []             # <<<<<<<<<<<<<<
 * 
 *         # For additional tables, try to find join conditions with any previously added table
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_join_clauses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":619
 * 
 *         # For additional tables, try to find join conditions with any previously added table
 *         for i in range(1, len(table_nodes)):             # <<<<<<<<<<<<<<
 *             current_table = table_nodes[i].replace("table:", "")
 *             join_found = False
 */
  __pyx_t_4 = __Pyx_PyList_GET_SIZE(__pyx_v_table_nodes); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 619, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_4;
  for (__pyx_t_9 = 1; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "sQQideR/DatabaseContext.pyx":620
 *         # For additional tables, try to find join conditions with any previously added table
 *         for i in range(1, len(table_nodes)):
 *             current_table = table_nodes[i].replace("table:", "")             # <<<<<<<<<<<<<<
 *             join_found = False
 *             for prev in table_nodes[:i]:
 */
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_table_nodes, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_replace); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_current_table, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":621
 *         for i in range(1, len(table_nodes)):
 *             current_table = table_nodes[i].replace("table:", "")
 *             join_found = False             # <<<<<<<<<<<<<<
 *             for prev in table_nodes[:i]:
 *                 prev_table = prev.replace("table:", "")
 */
    __pyx_v_join_found = 0;

    /* "sQQideR/DatabaseContext.pyx":622
 *             current_table = table_nodes[i].replace("table:", "")
 *             join_found = False
 *             for prev in table_nodes[:i]:             # <<<<<<<<<<<<<<
 *                 prev_table = prev.replace("table:", "")
 *                 # Check for a direct relationship between the tables
 */
    __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_table_nodes, 0, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __pyx_t_1; __Pyx_INCREF(__pyx_t_6);
    __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 622, __pyx_L1_error)
        #endif
        if (__pyx_t_10 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_10); __Pyx_INCREF(__pyx_t_1); __pyx_t_10++; if (unlikely((0 < 0))) __PYX_ERR(0, 622, __pyx_L1_error)
      #else
      __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_prev, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":623
 *             join_found = False
 *             for prev in table_nodes[:i]:
 *                 prev_table = prev.replace("table:", "")             # <<<<<<<<<<<<<<
 *                 # Check for a direct relationship between the tables
 *                 edge_data = self.graph.get_edge_data(prev, table_nodes[i])
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_prev, __pyx_n_s_replace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 623, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_prev_table, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":625
 *                 prev_table = prev.replace("table:", "")
 *                 # Check for a direct relationship between the tables
 *                 edge_data = self.graph.get_edge_data(prev, table_nodes[i])             # <<<<<<<<<<<<<<
 *                 if edge_data and 'join_condition' in edge_data:
 *                     join_condition = edge_data['join_condition']
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_get_edge_data); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 625, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_table_nodes, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_12 = NULL;
      __pyx_t_13 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
          __pyx_t_13 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_12, __pyx_v_prev, __pyx_t_1};
        __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_XDECREF_SET(__pyx_v_edge_data, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":626
 *                 # Check for a direct relationship between the tables
 *                 edge_data = self.graph.get_edge_data(prev, table_nodes[i])
 *                 if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                     join_condition = edge_data['join_condition']
 *                     # Replace table names with aliases
 */
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_edge_data); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 626, __pyx_L1_error)
      if (__pyx_t_2) {
      } else {
        __pyx_t_3 = __pyx_t_2;
        goto __pyx_L13_bool_binop_done;
      }
      __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_join_condition, __pyx_v_edge_data, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 626, __pyx_L1_error)
      __pyx_t_3 = __pyx_t_2;
      __pyx_L13_bool_binop_done:;
      if (__pyx_t_3) {

        /* "sQQideR/DatabaseContext.pyx":627
 *                 edge_data = self.graph.get_edge_data(prev, table_nodes[i])
 *                 if edge_data and 'join_condition' in edge_data:
 *                     join_condition = edge_data['join_condition']             # <<<<<<<<<<<<<<
 *                     # Replace table names with aliases
 *                     for tbl, alias in aliases.items():
 */
        __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_edge_data, __pyx_n_u_join_condition); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 627, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_XDECREF_SET(__pyx_v_join_condition, __pyx_t_5);
        __pyx_t_5 = 0;

        /* "sQQideR/DatabaseContext.pyx":629
 *                     join_condition = edge_data['join_condition']
 *                     # Replace table names with aliases
 *                     for tbl, alias in aliases.items():             # <<<<<<<<<<<<<<
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 */
        __pyx_t_14 = 0;
        __pyx_t_11 = __Pyx_dict_iterator(__pyx_v_aliases, 1, __pyx_n_s_items, (&__pyx_t_15), (&__pyx_t_16)); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 629, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF(__pyx_t_5);
        __pyx_t_5 = __pyx_t_11;
        __pyx_t_11 = 0;
        while (1) {
          __pyx_t_17 = __Pyx_dict_iter_next(__pyx_t_5, __pyx_t_15, &__pyx_t_14, &__pyx_t_11, &__pyx_t_1, NULL, __pyx_t_16);
          if (unlikely(__pyx_t_17 == 0)) break;
          if (unlikely(__pyx_t_17 == -1)) __PYX_ERR(0, 629, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_tbl, __pyx_t_11);
          __pyx_t_11 = 0;
          __Pyx_XDECREF_SET(__pyx_v_alias, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "sQQideR/DatabaseContext.pyx":630
 *                     # Replace table names with aliases
 *                     for tbl, alias in aliases.items():
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")             # <<<<<<<<<<<<<<
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True
 */
          __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_join_condition, __pyx_n_s_replace); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 630, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_FormatSimple(__pyx_v_tbl, __pyx_empty_unicode); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 630, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_18 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_12, __pyx_kp_u__15); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 630, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_12 = __Pyx_PyObject_FormatSimple(__pyx_v_alias, __pyx_empty_unicode); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 630, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_19 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_12, __pyx_kp_u__15); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 630, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_19);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_12 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_11))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
            if (likely(__pyx_t_12)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
              __Pyx_INCREF(__pyx_t_12);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_11, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_12, __pyx_t_18, __pyx_t_19};
            __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          }
          __Pyx_DECREF_SET(__pyx_v_join_condition, __pyx_t_1);
          __pyx_t_1 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "sQQideR/DatabaseContext.pyx":631
 *                     for tbl, alias in aliases.items():
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")             # <<<<<<<<<<<<<<
 *                     join_found = True
 *                     break
 */
        __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_15 = 0;
        __pyx_t_20 = 127;
        __Pyx_INCREF(__pyx_kp_u_JOIN);
        __pyx_t_15 += 5;
        __Pyx_GIVEREF(__pyx_kp_u_JOIN);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_JOIN);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_current_table, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_20;
        __pyx_t_15 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u__27);
        __pyx_t_15 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__27);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u__27);
        __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_aliases, __pyx_v_current_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_11 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) : __pyx_t_20;
        __pyx_t_15 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_11);
        __pyx_t_11 = 0;
        __Pyx_INCREF(__pyx_kp_u_ON);
        __pyx_t_15 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_ON);
        PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_kp_u_ON);
        __pyx_t_11 = __Pyx_PyObject_FormatSimple(__pyx_v_join_condition, __pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) : __pyx_t_20;
        __pyx_t_15 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_11);
        __pyx_t_11 = 0;
        __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_5, 6, __pyx_t_15, __pyx_t_20); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_join_clauses, __pyx_t_11); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 631, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "sQQideR/DatabaseContext.pyx":632
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True             # <<<<<<<<<<<<<<
 *                     break
 *                 # Check in the reverse direction
 */
        __pyx_v_join_found = 1;

        /* "sQQideR/DatabaseContext.pyx":633
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True
 *                     break             # <<<<<<<<<<<<<<
 *                 # Check in the reverse direction
 *                 edge_data = self.graph.get_edge_data(table_nodes[i], prev)
 */
        goto __pyx_L11_break;

        /* "sQQideR/DatabaseContext.pyx":626
 *                 # Check for a direct relationship between the tables
 *                 edge_data = self.graph.get_edge_data(prev, table_nodes[i])
 *                 if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                     join_condition = edge_data['join_condition']
 *                     # Replace table names with aliases
 */
      }

      /* "sQQideR/DatabaseContext.pyx":635
 *                     break
 *                 # Check in the reverse direction
 *                 edge_data = self.graph.get_edge_data(table_nodes[i], prev)             # <<<<<<<<<<<<<<
 *                 if edge_data and 'join_condition' in edge_data:
 *                     join_condition = edge_data['join_condition']
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 635, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_get_edge_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 635, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_table_nodes, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 635, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_19 = NULL;
      __pyx_t_13 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_19)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_19);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_13 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_19, __pyx_t_5, __pyx_v_prev};
        __pyx_t_11 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 635, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF_SET(__pyx_v_edge_data, __pyx_t_11);
      __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":636
 *                 # Check in the reverse direction
 *                 edge_data = self.graph.get_edge_data(table_nodes[i], prev)
 *                 if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                     join_condition = edge_data['join_condition']
 *                     for tbl, alias in aliases.items():
 */
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_edge_data); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 636, __pyx_L1_error)
      if (__pyx_t_2) {
      } else {
        __pyx_t_3 = __pyx_t_2;
        goto __pyx_L18_bool_binop_done;
      }
      __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_join_condition, __pyx_v_edge_data, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 636, __pyx_L1_error)
      __pyx_t_3 = __pyx_t_2;
      __pyx_L18_bool_binop_done:;
      if (__pyx_t_3) {

        /* "sQQideR/DatabaseContext.pyx":637
 *                 edge_data = self.graph.get_edge_data(table_nodes[i], prev)
 *                 if edge_data and 'join_condition' in edge_data:
 *                     join_condition = edge_data['join_condition']             # <<<<<<<<<<<<<<
 *                     for tbl, alias in aliases.items():
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 */
        __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_v_edge_data, __pyx_n_u_join_condition); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 637, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_join_condition, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "sQQideR/DatabaseContext.pyx":638
 *                 if edge_data and 'join_condition' in edge_data:
 *                     join_condition = edge_data['join_condition']
 *                     for tbl, alias in aliases.items():             # <<<<<<<<<<<<<<
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 */
        __pyx_t_15 = 0;
        __pyx_t_1 = __Pyx_dict_iterator(__pyx_v_aliases, 1, __pyx_n_s_items, (&__pyx_t_14), (&__pyx_t_16)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 638, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XDECREF(__pyx_t_11);
        __pyx_t_11 = __pyx_t_1;
        __pyx_t_1 = 0;
        while (1) {
          __pyx_t_17 = __Pyx_dict_iter_next(__pyx_t_11, __pyx_t_14, &__pyx_t_15, &__pyx_t_1, &__pyx_t_5, NULL, __pyx_t_16);
          if (unlikely(__pyx_t_17 == 0)) break;
          if (unlikely(__pyx_t_17 == -1)) __PYX_ERR(0, 638, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_XDECREF_SET(__pyx_v_tbl, __pyx_t_1);
          __pyx_t_1 = 0;
          __Pyx_XDECREF_SET(__pyx_v_alias, __pyx_t_5);
          __pyx_t_5 = 0;

          /* "sQQideR/DatabaseContext.pyx":639
 *                     join_condition = edge_data['join_condition']
 *                     for tbl, alias in aliases.items():
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")             # <<<<<<<<<<<<<<
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_join_condition, __pyx_n_s_replace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_19 = __Pyx_PyObject_FormatSimple(__pyx_v_tbl, __pyx_empty_unicode); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_18 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_19, __pyx_kp_u__15); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
          __pyx_t_19 = __Pyx_PyObject_FormatSimple(__pyx_v_alias, __pyx_empty_unicode); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_12 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_19, __pyx_kp_u__15); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
          __pyx_t_19 = NULL;
          __pyx_t_13 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_19)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_19);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
              __pyx_t_13 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_19, __pyx_t_18, __pyx_t_12};
            __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
            __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 639, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          }
          __Pyx_DECREF_SET(__pyx_v_join_condition, __pyx_t_5);
          __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "sQQideR/DatabaseContext.pyx":640
 *                     for tbl, alias in aliases.items():
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")             # <<<<<<<<<<<<<<
 *                     join_found = True
 *                     break
 */
        __pyx_t_11 = PyTuple_New(6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_14 = 0;
        __pyx_t_20 = 127;
        __Pyx_INCREF(__pyx_kp_u_JOIN);
        __pyx_t_14 += 5;
        __Pyx_GIVEREF(__pyx_kp_u_JOIN);
        PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_kp_u_JOIN);
        __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_current_table, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_20;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_5);
        __pyx_t_5 = 0;
        __Pyx_INCREF(__pyx_kp_u__27);
        __pyx_t_14 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__27);
        PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_kp_u__27);
        __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_aliases, __pyx_v_current_table); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_20;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u_ON);
        __pyx_t_14 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_ON);
        PyTuple_SET_ITEM(__pyx_t_11, 4, __pyx_kp_u_ON);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_join_condition, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_20;
        __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_11, 5, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_11, 6, __pyx_t_14, __pyx_t_20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_join_clauses, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 640, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":641
 *                         join_condition = join_condition.replace(f"{tbl}.", f"{alias}.")
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True             # <<<<<<<<<<<<<<
 *                     break
 *             if not join_found:
 */
        __pyx_v_join_found = 1;

        /* "sQQideR/DatabaseContext.pyx":642
 *                     join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON {join_condition}")
 *                     join_found = True
 *                     break             # <<<<<<<<<<<<<<
 *             if not join_found:
 *                 join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON /[inserted by cython to avoid comment start]* Missing join condition - verify relationship *[inserted by cython to avoid comment closer]/")
 */
        goto __pyx_L11_break;

        /* "sQQideR/DatabaseContext.pyx":636
 *                 # Check in the reverse direction
 *                 edge_data = self.graph.get_edge_data(table_nodes[i], prev)
 *                 if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                     join_condition = edge_data['join_condition']
 *                     for tbl, alias in aliases.items():
 */
      }

      /* "sQQideR/DatabaseContext.pyx":622
 *             current_table = table_nodes[i].replace("table:", "")
 *             join_found = False
 *             for prev in table_nodes[:i]:             # <<<<<<<<<<<<<<
 *                 prev_table = prev.replace("table:", "")
 *                 # Check for a direct relationship between the tables
 */
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L22_for_end;
    __pyx_L11_break:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L22_for_end;
    __pyx_L22_for_end:;

    /* "sQQideR/DatabaseContext.pyx":643
 *                     join_found = True
 *                     break
 *             if not join_found:             # <<<<<<<<<<<<<<
 *                 join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON /[inserted by cython to avoid comment start]* Missing join condition - verify relationship *[inserted by cython to avoid comment closer]/")
 * 
 */
    __pyx_t_3 = (!__pyx_v_join_found);
    if (__pyx_t_3) {

      /* "sQQideR/DatabaseContext.pyx":644
 *                     break
 *             if not join_found:
 *                 join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON /[inserted by cython to avoid comment start]* Missing join condition - verify relationship *[inserted by cython to avoid comment closer]/")             # <<<<<<<<<<<<<<
 * 
 *         # Add conditions from condition nodes
 */
      __pyx_t_6 = PyTuple_New(5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_10 = 0;
      __pyx_t_20 = 127;
      __Pyx_INCREF(__pyx_kp_u_JOIN);
      __pyx_t_10 += 5;
      __Pyx_GIVEREF(__pyx_kp_u_JOIN);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_kp_u_JOIN);
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_current_table, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_20;
      __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_INCREF(__pyx_kp_u__27);
      __pyx_t_10 += 1;
      __Pyx_GIVEREF(__pyx_kp_u__27);
      PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_kp_u__27);
      __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_aliases, __pyx_v_current_table); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_11 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) : __pyx_t_20;
      __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11);
      __Pyx_GIVEREF(__pyx_t_11);
      PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_kp_u_ON_Missing_join_condition_verif);
      __pyx_t_10 += 54;
      __Pyx_GIVEREF(__pyx_kp_u_ON_Missing_join_condition_verif);
      PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_kp_u_ON_Missing_join_condition_verif);
      __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_6, 5, __pyx_t_10, __pyx_t_20); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_join_clauses, __pyx_t_11); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 644, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "sQQideR/DatabaseContext.pyx":643
 *                     join_found = True
 *                     break
 *             if not join_found:             # <<<<<<<<<<<<<<
 *                 join_clauses.append(f"JOIN {current_table} {aliases[current_table]} ON /[inserted by cython to avoid comment start]* Missing join condition - verify relationship *[inserted by cython to avoid comment closer]/")
 * 
 */
    }
  }

  /* "sQQideR/DatabaseContext.pyx":647
 * 
 *         # Add conditions from condition nodes
 *         where_clauses = []             # <<<<<<<<<<<<<<
 *         if condition_nodes:
 *             for node in condition_nodes:
 */
  __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_where_clauses = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "sQQideR/DatabaseContext.pyx":648
 *         # Add conditions from condition nodes
 *         where_clauses = []
 *         if condition_nodes:             # <<<<<<<<<<<<<<
 *             for node in condition_nodes:
 *                 node_data = self.graph.nodes[node]
 */
  __pyx_t_3 = (__pyx_v_condition_nodes != Py_None)&&(PyList_GET_SIZE(__pyx_v_condition_nodes) != 0);
  if (__pyx_t_3) {

    /* "sQQideR/DatabaseContext.pyx":649
 *         where_clauses = []
 *         if condition_nodes:
 *             for node in condition_nodes:             # <<<<<<<<<<<<<<
 *                 node_data = self.graph.nodes[node]
 *                 if node_data['type'] == 'column':
 */
    if (unlikely(__pyx_v_condition_nodes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 649, __pyx_L1_error)
    }
    __pyx_t_11 = __pyx_v_condition_nodes; __Pyx_INCREF(__pyx_t_11);
    __pyx_t_4 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_11);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 649, __pyx_L1_error)
        #endif
        if (__pyx_t_4 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_4); __Pyx_INCREF(__pyx_t_6); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 649, __pyx_L1_error)
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_11, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_node, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "sQQideR/DatabaseContext.pyx":650
 *         if condition_nodes:
 *             for node in condition_nodes:
 *                 node_data = self.graph.nodes[node]             # <<<<<<<<<<<<<<
 *                 if node_data['type'] == 'column':
 *                     col_name = node.replace("column:", "")
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_nodes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_node); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_node_data, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "sQQideR/DatabaseContext.pyx":651
 *             for node in condition_nodes:
 *                 node_data = self.graph.nodes[node]
 *                 if node_data['type'] == 'column':             # <<<<<<<<<<<<<<
 *                     col_name = node.replace("column:", "")
 *                     where_clauses.append(f"{col_name} = ?")
 */
      __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_v_node_data, __pyx_n_u_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_6, __pyx_n_u_column_2, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_3) {

        /* "sQQideR/DatabaseContext.pyx":652
 *                 node_data = self.graph.nodes[node]
 *                 if node_data['type'] == 'column':
 *                     col_name = node.replace("column:", "")             # <<<<<<<<<<<<<<
 *                     where_clauses.append(f"{col_name} = ?")
 * 
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_replace); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 652, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 652, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_col_name, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":653
 *                 if node_data['type'] == 'column':
 *                     col_name = node.replace("column:", "")
 *                     where_clauses.append(f"{col_name} = ?")             # <<<<<<<<<<<<<<
 * 
 *         # Build the final SQL statement
 */
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_col_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_1, __pyx_kp_u__28); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_where_clauses, __pyx_t_6); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 653, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":651
 *             for node in condition_nodes:
 *                 node_data = self.graph.nodes[node]
 *                 if node_data['type'] == 'column':             # <<<<<<<<<<<<<<
 *                     col_name = node.replace("column:", "")
 *                     where_clauses.append(f"{col_name} = ?")
 */
      }

      /* "sQQideR/DatabaseContext.pyx":649
 *         where_clauses = []
 *         if condition_nodes:
 *             for node in condition_nodes:             # <<<<<<<<<<<<<<
 *                 node_data = self.graph.nodes[node]
 *                 if node_data['type'] == 'column':
 */
    }
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "sQQideR/DatabaseContext.pyx":648
 *         # Add conditions from condition nodes
 *         where_clauses = []
 *         if condition_nodes:             # <<<<<<<<<<<<<<
 *             for node in condition_nodes:
 *                 node_data = self.graph.nodes[node]
 */
  }

  /* "sQQideR/DatabaseContext.pyx":656
 * 
 *         # Build the final SQL statement
 *         sql_lines = []             # <<<<<<<<<<<<<<
 *         # Explicit SELECT clause using table aliases
 *         select_clause = "SELECT " + ", ".join([f"{alias}.*" for alias in aliases.values()])
 */
  __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_sql_lines = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "sQQideR/DatabaseContext.pyx":658
 *         sql_lines = []
 *         # Explicit SELECT clause using table aliases
 *         select_clause = "SELECT " + ", ".join([f"{alias}.*" for alias in aliases.values()])             # <<<<<<<<<<<<<<
 *         sql_lines.append(select_clause)
 * 
 */
  { /* enter inner scope */
    __pyx_t_11 = PyList_New(0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 658, __pyx_L31_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_dict_iterator(__pyx_v_aliases, 1, __pyx_n_s_values, (&__pyx_t_8), (&__pyx_t_16)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 658, __pyx_L31_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_6);
    __pyx_t_6 = __pyx_t_1;
    __pyx_t_1 = 0;
    while (1) {
      __pyx_t_17 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_8, &__pyx_t_4, NULL, &__pyx_t_1, NULL, __pyx_t_16);
      if (unlikely(__pyx_t_17 == 0)) break;
      if (unlikely(__pyx_t_17 == -1)) __PYX_ERR(0, 658, __pyx_L31_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_8genexpr4__pyx_v_alias, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_8genexpr4__pyx_v_alias, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 658, __pyx_L31_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_1, __pyx_kp_u__29); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 658, __pyx_L31_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_11, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 658, __pyx_L31_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_alias); __pyx_8genexpr4__pyx_v_alias = 0;
    goto __pyx_L34_exit_scope;
    __pyx_L31_error:;
    __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_alias); __pyx_8genexpr4__pyx_v_alias = 0;
    goto __pyx_L1_error;
    __pyx_L34_exit_scope:;
  } /* exit inner scope */
  __pyx_t_6 = PyUnicode_Join(__pyx_kp_u__2, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyUnicode_Concat(__pyx_kp_u_SELECT, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_select_clause = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "sQQideR/DatabaseContext.pyx":659
 *         # Explicit SELECT clause using table aliases
 *         select_clause = "SELECT " + ", ".join([f"{alias}.*" for alias in aliases.values()])
 *         sql_lines.append(select_clause)             # <<<<<<<<<<<<<<
 * 
 *         # FROM clause
 */
  __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sql_lines, __pyx_v_select_clause); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 659, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":662
 * 
 *         # FROM clause
 *         first_table = from_tables[0]             # <<<<<<<<<<<<<<
 *         sql_lines.append(f"FROM {first_table} {aliases[first_table]}")
 * 
 */
  __pyx_t_11 = __Pyx_GetItemInt_List(__pyx_v_from_tables, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 662, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_first_table = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "sQQideR/DatabaseContext.pyx":663
 *         # FROM clause
 *         first_table = from_tables[0]
 *         sql_lines.append(f"FROM {first_table} {aliases[first_table]}")             # <<<<<<<<<<<<<<
 * 
 *         # Add JOIN clauses
 */
  __pyx_t_11 = PyTuple_New(4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_8 = 0;
  __pyx_t_20 = 127;
  __Pyx_INCREF(__pyx_kp_u_FROM);
  __pyx_t_8 += 5;
  __Pyx_GIVEREF(__pyx_kp_u_FROM);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_kp_u_FROM);
  __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_first_table, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_20;
  __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_6);
  __pyx_t_6 = 0;
  __Pyx_INCREF(__pyx_kp_u__27);
  __pyx_t_8 += 1;
  __Pyx_GIVEREF(__pyx_kp_u__27);
  PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_kp_u__27);
  __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_aliases, __pyx_v_first_table); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_t_6, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_20 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_20) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_20;
  __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, __pyx_t_8, __pyx_t_20); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sql_lines, __pyx_t_5); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "sQQideR/DatabaseContext.pyx":666
 * 
 *         # Add JOIN clauses
 *         sql_lines.extend(join_clauses)             # <<<<<<<<<<<<<<
 * 
 *         # WHERE clause (if any conditions)
 */
  __pyx_t_7 = __Pyx_PyList_Extend(__pyx_v_sql_lines, __pyx_v_join_clauses); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 666, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":669
 * 
 *         # WHERE clause (if any conditions)
 *         if where_clauses:             # <<<<<<<<<<<<<<
 *             sql_lines.append("WHERE " + " AND ".join(where_clauses))
 * 
 */
  __pyx_t_3 = (PyList_GET_SIZE(__pyx_v_where_clauses) != 0);
  if (__pyx_t_3) {

    /* "sQQideR/DatabaseContext.pyx":670
 *         # WHERE clause (if any conditions)
 *         if where_clauses:
 *             sql_lines.append("WHERE " + " AND ".join(where_clauses))             # <<<<<<<<<<<<<<
 * 
 *         sql = "\n".join(sql_lines)
 */
    __pyx_t_5 = PyUnicode_Join(__pyx_kp_u_AND, __pyx_v_where_clauses); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = __Pyx_PyUnicode_Concat(__pyx_kp_u_WHERE, __pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_sql_lines, __pyx_t_11); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "sQQideR/DatabaseContext.pyx":669
 * 
 *         # WHERE clause (if any conditions)
 *         if where_clauses:             # <<<<<<<<<<<<<<
 *             sql_lines.append("WHERE " + " AND ".join(where_clauses))
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":672
 *             sql_lines.append("WHERE " + " AND ".join(where_clauses))
 * 
 *         sql = "\n".join(sql_lines)             # <<<<<<<<<<<<<<
 *         return sql
 * 
 */
  __pyx_t_11 = PyUnicode_Join(__pyx_kp_u__9, __pyx_v_sql_lines); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 672, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_sql = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "sQQideR/DatabaseContext.pyx":673
 * 
 *         sql = "\n".join(sql_lines)
 *         return sql             # <<<<<<<<<<<<<<
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_sql);
  __pyx_r = __pyx_v_sql;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_from_tables);
  __Pyx_XDECREF(__pyx_v_aliases);
  __Pyx_XDECREF(__pyx_v_get_alias);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_join_clauses);
  __Pyx_XDECREF(__pyx_v_current_table);
  __Pyx_XDECREF(__pyx_v_prev);
  __Pyx_XDECREF(__pyx_v_prev_table);
  __Pyx_XDECREF(__pyx_v_edge_data);
  __Pyx_XDECREF(__pyx_v_join_condition);
  __Pyx_XDECREF(__pyx_v_tbl);
  __Pyx_XDECREF(__pyx_v_alias);
  __Pyx_XDECREF(__pyx_v_where_clauses);
  __Pyx_XDECREF(__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_node_data);
  __Pyx_XDECREF(__pyx_v_col_name);
  __Pyx_XDECREF(__pyx_v_sql_lines);
  __Pyx_XDECREF(__pyx_v_select_clause);
  __Pyx_XDECREF(__pyx_v_first_table);
  __Pyx_XDECREF(__pyx_v_sql);
  __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_alias);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_19generate_sql_from_nl(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_18generate_sql_from_nl, "\n        Generate SQL based on natural language query using graph knowledge and examples.\n        \n        Args:\n            natural_language_query: Natural language query string\n            \n        Returns:\n            Generated SQL query string\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_19generate_sql_from_nl = {"generate_sql_from_nl", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_19generate_sql_from_nl, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_18generate_sql_from_nl};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_19generate_sql_from_nl(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_natural_language_query = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate_sql_from_nl (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_natural_language_query,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 675, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_natural_language_query)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 675, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_sql_from_nl", 1, 2, 2, 1); __PYX_ERR(0, 675, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "generate_sql_from_nl") < 0)) __PYX_ERR(0, 675, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_natural_language_query = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("generate_sql_from_nl", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 675, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_natural_language_query), (&PyUnicode_Type), 0, "natural_language_query", 1))) __PYX_ERR(0, 675, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18generate_sql_from_nl(__pyx_self, __pyx_v_self, __pyx_v_natural_language_query);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":733
 *         # Use the best matching example as a template if available
 *         if example_scores:
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]             # <<<<<<<<<<<<<<
 *             template_query = self.graph.nodes[best_example]['query']
 *             if self.verbose:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_lambda2(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_lambda2 = {"lambda2", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_lambda2, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_lambda2(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_x = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda2 (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_x)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 733, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "lambda2") < 0)) __PYX_ERR(0, 733, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_x = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda2", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 733, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl.lambda2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda2(__pyx_self, __pyx_v_x);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda2(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda2", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_x, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl.lambda2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_3generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "sQQideR/DatabaseContext.pyx":758
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):             # <<<<<<<<<<<<<<
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)
 * 
 */

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_1genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 758, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *) __pyx_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_3generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_DatabaseContextService_generate_2, __pyx_n_s_sQQideR_DatabaseContext); if (unlikely(!gen)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_3generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *__pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 758, __pyx_L1_error)
  __pyx_t_1 = __pyx_tuple__30; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 2) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 758, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_word);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_word, ((PyObject*)__pyx_t_3));
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_natural_language_query)) { __Pyx_RaiseClosureNameError("natural_language_query"); __PYX_ERR(0, 758, __pyx_L1_error) }
    __pyx_t_3 = __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyUnicode_Type_lower, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_natural_language_query); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_cur_scope->__pyx_v_word, __pyx_t_3, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_4) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_Generator_Replace_StopIteration(0);
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_18generate_sql_from_nl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_natural_language_query) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *__pyx_cur_scope;
  CYTHON_UNUSED PyObject *__pyx_v_context = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_keywords = NULL;
  PyObject *__pyx_v_example_scores = NULL;
  PyObject *__pyx_v_example_nodes = NULL;
  PyObject *__pyx_v_example_node = NULL;
  PyObject *__pyx_v_example_data = NULL;
  PyObject *__pyx_v_score = NULL;
  PyObject *__pyx_v_keyword = NULL;
  PyObject *__pyx_v_best_example = NULL;
  PyObject *__pyx_v_template_query = NULL;
  PyObject *__pyx_v_adapted_query = NULL;
  PyObject *__pyx_v_sql = NULL;
  PyObject *__pyx_v_tables = NULL;
  PyObject *__pyx_v_columns = NULL;
  PyObject *__pyx_v_results = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_matches = NULL;
  PyObject *__pyx_v_node_id = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_8genexpr5__pyx_v_word = NULL;
  PyObject *__pyx_8genexpr6__pyx_v_n = NULL;
  PyObject *__pyx_8genexpr6__pyx_v_d = NULL;
  PyObject *__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_3generator1 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_UCS4 __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  char const *__pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  Py_ssize_t __pyx_t_23;
  PyObject *(*__pyx_t_24)(PyObject *);
  PyObject *__pyx_t_25 = NULL;
  PyObject *(*__pyx_t_26)(PyObject *);
  int __pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("generate_sql_from_nl", 0);
  __pyx_cur_scope = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 675, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_natural_language_query = __pyx_v_natural_language_query;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_natural_language_query);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_natural_language_query);

  /* "sQQideR/DatabaseContext.pyx":685
 *             Generated SQL query string
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":686
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         # Try to use AI for SQL generation if available
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 686, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":685
 *             Generated SQL query string
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":689
 * 
 *         # Try to use AI for SQL generation if available
 *         if self.ai_base and hasattr(self.ai_base, 'get_response'):             # <<<<<<<<<<<<<<
 *             try:
 *                 # Prepare context for AI
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_HasAttr(__pyx_t_1, __pyx_n_u_get_response); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":690
 *         # Try to use AI for SQL generation if available
 *         if self.ai_base and hasattr(self.ai_base, 'get_response'):
 *             try:             # <<<<<<<<<<<<<<
 *                 # Prepare context for AI
 *                 context = {
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      /*try:*/ {

        /* "sQQideR/DatabaseContext.pyx":693
 *                 # Prepare context for AI
 *                 context = {
 *                     "query": natural_language_query,             # <<<<<<<<<<<<<<
 *                     "database_schema": self._get_schema_summary()
 *                 }
 */
        __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 693, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_query, __pyx_cur_scope->__pyx_v_natural_language_query) < 0) __PYX_ERR(0, 693, __pyx_L7_error)

        /* "sQQideR/DatabaseContext.pyx":694
 *                 context = {
 *                     "query": natural_language_query,
 *                     "database_schema": self._get_schema_summary()             # <<<<<<<<<<<<<<
 *                 }
 * 
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_get_schema_summary); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 694, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, NULL};
          __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 694, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_database_schema, __pyx_t_7) < 0) __PYX_ERR(0, 693, __pyx_L7_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_v_context = ((PyObject*)__pyx_t_1);
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":690
 *         # Try to use AI for SQL generation if available
 *         if self.ai_base and hasattr(self.ai_base, 'get_response'):
 *             try:             # <<<<<<<<<<<<<<
 *                 # Prepare context for AI
 *                 context = {
 */
      }
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L12_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":701
 *                 # This is just a placeholder - we'll continue with the existing logic
 *                 pass
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"AI SQL generation failed: {e}. Falling back to rule-based approach.")
 */
      __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
      if (__pyx_t_11) {
        __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 701, __pyx_L9_except_error)
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __pyx_v_e = __pyx_t_7;
        /*try:*/ {

          /* "sQQideR/DatabaseContext.pyx":702
 *                 pass
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"AI SQL generation failed: {e}. Falling back to rule-based approach.")
 * 
 */
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 702, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 702, __pyx_L18_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":703
 *             except Exception as e:
 *                 if self.verbose:
 *                     print(f"AI SQL generation failed: {e}. Falling back to rule-based approach.")             # <<<<<<<<<<<<<<
 * 
 *         # Extract keywords from the query
 */
            __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 703, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_12 = 0;
            __pyx_t_13 = 127;
            __Pyx_INCREF(__pyx_kp_u_AI_SQL_generation_failed);
            __pyx_t_12 += 26;
            __Pyx_GIVEREF(__pyx_kp_u_AI_SQL_generation_failed);
            PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_kp_u_AI_SQL_generation_failed);
            __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 703, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_13 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) > __pyx_t_13) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) : __pyx_t_13;
            __pyx_t_12 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14);
            __Pyx_GIVEREF(__pyx_t_14);
            PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_14);
            __pyx_t_14 = 0;
            __Pyx_INCREF(__pyx_kp_u_Falling_back_to_rule_based_appr);
            __pyx_t_12 += 38;
            __Pyx_GIVEREF(__pyx_kp_u_Falling_back_to_rule_based_appr);
            PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_kp_u_Falling_back_to_rule_based_appr);
            __pyx_t_14 = __Pyx_PyUnicode_Join(__pyx_t_9, 3, __pyx_t_12, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 703, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_14); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 703, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

            /* "sQQideR/DatabaseContext.pyx":702
 *                 pass
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"AI SQL generation failed: {e}. Falling back to rule-based approach.")
 * 
 */
          }
        }

        /* "sQQideR/DatabaseContext.pyx":701
 *                 # This is just a placeholder - we'll continue with the existing logic
 *                 pass
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"AI SQL generation failed: {e}. Falling back to rule-based approach.")
 */
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            goto __pyx_L19;
          }
          __pyx_L18_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
            if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_22);
            __pyx_t_11 = __pyx_lineno; __pyx_t_15 = __pyx_clineno; __pyx_t_16 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            }
            if (PY_MAJOR_VERSION >= 3) {
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_XGIVEREF(__pyx_t_21);
              __Pyx_XGIVEREF(__pyx_t_22);
              __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
            }
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_ErrRestore(__pyx_t_17, __pyx_t_18, __pyx_t_19);
            __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
            __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_15; __pyx_filename = __pyx_t_16;
            goto __pyx_L9_except_error;
          }
          __pyx_L19:;
        }
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L8_exception_handled;
      }
      goto __pyx_L9_except_error;

      /* "sQQideR/DatabaseContext.pyx":690
 *         # Try to use AI for SQL generation if available
 *         if self.ai_base and hasattr(self.ai_base, 'get_response'):
 *             try:             # <<<<<<<<<<<<<<
 *                 # Prepare context for AI
 *                 context = {
 */
      __pyx_L9_except_error:;
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
      goto __pyx_L1_error;
      __pyx_L8_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
      __pyx_L12_try_end:;
    }

    /* "sQQideR/DatabaseContext.pyx":689
 * 
 *         # Try to use AI for SQL generation if available
 *         if self.ai_base and hasattr(self.ai_base, 'get_response'):             # <<<<<<<<<<<<<<
 *             try:
 *                 # Prepare context for AI
 */
  }

  /* "sQQideR/DatabaseContext.pyx":706
 * 
 *         # Extract keywords from the query
 *         keywords = [word.lower() for word in natural_language_query.split() if len(word) > 3]             # <<<<<<<<<<<<<<
 *         if self.verbose:
 *             print(f"Extracted keywords: {keywords}")
 */
  { /* enter inner scope */
    __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 706, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = PyUnicode_Split(__pyx_cur_scope->__pyx_v_natural_language_query, ((PyObject *)NULL), -1L); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 706, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __pyx_t_7; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 706, __pyx_L27_error)
        #endif
        if (__pyx_t_12 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_7 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_12); __Pyx_INCREF(__pyx_t_7); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 706, __pyx_L27_error)
      #else
      __pyx_t_7 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 706, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_XDECREF_SET(__pyx_8genexpr5__pyx_v_word, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_23 = PyObject_Length(__pyx_8genexpr5__pyx_v_word); if (unlikely(__pyx_t_23 == ((Py_ssize_t)-1))) __PYX_ERR(0, 706, __pyx_L27_error)
      __pyx_t_2 = (__pyx_t_23 > 3);
      if (__pyx_t_2) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr5__pyx_v_word, __pyx_n_s_lower); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 706, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_14 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_14, NULL};
          __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 706, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_8, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 706, __pyx_L27_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_word); __pyx_8genexpr5__pyx_v_word = 0;
    goto __pyx_L32_exit_scope;
    __pyx_L27_error:;
    __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_word); __pyx_8genexpr5__pyx_v_word = 0;
    goto __pyx_L1_error;
    __pyx_L32_exit_scope:;
  } /* exit inner scope */
  __pyx_v_keywords = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":707
 *         # Extract keywords from the query
 *         keywords = [word.lower() for word in natural_language_query.split() if len(word) > 3]
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Extracted keywords: {keywords}")
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":708
 *         keywords = [word.lower() for word in natural_language_query.split() if len(word) > 3]
 *         if self.verbose:
 *             print(f"Extracted keywords: {keywords}")             # <<<<<<<<<<<<<<
 * 
 *         # Find relevant examples for SQL generation if available
 */
    __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_keywords, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Extracted_keywords, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":707
 *         # Extract keywords from the query
 *         keywords = [word.lower() for word in natural_language_query.split() if len(word) > 3]
 *         if self.verbose:             # <<<<<<<<<<<<<<
 *             print(f"Extracted keywords: {keywords}")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":711
 * 
 *         # Find relevant examples for SQL generation if available
 *         example_scores = {}             # <<<<<<<<<<<<<<
 *         example_nodes = [n for n, d in self.graph.nodes(data=True) if d.get('type') == 'example']
 * 
 */
  __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_example_scores = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":712
 *         # Find relevant examples for SQL generation if available
 *         example_scores = {}
 *         example_nodes = [n for n, d in self.graph.nodes(data=True) if d.get('type') == 'example']             # <<<<<<<<<<<<<<
 * 
 *         for example_node in example_nodes:
 */
  { /* enter inner scope */
    __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 712, __pyx_L36_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L36_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 712, __pyx_L36_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L36_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_data, Py_True) < 0) __PYX_ERR(0, 712, __pyx_L36_error)
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 712, __pyx_L36_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_9)) || PyTuple_CheckExact(__pyx_t_9)) {
      __pyx_t_1 = __pyx_t_9; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_12 = 0;
      __pyx_t_24 = NULL;
    } else {
      __pyx_t_12 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L36_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_24 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 712, __pyx_L36_error)
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    for (;;) {
      if (likely(!__pyx_t_24)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 712, __pyx_L36_error)
            #endif
            if (__pyx_t_12 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_12); __Pyx_INCREF(__pyx_t_9); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 712, __pyx_L36_error)
          #else
          __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 712, __pyx_L36_error)
          __Pyx_GOTREF(__pyx_t_9);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 712, __pyx_L36_error)
            #endif
            if (__pyx_t_12 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_12); __Pyx_INCREF(__pyx_t_9); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 712, __pyx_L36_error)
          #else
          __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 712, __pyx_L36_error)
          __Pyx_GOTREF(__pyx_t_9);
          #endif
        }
      } else {
        __pyx_t_9 = __pyx_t_24(__pyx_t_1);
        if (unlikely(!__pyx_t_9)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 712, __pyx_L36_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_9);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_9))) || (PyList_CheckExact(__pyx_t_9))) {
        PyObject* sequence = __pyx_t_9;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 712, __pyx_L36_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_14);
        #else
        __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 712, __pyx_L36_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 712, __pyx_L36_error)
        __Pyx_GOTREF(__pyx_t_14);
        #endif
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_25 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 712, __pyx_L36_error)
        __Pyx_GOTREF(__pyx_t_25);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_26 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_25);
        index = 0; __pyx_t_7 = __pyx_t_26(__pyx_t_25); if (unlikely(!__pyx_t_7)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        index = 1; __pyx_t_14 = __pyx_t_26(__pyx_t_25); if (unlikely(!__pyx_t_14)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_14);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_26(__pyx_t_25), 2) < 0) __PYX_ERR(0, 712, __pyx_L36_error)
        __pyx_t_26 = NULL;
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        __pyx_t_26 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 712, __pyx_L36_error)
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr6__pyx_v_n, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr6__pyx_v_d, __pyx_t_14);
      __pyx_t_14 = 0;
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr6__pyx_v_d, __pyx_n_s_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 712, __pyx_L36_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_7 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_n_u_type};
        __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 712, __pyx_L36_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_9, __pyx_n_u_example_2, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 712, __pyx_L36_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (__pyx_t_2) {
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_8, (PyObject*)__pyx_8genexpr6__pyx_v_n))) __PYX_ERR(0, 712, __pyx_L36_error)
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_d); __pyx_8genexpr6__pyx_v_d = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_n); __pyx_8genexpr6__pyx_v_n = 0;
    goto __pyx_L43_exit_scope;
    __pyx_L36_error:;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_d); __pyx_8genexpr6__pyx_v_d = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_n); __pyx_8genexpr6__pyx_v_n = 0;
    goto __pyx_L1_error;
    __pyx_L43_exit_scope:;
  } /* exit inner scope */
  __pyx_v_example_nodes = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":714
 *         example_nodes = [n for n, d in self.graph.nodes(data=True) if d.get('type') == 'example']
 * 
 *         for example_node in example_nodes:             # <<<<<<<<<<<<<<
 *             example_data = self.graph.nodes[example_node]
 *             score = 0
 */
  __pyx_t_8 = __pyx_v_example_nodes; __Pyx_INCREF(__pyx_t_8);
  __pyx_t_12 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 714, __pyx_L1_error)
      #endif
      if (__pyx_t_12 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_12); __Pyx_INCREF(__pyx_t_1); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 714, __pyx_L1_error)
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 714, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_example_node, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":715
 * 
 *         for example_node in example_nodes:
 *             example_data = self.graph.nodes[example_node]             # <<<<<<<<<<<<<<
 *             score = 0
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_example_node); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_example_data, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":716
 *         for example_node in example_nodes:
 *             example_data = self.graph.nodes[example_node]
 *             score = 0             # <<<<<<<<<<<<<<
 * 
 *             # Check description match
 */
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_XDECREF_SET(__pyx_v_score, __pyx_int_0);

    /* "sQQideR/DatabaseContext.pyx":719
 * 
 *             # Check description match
 *             for keyword in keywords:             # <<<<<<<<<<<<<<
 *                 if keyword in example_data['description'].lower():
 *                     score += 2
 */
    __pyx_t_1 = __pyx_v_keywords; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_23 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 719, __pyx_L1_error)
        #endif
        if (__pyx_t_23 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_23); __Pyx_INCREF(__pyx_t_9); __pyx_t_23++; if (unlikely((0 < 0))) __PYX_ERR(0, 719, __pyx_L1_error)
      #else
      __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_23); __pyx_t_23++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 719, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_keyword, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":720
 *             # Check description match
 *             for keyword in keywords:
 *                 if keyword in example_data['description'].lower():             # <<<<<<<<<<<<<<
 *                     score += 2
 * 
 */
      __pyx_t_14 = __Pyx_PyObject_Dict_GetItem(__pyx_v_example_data, __pyx_n_u_description); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 720, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_lower); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 720, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_14)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_14, NULL};
        __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_keyword, __pyx_t_9, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 720, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":721
 *             for keyword in keywords:
 *                 if keyword in example_data['description'].lower():
 *                     score += 2             # <<<<<<<<<<<<<<
 * 
 *             # Check query match
 */
        __pyx_t_9 = __Pyx_PyInt_AddObjC(__pyx_v_score, __pyx_int_2, 2, 1, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 721, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF_SET(__pyx_v_score, __pyx_t_9);
        __pyx_t_9 = 0;

        /* "sQQideR/DatabaseContext.pyx":720
 *             # Check description match
 *             for keyword in keywords:
 *                 if keyword in example_data['description'].lower():             # <<<<<<<<<<<<<<
 *                     score += 2
 * 
 */
      }

      /* "sQQideR/DatabaseContext.pyx":719
 * 
 *             # Check description match
 *             for keyword in keywords:             # <<<<<<<<<<<<<<
 *                 if keyword in example_data['description'].lower():
 *                     score += 2
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":724
 * 
 *             # Check query match
 *             for keyword in keywords:             # <<<<<<<<<<<<<<
 *                 if keyword in example_data['query'].lower():
 *                     score += 1
 */
    __pyx_t_1 = __pyx_v_keywords; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_23 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 724, __pyx_L1_error)
        #endif
        if (__pyx_t_23 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_23); __Pyx_INCREF(__pyx_t_9); __pyx_t_23++; if (unlikely((0 < 0))) __PYX_ERR(0, 724, __pyx_L1_error)
      #else
      __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_23); __pyx_t_23++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 724, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_keyword, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":725
 *             # Check query match
 *             for keyword in keywords:
 *                 if keyword in example_data['query'].lower():             # <<<<<<<<<<<<<<
 *                     score += 1
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_v_example_data, __pyx_n_u_query); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 725, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_lower); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 725, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
        __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 725, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_keyword, __pyx_t_9, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 725, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":726
 *             for keyword in keywords:
 *                 if keyword in example_data['query'].lower():
 *                     score += 1             # <<<<<<<<<<<<<<
 * 
 *             if score > 0:
 */
        __pyx_t_9 = __Pyx_PyInt_AddObjC(__pyx_v_score, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 726, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF_SET(__pyx_v_score, __pyx_t_9);
        __pyx_t_9 = 0;

        /* "sQQideR/DatabaseContext.pyx":725
 *             # Check query match
 *             for keyword in keywords:
 *                 if keyword in example_data['query'].lower():             # <<<<<<<<<<<<<<
 *                     score += 1
 * 
 */
      }

      /* "sQQideR/DatabaseContext.pyx":724
 * 
 *             # Check query match
 *             for keyword in keywords:             # <<<<<<<<<<<<<<
 *                 if keyword in example_data['query'].lower():
 *                     score += 1
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":728
 *                     score += 1
 * 
 *             if score > 0:             # <<<<<<<<<<<<<<
 *                 example_scores[example_node] = score
 * 
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_score, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":729
 * 
 *             if score > 0:
 *                 example_scores[example_node] = score             # <<<<<<<<<<<<<<
 * 
 *         # Use the best matching example as a template if available
 */
      if (unlikely((PyDict_SetItem(__pyx_v_example_scores, __pyx_v_example_node, __pyx_v_score) < 0))) __PYX_ERR(0, 729, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":728
 *                     score += 1
 * 
 *             if score > 0:             # <<<<<<<<<<<<<<
 *                 example_scores[example_node] = score
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":714
 *         example_nodes = [n for n, d in self.graph.nodes(data=True) if d.get('type') == 'example']
 * 
 *         for example_node in example_nodes:             # <<<<<<<<<<<<<<
 *             example_data = self.graph.nodes[example_node]
 *             score = 0
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":732
 * 
 *         # Use the best matching example as a template if available
 *         if example_scores:             # <<<<<<<<<<<<<<
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]
 *             template_query = self.graph.nodes[best_example]['query']
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_example_scores); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 732, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":733
 *         # Use the best matching example as a template if available
 *         if example_scores:
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]             # <<<<<<<<<<<<<<
 *             template_query = self.graph.nodes[best_example]['query']
 *             if self.verbose:
 */
    __pyx_t_8 = __Pyx_PyDict_Items(__pyx_v_example_scores); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_8)) __PYX_ERR(0, 733, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_lambda2, 0, __pyx_n_s_DatabaseContextService_generate_3, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_key, __pyx_t_9) < 0) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_max, __pyx_t_1, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_GetItemInt(__pyx_t_9, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 733, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_v_best_example = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":734
 *         if example_scores:
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]
 *             template_query = self.graph.nodes[best_example]['query']             # <<<<<<<<<<<<<<
 *             if self.verbose:
 *                 print(f"Using template query: {template_query}")
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_nodes); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_best_example); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_n_u_query); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_template_query = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":735
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]
 *             template_query = self.graph.nodes[best_example]['query']
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using template query: {template_query}")
 * 
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 735, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 735, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":736
 *             template_query = self.graph.nodes[best_example]['query']
 *             if self.verbose:
 *                 print(f"Using template query: {template_query}")             # <<<<<<<<<<<<<<
 * 
 *             # Adapt the template to the current context
 */
      __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_v_template_query, __pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 736, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Using_template_query, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 736, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 736, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":735
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]
 *             template_query = self.graph.nodes[best_example]['query']
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using template query: {template_query}")
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":741
 *             # This would require more sophisticated NLP to do well
 *             # For now, just substitute table names
 *             adapted_query = template_query             # <<<<<<<<<<<<<<
 *             sql = adapted_query
 *         else:
 */
    __Pyx_INCREF(__pyx_v_template_query);
    __pyx_v_adapted_query = __pyx_v_template_query;

    /* "sQQideR/DatabaseContext.pyx":742
 *             # For now, just substitute table names
 *             adapted_query = template_query
 *             sql = adapted_query             # <<<<<<<<<<<<<<
 *         else:
 *             # Find matching tables and columns if no good example
 */
    __Pyx_INCREF(__pyx_v_adapted_query);
    __pyx_v_sql = __pyx_v_adapted_query;

    /* "sQQideR/DatabaseContext.pyx":732
 * 
 *         # Use the best matching example as a template if available
 *         if example_scores:             # <<<<<<<<<<<<<<
 *             best_example = max(example_scores.items(), key=lambda x: x[1])[0]
 *             template_query = self.graph.nodes[best_example]['query']
 */
    goto __pyx_L56;
  }

  /* "sQQideR/DatabaseContext.pyx":745
 *         else:
 *             # Find matching tables and columns if no good example
 *             tables = []             # <<<<<<<<<<<<<<
 *             columns = []
 *             results, matches = self.search_by_keywords(keywords)
 */
  /*else*/ {
    __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 745, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_tables = ((PyObject*)__pyx_t_9);
    __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":746
 *             # Find matching tables and columns if no good example
 *             tables = []
 *             columns = []             # <<<<<<<<<<<<<<
 *             results, matches = self.search_by_keywords(keywords)
 *             for node_id, data in results:
 */
    __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_columns = ((PyObject*)__pyx_t_9);
    __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":747
 *             tables = []
 *             columns = []
 *             results, matches = self.search_by_keywords(keywords)             # <<<<<<<<<<<<<<
 *             for node_id, data in results:
 *                 if node_id.startswith("table:"):
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_search_by_keywords); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 747, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_keywords};
      __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_9))) || (PyList_CheckExact(__pyx_t_9))) {
      PyObject* sequence = __pyx_t_9;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 747, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      #else
      __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_14 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 747, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_26 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_14);
      index = 0; __pyx_t_8 = __pyx_t_26(__pyx_t_14); if (unlikely(!__pyx_t_8)) goto __pyx_L58_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 1; __pyx_t_1 = __pyx_t_26(__pyx_t_14); if (unlikely(!__pyx_t_1)) goto __pyx_L58_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_26(__pyx_t_14), 2) < 0) __PYX_ERR(0, 747, __pyx_L1_error)
      __pyx_t_26 = NULL;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      goto __pyx_L59_unpacking_done;
      __pyx_L58_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_26 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 747, __pyx_L1_error)
      __pyx_L59_unpacking_done:;
    }
    __pyx_v_results = __pyx_t_8;
    __pyx_t_8 = 0;
    __pyx_v_matches = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":748
 *             columns = []
 *             results, matches = self.search_by_keywords(keywords)
 *             for node_id, data in results:             # <<<<<<<<<<<<<<
 *                 if node_id.startswith("table:"):
 *                     tables.append(node_id)
 */
    if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
      __pyx_t_9 = __pyx_v_results; __Pyx_INCREF(__pyx_t_9);
      __pyx_t_12 = 0;
      __pyx_t_24 = NULL;
    } else {
      __pyx_t_12 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 748, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_24 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_9); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 748, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_24)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_9);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 748, __pyx_L1_error)
            #endif
            if (__pyx_t_12 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_12); __Pyx_INCREF(__pyx_t_1); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 748, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_9, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_9);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 748, __pyx_L1_error)
            #endif
            if (__pyx_t_12 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_12); __Pyx_INCREF(__pyx_t_1); __pyx_t_12++; if (unlikely((0 < 0))) __PYX_ERR(0, 748, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_9, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_24(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 748, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 748, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_14);
        #else
        __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_26 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
        index = 0; __pyx_t_8 = __pyx_t_26(__pyx_t_7); if (unlikely(!__pyx_t_8)) goto __pyx_L62_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        index = 1; __pyx_t_14 = __pyx_t_26(__pyx_t_7); if (unlikely(!__pyx_t_14)) goto __pyx_L62_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_14);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_26(__pyx_t_7), 2) < 0) __PYX_ERR(0, 748, __pyx_L1_error)
        __pyx_t_26 = NULL;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L63_unpacking_done;
        __pyx_L62_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_26 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 748, __pyx_L1_error)
        __pyx_L63_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_14);
      __pyx_t_14 = 0;

      /* "sQQideR/DatabaseContext.pyx":749
 *             results, matches = self.search_by_keywords(keywords)
 *             for node_id, data in results:
 *                 if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                     tables.append(node_id)
 *                 elif node_id.startswith("column:"):
 */
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 749, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_8 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u_table};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 749, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 749, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":750
 *             for node_id, data in results:
 *                 if node_id.startswith("table:"):
 *                     tables.append(node_id)             # <<<<<<<<<<<<<<
 *                 elif node_id.startswith("column:"):
 *                     columns.append(node_id)
 */
        __pyx_t_27 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_node_id); if (unlikely(__pyx_t_27 == ((int)-1))) __PYX_ERR(0, 750, __pyx_L1_error)

        /* "sQQideR/DatabaseContext.pyx":749
 *             results, matches = self.search_by_keywords(keywords)
 *             for node_id, data in results:
 *                 if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                     tables.append(node_id)
 *                 elif node_id.startswith("column:"):
 */
        goto __pyx_L64;
      }

      /* "sQQideR/DatabaseContext.pyx":751
 *                 if node_id.startswith("table:"):
 *                     tables.append(node_id)
 *                 elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                     columns.append(node_id)
 * 
 */
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 751, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_8 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_kp_u_column};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 751, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 751, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_2) {

        /* "sQQideR/DatabaseContext.pyx":752
 *                     tables.append(node_id)
 *                 elif node_id.startswith("column:"):
 *                     columns.append(node_id)             # <<<<<<<<<<<<<<
 * 
 *             # Fall back to the original query generation if no good example
 */
        __pyx_t_27 = __Pyx_PyList_Append(__pyx_v_columns, __pyx_v_node_id); if (unlikely(__pyx_t_27 == ((int)-1))) __PYX_ERR(0, 752, __pyx_L1_error)

        /* "sQQideR/DatabaseContext.pyx":751
 *                 if node_id.startswith("table:"):
 *                     tables.append(node_id)
 *                 elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                     columns.append(node_id)
 * 
 */
      }
      __pyx_L64:;

      /* "sQQideR/DatabaseContext.pyx":748
 *             columns = []
 *             results, matches = self.search_by_keywords(keywords)
 *             for node_id, data in results:             # <<<<<<<<<<<<<<
 *                 if node_id.startswith("table:"):
 *                     tables.append(node_id)
 */
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":755
 * 
 *             # Fall back to the original query generation if no good example
 *             sql = self.generate_sql(tables, columns)             # <<<<<<<<<<<<<<
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_sql); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 755, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_14 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_v_tables, __pyx_v_columns};
      __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 755, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_v_sql = __pyx_t_9;
    __pyx_t_9 = 0;
  }
  __pyx_L56:;

  /* "sQQideR/DatabaseContext.pyx":758
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):             # <<<<<<<<<<<<<<
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)
 * 
 */
  __pyx_t_9 = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_1genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_Generator_Next(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "sQQideR/DatabaseContext.pyx":759
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)             # <<<<<<<<<<<<<<
 * 
 *         # Remove any LIMIT clauses from the SQL
 */
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_re); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 759, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_sub); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 759, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_14))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_14);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_14, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_9, __pyx_kp_u_d_4_d_2_d_2, __pyx_kp_u__31, __pyx_v_sql};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 3+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_sql, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":758
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):             # <<<<<<<<<<<<<<
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":762
 * 
 *         # Remove any LIMIT clauses from the SQL
 *         sql = re.sub(r"\s+LIMIT\s+\d+\s*;?", "", sql, flags=re.IGNORECASE)             # <<<<<<<<<<<<<<
 * 
 *         return sql
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_re); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sub); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_kp_u_s_LIMIT_s_d_s);
  __Pyx_GIVEREF(__pyx_kp_u_s_LIMIT_s_d_s);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_s_LIMIT_s_d_s)) __PYX_ERR(0, 762, __pyx_L1_error);
  __Pyx_INCREF(__pyx_kp_u__12);
  __Pyx_GIVEREF(__pyx_kp_u__12);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_kp_u__12)) __PYX_ERR(0, 762, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_sql);
  __Pyx_GIVEREF(__pyx_v_sql);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_sql)) __PYX_ERR(0, 762, __pyx_L1_error);
  __pyx_t_9 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_re); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_IGNORECASE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_flags, __pyx_t_7) < 0) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_1, __pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF_SET(__pyx_v_sql, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "sQQideR/DatabaseContext.pyx":764
 *         sql = re.sub(r"\s+LIMIT\s+\d+\s*;?", "", sql, flags=re.IGNORECASE)
 * 
 *         return sql             # <<<<<<<<<<<<<<
 * 
 *     def process_query_with_ai(self, query_text: str, output_format: str = "json") -> Dict:
 */
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_sql))||((__pyx_v_sql) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_sql))) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_INCREF(__pyx_v_sql);
  __pyx_r = ((PyObject*)__pyx_v_sql);
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_sql_from_nl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_context);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_keywords);
  __Pyx_XDECREF(__pyx_v_example_scores);
  __Pyx_XDECREF(__pyx_v_example_nodes);
  __Pyx_XDECREF(__pyx_v_example_node);
  __Pyx_XDECREF(__pyx_v_example_data);
  __Pyx_XDECREF(__pyx_v_score);
  __Pyx_XDECREF(__pyx_v_keyword);
  __Pyx_XDECREF(__pyx_v_best_example);
  __Pyx_XDECREF(__pyx_v_template_query);
  __Pyx_XDECREF(__pyx_v_adapted_query);
  __Pyx_XDECREF(__pyx_v_sql);
  __Pyx_XDECREF(__pyx_v_tables);
  __Pyx_XDECREF(__pyx_v_columns);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_matches);
  __Pyx_XDECREF(__pyx_v_node_id);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_word);
  __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_n);
  __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_d);
  __Pyx_XDECREF(__pyx_gb_7sQQideR_15DatabaseContext_22DatabaseContextService_20generate_sql_from_nl_3generator1);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":766
 *         return sql
 * 
 *     def process_query_with_ai(self, query_text: str, output_format: str = "json") -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query using AI enhancement and return context information.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_21process_query_with_ai(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_20process_query_with_ai, "\n        Process a natural language query using AI enhancement and return context information.\n        \n        Args:\n            query_text: Natural language query string\n            output_format: Output format, 'json' or 'markdown'\n            \n        Returns:\n            Dictionary with context information\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_21process_query_with_ai = {"process_query_with_ai", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_21process_query_with_ai, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_20process_query_with_ai};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_21process_query_with_ai(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_query_text = 0;
  PyObject *__pyx_v_output_format = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("process_query_with_ai (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_query_text,&__pyx_n_s_output_format,0};
    values[2] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)((PyObject*)__pyx_n_u_json)));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 766, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_query_text)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 766, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("process_query_with_ai", 0, 2, 3, 1); __PYX_ERR(0, 766, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_output_format);
          if (value) { values[2] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 766, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "process_query_with_ai") < 0)) __PYX_ERR(0, 766, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_query_text = ((PyObject*)values[1]);
    __pyx_v_output_format = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("process_query_with_ai", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 766, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_ai", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query_text), (&PyUnicode_Type), 0, "query_text", 1))) __PYX_ERR(0, 766, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_output_format), (&PyUnicode_Type), 0, "output_format", 1))) __PYX_ERR(0, 766, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20process_query_with_ai(__pyx_self, __pyx_v_self, __pyx_v_query_text, __pyx_v_output_format);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_20process_query_with_ai(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, PyObject *__pyx_v_output_format) {
  PyObject *__pyx_v_enhanced_query = NULL;
  PyObject *__pyx_v_prompt = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_clean_response = NULL;
  PyObject *__pyx_v_first_newline = NULL;
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_keywords = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_matches = NULL;
  PyObject *__pyx_v_ai_context = NULL;
  PyObject *__pyx_v_tables = NULL;
  PyObject *__pyx_v_columns = NULL;
  PyObject *__pyx_v_concepts = NULL;
  PyObject *__pyx_v_node_id = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_domain = NULL;
  PyObject *__pyx_v_domain_results = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_concept = NULL;
  PyObject *__pyx_v_neighbor = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_table_id = NULL;
  PyObject *__pyx_v_all_tables = NULL;
  PyObject *__pyx_v_all_rules = NULL;
  PyObject *__pyx_v_node_rules = NULL;
  PyObject *__pyx_v_unique_rules = NULL;
  PyObject *__pyx_v_rule_id = NULL;
  PyObject *__pyx_v_rule_desc = NULL;
  PyObject *__pyx_v_filtered_rules = NULL;
  PyObject *__pyx_v_sql = NULL;
  PyObject *__pyx_v_output = NULL;
  PyObject *__pyx_8genexpr8__pyx_v_word = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_UCS4 __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  char const *__pyx_t_26;
  Py_ssize_t __pyx_t_27;
  PyObject *(*__pyx_t_28)(PyObject *);
  PyObject *(*__pyx_t_29)(PyObject *);
  int __pyx_t_30;
  PyObject *(*__pyx_t_31)(PyObject *);
  int __pyx_t_32;
  int __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_query_with_ai", 1);

  /* "sQQideR/DatabaseContext.pyx":777
 *             Dictionary with context information
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "sQQideR/DatabaseContext.pyx":778
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         # Get AI-enhanced query information using the synchronous run method
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 778, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 778, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":777
 *             Dictionary with context information
 *         """
 *         if self.graph is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("Graph not built. Call build_graph first.")
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":781
 * 
 *         # Get AI-enhanced query information using the synchronous run method
 *         enhanced_query = None             # <<<<<<<<<<<<<<
 *         if self.ai_base:
 *             try:
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_enhanced_query = Py_None;

  /* "sQQideR/DatabaseContext.pyx":782
 *         # Get AI-enhanced query information using the synchronous run method
 *         enhanced_query = None
 *         if self.ai_base:             # <<<<<<<<<<<<<<
 *             try:
 *                 # Use the synchronous run method instead of async methods
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":783
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Use the synchronous run method instead of async methods
 *                 prompt = f"""
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "sQQideR/DatabaseContext.pyx":785
 *             try:
 *                 # Use the synchronous run method instead of async methods
 *                 prompt = f"""             # <<<<<<<<<<<<<<
 *                 You are an expert in project management and database systems. Analyze this question:
 * 
 */
        __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 785, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = 0;
        __pyx_t_7 = 127;
        __Pyx_INCREF(__pyx_kp_u_You_are_an_expert_in_project_ma);
        __pyx_t_6 += 136;
        __Pyx_GIVEREF(__pyx_kp_u_You_are_an_expert_in_project_ma);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_You_are_an_expert_in_project_ma);
        __Pyx_INCREF(__pyx_v_query_text);
        __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_query_text) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_query_text) : __pyx_t_7;
        __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_v_query_text);
        __Pyx_GIVEREF(__pyx_v_query_text);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_query_text);
        __Pyx_INCREF(__pyx_kp_u_1_Extract_the_key_business_doma_2);
        __pyx_t_6 += 831;
        __Pyx_GIVEREF(__pyx_kp_u_1_Extract_the_key_business_doma_2);
        PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u_1_Extract_the_key_business_doma_2);
        __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 785, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v_prompt = ((PyObject*)__pyx_t_8);
        __pyx_t_8 = 0;

        /* "sQQideR/DatabaseContext.pyx":805
 * 
 *                 # Use the synchronous run method
 *                 response = self.ai_base.run(prompt)             # <<<<<<<<<<<<<<
 * 
 *                 # Clean the response in case it contains markdown formatting
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_ai_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_run); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 805, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_prompt};
          __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 805, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __pyx_v_response = __pyx_t_8;
        __pyx_t_8 = 0;

        /* "sQQideR/DatabaseContext.pyx":808
 * 
 *                 # Clean the response in case it contains markdown formatting
 *                 clean_response = response.strip()             # <<<<<<<<<<<<<<
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):
 */
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_response, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 808, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_1 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
          __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 808, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __pyx_v_clean_response = __pyx_t_8;
        __pyx_t_8 = 0;

        /* "sQQideR/DatabaseContext.pyx":810
 *                 clean_response = response.strip()
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):             # <<<<<<<<<<<<<<
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 */
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_startswith); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 810, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_1 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_kp_u__10};
          __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 810, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 810, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":811
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")             # <<<<<<<<<<<<<<
 *                     if first_newline != -1:
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()
 */
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_find); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 811, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_1 = NULL;
          __pyx_t_10 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
              __pyx_t_10 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_kp_u__9};
            __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 811, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          __pyx_v_first_newline = __pyx_t_8;
          __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":812
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:             # <<<<<<<<<<<<<<
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()
 *                 # Remove any trailing markdown code block identifiers
 */
          __pyx_t_2 = (__Pyx_PyInt_BoolNeObjC(__pyx_v_first_newline, __pyx_int_neg_1, -1L, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 812, __pyx_L5_error)
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":813
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()             # <<<<<<<<<<<<<<
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):
 */
            __pyx_t_9 = __Pyx_PyObject_GetSlice(__pyx_v_clean_response, 0, 0, &__pyx_v_first_newline, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 813, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_strip); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 813, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __pyx_t_9 = NULL;
            __pyx_t_10 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_9)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
                __pyx_t_10 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_9, NULL};
              __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 813, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            }
            __Pyx_DECREF_SET(__pyx_v_clean_response, __pyx_t_8);
            __pyx_t_8 = 0;

            /* "sQQideR/DatabaseContext.pyx":812
 *                 if clean_response.startswith("```"):
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:             # <<<<<<<<<<<<<<
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()
 *                 # Remove any trailing markdown code block identifiers
 */
          }

          /* "sQQideR/DatabaseContext.pyx":810
 *                 clean_response = response.strip()
 *                 # Remove any leading markdown code block identifiers
 *                 if clean_response.startswith("```"):             # <<<<<<<<<<<<<<
 *                     first_newline = clean_response.find("\n")
 *                     if first_newline != -1:
 */
        }

        /* "sQQideR/DatabaseContext.pyx":815
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):             # <<<<<<<<<<<<<<
 *                     clean_response = clean_response[:-3].strip()
 * 
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_clean_response, __pyx_n_s_endswith); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_kp_u__10};
          __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 815, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 815, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":816
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):
 *                     clean_response = clean_response[:-3].strip()             # <<<<<<<<<<<<<<
 * 
 *                 try:
 */
          __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_clean_response, 0, -3L, NULL, NULL, &__pyx_slice__11, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 816, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = NULL;
          __pyx_t_10 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
              __pyx_t_10 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
            __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 816, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          __Pyx_DECREF_SET(__pyx_v_clean_response, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":815
 *                         clean_response = clean_response[first_newline:].strip()  # Fix: changed trip() to strip()
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):             # <<<<<<<<<<<<<<
 *                     clean_response = clean_response[:-3].strip()
 * 
 */
        }

        /* "sQQideR/DatabaseContext.pyx":818
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
          __Pyx_XGOTREF(__pyx_t_11);
          __Pyx_XGOTREF(__pyx_t_12);
          __Pyx_XGOTREF(__pyx_t_13);
          /*try:*/ {

            /* "sQQideR/DatabaseContext.pyx":820
 *                 try:
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)             # <<<<<<<<<<<<<<
 * 
 *                     if self.verbose:
 */
            __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_json); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 820, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_loads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __pyx_t_9 = NULL;
            __pyx_t_10 = 0;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_9)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
                __pyx_t_10 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_v_clean_response};
              __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 820, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            }
            __pyx_v_parsed = __pyx_t_8;
            __pyx_t_8 = 0;

            /* "sQQideR/DatabaseContext.pyx":822
 *                     parsed = json.loads(clean_response)
 * 
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 */
            __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 822, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 822, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (__pyx_t_2) {

              /* "sQQideR/DatabaseContext.pyx":823
 * 
 *                     if self.verbose:
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")             # <<<<<<<<<<<<<<
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 */
              __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 823, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 823, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 823, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_enhanced_query, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 823, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 823, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

              /* "sQQideR/DatabaseContext.pyx":824
 *                     if self.verbose:
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")             # <<<<<<<<<<<<<<
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 * 
 */
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 824, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);
              __pyx_t_14 = NULL;
              __pyx_t_10 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_1))) {
                __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
                if (likely(__pyx_t_14)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                  __Pyx_INCREF(__pyx_t_14);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_1, function);
                  __pyx_t_10 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_n_u_keywords, __pyx_t_9};
                __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 824, __pyx_L14_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              }
              __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_extracted_keywords, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 824, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "sQQideR/DatabaseContext.pyx":825
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")             # <<<<<<<<<<<<<<
 * 
 *                     enhanced_query = SQLPrompt(
 */
              __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 825, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 825, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);
              __pyx_t_14 = NULL;
              __pyx_t_10 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_8))) {
                __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_8);
                if (likely(__pyx_t_14)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
                  __Pyx_INCREF(__pyx_t_14);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_8, function);
                  __pyx_t_10 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_n_u_business_domains, __pyx_t_9};
                __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 825, __pyx_L14_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              }
              __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 825, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_kp_u_AI_identified_domains, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 825, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 825, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

              /* "sQQideR/DatabaseContext.pyx":822
 *                     parsed = json.loads(clean_response)
 * 
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 */
            }

            /* "sQQideR/DatabaseContext.pyx":827
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 * 
 *                     enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 */
            __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_SQLPrompt); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 827, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_8);

            /* "sQQideR/DatabaseContext.pyx":828
 * 
 *                     enhanced_query = SQLPrompt(
 *                         user_prompt=query_text,             # <<<<<<<<<<<<<<
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),
 */
            __pyx_t_1 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 828, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_user_prompt, __pyx_v_query_text) < 0) __PYX_ERR(0, 828, __pyx_L14_error)

            /* "sQQideR/DatabaseContext.pyx":829
 *                     enhanced_query = SQLPrompt(
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),             # <<<<<<<<<<<<<<
 *                         buisness_domains=parsed.get("business_domains", []),
 *                         keywords=parsed.get("keywords", [])
 */
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 829, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_15 = NULL;
            __pyx_t_10 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_15)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
                __pyx_t_10 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_15, __pyx_n_u_enhanced_query, __pyx_v_query_text};
              __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 829, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
            if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_ai_context, __pyx_t_9) < 0) __PYX_ERR(0, 828, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

            /* "sQQideR/DatabaseContext.pyx":830
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),             # <<<<<<<<<<<<<<
 *                         keywords=parsed.get("keywords", [])
 *                     )
 */
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 830, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_15 = PyList_New(0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 830, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_16 = NULL;
            __pyx_t_10 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_16)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_16);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
                __pyx_t_10 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_16, __pyx_n_u_business_domains, __pyx_t_15};
              __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 830, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
            if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_buisness_domains, __pyx_t_9) < 0) __PYX_ERR(0, 828, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

            /* "sQQideR/DatabaseContext.pyx":831
 *                         ai_context=parsed.get("enhanced_query", query_text),
 *                         buisness_domains=parsed.get("business_domains", []),
 *                         keywords=parsed.get("keywords", [])             # <<<<<<<<<<<<<<
 *                     )
 *                 except json.JSONDecodeError as e:
 */
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_n_s_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 831, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_15 = PyList_New(0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 831, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_16 = NULL;
            __pyx_t_10 = 0;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_16)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_16);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
                __pyx_t_10 = 1;
              }
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_16, __pyx_n_u_keywords, __pyx_t_15};
              __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 831, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_9);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
            if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_keywords, __pyx_t_9) < 0) __PYX_ERR(0, 828, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

            /* "sQQideR/DatabaseContext.pyx":827
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 * 
 *                     enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                         user_prompt=query_text,
 *                         ai_context=parsed.get("enhanced_query", query_text),
 */
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 827, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF_SET(__pyx_v_enhanced_query, __pyx_t_9);
            __pyx_t_9 = 0;

            /* "sQQideR/DatabaseContext.pyx":818
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
          }
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          goto __pyx_L19_try_end;
          __pyx_L14_error:;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

          /* "sQQideR/DatabaseContext.pyx":833
 *                         keywords=parsed.get("keywords", [])
 *                     )
 *                 except json.JSONDecodeError as e:             # <<<<<<<<<<<<<<
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 */
          __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_1, &__pyx_t_8);
          __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_n_s_json); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 833, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_JSONDecodeError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 833, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_17 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_9, __pyx_t_15);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_ErrRestore(__pyx_t_9, __pyx_t_1, __pyx_t_8);
          __pyx_t_9 = 0; __pyx_t_1 = 0; __pyx_t_8 = 0;
          if (__pyx_t_17) {
            __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_ai", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_1, &__pyx_t_9) < 0) __PYX_ERR(0, 833, __pyx_L16_except_error)
            __Pyx_XGOTREF(__pyx_t_8);
            __Pyx_XGOTREF(__pyx_t_1);
            __Pyx_XGOTREF(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_1);
            __pyx_v_e = __pyx_t_1;
            /*try:*/ {

              /* "sQQideR/DatabaseContext.pyx":834
 *                     )
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 */
              __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 834, __pyx_L26_error)
              __Pyx_GOTREF(__pyx_t_15);
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_15); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 834, __pyx_L26_error)
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (__pyx_t_2) {

                /* "sQQideR/DatabaseContext.pyx":835
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")             # <<<<<<<<<<<<<<
 *                         print(f"Raw response: {response}")
 *                         print(f"Cleaned response: {clean_response}")
 */
                __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 835, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Error_parsing_AI_response, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 835, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_14);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 835, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

                /* "sQQideR/DatabaseContext.pyx":836
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")             # <<<<<<<<<<<<<<
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 */
                __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_response, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 836, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Raw_response, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 836, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_14);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 836, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

                /* "sQQideR/DatabaseContext.pyx":837
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 *                         print(f"Cleaned response: {clean_response}")             # <<<<<<<<<<<<<<
 *                     # Fallback handled below
 *             except Exception as e:
 */
                __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_clean_response, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 837, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Cleaned_response, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 837, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_14);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 837, __pyx_L26_error)
                __Pyx_GOTREF(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

                /* "sQQideR/DatabaseContext.pyx":834
 *                     )
 *                 except json.JSONDecodeError as e:
 *                     if self.verbose:             # <<<<<<<<<<<<<<
 *                         print(f"Error parsing AI response: {e}")
 *                         print(f"Raw response: {response}")
 */
              }
            }

            /* "sQQideR/DatabaseContext.pyx":833
 *                         keywords=parsed.get("keywords", [])
 *                     )
 *                 except json.JSONDecodeError as e:             # <<<<<<<<<<<<<<
 *                     if self.verbose:
 *                         print(f"Error parsing AI response: {e}")
 */
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                goto __pyx_L27;
              }
              __pyx_L26_error:;
              /*exception exit:*/{
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
                __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
                if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
                if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
                __Pyx_XGOTREF(__pyx_t_20);
                __Pyx_XGOTREF(__pyx_t_21);
                __Pyx_XGOTREF(__pyx_t_22);
                __Pyx_XGOTREF(__pyx_t_23);
                __Pyx_XGOTREF(__pyx_t_24);
                __Pyx_XGOTREF(__pyx_t_25);
                __pyx_t_17 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
                {
                  __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
                }
                if (PY_MAJOR_VERSION >= 3) {
                  __Pyx_XGIVEREF(__pyx_t_23);
                  __Pyx_XGIVEREF(__pyx_t_24);
                  __Pyx_XGIVEREF(__pyx_t_25);
                  __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
                }
                __Pyx_XGIVEREF(__pyx_t_20);
                __Pyx_XGIVEREF(__pyx_t_21);
                __Pyx_XGIVEREF(__pyx_t_22);
                __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
                __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
                __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
                goto __pyx_L16_except_error;
              }
              __pyx_L27:;
            }
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L15_exception_handled;
          }
          goto __pyx_L16_except_error;

          /* "sQQideR/DatabaseContext.pyx":818
 *                     clean_response = clean_response[:-3].strip()
 * 
 *                 try:             # <<<<<<<<<<<<<<
 *                     # Parse the cleaned JSON response
 *                     parsed = json.loads(clean_response)
 */
          __pyx_L16_except_error:;
          __Pyx_XGIVEREF(__pyx_t_11);
          __Pyx_XGIVEREF(__pyx_t_12);
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
          goto __pyx_L5_error;
          __pyx_L15_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_11);
          __Pyx_XGIVEREF(__pyx_t_12);
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
          __pyx_L19_try_end:;
        }

        /* "sQQideR/DatabaseContext.pyx":783
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Use the synchronous run method instead of async methods
 *                 prompt = f"""
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L10_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":839
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")
 */
      __pyx_t_18 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
      if (__pyx_t_18) {
        __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_ai", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_1, &__pyx_t_8) < 0) __PYX_ERR(0, 839, __pyx_L7_except_error)
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_e, __pyx_t_1);
        /*try:*/ {

          /* "sQQideR/DatabaseContext.pyx":840
 *                     # Fallback handled below
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"Error enhancing query with AI: {e}")
 *                 # Fallback handled below
 */
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 840, __pyx_L38_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_15); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 840, __pyx_L38_error)
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (__pyx_t_2) {

            /* "sQQideR/DatabaseContext.pyx":841
 *             except Exception as e:
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")             # <<<<<<<<<<<<<<
 *                 # Fallback handled below
 * 
 */
            __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 841, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Error_enhancing_query_with_AI, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 841, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            __pyx_t_15 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 841, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_15);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

            /* "sQQideR/DatabaseContext.pyx":840
 *                     # Fallback handled below
 *             except Exception as e:
 *                 if self.verbose:             # <<<<<<<<<<<<<<
 *                     print(f"Error enhancing query with AI: {e}")
 *                 # Fallback handled below
 */
          }
        }

        /* "sQQideR/DatabaseContext.pyx":839
 *                         print(f"Cleaned response: {clean_response}")
 *                     # Fallback handled below
 *             except Exception as e:             # <<<<<<<<<<<<<<
 *                 if self.verbose:
 *                     print(f"Error enhancing query with AI: {e}")
 */
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            goto __pyx_L39;
          }
          __pyx_L38_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23);
            if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_25);
            __Pyx_XGOTREF(__pyx_t_24);
            __Pyx_XGOTREF(__pyx_t_23);
            __pyx_t_18 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_26 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
            }
            if (PY_MAJOR_VERSION >= 3) {
              __Pyx_XGIVEREF(__pyx_t_25);
              __Pyx_XGIVEREF(__pyx_t_24);
              __Pyx_XGIVEREF(__pyx_t_23);
              __Pyx_ExceptionReset(__pyx_t_25, __pyx_t_24, __pyx_t_23);
            }
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ErrRestore(__pyx_t_13, __pyx_t_12, __pyx_t_11);
            __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0;
            __pyx_lineno = __pyx_t_18; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_26;
            goto __pyx_L7_except_error;
          }
          __pyx_L39:;
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L6_exception_handled;
      }
      goto __pyx_L7_except_error;

      /* "sQQideR/DatabaseContext.pyx":783
 *         enhanced_query = None
 *         if self.ai_base:
 *             try:             # <<<<<<<<<<<<<<
 *                 # Use the synchronous run method instead of async methods
 *                 prompt = f"""
 */
      __pyx_L7_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L6_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      __pyx_L10_try_end:;
    }

    /* "sQQideR/DatabaseContext.pyx":782
 *         # Get AI-enhanced query information using the synchronous run method
 *         enhanced_query = None
 *         if self.ai_base:             # <<<<<<<<<<<<<<
 *             try:
 *                 # Use the synchronous run method instead of async methods
 */
  }

  /* "sQQideR/DatabaseContext.pyx":845
 * 
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:             # <<<<<<<<<<<<<<
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 */
  __pyx_t_2 = (__pyx_v_enhanced_query == Py_None);
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":846
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]             # <<<<<<<<<<<<<<
 *             if self.verbose:
 *                 print(f"Using basic keyword extraction: {keywords}")
 */
    { /* enter inner scope */
      __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 846, __pyx_L48_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = PyUnicode_Split(__pyx_v_query_text, ((PyObject *)NULL), -1L); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L48_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9);
      __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_9);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 846, __pyx_L48_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 846, __pyx_L48_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_9, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L48_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
        __Pyx_XDECREF_SET(__pyx_8genexpr8__pyx_v_word, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_27 = PyObject_Length(__pyx_8genexpr8__pyx_v_word); if (unlikely(__pyx_t_27 == ((Py_ssize_t)-1))) __PYX_ERR(0, 846, __pyx_L48_error)
        __pyx_t_2 = (__pyx_t_27 > 3);
        if (__pyx_t_2) {
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr8__pyx_v_word, __pyx_n_s_lower); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 846, __pyx_L48_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_14 = NULL;
          __pyx_t_10 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_15))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_15);
            if (likely(__pyx_t_14)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
              __Pyx_INCREF(__pyx_t_14);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_15, function);
              __pyx_t_10 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_14, NULL};
            __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 0+__pyx_t_10);
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L48_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          }
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_8, (PyObject*)__pyx_t_1))) __PYX_ERR(0, 846, __pyx_L48_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_word); __pyx_8genexpr8__pyx_v_word = 0;
      goto __pyx_L53_exit_scope;
      __pyx_L48_error:;
      __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_word); __pyx_8genexpr8__pyx_v_word = 0;
      goto __pyx_L1_error;
      __pyx_L53_exit_scope:;
    } /* exit inner scope */
    __pyx_v_keywords = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":847
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_verbose); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":848
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 *                 print(f"Using basic keyword extraction: {keywords}")             # <<<<<<<<<<<<<<
 * 
 *             enhanced_query = SQLPrompt(
 */
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_keywords, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 848, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Using_basic_keyword_extraction, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 848, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 848, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":847
 *         if enhanced_query is None:
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:             # <<<<<<<<<<<<<<
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":850
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 *             enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_SQLPrompt); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);

    /* "sQQideR/DatabaseContext.pyx":851
 * 
 *             enhanced_query = SQLPrompt(
 *                 user_prompt=query_text,             # <<<<<<<<<<<<<<
 *                 ai_context=query_text,
 *                 buisness_domains=[],
 */
    __pyx_t_9 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_user_prompt, __pyx_v_query_text) < 0) __PYX_ERR(0, 851, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":852
 *             enhanced_query = SQLPrompt(
 *                 user_prompt=query_text,
 *                 ai_context=query_text,             # <<<<<<<<<<<<<<
 *                 buisness_domains=[],
 *                 keywords=keywords
 */
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_ai_context, __pyx_v_query_text) < 0) __PYX_ERR(0, 851, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":853
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 *                 buisness_domains=[],             # <<<<<<<<<<<<<<
 *                 keywords=keywords
 *             )
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_buisness_domains, __pyx_t_1) < 0) __PYX_ERR(0, 851, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":854
 *                 ai_context=query_text,
 *                 buisness_domains=[],
 *                 keywords=keywords             # <<<<<<<<<<<<<<
 *             )
 * 
 */
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_keywords, __pyx_v_keywords) < 0) __PYX_ERR(0, 851, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":850
 *                 print(f"Using basic keyword extraction: {keywords}")
 * 
 *             enhanced_query = SQLPrompt(             # <<<<<<<<<<<<<<
 *                 user_prompt=query_text,
 *                 ai_context=query_text,
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_empty_tuple, __pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF_SET(__pyx_v_enhanced_query, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":845
 * 
 *         # Fallback to basic keyword extraction if AI enhancement failed or AI is not available
 *         if enhanced_query is None:             # <<<<<<<<<<<<<<
 *             keywords = [word.lower() for word in query_text.split() if len(word) > 3]
 *             if self.verbose:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":858
 * 
 *         # Use the enhanced query for keyword search
 *         results, matches = self.search_by_keywords(enhanced_query.keywords)             # <<<<<<<<<<<<<<
 * 
 *         # Store the original query information
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_search_by_keywords); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_keywords); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_15 = NULL;
  __pyx_t_10 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_15)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_15);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_10 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_8};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 858, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_8);
    #else
    __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_15 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_15);
    index = 0; __pyx_t_9 = __pyx_t_28(__pyx_t_15); if (unlikely(!__pyx_t_9)) goto __pyx_L55_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_9);
    index = 1; __pyx_t_8 = __pyx_t_28(__pyx_t_15); if (unlikely(!__pyx_t_8)) goto __pyx_L55_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_15), 2) < 0) __PYX_ERR(0, 858, __pyx_L1_error)
    __pyx_t_28 = NULL;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    goto __pyx_L56_unpacking_done;
    __pyx_L55_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __pyx_t_28 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 858, __pyx_L1_error)
    __pyx_L56_unpacking_done:;
  }
  __pyx_v_results = __pyx_t_9;
  __pyx_t_9 = 0;
  __pyx_v_matches = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":861
 * 
 *         # Store the original query information
 *         ai_context = enhanced_query.ai_context if enhanced_query.ai_context else query_text             # <<<<<<<<<<<<<<
 * 
 *         # Group results by type
 */
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 861, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 861, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __pyx_t_8;
    __pyx_t_8 = 0;
  } else {
    __Pyx_INCREF(__pyx_v_query_text);
    __pyx_t_1 = __pyx_v_query_text;
  }
  __pyx_v_ai_context = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":864
 * 
 *         # Group results by type
 *         tables = []             # <<<<<<<<<<<<<<
 *         columns = []
 *         concepts = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tables = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":865
 *         # Group results by type
 *         tables = []
 *         columns = []             # <<<<<<<<<<<<<<
 *         concepts = []
 *         for node_id, data in results:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_columns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":866
 *         tables = []
 *         columns = []
 *         concepts = []             # <<<<<<<<<<<<<<
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_concepts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":867
 *         columns = []
 *         concepts = []
 *         for node_id, data in results:             # <<<<<<<<<<<<<<
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 */
  if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
    __pyx_t_1 = __pyx_v_results; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_29 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 867, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_29)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 867, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 867, __pyx_L1_error)
        #else
        __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 867, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 867, __pyx_L1_error)
        #else
        __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 867, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      }
    } else {
      __pyx_t_8 = __pyx_t_29(__pyx_t_1);
      if (unlikely(!__pyx_t_8)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 867, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_8);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
      PyObject* sequence = __pyx_t_8;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 867, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_15 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_15 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_15);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 867, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      #endif
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_14 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 867, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_14);
      index = 0; __pyx_t_9 = __pyx_t_28(__pyx_t_14); if (unlikely(!__pyx_t_9)) goto __pyx_L59_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_15 = __pyx_t_28(__pyx_t_14); if (unlikely(!__pyx_t_15)) goto __pyx_L59_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_15);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_14), 2) < 0) __PYX_ERR(0, 867, __pyx_L1_error)
      __pyx_t_28 = NULL;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      goto __pyx_L60_unpacking_done;
      __pyx_L59_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_28 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 867, __pyx_L1_error)
      __pyx_L60_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_15);
    __pyx_t_15 = 0;

    /* "sQQideR/DatabaseContext.pyx":868
 *         concepts = []
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 */
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_15))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_15);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_15, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_kp_u_table};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 868, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":869
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)             # <<<<<<<<<<<<<<
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 */
      __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 869, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":868
 *         concepts = []
 *         for node_id, data in results:
 *             if node_id.startswith("table:"):             # <<<<<<<<<<<<<<
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 */
      goto __pyx_L61;
    }

    /* "sQQideR/DatabaseContext.pyx":870
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 */
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 870, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_15))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_15);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_15, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_kp_u_column};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 870, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":871
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)             # <<<<<<<<<<<<<<
 *             elif node_id.startswith("concept:"):
 *                 concepts.append(node_id)
 */
      __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_columns, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 871, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":870
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 *             elif node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 */
      goto __pyx_L61;
    }

    /* "sQQideR/DatabaseContext.pyx":872
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):             # <<<<<<<<<<<<<<
 *                 concepts.append(node_id)
 * 
 */
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_15))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_15);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_15, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_kp_u_concept};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 872, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {

      /* "sQQideR/DatabaseContext.pyx":873
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):
 *                 concepts.append(node_id)             # <<<<<<<<<<<<<<
 * 
 *         # Look for domain-specific matches based on business domains
 */
      __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_concepts, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 873, __pyx_L1_error)

      /* "sQQideR/DatabaseContext.pyx":872
 *             elif node_id.startswith("column:"):
 *                 columns.append(node_id)
 *             elif node_id.startswith("concept:"):             # <<<<<<<<<<<<<<
 *                 concepts.append(node_id)
 * 
 */
    }
    __pyx_L61:;

    /* "sQQideR/DatabaseContext.pyx":867
 *         columns = []
 *         concepts = []
 *         for node_id, data in results:             # <<<<<<<<<<<<<<
 *             if node_id.startswith("table:"):
 *                 tables.append(node_id)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":876
 * 
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 876, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":877
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8);
      __pyx_t_6 = 0;
      __pyx_t_29 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 877, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_29)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 877, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 877, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 877, __pyx_L1_error)
            #endif
            if (__pyx_t_6 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 877, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 877, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_29(__pyx_t_8);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 877, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_domain, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":878
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])             # <<<<<<<<<<<<<<
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:
 */
      __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_search_by_keywords); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_9 = PyList_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_v_domain);
      __Pyx_GIVEREF(__pyx_v_domain);
      if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 0, __pyx_v_domain)) __PYX_ERR(0, 878, __pyx_L1_error);
      __pyx_t_14 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_15))) {
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_15);
        if (likely(__pyx_t_14)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_15, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 878, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_15 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_15 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_15);
        __Pyx_INCREF(__pyx_t_9);
        #else
        __pyx_t_15 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_14 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_14);
        index = 0; __pyx_t_15 = __pyx_t_28(__pyx_t_14); if (unlikely(!__pyx_t_15)) goto __pyx_L66_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_15);
        index = 1; __pyx_t_9 = __pyx_t_28(__pyx_t_14); if (unlikely(!__pyx_t_9)) goto __pyx_L66_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_9);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_14), 2) < 0) __PYX_ERR(0, 878, __pyx_L1_error)
        __pyx_t_28 = NULL;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        goto __pyx_L67_unpacking_done;
        __pyx_L66_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_28 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 878, __pyx_L1_error)
        __pyx_L67_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_domain_results, __pyx_t_15);
      __pyx_t_15 = 0;
      __Pyx_XDECREF_SET(__pyx_v__, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":879
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 */
      if (likely(PyList_CheckExact(__pyx_v_domain_results)) || PyTuple_CheckExact(__pyx_v_domain_results)) {
        __pyx_t_1 = __pyx_v_domain_results; __Pyx_INCREF(__pyx_t_1);
        __pyx_t_27 = 0;
        __pyx_t_31 = NULL;
      } else {
        __pyx_t_27 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_domain_results); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 879, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_31 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 879, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_31)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 879, __pyx_L1_error)
              #endif
              if (__pyx_t_27 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_27); __Pyx_INCREF(__pyx_t_9); __pyx_t_27++; if (unlikely((0 < 0))) __PYX_ERR(0, 879, __pyx_L1_error)
            #else
            __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_27); __pyx_t_27++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 879, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 879, __pyx_L1_error)
              #endif
              if (__pyx_t_27 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_27); __Pyx_INCREF(__pyx_t_9); __pyx_t_27++; if (unlikely((0 < 0))) __PYX_ERR(0, 879, __pyx_L1_error)
            #else
            __pyx_t_9 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_27); __pyx_t_27++; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 879, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            #endif
          }
        } else {
          __pyx_t_9 = __pyx_t_31(__pyx_t_1);
          if (unlikely(!__pyx_t_9)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 879, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_9);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_9))) || (PyList_CheckExact(__pyx_t_9))) {
          PyObject* sequence = __pyx_t_9;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 879, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_15 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_15 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_14);
          #else
          __pyx_t_15 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 879, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 879, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          #endif
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_16 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 879, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_16);
          index = 0; __pyx_t_15 = __pyx_t_28(__pyx_t_16); if (unlikely(!__pyx_t_15)) goto __pyx_L70_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_15);
          index = 1; __pyx_t_14 = __pyx_t_28(__pyx_t_16); if (unlikely(!__pyx_t_14)) goto __pyx_L70_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_14);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_16), 2) < 0) __PYX_ERR(0, 879, __pyx_L1_error)
          __pyx_t_28 = NULL;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L71_unpacking_done;
          __pyx_L70_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __pyx_t_28 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 879, __pyx_L1_error)
          __pyx_L71_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_15);
        __pyx_t_15 = 0;
        __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_14);
        __pyx_t_14 = 0;

        /* "sQQideR/DatabaseContext.pyx":880
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:             # <<<<<<<<<<<<<<
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 */
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_kp_u_table};
          __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 880, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }
        __pyx_t_32 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (__pyx_t_32) {
        } else {
          __pyx_t_2 = __pyx_t_32;
          goto __pyx_L73_bool_binop_done;
        }
        __pyx_t_32 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_tables, Py_NE)); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 880, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_32;
        __pyx_L73_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":881
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)             # <<<<<<<<<<<<<<
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 */
          __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 881, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":880
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 *                     if node_id.startswith("table:") and node_id not in tables:             # <<<<<<<<<<<<<<
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 */
          goto __pyx_L72;
        }

        /* "sQQideR/DatabaseContext.pyx":882
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:             # <<<<<<<<<<<<<<
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 */
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_kp_u_column};
          __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 882, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }
        __pyx_t_32 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (__pyx_t_32) {
        } else {
          __pyx_t_2 = __pyx_t_32;
          goto __pyx_L75_bool_binop_done;
        }
        __pyx_t_32 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_columns, Py_NE)); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 882, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_32;
        __pyx_L75_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":883
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)             # <<<<<<<<<<<<<<
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 *                         concepts.append(node_id)
 */
          __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_columns, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 883, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":882
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 *                     elif node_id.startswith("column:") and node_id not in columns:             # <<<<<<<<<<<<<<
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 */
          goto __pyx_L72;
        }

        /* "sQQideR/DatabaseContext.pyx":884
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:             # <<<<<<<<<<<<<<
 *                         concepts.append(node_id)
 * 
 */
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_15)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_kp_u_concept};
          __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }
        __pyx_t_32 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (__pyx_t_32) {
        } else {
          __pyx_t_2 = __pyx_t_32;
          goto __pyx_L77_bool_binop_done;
        }
        __pyx_t_32 = (__Pyx_PySequence_ContainsTF(__pyx_v_node_id, __pyx_v_concepts, Py_NE)); if (unlikely((__pyx_t_32 < 0))) __PYX_ERR(0, 884, __pyx_L1_error)
        __pyx_t_2 = __pyx_t_32;
        __pyx_L77_bool_binop_done:;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":885
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:
 *                         concepts.append(node_id)             # <<<<<<<<<<<<<<
 * 
 *         # If we found concepts but no tables, find related tables
 */
          __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_concepts, __pyx_v_node_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 885, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":884
 *                     elif node_id.startswith("column:") and node_id not in columns:
 *                         columns.append(node_id)
 *                     elif node_id.startswith("concept:") and node_id not in concepts:             # <<<<<<<<<<<<<<
 *                         concepts.append(node_id)
 * 
 */
        }
        __pyx_L72:;

        /* "sQQideR/DatabaseContext.pyx":879
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("table:") and node_id not in tables:
 *                         tables.append(node_id)
 */
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":877
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:
 *             for domain in enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *                 domain_results, _ = self.search_by_keywords([domain])
 *                 for node_id, data in domain_results:
 */
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":876
 * 
 *         # Look for domain-specific matches based on business domains
 *         if enhanced_query.buisness_domains:             # <<<<<<<<<<<<<<
 *             for domain in enhanced_query.buisness_domains:
 *                 domain_results, _ = self.search_by_keywords([domain])
 */
  }

  /* "sQQideR/DatabaseContext.pyx":888
 * 
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:             # <<<<<<<<<<<<<<
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 */
  __pyx_t_32 = (PyList_GET_SIZE(__pyx_v_concepts) != 0);
  if (__pyx_t_32) {
  } else {
    __pyx_t_2 = __pyx_t_32;
    goto __pyx_L82_bool_binop_done;
  }
  __pyx_t_32 = (PyList_GET_SIZE(__pyx_v_tables) != 0);
  __pyx_t_33 = (!__pyx_t_32);
  __pyx_t_2 = __pyx_t_33;
  __pyx_L82_bool_binop_done:;
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":889
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:
 *             for concept in concepts:             # <<<<<<<<<<<<<<
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 */
    __pyx_t_8 = __pyx_v_concepts; __Pyx_INCREF(__pyx_t_8);
    __pyx_t_6 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 889, __pyx_L1_error)
        #endif
        if (__pyx_t_6 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 889, __pyx_L1_error)
      #else
      __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_concept, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":890
 *         if concepts and not tables:
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_nodes); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_data, Py_True) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
      __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (likely(PyList_CheckExact(__pyx_t_14)) || PyTuple_CheckExact(__pyx_t_14)) {
        __pyx_t_1 = __pyx_t_14; __Pyx_INCREF(__pyx_t_1);
        __pyx_t_27 = 0;
        __pyx_t_29 = NULL;
      } else {
        __pyx_t_27 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_14); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 890, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_29 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 890, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      for (;;) {
        if (likely(!__pyx_t_29)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 890, __pyx_L1_error)
              #endif
              if (__pyx_t_27 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_14 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_27); __Pyx_INCREF(__pyx_t_14); __pyx_t_27++; if (unlikely((0 < 0))) __PYX_ERR(0, 890, __pyx_L1_error)
            #else
            __pyx_t_14 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_27); __pyx_t_27++; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 890, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 890, __pyx_L1_error)
              #endif
              if (__pyx_t_27 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_14 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_27); __Pyx_INCREF(__pyx_t_14); __pyx_t_27++; if (unlikely((0 < 0))) __PYX_ERR(0, 890, __pyx_L1_error)
            #else
            __pyx_t_14 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_27); __pyx_t_27++; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 890, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            #endif
          }
        } else {
          __pyx_t_14 = __pyx_t_29(__pyx_t_1);
          if (unlikely(!__pyx_t_14)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 890, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_14);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_14))) || (PyList_CheckExact(__pyx_t_14))) {
          PyObject* sequence = __pyx_t_14;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 890, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_15 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_15 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_15);
          #else
          __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_15 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          #endif
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_16 = PyObject_GetIter(__pyx_t_14); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_16);
          index = 0; __pyx_t_9 = __pyx_t_28(__pyx_t_16); if (unlikely(!__pyx_t_9)) goto __pyx_L88_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_9);
          index = 1; __pyx_t_15 = __pyx_t_28(__pyx_t_16); if (unlikely(!__pyx_t_15)) goto __pyx_L88_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_15);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_16), 2) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
          __pyx_t_28 = NULL;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L89_unpacking_done;
          __pyx_L88_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __pyx_t_28 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 890, __pyx_L1_error)
          __pyx_L89_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_9);
        __pyx_t_9 = 0;
        __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_15);
        __pyx_t_15 = 0;

        /* "sQQideR/DatabaseContext.pyx":891
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 */
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_startswith); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_9 = NULL;
        __pyx_t_10 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_15);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_15, function);
            __pyx_t_10 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_kp_u_column};
          __pyx_t_14 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (__pyx_t_2) {

          /* "sQQideR/DatabaseContext.pyx":892
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 */
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 892, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_neighbors); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 892, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_15 = NULL;
          __pyx_t_10 = 0;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_15)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_15);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
              __pyx_t_10 = 1;
            }
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_v_node_id};
            __pyx_t_14 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 892, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          if (likely(PyList_CheckExact(__pyx_t_14)) || PyTuple_CheckExact(__pyx_t_14)) {
            __pyx_t_9 = __pyx_t_14; __Pyx_INCREF(__pyx_t_9);
            __pyx_t_34 = 0;
            __pyx_t_31 = NULL;
          } else {
            __pyx_t_34 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_14); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 892, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_31 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_9); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 892, __pyx_L1_error)
          }
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          for (;;) {
            if (likely(!__pyx_t_31)) {
              if (likely(PyList_CheckExact(__pyx_t_9))) {
                {
                  Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_9);
                  #if !CYTHON_ASSUME_SAFE_MACROS
                  if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 892, __pyx_L1_error)
                  #endif
                  if (__pyx_t_34 >= __pyx_temp) break;
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_14 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_34); __Pyx_INCREF(__pyx_t_14); __pyx_t_34++; if (unlikely((0 < 0))) __PYX_ERR(0, 892, __pyx_L1_error)
                #else
                __pyx_t_14 = __Pyx_PySequence_ITEM(__pyx_t_9, __pyx_t_34); __pyx_t_34++; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 892, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_14);
                #endif
              } else {
                {
                  Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_9);
                  #if !CYTHON_ASSUME_SAFE_MACROS
                  if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 892, __pyx_L1_error)
                  #endif
                  if (__pyx_t_34 >= __pyx_temp) break;
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_14 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_34); __Pyx_INCREF(__pyx_t_14); __pyx_t_34++; if (unlikely((0 < 0))) __PYX_ERR(0, 892, __pyx_L1_error)
                #else
                __pyx_t_14 = __Pyx_PySequence_ITEM(__pyx_t_9, __pyx_t_34); __pyx_t_34++; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 892, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_14);
                #endif
              }
            } else {
              __pyx_t_14 = __pyx_t_31(__pyx_t_9);
              if (unlikely(!__pyx_t_14)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 892, __pyx_L1_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_14);
            }
            __Pyx_XDECREF_SET(__pyx_v_neighbor, __pyx_t_14);
            __pyx_t_14 = 0;

            /* "sQQideR/DatabaseContext.pyx":893
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:             # <<<<<<<<<<<<<<
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 */
            __pyx_t_14 = PyObject_RichCompare(__pyx_v_neighbor, __pyx_v_concept, Py_EQ); __Pyx_XGOTREF(__pyx_t_14); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 893, __pyx_L1_error)
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 893, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            if (__pyx_t_2) {

              /* "sQQideR/DatabaseContext.pyx":894
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")             # <<<<<<<<<<<<<<
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:
 */
              __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_node_id, __pyx_n_s_split); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 894, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __pyx_t_16 = NULL;
              __pyx_t_10 = 0;
              #if CYTHON_UNPACK_METHODS
              if (likely(PyMethod_Check(__pyx_t_15))) {
                __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_15);
                if (likely(__pyx_t_16)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
                  __Pyx_INCREF(__pyx_t_16);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_15, function);
                  __pyx_t_10 = 1;
                }
              }
              #endif
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_16, __pyx_kp_u__15};
                __pyx_t_14 = __Pyx_PyObject_FastCall(__pyx_t_15, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
                __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
                if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 894, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_14);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              }
              __pyx_t_15 = __Pyx_GetItemInt(__pyx_t_14, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 894, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_replace); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 894, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 894, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_15);
              __pyx_t_15 = 0;

              /* "sQQideR/DatabaseContext.pyx":895
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"             # <<<<<<<<<<<<<<
 *                                 if table_id not in tables:
 *                                     tables.append(table_id)
 */
              __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_table_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 895, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_15);
              __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_kp_u_table, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 895, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF_SET(__pyx_v_table_id, ((PyObject*)__pyx_t_14));
              __pyx_t_14 = 0;

              /* "sQQideR/DatabaseContext.pyx":896
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:             # <<<<<<<<<<<<<<
 *                                     tables.append(table_id)
 * 
 */
              __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_table_id, __pyx_v_tables, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 896, __pyx_L1_error)
              if (__pyx_t_2) {

                /* "sQQideR/DatabaseContext.pyx":897
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:
 *                                     tables.append(table_id)             # <<<<<<<<<<<<<<
 * 
 *         # Store all tables before filtering for relationship detection
 */
                __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_tables, __pyx_v_table_id); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 897, __pyx_L1_error)

                /* "sQQideR/DatabaseContext.pyx":896
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:             # <<<<<<<<<<<<<<
 *                                     tables.append(table_id)
 * 
 */
              }

              /* "sQQideR/DatabaseContext.pyx":893
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:             # <<<<<<<<<<<<<<
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 *                                 table_id = f"table:{table_name}"
 */
            }

            /* "sQQideR/DatabaseContext.pyx":892
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):             # <<<<<<<<<<<<<<
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")
 */
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

          /* "sQQideR/DatabaseContext.pyx":891
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):             # <<<<<<<<<<<<<<
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 */
        }

        /* "sQQideR/DatabaseContext.pyx":890
 *         if concepts and not tables:
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):             # <<<<<<<<<<<<<<
 *                     if node_id.startswith("column:"):
 *                         for neighbor in self.graph.neighbors(node_id):
 */
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":889
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:
 *             for concept in concepts:             # <<<<<<<<<<<<<<
 *                 for node_id, data in self.graph.nodes(data=True):
 *                     if node_id.startswith("column:"):
 */
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":888
 * 
 *         # If we found concepts but no tables, find related tables
 *         if concepts and not tables:             # <<<<<<<<<<<<<<
 *             for concept in concepts:
 *                 for node_id, data in self.graph.nodes(data=True):
 */
  }

  /* "sQQideR/DatabaseContext.pyx":900
 * 
 *         # Store all tables before filtering for relationship detection
 *         all_tables = tables.copy()             # <<<<<<<<<<<<<<
 * 
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 */
  __pyx_t_8 = __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyList_Type_copy, __pyx_v_tables); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_all_tables = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":903
 * 
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 *         tables = tables[:2]             # <<<<<<<<<<<<<<
 *         columns = columns[:2]
 *         concepts = concepts[:2]
 */
  __pyx_t_8 = __Pyx_PyList_GetSlice(__pyx_v_tables, 0, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF_SET(__pyx_v_tables, ((PyObject*)__pyx_t_8));
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":904
 *         # --- NEW FILTERING: Only include top 2 most relevant entries per category ---
 *         tables = tables[:2]
 *         columns = columns[:2]             # <<<<<<<<<<<<<<
 *         concepts = concepts[:2]
 * 
 */
  __pyx_t_8 = __Pyx_PyList_GetSlice(__pyx_v_columns, 0, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF_SET(__pyx_v_columns, ((PyObject*)__pyx_t_8));
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":905
 *         tables = tables[:2]
 *         columns = columns[:2]
 *         concepts = concepts[:2]             # <<<<<<<<<<<<<<
 * 
 *         # Get related business rules only from tables, columns, and concepts
 */
  __pyx_t_8 = __Pyx_PyList_GetSlice(__pyx_v_concepts, 0, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF_SET(__pyx_v_concepts, ((PyObject*)__pyx_t_8));
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":908
 * 
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []             # <<<<<<<<<<<<<<
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)
 */
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_all_rules = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":909
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []
 *         for node_id in tables + columns + concepts:             # <<<<<<<<<<<<<<
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)
 */
  __pyx_t_8 = PyNumber_Add(__pyx_v_tables, __pyx_v_columns); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = PyNumber_Add(__pyx_t_8, __pyx_v_concepts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8);
  __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 909, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 909, __pyx_L1_error)
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 909, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_node_id, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":910
 *         all_rules = []
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)             # <<<<<<<<<<<<<<
 *             all_rules.extend(node_rules)
 * 
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_get_related_rules); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_14 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_node_id};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_node_rules, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":911
 *         for node_id in tables + columns + concepts:
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)             # <<<<<<<<<<<<<<
 * 
 *         # Deduplicate and limit to 2 business rules
 */
    __pyx_t_30 = __Pyx_PyList_Extend(__pyx_v_all_rules, __pyx_v_node_rules); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 911, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":909
 *         # Get related business rules only from tables, columns, and concepts
 *         all_rules = []
 *         for node_id in tables + columns + concepts:             # <<<<<<<<<<<<<<
 *             node_rules = self.get_related_rules(node_id)
 *             all_rules.extend(node_rules)
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":914
 * 
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}             # <<<<<<<<<<<<<<
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc
 */
  __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_unique_rules = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":915
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:             # <<<<<<<<<<<<<<
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]
 */
  __pyx_t_8 = __pyx_v_all_rules; __Pyx_INCREF(__pyx_t_8);
  __pyx_t_6 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 915, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 915, __pyx_L1_error)
    #else
    __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 915, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 915, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_14);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_15 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_28 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_15);
      index = 0; __pyx_t_9 = __pyx_t_28(__pyx_t_15); if (unlikely(!__pyx_t_9)) goto __pyx_L103_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_14 = __pyx_t_28(__pyx_t_15); if (unlikely(!__pyx_t_14)) goto __pyx_L103_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_14);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_28(__pyx_t_15), 2) < 0) __PYX_ERR(0, 915, __pyx_L1_error)
      __pyx_t_28 = NULL;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      goto __pyx_L104_unpacking_done;
      __pyx_L103_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_28 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 915, __pyx_L1_error)
      __pyx_L104_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_rule_id, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_rule_desc, __pyx_t_14);
    __pyx_t_14 = 0;

    /* "sQQideR/DatabaseContext.pyx":916
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc             # <<<<<<<<<<<<<<
 *         filtered_rules = list(unique_rules.keys())[:2]
 * 
 */
    if (unlikely((PyDict_SetItem(__pyx_v_unique_rules, __pyx_v_rule_id, __pyx_v_rule_desc) < 0))) __PYX_ERR(0, 916, __pyx_L1_error)

    /* "sQQideR/DatabaseContext.pyx":915
 *         # Deduplicate and limit to 2 business rules
 *         unique_rules = {}
 *         for rule_id, rule_desc in all_rules:             # <<<<<<<<<<<<<<
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":917
 *         for rule_id, rule_desc in all_rules:
 *             unique_rules[rule_id] = rule_desc
 *         filtered_rules = list(unique_rules.keys())[:2]             # <<<<<<<<<<<<<<
 * 
 *         # Generate SQL if we have tables
 */
  __pyx_t_8 = __Pyx_PyDict_Keys(__pyx_v_unique_rules); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = __Pyx_PySequence_ListKeepNew(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyList_GetSlice(__pyx_t_1, 0, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_filtered_rules = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":920
 * 
 *         # Generate SQL if we have tables
 *         sql = None             # <<<<<<<<<<<<<<
 *         if tables:
 *             sql = self.generate_sql_from_nl(ai_context)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_sql = Py_None;

  /* "sQQideR/DatabaseContext.pyx":921
 *         # Generate SQL if we have tables
 *         sql = None
 *         if tables:             # <<<<<<<<<<<<<<
 *             sql = self.generate_sql_from_nl(ai_context)
 * 
 */
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_tables) != 0);
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":922
 *         sql = None
 *         if tables:
 *             sql = self.generate_sql_from_nl(ai_context)             # <<<<<<<<<<<<<<
 * 
 *         # Generate output with the filtered lists and all_tables for relationship detection
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_sql_from_nl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 922, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_14 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_ai_context};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 922, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF_SET(__pyx_v_sql, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":921
 *         # Generate SQL if we have tables
 *         sql = None
 *         if tables:             # <<<<<<<<<<<<<<
 *             sql = self.generate_sql_from_nl(ai_context)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":925
 * 
 *         # Generate output with the filtered lists and all_tables for relationship detection
 *         output = self.generate_output(             # <<<<<<<<<<<<<<
 *             query_text, results, matches, tables, columns, concepts,
 *             filtered_rules, sql, all_tables
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_generate_output); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "sQQideR/DatabaseContext.pyx":927
 *         output = self.generate_output(
 *             query_text, results, matches, tables, columns, concepts,
 *             filtered_rules, sql, all_tables             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_14 = NULL;
  __pyx_t_10 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_14)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_14);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_10 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[10] = {__pyx_t_14, __pyx_v_query_text, __pyx_v_results, __pyx_v_matches, __pyx_v_tables, __pyx_v_columns, __pyx_v_concepts, __pyx_v_filtered_rules, __pyx_v_sql, __pyx_v_all_tables};
    __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 9+__pyx_t_10);
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 925, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_v_output = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":932
 *         # Add the AI enhanced query information to the output
 *         output["ai_enhanced"] = {
 *             "keywords": enhanced_query.keywords,             # <<<<<<<<<<<<<<
 *             "business_domains": enhanced_query.buisness_domains,
 *             "enhanced_query": enhanced_query.ai_context
 */
  __pyx_t_8 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_keywords); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_keywords, __pyx_t_1) < 0) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":933
 *         output["ai_enhanced"] = {
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,             # <<<<<<<<<<<<<<
 *             "enhanced_query": enhanced_query.ai_context
 *         }
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_buisness_domains); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_business_domains, __pyx_t_1) < 0) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":934
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,
 *             "enhanced_query": enhanced_query.ai_context             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_enhanced_query, __pyx_n_s_ai_context); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_enhanced_query, __pyx_t_1) < 0) __PYX_ERR(0, 932, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":931
 * 
 *         # Add the AI enhanced query information to the output
 *         output["ai_enhanced"] = {             # <<<<<<<<<<<<<<
 *             "keywords": enhanced_query.keywords,
 *             "business_domains": enhanced_query.buisness_domains,
 */
  if (unlikely((PyObject_SetItem(__pyx_v_output, __pyx_n_u_ai_enhanced, __pyx_t_8) < 0))) __PYX_ERR(0, 931, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":937
 *         }
 * 
 *         if output_format == "markdown":             # <<<<<<<<<<<<<<
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_output_format, __pyx_n_u_markdown, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 937, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "sQQideR/DatabaseContext.pyx":938
 * 
 *         if output_format == "markdown":
 *             output["markdown"] = self.format_as_markdown(output)             # <<<<<<<<<<<<<<
 * 
 *         return output
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_format_as_markdown); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 938, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_14 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_output};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 938, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    if (unlikely((PyObject_SetItem(__pyx_v_output, __pyx_n_u_markdown, __pyx_t_8) < 0))) __PYX_ERR(0, 938, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":937
 *         }
 * 
 *         if output_format == "markdown":             # <<<<<<<<<<<<<<
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":940
 *             output["markdown"] = self.format_as_markdown(output)
 * 
 *         return output             # <<<<<<<<<<<<<<
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,
 */
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(PyDict_CheckExact(__pyx_v_output))||((__pyx_v_output) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_v_output))) __PYX_ERR(0, 940, __pyx_L1_error)
  __Pyx_INCREF(__pyx_v_output);
  __pyx_r = ((PyObject*)__pyx_v_output);
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":766
 *         return sql
 * 
 *     def process_query_with_ai(self, query_text: str, output_format: str = "json") -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query using AI enhancement and return context information.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.process_query_with_ai", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_enhanced_query);
  __Pyx_XDECREF(__pyx_v_prompt);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_clean_response);
  __Pyx_XDECREF(__pyx_v_first_newline);
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_keywords);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_matches);
  __Pyx_XDECREF(__pyx_v_ai_context);
  __Pyx_XDECREF(__pyx_v_tables);
  __Pyx_XDECREF(__pyx_v_columns);
  __Pyx_XDECREF(__pyx_v_concepts);
  __Pyx_XDECREF(__pyx_v_node_id);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_domain);
  __Pyx_XDECREF(__pyx_v_domain_results);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_concept);
  __Pyx_XDECREF(__pyx_v_neighbor);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_table_id);
  __Pyx_XDECREF(__pyx_v_all_tables);
  __Pyx_XDECREF(__pyx_v_all_rules);
  __Pyx_XDECREF(__pyx_v_node_rules);
  __Pyx_XDECREF(__pyx_v_unique_rules);
  __Pyx_XDECREF(__pyx_v_rule_id);
  __Pyx_XDECREF(__pyx_v_rule_desc);
  __Pyx_XDECREF(__pyx_v_filtered_rules);
  __Pyx_XDECREF(__pyx_v_sql);
  __Pyx_XDECREF(__pyx_v_output);
  __Pyx_XDECREF(__pyx_8genexpr8__pyx_v_word);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":942
 *         return output
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,             # <<<<<<<<<<<<<<
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_23generate_output(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_22generate_output, "\n        Generate structured output with all context information.\n        \n        Args:\n            query_text: Original query text\n            results: Search results from search_by_keywords\n            matches: Match information from search_by_keywords\n            tables: List of relevant table node IDs\n            columns: List of relevant column node IDs\n            concepts: List of relevant concept node IDs\n            rules: List of relevant rule node IDs\n            sql: Optional generated SQL query\n            all_tables: Optional list of all tables before filtering (for relationship detection)\n            \n        Returns:\n            Dictionary with structured context information\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_23generate_output = {"generate_output", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_23generate_output, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_22generate_output};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_23generate_output(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_query_text = 0;
  CYTHON_UNUSED PyObject *__pyx_v_results = 0;
  CYTHON_UNUSED PyObject *__pyx_v_matches = 0;
  PyObject *__pyx_v_tables = 0;
  PyObject *__pyx_v_columns = 0;
  PyObject *__pyx_v_concepts = 0;
  PyObject *__pyx_v_rules = 0;
  CYTHON_UNUSED PyObject *__pyx_v_sql = 0;
  PyObject *__pyx_v_all_tables = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate_output (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_query_text,&__pyx_n_s_results,&__pyx_n_s_matches,&__pyx_n_s_tables,&__pyx_n_s_columns,&__pyx_n_s_concepts,&__pyx_n_s_rules,&__pyx_n_s_sql,&__pyx_n_s_all_tables,0};

    /* "sQQideR/DatabaseContext.pyx":944
 *     def generate_output(self, query_text: str, results: List, matches: Dict,
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Generate structured output with all context information.
 */
    values[8] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    values[9] = __Pyx_Arg_NewRef_FASTCALL(((PyObject*)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case 10: values[9] = __Pyx_Arg_FASTCALL(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = __Pyx_Arg_FASTCALL(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = __Pyx_Arg_FASTCALL(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = __Pyx_Arg_FASTCALL(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_query_text)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 1); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_results)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 2); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_matches)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[3]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 3); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_tables)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[4]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 4); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_columns)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[5]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 5); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_concepts)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[6]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 6); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_rules)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[7]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, 7); __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_sql);
          if (value) { values[8] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_all_tables);
          if (value) { values[9] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 942, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "generate_output") < 0)) __PYX_ERR(0, 942, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case 10: values[9] = __Pyx_Arg_FASTCALL(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = __Pyx_Arg_FASTCALL(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = __Pyx_Arg_FASTCALL(__pyx_args, 7);
        values[6] = __Pyx_Arg_FASTCALL(__pyx_args, 6);
        values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_query_text = ((PyObject*)values[1]);
    __pyx_v_results = ((PyObject*)values[2]);
    __pyx_v_matches = ((PyObject*)values[3]);
    __pyx_v_tables = ((PyObject*)values[4]);
    __pyx_v_columns = ((PyObject*)values[5]);
    __pyx_v_concepts = ((PyObject*)values[6]);
    __pyx_v_rules = ((PyObject*)values[7]);
    __pyx_v_sql = ((PyObject*)values[8]);
    __pyx_v_all_tables = ((PyObject*)values[9]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("generate_output", 0, 8, 10, __pyx_nargs); __PYX_ERR(0, 942, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_output", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_query_text), (&PyUnicode_Type), 0, "query_text", 1))) __PYX_ERR(0, 942, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_results), (&PyList_Type), 0, "results", 1))) __PYX_ERR(0, 942, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_matches), (&PyDict_Type), 0, "matches", 1))) __PYX_ERR(0, 942, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tables), (&PyList_Type), 0, "tables", 1))) __PYX_ERR(0, 943, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_columns), (&PyList_Type), 0, "columns", 1))) __PYX_ERR(0, 943, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_concepts), (&PyList_Type), 0, "concepts", 1))) __PYX_ERR(0, 943, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rules), (&PyList_Type), 0, "rules", 1))) __PYX_ERR(0, 944, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sql), (&PyUnicode_Type), 1, "sql", 1))) __PYX_ERR(0, 944, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_all_tables), (&PyList_Type), 1, "all_tables", 1))) __PYX_ERR(0, 944, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_22generate_output(__pyx_self, __pyx_v_self, __pyx_v_query_text, __pyx_v_results, __pyx_v_matches, __pyx_v_tables, __pyx_v_columns, __pyx_v_concepts, __pyx_v_rules, __pyx_v_sql, __pyx_v_all_tables);

  /* "sQQideR/DatabaseContext.pyx":942
 *         return output
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,             # <<<<<<<<<<<<<<
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_22generate_output(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_query_text, CYTHON_UNUSED PyObject *__pyx_v_results, CYTHON_UNUSED PyObject *__pyx_v_matches, PyObject *__pyx_v_tables, PyObject *__pyx_v_columns, PyObject *__pyx_v_concepts, PyObject *__pyx_v_rules, CYTHON_UNUSED PyObject *__pyx_v_sql, PyObject *__pyx_v_all_tables) {
  PyObject *__pyx_v_output = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_j = NULL;
  PyObject *__pyx_v_edge_data = NULL;
  PyObject *__pyx_v_table_names = NULL;
  PyObject *__pyx_v_src_name = NULL;
  PyObject *__pyx_v_tgt_name = NULL;
  PyObject *__pyx_v_relationship = NULL;
  PyObject *__pyx_v_rel = NULL;
  PyObject *__pyx_v_table = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_column = NULL;
  PyObject *__pyx_v_col_name = NULL;
  PyObject *__pyx_8genexpr9__pyx_v_t = NULL;
  PyObject *__pyx_9genexpr10__pyx_v_c = NULL;
  PyObject *__pyx_9genexpr11__pyx_v_c = NULL;
  PyObject *__pyx_9genexpr12__pyx_v_r = NULL;
  PyObject *__pyx_9genexpr13__pyx_v_t = NULL;
  PyObject *__pyx_9genexpr14__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  unsigned int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("generate_output", 1);

  /* "sQQideR/DatabaseContext.pyx":963
 *         """
 *         output = {
 *             "query": query_text,             # <<<<<<<<<<<<<<
 *             "database_objects": {
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_query, __pyx_v_query_text) < 0) __PYX_ERR(0, 963, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":965
 *             "query": query_text,
 *             "database_objects": {
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 965, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "sQQideR/DatabaseContext.pyx":966
 *             "database_objects": {
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],             # <<<<<<<<<<<<<<
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 */
    __pyx_t_4 = __pyx_v_tables; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 966, __pyx_L5_error)
        #endif
        if (__pyx_t_5 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 966, __pyx_L5_error)
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_8genexpr9__pyx_v_t, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "sQQideR/DatabaseContext.pyx":965
 *             "query": query_text,
 *             "database_objects": {
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 */
      __pyx_t_6 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_id, __pyx_8genexpr9__pyx_v_t) < 0) __PYX_ERR(0, 965, __pyx_L5_error)
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr9__pyx_v_t, __pyx_n_s_replace); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_name, __pyx_t_8) < 0) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":966
 *             "database_objects": {
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],             # <<<<<<<<<<<<<<
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_nodes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_7, __pyx_8genexpr9__pyx_v_t); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 966, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_description, __pyx_t_8) < 0) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 965, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_t); __pyx_8genexpr9__pyx_v_t = 0;
    goto __pyx_L9_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_t); __pyx_8genexpr9__pyx_v_t = 0;
    goto __pyx_L1_error;
    __pyx_L9_exit_scope:;
  } /* exit inner scope */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_tables, __pyx_t_3) < 0) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  { /* enter inner scope */

    /* "sQQideR/DatabaseContext.pyx":967
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),             # <<<<<<<<<<<<<<
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 967, __pyx_L12_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "sQQideR/DatabaseContext.pyx":969
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],             # <<<<<<<<<<<<<<
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 */
    __pyx_t_4 = __pyx_v_columns; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 969, __pyx_L12_error)
        #endif
        if (__pyx_t_5 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 969, __pyx_L12_error)
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_9genexpr10__pyx_v_c, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "sQQideR/DatabaseContext.pyx":967
 *                 "tables": [{"id": t, "name": t.replace("table:", ""),
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),             # <<<<<<<<<<<<<<
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 */
      __pyx_t_6 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_id, __pyx_9genexpr10__pyx_v_c) < 0) __PYX_ERR(0, 967, __pyx_L12_error)
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_9genexpr10__pyx_v_c, __pyx_n_s_replace); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_name, __pyx_t_7) < 0) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "sQQideR/DatabaseContext.pyx":968
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),             # <<<<<<<<<<<<<<
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 968, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_nodes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 968, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_9genexpr10__pyx_v_c); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 968, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 968, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 968, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_data_type, __pyx_t_7) < 0) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "sQQideR/DatabaseContext.pyx":969
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],             # <<<<<<<<<<<<<<
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_nodes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_9genexpr10__pyx_v_c); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 969, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_description, __pyx_t_7) < 0) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 967, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_c); __pyx_9genexpr10__pyx_v_c = 0;
    goto __pyx_L16_exit_scope;
    __pyx_L12_error:;
    __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_c); __pyx_9genexpr10__pyx_v_c = 0;
    goto __pyx_L1_error;
    __pyx_L16_exit_scope:;
  } /* exit inner scope */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_columns, __pyx_t_3) < 0) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  { /* enter inner scope */

    /* "sQQideR/DatabaseContext.pyx":970
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),             # <<<<<<<<<<<<<<
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 *             },
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 970, __pyx_L19_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "sQQideR/DatabaseContext.pyx":971
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],             # <<<<<<<<<<<<<<
 *             },
 *             "business_rules": [{"id": r, "description": self.graph.nodes[r].get("description", "")} for r in rules]
 */
    __pyx_t_4 = __pyx_v_concepts; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 971, __pyx_L19_error)
        #endif
        if (__pyx_t_5 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 971, __pyx_L19_error)
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_9genexpr11__pyx_v_c, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "sQQideR/DatabaseContext.pyx":970
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),             # <<<<<<<<<<<<<<
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 *             },
 */
      __pyx_t_6 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_id, __pyx_9genexpr11__pyx_v_c) < 0) __PYX_ERR(0, 970, __pyx_L19_error)
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_9genexpr11__pyx_v_c, __pyx_n_s_replace); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_name, __pyx_t_8) < 0) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":971
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],             # <<<<<<<<<<<<<<
 *             },
 *             "business_rules": [{"id": r, "description": self.graph.nodes[r].get("description", "")} for r in rules]
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_nodes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_7, __pyx_9genexpr11__pyx_v_c); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 971, __pyx_L19_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_description, __pyx_t_8) < 0) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 970, __pyx_L19_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_c); __pyx_9genexpr11__pyx_v_c = 0;
    goto __pyx_L23_exit_scope;
    __pyx_L19_error:;
    __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_c); __pyx_9genexpr11__pyx_v_c = 0;
    goto __pyx_L1_error;
    __pyx_L23_exit_scope:;
  } /* exit inner scope */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_concepts, __pyx_t_3) < 0) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_database_objects, __pyx_t_2) < 0) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  { /* enter inner scope */

    /* "sQQideR/DatabaseContext.pyx":973
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 *             },
 *             "business_rules": [{"id": r, "description": self.graph.nodes[r].get("description", "")} for r in rules]             # <<<<<<<<<<<<<<
 *         }
 * 
 */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L26_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_v_rules; __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 973, __pyx_L26_error)
        #endif
        if (__pyx_t_5 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 973, __pyx_L26_error)
      #else
      __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_9genexpr12__pyx_v_r, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_id, __pyx_9genexpr12__pyx_v_r) < 0) __PYX_ERR(0, 973, __pyx_L26_error)
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_nodes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_9genexpr12__pyx_v_r); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_description, __pyx_t_6) < 0) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 973, __pyx_L26_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_r); __pyx_9genexpr12__pyx_v_r = 0;
    goto __pyx_L30_exit_scope;
    __pyx_L26_error:;
    __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_r); __pyx_9genexpr12__pyx_v_r = 0;
    goto __pyx_L1_error;
    __pyx_L30_exit_scope:;
  } /* exit inner scope */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_business_rules, __pyx_t_2) < 0) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_output = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":977
 * 
 *         # Add relationships between tables
 *         output["relationships"] = []             # <<<<<<<<<<<<<<
 * 
 *         # First, try to find direct relationships between the filtered tables
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 977, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyDict_SetItem(__pyx_v_output, __pyx_n_u_relationships, __pyx_t_1) < 0))) __PYX_ERR(0, 977, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":980
 * 
 *         # First, try to find direct relationships between the filtered tables
 *         if len(tables) > 1:             # <<<<<<<<<<<<<<
 *             for i in range(len(tables)-1):
 *                 for j in range(i+1, len(tables)):
 */
  __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_tables); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 980, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_5 > 1);
  if (__pyx_t_9) {

    /* "sQQideR/DatabaseContext.pyx":981
 *         # First, try to find direct relationships between the filtered tables
 *         if len(tables) > 1:
 *             for i in range(len(tables)-1):             # <<<<<<<<<<<<<<
 *                 for j in range(i+1, len(tables)):
 *                     # Check direct edge in both directions
 */
    __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_tables); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 981, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t((__pyx_t_5 - 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 981, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 981, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 981, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 981, __pyx_L1_error)
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 981, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 981, __pyx_L1_error)
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 981, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":982
 *         if len(tables) > 1:
 *             for i in range(len(tables)-1):
 *                 for j in range(i+1, len(tables)):             # <<<<<<<<<<<<<<
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 */
      __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_i, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = __Pyx_PyList_GET_SIZE(__pyx_v_tables); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 982, __pyx_L1_error)
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2)) __PYX_ERR(0, 982, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error);
      __pyx_t_2 = 0;
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
        __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4);
        __pyx_t_11 = 0;
        __pyx_t_12 = NULL;
      } else {
        __pyx_t_11 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 982, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_12 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 982, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      for (;;) {
        if (likely(!__pyx_t_12)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 982, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_11); __Pyx_INCREF(__pyx_t_3); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 982, __pyx_L1_error)
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 982, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_11); __Pyx_INCREF(__pyx_t_3); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 982, __pyx_L1_error)
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 982, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            #endif
          }
        } else {
          __pyx_t_3 = __pyx_t_12(__pyx_t_4);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 982, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":984
 *                 for j in range(i+1, len(tables)):
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])             # <<<<<<<<<<<<<<
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 984, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get_edge_data); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 984, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 984, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_j); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 984, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_7 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_2, __pyx_t_8};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 984, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_XDECREF_SET(__pyx_v_edge_data, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":985
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append({
 *                             "source": tables[i].replace("table:", ""),
 */
        __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_v_edge_data); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 985, __pyx_L1_error)
        if (__pyx_t_14) {
        } else {
          __pyx_t_9 = __pyx_t_14;
          goto __pyx_L37_bool_binop_done;
        }
        __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_join_condition, __pyx_v_edge_data, Py_EQ)); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 985, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_14;
        __pyx_L37_bool_binop_done:;
        if (__pyx_t_9) {

          /* "sQQideR/DatabaseContext.pyx":986
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({             # <<<<<<<<<<<<<<
 *                             "source": tables[i].replace("table:", ""),
 *                             "target": tables[j].replace("table:", ""),
 */
          __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 986, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);

          /* "sQQideR/DatabaseContext.pyx":987
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({
 *                             "source": tables[i].replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "target": tables[j].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),
 */
          __pyx_t_6 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_replace); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_source, __pyx_t_8) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":988
 *                         output["relationships"].append({
 *                             "source": tables[i].replace("table:", ""),
 *                             "target": tables[j].replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")
 */
          __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_j); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 988, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_replace); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 988, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 988, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_target, __pyx_t_8) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":989
 *                             "source": tables[i].replace("table:", ""),
 *                             "target": tables[j].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),             # <<<<<<<<<<<<<<
 *                             "description": edge_data.get("description", "")
 *                         })
 */
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 989, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 989, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_join_condition, __pyx_t_2) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "sQQideR/DatabaseContext.pyx":990
 *                             "target": tables[j].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")             # <<<<<<<<<<<<<<
 *                         })
 *                         continue
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 990, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 990, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_description, __pyx_t_8) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":986
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({             # <<<<<<<<<<<<<<
 *                             "source": tables[i].replace("table:", ""),
 *                             "target": tables[j].replace("table:", ""),
 */
          __pyx_t_15 = __Pyx_PyObject_Append(__pyx_t_3, __pyx_t_6); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 986, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "sQQideR/DatabaseContext.pyx":992
 *                             "description": edge_data.get("description", "")
 *                         })
 *                         continue             # <<<<<<<<<<<<<<
 *                     # Check reverse direction
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])
 */
          goto __pyx_L34_continue;

          /* "sQQideR/DatabaseContext.pyx":985
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append({
 *                             "source": tables[i].replace("table:", ""),
 */
        }

        /* "sQQideR/DatabaseContext.pyx":994
 *                         continue
 *                     # Check reverse direction
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])             # <<<<<<<<<<<<<<
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 994, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_get_edge_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 994, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_j); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 994, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 994, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_3, __pyx_t_2};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 994, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF_SET(__pyx_v_edge_data, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":995
 *                     # Check reverse direction
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append({
 *                             "source": tables[j].replace("table:", ""),
 */
        __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_v_edge_data); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 995, __pyx_L1_error)
        if (__pyx_t_14) {
        } else {
          __pyx_t_9 = __pyx_t_14;
          goto __pyx_L40_bool_binop_done;
        }
        __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_join_condition, __pyx_v_edge_data, Py_EQ)); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 995, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_14;
        __pyx_L40_bool_binop_done:;
        if (__pyx_t_9) {

          /* "sQQideR/DatabaseContext.pyx":996
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({             # <<<<<<<<<<<<<<
 *                             "source": tables[j].replace("table:", ""),
 *                             "target": tables[i].replace("table:", ""),
 */
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 996, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);

          /* "sQQideR/DatabaseContext.pyx":997
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({
 *                             "source": tables[j].replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "target": tables[i].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),
 */
          __pyx_t_8 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_j); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_replace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_source, __pyx_t_2) < 0) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "sQQideR/DatabaseContext.pyx":998
 *                         output["relationships"].append({
 *                             "source": tables[j].replace("table:", ""),
 *                             "target": tables[i].replace("table:", ""),             # <<<<<<<<<<<<<<
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")
 */
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_tables, __pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 998, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_replace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 998, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 998, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_target, __pyx_t_2) < 0) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "sQQideR/DatabaseContext.pyx":999
 *                             "source": tables[j].replace("table:", ""),
 *                             "target": tables[i].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),             # <<<<<<<<<<<<<<
 *                             "description": edge_data.get("description", "")
 *                         })
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 999, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 999, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_join_condition, __pyx_t_3) < 0) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":1000
 *                             "target": tables[i].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")             # <<<<<<<<<<<<<<
 *                         })
 * 
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1000, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1000, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_description, __pyx_t_2) < 0) __PYX_ERR(0, 997, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "sQQideR/DatabaseContext.pyx":996
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])
 *                     if edge_data and 'join_condition' in edge_data:
 *                         output["relationships"].append({             # <<<<<<<<<<<<<<
 *                             "source": tables[j].replace("table:", ""),
 *                             "target": tables[i].replace("table:", ""),
 */
          __pyx_t_15 = __Pyx_PyObject_Append(__pyx_t_6, __pyx_t_8); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 996, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "sQQideR/DatabaseContext.pyx":995
 *                     # Check reverse direction
 *                     edge_data = self.graph.get_edge_data(tables[j], tables[i])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append({
 *                             "source": tables[j].replace("table:", ""),
 */
        }

        /* "sQQideR/DatabaseContext.pyx":982
 *         if len(tables) > 1:
 *             for i in range(len(tables)-1):
 *                 for j in range(i+1, len(tables)):             # <<<<<<<<<<<<<<
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(tables[i], tables[j])
 */
        __pyx_L34_continue:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":981
 *         # First, try to find direct relationships between the filtered tables
 *         if len(tables) > 1:
 *             for i in range(len(tables)-1):             # <<<<<<<<<<<<<<
 *                 for j in range(i+1, len(tables)):
 *                     # Check direct edge in both directions
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":980
 * 
 *         # First, try to find direct relationships between the filtered tables
 *         if len(tables) > 1:             # <<<<<<<<<<<<<<
 *             for i in range(len(tables)-1):
 *                 for j in range(i+1, len(tables)):
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1004
 * 
 *         # If we didn't find relationships and we have access to all tables, check those too
 *         if not output["relationships"] and all_tables and len(all_tables) > 1:             # <<<<<<<<<<<<<<
 *             table_names = [t.replace("table:", "") for t in tables]
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 */
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_16 = (!__pyx_t_14);
  if (__pyx_t_16) {
  } else {
    __pyx_t_9 = __pyx_t_16;
    goto __pyx_L45_bool_binop_done;
  }
  __pyx_t_16 = (__pyx_v_all_tables != Py_None)&&(PyList_GET_SIZE(__pyx_v_all_tables) != 0);
  if (__pyx_t_16) {
  } else {
    __pyx_t_9 = __pyx_t_16;
    goto __pyx_L45_bool_binop_done;
  }
  if (unlikely(__pyx_v_all_tables == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1004, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_all_tables); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1004, __pyx_L1_error)
  __pyx_t_16 = (__pyx_t_5 > 1);
  __pyx_t_9 = __pyx_t_16;
  __pyx_L45_bool_binop_done:;
  if (__pyx_t_9) {

    /* "sQQideR/DatabaseContext.pyx":1005
 *         # If we didn't find relationships and we have access to all tables, check those too
 *         if not output["relationships"] and all_tables and len(all_tables) > 1:
 *             table_names = [t.replace("table:", "") for t in tables]             # <<<<<<<<<<<<<<
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 *             for i in range(len(all_tables)):
 */
    { /* enter inner scope */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1005, __pyx_L50_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __pyx_v_tables; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1005, __pyx_L50_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_8); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1005, __pyx_L50_error)
        #else
        __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1005, __pyx_L50_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
        __Pyx_XDECREF_SET(__pyx_9genexpr13__pyx_v_t, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_9genexpr13__pyx_v_t, __pyx_n_s_replace); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1005, __pyx_L50_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1005, __pyx_L50_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 1005, __pyx_L50_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_t); __pyx_9genexpr13__pyx_v_t = 0;
      goto __pyx_L54_exit_scope;
      __pyx_L50_error:;
      __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_t); __pyx_9genexpr13__pyx_v_t = 0;
      goto __pyx_L1_error;
      __pyx_L54_exit_scope:;
    } /* exit inner scope */
    __pyx_v_table_names = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1007
 *             table_names = [t.replace("table:", "") for t in tables]
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 *             for i in range(len(all_tables)):             # <<<<<<<<<<<<<<
 *                 src_name = all_tables[i].replace("table:", "")
 *                 if src_name not in table_names:
 */
    if (unlikely(__pyx_v_all_tables == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1007, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_all_tables); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1007, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1007, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1007, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1007, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1007, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1007, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1007, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1007, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1007, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1007, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1007, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1007, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1008
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 *             for i in range(len(all_tables)):
 *                 src_name = all_tables[i].replace("table:", "")             # <<<<<<<<<<<<<<
 *                 if src_name not in table_names:
 *                     continue
 */
      if (unlikely(__pyx_v_all_tables == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1008, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_all_tables, __pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_replace); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF_SET(__pyx_v_src_name, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1009
 *             for i in range(len(all_tables)):
 *                 src_name = all_tables[i].replace("table:", "")
 *                 if src_name not in table_names:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j in range(len(all_tables)):
 */
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_src_name, __pyx_v_table_names, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1009, __pyx_L1_error)
      if (__pyx_t_9) {

        /* "sQQideR/DatabaseContext.pyx":1010
 *                 src_name = all_tables[i].replace("table:", "")
 *                 if src_name not in table_names:
 *                     continue             # <<<<<<<<<<<<<<
 *                 for j in range(len(all_tables)):
 *                     if i == j:
 */
        goto __pyx_L55_continue;

        /* "sQQideR/DatabaseContext.pyx":1009
 *             for i in range(len(all_tables)):
 *                 src_name = all_tables[i].replace("table:", "")
 *                 if src_name not in table_names:             # <<<<<<<<<<<<<<
 *                     continue
 *                 for j in range(len(all_tables)):
 */
      }

      /* "sQQideR/DatabaseContext.pyx":1011
 *                 if src_name not in table_names:
 *                     continue
 *                 for j in range(len(all_tables)):             # <<<<<<<<<<<<<<
 *                     if i == j:
 *                         continue
 */
      if (unlikely(__pyx_v_all_tables == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 1011, __pyx_L1_error)
      }
      __pyx_t_11 = __Pyx_PyList_GET_SIZE(__pyx_v_all_tables); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1011, __pyx_L1_error)
      __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1011, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1011, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
        __pyx_t_4 = __pyx_t_6; __Pyx_INCREF(__pyx_t_4);
        __pyx_t_11 = 0;
        __pyx_t_12 = NULL;
      } else {
        __pyx_t_11 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1011, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_12 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1011, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      for (;;) {
        if (likely(!__pyx_t_12)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1011, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 1011, __pyx_L1_error)
            #else
            __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1011, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
              #if !CYTHON_ASSUME_SAFE_MACROS
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1011, __pyx_L1_error)
              #endif
              if (__pyx_t_11 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely((0 < 0))) __PYX_ERR(0, 1011, __pyx_L1_error)
            #else
            __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1011, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            #endif
          }
        } else {
          __pyx_t_6 = __pyx_t_12(__pyx_t_4);
          if (unlikely(!__pyx_t_6)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 1011, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":1012
 *                     continue
 *                 for j in range(len(all_tables)):
 *                     if i == j:             # <<<<<<<<<<<<<<
 *                         continue
 *                     tgt_name = all_tables[j].replace("table:", "")
 */
        __pyx_t_6 = PyObject_RichCompare(__pyx_v_i, __pyx_v_j, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1012, __pyx_L1_error)
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1012, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (__pyx_t_9) {

          /* "sQQideR/DatabaseContext.pyx":1013
 *                 for j in range(len(all_tables)):
 *                     if i == j:
 *                         continue             # <<<<<<<<<<<<<<
 *                     tgt_name = all_tables[j].replace("table:", "")
 *                     # Check direct edge in both directions
 */
          goto __pyx_L58_continue;

          /* "sQQideR/DatabaseContext.pyx":1012
 *                     continue
 *                 for j in range(len(all_tables)):
 *                     if i == j:             # <<<<<<<<<<<<<<
 *                         continue
 *                     tgt_name = all_tables[j].replace("table:", "")
 */
        }

        /* "sQQideR/DatabaseContext.pyx":1014
 *                     if i == j:
 *                         continue
 *                     tgt_name = all_tables[j].replace("table:", "")             # <<<<<<<<<<<<<<
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(all_tables[i], all_tables[j])
 */
        if (unlikely(__pyx_v_all_tables == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1014, __pyx_L1_error)
        }
        __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_all_tables, __pyx_v_j); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1014, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_replace); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1014, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1014, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF_SET(__pyx_v_tgt_name, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":1016
 *                     tgt_name = all_tables[j].replace("table:", "")
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(all_tables[i], all_tables[j])             # <<<<<<<<<<<<<<
 *                     if edge_data and 'join_condition' in edge_data:
 *                         relationship = {
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1016, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get_edge_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1016, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(__pyx_v_all_tables == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1016, __pyx_L1_error)
        }
        __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_all_tables, __pyx_v_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1016, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (unlikely(__pyx_v_all_tables == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 1016, __pyx_L1_error)
        }
        __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_all_tables, __pyx_v_j); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1016, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = NULL;
        __pyx_t_13 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
            __pyx_t_13 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_8, __pyx_t_3};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_13, 2+__pyx_t_13);
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1016, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }
        __Pyx_XDECREF_SET(__pyx_v_edge_data, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "sQQideR/DatabaseContext.pyx":1017
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(all_tables[i], all_tables[j])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         relationship = {
 *                             "source": src_name,
 */
        __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_v_edge_data); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 1017, __pyx_L1_error)
        if (__pyx_t_16) {
        } else {
          __pyx_t_9 = __pyx_t_16;
          goto __pyx_L62_bool_binop_done;
        }
        __pyx_t_16 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_join_condition, __pyx_v_edge_data, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 1017, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_16;
        __pyx_L62_bool_binop_done:;
        if (__pyx_t_9) {

          /* "sQQideR/DatabaseContext.pyx":1019
 *                     if edge_data and 'join_condition' in edge_data:
 *                         relationship = {
 *                             "source": src_name,             # <<<<<<<<<<<<<<
 *                             "target": tgt_name,
 *                             "join_condition": edge_data.get("join_condition", ""),
 */
          __pyx_t_6 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1019, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_source, __pyx_v_src_name) < 0) __PYX_ERR(0, 1019, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":1020
 *                         relationship = {
 *                             "source": src_name,
 *                             "target": tgt_name,             # <<<<<<<<<<<<<<
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")
 */
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_target, __pyx_v_tgt_name) < 0) __PYX_ERR(0, 1019, __pyx_L1_error)

          /* "sQQideR/DatabaseContext.pyx":1021
 *                             "source": src_name,
 *                             "target": tgt_name,
 *                             "join_condition": edge_data.get("join_condition", ""),             # <<<<<<<<<<<<<<
 *                             "description": edge_data.get("description", "")
 *                         }
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1021, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1021, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_join_condition, __pyx_t_3) < 0) __PYX_ERR(0, 1019, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "sQQideR/DatabaseContext.pyx":1022
 *                             "target": tgt_name,
 *                             "join_condition": edge_data.get("join_condition", ""),
 *                             "description": edge_data.get("description", "")             # <<<<<<<<<<<<<<
 *                         }
 *                         if relationship not in output["relationships"]:
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_edge_data, __pyx_n_s_get); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1022, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (PyDict_SetItem(__pyx_t_6, __pyx_n_u_description, __pyx_t_2) < 0) __PYX_ERR(0, 1019, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF_SET(__pyx_v_relationship, ((PyObject*)__pyx_t_6));
          __pyx_t_6 = 0;

          /* "sQQideR/DatabaseContext.pyx":1024
 *                             "description": edge_data.get("description", "")
 *                         }
 *                         if relationship not in output["relationships"]:             # <<<<<<<<<<<<<<
 *                             output["relationships"].append(relationship)
 * 
 */
          __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1024, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_relationship, __pyx_t_6, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1024, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (__pyx_t_9) {

            /* "sQQideR/DatabaseContext.pyx":1025
 *                         }
 *                         if relationship not in output["relationships"]:
 *                             output["relationships"].append(relationship)             # <<<<<<<<<<<<<<
 * 
 *         # If still no relationships found, look for joins in the metadata
 */
            __pyx_t_6 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1025, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_15 = __Pyx_PyObject_Append(__pyx_t_6, __pyx_v_relationship); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 1025, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

            /* "sQQideR/DatabaseContext.pyx":1024
 *                             "description": edge_data.get("description", "")
 *                         }
 *                         if relationship not in output["relationships"]:             # <<<<<<<<<<<<<<
 *                             output["relationships"].append(relationship)
 * 
 */
          }

          /* "sQQideR/DatabaseContext.pyx":1017
 *                     # Check direct edge in both directions
 *                     edge_data = self.graph.get_edge_data(all_tables[i], all_tables[j])
 *                     if edge_data and 'join_condition' in edge_data:             # <<<<<<<<<<<<<<
 *                         relationship = {
 *                             "source": src_name,
 */
        }

        /* "sQQideR/DatabaseContext.pyx":1011
 *                 if src_name not in table_names:
 *                     continue
 *                 for j in range(len(all_tables)):             # <<<<<<<<<<<<<<
 *                     if i == j:
 *                         continue
 */
        __pyx_L58_continue:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1007
 *             table_names = [t.replace("table:", "") for t in tables]
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 *             for i in range(len(all_tables)):             # <<<<<<<<<<<<<<
 *                 src_name = all_tables[i].replace("table:", "")
 *                 if src_name not in table_names:
 */
      __pyx_L55_continue:;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1004
 * 
 *         # If we didn't find relationships and we have access to all tables, check those too
 *         if not output["relationships"] and all_tables and len(all_tables) > 1:             # <<<<<<<<<<<<<<
 *             table_names = [t.replace("table:", "") for t in tables]
 *             # Check all possible pairs from all_tables that include at least one of our filtered tables
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1028
 * 
 *         # If still no relationships found, look for joins in the metadata
 *         if not output["relationships"] and self.metadata and "relationships" in self.metadata:             # <<<<<<<<<<<<<<
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:
 */
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = (!__pyx_t_16);
  if (__pyx_t_14) {
  } else {
    __pyx_t_9 = __pyx_t_14;
    goto __pyx_L68_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_14) {
  } else {
    __pyx_t_9 = __pyx_t_14;
    goto __pyx_L68_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_relationships, __pyx_t_1, Py_EQ)); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 1028, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = __pyx_t_14;
  __pyx_L68_bool_binop_done:;
  if (__pyx_t_9) {

    /* "sQQideR/DatabaseContext.pyx":1029
 *         # If still no relationships found, look for joins in the metadata
 *         if not output["relationships"] and self.metadata and "relationships" in self.metadata:
 *             table_names = [t.replace("table:", "") for t in tables]             # <<<<<<<<<<<<<<
 *             for rel in self.metadata["relationships"]:
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:
 */
    { /* enter inner scope */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1029, __pyx_L73_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __pyx_v_tables; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1029, __pyx_L73_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1029, __pyx_L73_error)
        #else
        __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1029, __pyx_L73_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_XDECREF_SET(__pyx_9genexpr14__pyx_v_t, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_9genexpr14__pyx_v_t, __pyx_n_s_replace); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1029, __pyx_L73_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1029, __pyx_L73_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 1029, __pyx_L73_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_9genexpr14__pyx_v_t); __pyx_9genexpr14__pyx_v_t = 0;
      goto __pyx_L77_exit_scope;
      __pyx_L73_error:;
      __Pyx_XDECREF(__pyx_9genexpr14__pyx_v_t); __pyx_9genexpr14__pyx_v_t = 0;
      goto __pyx_L1_error;
      __pyx_L77_exit_scope:;
    } /* exit inner scope */
    __Pyx_XDECREF_SET(__pyx_v_table_names, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1030
 *         if not output["relationships"] and self.metadata and "relationships" in self.metadata:
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:             # <<<<<<<<<<<<<<
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:
 *                     relationship = {
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1030, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_relationships); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1030, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1030, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1030, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1030, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1030, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1030, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1030, __pyx_L1_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1030, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1030, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1030, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_rel, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1031
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:             # <<<<<<<<<<<<<<
 *                     relationship = {
 *                         "source": rel["from_table"],
 */
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_from_table); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_t_4, __pyx_v_table_names, Py_EQ)); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 1031, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_14) {
      } else {
        __pyx_t_9 = __pyx_t_14;
        goto __pyx_L81_bool_binop_done;
      }
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_to_table); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_t_4, __pyx_v_table_names, Py_EQ)); if (unlikely((__pyx_t_14 < 0))) __PYX_ERR(0, 1031, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = __pyx_t_14;
      __pyx_L81_bool_binop_done:;
      if (__pyx_t_9) {

        /* "sQQideR/DatabaseContext.pyx":1033
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:
 *                     relationship = {
 *                         "source": rel["from_table"],             # <<<<<<<<<<<<<<
 *                         "target": rel["to_table"],
 *                         "join_condition": rel["join_condition"],
 */
        __pyx_t_4 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_from_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_source, __pyx_t_2) < 0) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sQQideR/DatabaseContext.pyx":1034
 *                     relationship = {
 *                         "source": rel["from_table"],
 *                         "target": rel["to_table"],             # <<<<<<<<<<<<<<
 *                         "join_condition": rel["join_condition"],
 *                         "description": rel["description"]
 */
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_to_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1034, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_target, __pyx_t_2) < 0) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sQQideR/DatabaseContext.pyx":1035
 *                         "source": rel["from_table"],
 *                         "target": rel["to_table"],
 *                         "join_condition": rel["join_condition"],             # <<<<<<<<<<<<<<
 *                         "description": rel["description"]
 *                     }
 */
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_join_condition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1035, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_join_condition, __pyx_t_2) < 0) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sQQideR/DatabaseContext.pyx":1036
 *                         "target": rel["to_table"],
 *                         "join_condition": rel["join_condition"],
 *                         "description": rel["description"]             # <<<<<<<<<<<<<<
 *                     }
 *                     if relationship not in output["relationships"]:
 */
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_description); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1036, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_description, __pyx_t_2) < 0) __PYX_ERR(0, 1033, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF_SET(__pyx_v_relationship, ((PyObject*)__pyx_t_4));
        __pyx_t_4 = 0;

        /* "sQQideR/DatabaseContext.pyx":1038
 *                         "description": rel["description"]
 *                     }
 *                     if relationship not in output["relationships"]:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append(relationship)
 * 
 */
        __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1038, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_relationship, __pyx_t_4, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1038, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (__pyx_t_9) {

          /* "sQQideR/DatabaseContext.pyx":1039
 *                     }
 *                     if relationship not in output["relationships"]:
 *                         output["relationships"].append(relationship)             # <<<<<<<<<<<<<<
 * 
 *         # Add sample data if available
 */
          __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_relationships); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1039, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_15 = __Pyx_PyObject_Append(__pyx_t_4, __pyx_v_relationship); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(0, 1039, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":1038
 *                         "description": rel["description"]
 *                     }
 *                     if relationship not in output["relationships"]:             # <<<<<<<<<<<<<<
 *                         output["relationships"].append(relationship)
 * 
 */
        }

        /* "sQQideR/DatabaseContext.pyx":1031
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:             # <<<<<<<<<<<<<<
 *                     relationship = {
 *                         "source": rel["from_table"],
 */
      }

      /* "sQQideR/DatabaseContext.pyx":1030
 *         if not output["relationships"] and self.metadata and "relationships" in self.metadata:
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:             # <<<<<<<<<<<<<<
 *                 if rel["from_table"] in table_names or rel["to_table"] in table_names:
 *                     relationship = {
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1028
 * 
 *         # If still no relationships found, look for joins in the metadata
 *         if not output["relationships"] and self.metadata and "relationships" in self.metadata:             # <<<<<<<<<<<<<<
 *             table_names = [t.replace("table:", "") for t in tables]
 *             for rel in self.metadata["relationships"]:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1042
 * 
 *         # Add sample data if available
 *         for table in tables:             # <<<<<<<<<<<<<<
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:
 */
  __pyx_t_1 = __pyx_v_tables; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1042, __pyx_L1_error)
      #endif
      if (__pyx_t_5 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1042, __pyx_L1_error)
    #else
    __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1042, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_table, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1043
 *         # Add sample data if available
 *         for table in tables:
 *             table_name = table.replace("table:", "")             # <<<<<<<<<<<<<<
 *             if "statistics" in self.graph.nodes[table]:
 *                 if "database_statistics" not in output:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_table, __pyx_n_s_replace); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1043, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1043, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "sQQideR/DatabaseContext.pyx":1044
 *         for table in tables:
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:             # <<<<<<<<<<<<<<
 *                 if "database_statistics" not in output:
 *                     output["database_statistics"] = {}
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_nodes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_v_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_statistics, __pyx_t_2, Py_EQ)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1044, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_9) {

      /* "sQQideR/DatabaseContext.pyx":1045
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:
 *                 if "database_statistics" not in output:             # <<<<<<<<<<<<<<
 *                     output["database_statistics"] = {}
 *                 output["database_statistics"][table_name] = self.graph.nodes[table]["statistics"]
 */
      __pyx_t_9 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_database_statistics, __pyx_v_output, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1045, __pyx_L1_error)
      if (__pyx_t_9) {

        /* "sQQideR/DatabaseContext.pyx":1046
 *             if "statistics" in self.graph.nodes[table]:
 *                 if "database_statistics" not in output:
 *                     output["database_statistics"] = {}             # <<<<<<<<<<<<<<
 *                 output["database_statistics"][table_name] = self.graph.nodes[table]["statistics"]
 * 
 */
        __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1046, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely((PyDict_SetItem(__pyx_v_output, __pyx_n_u_database_statistics, __pyx_t_2) < 0))) __PYX_ERR(0, 1046, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sQQideR/DatabaseContext.pyx":1045
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:
 *                 if "database_statistics" not in output:             # <<<<<<<<<<<<<<
 *                     output["database_statistics"] = {}
 *                 output["database_statistics"][table_name] = self.graph.nodes[table]["statistics"]
 */
      }

      /* "sQQideR/DatabaseContext.pyx":1047
 *                 if "database_statistics" not in output:
 *                     output["database_statistics"] = {}
 *                 output["database_statistics"][table_name] = self.graph.nodes[table]["statistics"]             # <<<<<<<<<<<<<<
 * 
 *         # Add column statistics if available
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_nodes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_v_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_statistics); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_database_statistics); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely((PyObject_SetItem(__pyx_t_2, __pyx_v_table_name, __pyx_t_4) < 0))) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1044
 *         for table in tables:
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:             # <<<<<<<<<<<<<<
 *                 if "database_statistics" not in output:
 *                     output["database_statistics"] = {}
 */
    }

    /* "sQQideR/DatabaseContext.pyx":1042
 * 
 *         # Add sample data if available
 *         for table in tables:             # <<<<<<<<<<<<<<
 *             table_name = table.replace("table:", "")
 *             if "statistics" in self.graph.nodes[table]:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":1050
 * 
 *         # Add column statistics if available
 *         for column in columns:             # <<<<<<<<<<<<<<
 *             if "statistics" in self.graph.nodes[column]:
 *                 col_name = column.replace("column:", "")
 */
  __pyx_t_1 = __pyx_v_columns; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1050, __pyx_L1_error)
      #endif
      if (__pyx_t_5 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely((0 < 0))) __PYX_ERR(0, 1050, __pyx_L1_error)
    #else
    __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1050, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_column, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1051
 *         # Add column statistics if available
 *         for column in columns:
 *             if "statistics" in self.graph.nodes[column]:             # <<<<<<<<<<<<<<
 *                 col_name = column.replace("column:", "")
 *                 if "column_statistics" not in output:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_nodes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_column); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_statistics, __pyx_t_4, Py_EQ)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1051, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_9) {

      /* "sQQideR/DatabaseContext.pyx":1052
 *         for column in columns:
 *             if "statistics" in self.graph.nodes[column]:
 *                 col_name = column.replace("column:", "")             # <<<<<<<<<<<<<<
 *                 if "column_statistics" not in output:
 *                     output["column_statistics"] = {}
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_column, __pyx_n_s_replace); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1052, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1052, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_v_col_name, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "sQQideR/DatabaseContext.pyx":1053
 *             if "statistics" in self.graph.nodes[column]:
 *                 col_name = column.replace("column:", "")
 *                 if "column_statistics" not in output:             # <<<<<<<<<<<<<<
 *                     output["column_statistics"] = {}
 *                 output["column_statistics"][col_name] = self.graph.nodes[column]["statistics"]
 */
      __pyx_t_9 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_column_statistics, __pyx_v_output, Py_NE)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1053, __pyx_L1_error)
      if (__pyx_t_9) {

        /* "sQQideR/DatabaseContext.pyx":1054
 *                 col_name = column.replace("column:", "")
 *                 if "column_statistics" not in output:
 *                     output["column_statistics"] = {}             # <<<<<<<<<<<<<<
 *                 output["column_statistics"][col_name] = self.graph.nodes[column]["statistics"]
 * 
 */
        __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1054, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely((PyDict_SetItem(__pyx_v_output, __pyx_n_u_column_statistics, __pyx_t_2) < 0))) __PYX_ERR(0, 1054, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "sQQideR/DatabaseContext.pyx":1053
 *             if "statistics" in self.graph.nodes[column]:
 *                 col_name = column.replace("column:", "")
 *                 if "column_statistics" not in output:             # <<<<<<<<<<<<<<
 *                     output["column_statistics"] = {}
 *                 output["column_statistics"][col_name] = self.graph.nodes[column]["statistics"]
 */
      }

      /* "sQQideR/DatabaseContext.pyx":1055
 *                 if "column_statistics" not in output:
 *                     output["column_statistics"] = {}
 *                 output["column_statistics"][col_name] = self.graph.nodes[column]["statistics"]             # <<<<<<<<<<<<<<
 * 
 *         return output
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_graph); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_nodes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_v_column); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_statistics); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_output, __pyx_n_u_column_statistics); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely((PyObject_SetItem(__pyx_t_2, __pyx_v_col_name, __pyx_t_4) < 0))) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1051
 *         # Add column statistics if available
 *         for column in columns:
 *             if "statistics" in self.graph.nodes[column]:             # <<<<<<<<<<<<<<
 *                 col_name = column.replace("column:", "")
 *                 if "column_statistics" not in output:
 */
    }

    /* "sQQideR/DatabaseContext.pyx":1050
 * 
 *         # Add column statistics if available
 *         for column in columns:             # <<<<<<<<<<<<<<
 *             if "statistics" in self.graph.nodes[column]:
 *                 col_name = column.replace("column:", "")
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":1057
 *                 output["column_statistics"][col_name] = self.graph.nodes[column]["statistics"]
 * 
 *         return output             # <<<<<<<<<<<<<<
 * 
 *     def format_as_markdown(self, output_data: Dict) -> str:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_output);
  __pyx_r = __pyx_v_output;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":942
 *         return output
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,             # <<<<<<<<<<<<<<
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.generate_output", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_output);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_j);
  __Pyx_XDECREF(__pyx_v_edge_data);
  __Pyx_XDECREF(__pyx_v_table_names);
  __Pyx_XDECREF(__pyx_v_src_name);
  __Pyx_XDECREF(__pyx_v_tgt_name);
  __Pyx_XDECREF(__pyx_v_relationship);
  __Pyx_XDECREF(__pyx_v_rel);
  __Pyx_XDECREF(__pyx_v_table);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_column);
  __Pyx_XDECREF(__pyx_v_col_name);
  __Pyx_XDECREF(__pyx_8genexpr9__pyx_v_t);
  __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_c);
  __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_c);
  __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_r);
  __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_t);
  __Pyx_XDECREF(__pyx_9genexpr14__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":1059
 *         return output
 * 
 *     def format_as_markdown(self, output_data: Dict) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Format context information as Markdown.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_25format_as_markdown(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_24format_as_markdown, "\n        Format context information as Markdown.\n        \n        Args:\n            output_data: Context data dictionary from generate_output\n            \n        Returns:\n            Markdown-formatted string\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_25format_as_markdown = {"format_as_markdown", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_25format_as_markdown, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_24format_as_markdown};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_25format_as_markdown(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_output_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("format_as_markdown (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_output_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1059, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_output_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1059, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("format_as_markdown", 1, 2, 2, 1); __PYX_ERR(0, 1059, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "format_as_markdown") < 0)) __PYX_ERR(0, 1059, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_output_data = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("format_as_markdown", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1059, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.format_as_markdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_output_data), (&PyDict_Type), 0, "output_data", 1))) __PYX_ERR(0, 1059, __pyx_L1_error)
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_24format_as_markdown(__pyx_self, __pyx_v_self, __pyx_v_output_data);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_24format_as_markdown(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, PyObject *__pyx_v_output_data) {
  PyObject *__pyx_v_md_output = NULL;
  PyObject *__pyx_v_table = NULL;
  PyObject *__pyx_v_column = NULL;
  PyObject *__pyx_v_concept = NULL;
  PyObject *__pyx_v_rel = NULL;
  PyObject *__pyx_v_rule = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_UCS4 __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("format_as_markdown", 1);

  /* "sQQideR/DatabaseContext.pyx":1069
 *             Markdown-formatted string
 *         """
 *         md_output = f"# Database Context for Query: {output_data['query']}\n\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add AI enhanced context if available
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = 0;
  __pyx_t_3 = 127;
  __Pyx_INCREF(__pyx_kp_u_Database_Context_for_Query);
  __pyx_t_2 += 30;
  __Pyx_GIVEREF(__pyx_kp_u_Database_Context_for_Query);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_Database_Context_for_Query);
  __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_query); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
  __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_kp_u__35);
  __pyx_t_2 += 2;
  __Pyx_GIVEREF(__pyx_kp_u__35);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__35);
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_md_output = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "sQQideR/DatabaseContext.pyx":1072
 * 
 *         # Add AI enhanced context if available
 *         if "ai_enhanced" in output_data and output_data["ai_enhanced"]["enhanced_query"]:             # <<<<<<<<<<<<<<
 *             md_output += f"## AI Enhanced Query\n\n"
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 */
  __pyx_t_7 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_ai_enhanced, __pyx_v_output_data, Py_EQ)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1072, __pyx_L1_error)
  if (__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_ai_enhanced); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_5, __pyx_n_u_enhanced_query); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1073
 *         # Add AI enhanced context if available
 *         if "ai_enhanced" in output_data and output_data["ai_enhanced"]["enhanced_query"]:
 *             md_output += f"## AI Enhanced Query\n\n"             # <<<<<<<<<<<<<<
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 *             if output_data["ai_enhanced"]["business_domains"]:
 */
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_AI_Enhanced_Query); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1073, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1074
 *         if "ai_enhanced" in output_data and output_data["ai_enhanced"]["enhanced_query"]:
 *             md_output += f"## AI Enhanced Query\n\n"
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"             # <<<<<<<<<<<<<<
 *             if output_data["ai_enhanced"]["business_domains"]:
 *                 md_output += f"**Business Domains**: {', '.join(output_data['ai_enhanced']['business_domains'])}\n\n"
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_ai_enhanced); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_enhanced_query); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_1, __pyx_kp_u__35); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1074, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1075
 *             md_output += f"## AI Enhanced Query\n\n"
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 *             if output_data["ai_enhanced"]["business_domains"]:             # <<<<<<<<<<<<<<
 *                 md_output += f"**Business Domains**: {', '.join(output_data['ai_enhanced']['business_domains'])}\n\n"
 * 
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_ai_enhanced); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1075, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_business_domains); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1075, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1075, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_6) {

      /* "sQQideR/DatabaseContext.pyx":1076
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 *             if output_data["ai_enhanced"]["business_domains"]:
 *                 md_output += f"**Business Domains**: {', '.join(output_data['ai_enhanced']['business_domains'])}\n\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add tables section
 */
      __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u_Business_Domains);
      __pyx_t_2 += 22;
      __Pyx_GIVEREF(__pyx_kp_u_Business_Domains);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_Business_Domains);
      __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_ai_enhanced); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_business_domains); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyUnicode_Join(__pyx_kp_u__2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_3;
      __pyx_t_2 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_INCREF(__pyx_kp_u__35);
      __pyx_t_2 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__35);
      PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u__35);
      __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_5, 3, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1076, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1075
 *             md_output += f"## AI Enhanced Query\n\n"
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 *             if output_data["ai_enhanced"]["business_domains"]:             # <<<<<<<<<<<<<<
 *                 md_output += f"**Business Domains**: {', '.join(output_data['ai_enhanced']['business_domains'])}\n\n"
 * 
 */
    }

    /* "sQQideR/DatabaseContext.pyx":1072
 * 
 *         # Add AI enhanced context if available
 *         if "ai_enhanced" in output_data and output_data["ai_enhanced"]["enhanced_query"]:             # <<<<<<<<<<<<<<
 *             md_output += f"## AI Enhanced Query\n\n"
 *             md_output += f"{output_data['ai_enhanced']['enhanced_query']}\n\n"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1079
 * 
 *         # Add tables section
 *         if output_data['database_objects']['tables']:             # <<<<<<<<<<<<<<
 *             md_output += "## Relevant Tables\n\n"
 *             for table in output_data['database_objects']['tables']:
 */
  __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_5, __pyx_n_u_tables); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1080
 *         # Add tables section
 *         if output_data['database_objects']['tables']:
 *             md_output += "## Relevant Tables\n\n"             # <<<<<<<<<<<<<<
 *             for table in output_data['database_objects']['tables']:
 *                 md_output += f"### {table['name']}\n\n"
 */
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Relevant_Tables); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1080, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1081
 *         if output_data['database_objects']['tables']:
 *             md_output += "## Relevant Tables\n\n"
 *             for table in output_data['database_objects']['tables']:             # <<<<<<<<<<<<<<
 *                 md_output += f"### {table['name']}\n\n"
 *                 md_output += f"{table['description']}\n\n"
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_tables); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1081, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
      __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1081, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1081, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1081, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_5); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1081, __pyx_L1_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1081, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1081, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_5); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1081, __pyx_L1_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1081, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1081, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_v_table, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1082
 *             md_output += "## Relevant Tables\n\n"
 *             for table in output_data['database_objects']['tables']:
 *                 md_output += f"### {table['name']}\n\n"             # <<<<<<<<<<<<<<
 *                 md_output += f"{table['description']}\n\n"
 * 
 */
      __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1082, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u__36);
      __pyx_t_9 += 4;
      __Pyx_GIVEREF(__pyx_kp_u__36);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u__36);
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table, __pyx_n_u_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1082, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1082, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_INCREF(__pyx_kp_u__35);
      __pyx_t_9 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__35);
      PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u__35);
      __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_5, 3, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1082, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1082, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1083
 *             for table in output_data['database_objects']['tables']:
 *                 md_output += f"### {table['name']}\n\n"
 *                 md_output += f"{table['description']}\n\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add columns section
 */
      __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_table, __pyx_n_u_description); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1083, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1083, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_10, __pyx_kp_u__35); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1083, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1083, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
      __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":1081
 *         if output_data['database_objects']['tables']:
 *             md_output += "## Relevant Tables\n\n"
 *             for table in output_data['database_objects']['tables']:             # <<<<<<<<<<<<<<
 *                 md_output += f"### {table['name']}\n\n"
 *                 md_output += f"{table['description']}\n\n"
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1079
 * 
 *         # Add tables section
 *         if output_data['database_objects']['tables']:             # <<<<<<<<<<<<<<
 *             md_output += "## Relevant Tables\n\n"
 *             for table in output_data['database_objects']['tables']:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1086
 * 
 *         # Add columns section
 *         if output_data['database_objects']['columns']:             # <<<<<<<<<<<<<<
 *             md_output += "## Relevant Columns\n\n"
 *             md_output += "| Column | Data Type | Description |\n"
 */
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_columns); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1086, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1087
 *         # Add columns section
 *         if output_data['database_objects']['columns']:
 *             md_output += "## Relevant Columns\n\n"             # <<<<<<<<<<<<<<
 *             md_output += "| Column | Data Type | Description |\n"
 *             md_output += "| ------ | --------- | ----------- |\n"
 */
    __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Relevant_Columns); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1087, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1088
 *         if output_data['database_objects']['columns']:
 *             md_output += "## Relevant Columns\n\n"
 *             md_output += "| Column | Data Type | Description |\n"             # <<<<<<<<<<<<<<
 *             md_output += "| ------ | --------- | ----------- |\n"
 *             for column in output_data['database_objects']['columns']:
 */
    __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Column_Data_Type_Description); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1088, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1089
 *             md_output += "## Relevant Columns\n\n"
 *             md_output += "| Column | Data Type | Description |\n"
 *             md_output += "| ------ | --------- | ----------- |\n"             # <<<<<<<<<<<<<<
 *             for column in output_data['database_objects']['columns']:
 *                 md_output += f"| {column['name']} | {column['data_type']} | {column['description']} |\n"
 */
    __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u__37); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1089, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1090
 *             md_output += "| Column | Data Type | Description |\n"
 *             md_output += "| ------ | --------- | ----------- |\n"
 *             for column in output_data['database_objects']['columns']:             # <<<<<<<<<<<<<<
 *                 md_output += f"| {column['name']} | {column['data_type']} | {column['description']} |\n"
 *             md_output += "\n"
 */
    __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1090, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_columns); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_10 = __pyx_t_1; __Pyx_INCREF(__pyx_t_10);
      __pyx_t_2 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1090, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_8 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1090, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_10))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_10);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1090, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1090, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_10, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_10);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1090, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1090, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_10, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_8(__pyx_t_10);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1090, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_column, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":1091
 *             md_output += "| ------ | --------- | ----------- |\n"
 *             for column in output_data['database_objects']['columns']:
 *                 md_output += f"| {column['name']} | {column['data_type']} | {column['description']} |\n"             # <<<<<<<<<<<<<<
 *             md_output += "\n"
 * 
 */
      __pyx_t_1 = PyTuple_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u__38);
      __pyx_t_9 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__38);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u__38);
      __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_column, __pyx_n_u_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__39);
      __pyx_t_9 += 3;
      __Pyx_GIVEREF(__pyx_kp_u__39);
      PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__39);
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_column, __pyx_n_u_data_type); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_kp_u__39);
      __pyx_t_9 += 3;
      __Pyx_GIVEREF(__pyx_kp_u__39);
      PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u__39);
      __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_column, __pyx_n_u_description); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__40);
      __pyx_t_9 += 3;
      __Pyx_GIVEREF(__pyx_kp_u__40);
      PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_kp_u__40);
      __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_1, 7, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":1090
 *             md_output += "| Column | Data Type | Description |\n"
 *             md_output += "| ------ | --------- | ----------- |\n"
 *             for column in output_data['database_objects']['columns']:             # <<<<<<<<<<<<<<
 *                 md_output += f"| {column['name']} | {column['data_type']} | {column['description']} |\n"
 *             md_output += "\n"
 */
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1092
 *             for column in output_data['database_objects']['columns']:
 *                 md_output += f"| {column['name']} | {column['data_type']} | {column['description']} |\n"
 *             md_output += "\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add business concepts
 */
    __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u__9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1092, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1086
 * 
 *         # Add columns section
 *         if output_data['database_objects']['columns']:             # <<<<<<<<<<<<<<
 *             md_output += "## Relevant Columns\n\n"
 *             md_output += "| Column | Data Type | Description |\n"
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1095
 * 
 *         # Add business concepts
 *         if output_data['database_objects']['concepts']:             # <<<<<<<<<<<<<<
 *             md_output += "## Business Concepts\n\n"
 *             for concept in output_data['database_objects']['concepts']:
 */
  __pyx_t_10 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_concepts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1095, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1096
 *         # Add business concepts
 *         if output_data['database_objects']['concepts']:
 *             md_output += "## Business Concepts\n\n"             # <<<<<<<<<<<<<<
 *             for concept in output_data['database_objects']['concepts']:
 *                 md_output += f"### {concept['name']}\n\n"
 */
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Business_Concepts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1096, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1097
 *         if output_data['database_objects']['concepts']:
 *             md_output += "## Business Concepts\n\n"
 *             for concept in output_data['database_objects']['concepts']:             # <<<<<<<<<<<<<<
 *                 md_output += f"### {concept['name']}\n\n"
 *                 md_output += f"{concept['description']}\n\n"
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_database_objects); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1097, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_concepts); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1097, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_10)) || PyTuple_CheckExact(__pyx_t_10)) {
      __pyx_t_1 = __pyx_t_10; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1097, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1097, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1097, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_10); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1097, __pyx_L1_error)
          #else
          __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1097, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1097, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_10); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1097, __pyx_L1_error)
          #else
          __pyx_t_10 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1097, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        }
      } else {
        __pyx_t_10 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_10)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1097, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_concept, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":1098
 *             md_output += "## Business Concepts\n\n"
 *             for concept in output_data['database_objects']['concepts']:
 *                 md_output += f"### {concept['name']}\n\n"             # <<<<<<<<<<<<<<
 *                 md_output += f"{concept['description']}\n\n"
 * 
 */
      __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1098, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u__36);
      __pyx_t_9 += 4;
      __Pyx_GIVEREF(__pyx_kp_u__36);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_kp_u__36);
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_concept, __pyx_n_u_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1098, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1098, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_kp_u__35);
      __pyx_t_9 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__35);
      PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_kp_u__35);
      __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_10, 3, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1098, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1098, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_10));
      __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":1099
 *             for concept in output_data['database_objects']['concepts']:
 *                 md_output += f"### {concept['name']}\n\n"
 *                 md_output += f"{concept['description']}\n\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add relationships
 */
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_concept, __pyx_n_u_description); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = __Pyx_PyUnicode_ConcatInPlace(__pyx_t_5, __pyx_kp_u__35); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1097
 *         if output_data['database_objects']['concepts']:
 *             md_output += "## Business Concepts\n\n"
 *             for concept in output_data['database_objects']['concepts']:             # <<<<<<<<<<<<<<
 *                 md_output += f"### {concept['name']}\n\n"
 *                 md_output += f"{concept['description']}\n\n"
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1095
 * 
 *         # Add business concepts
 *         if output_data['database_objects']['concepts']:             # <<<<<<<<<<<<<<
 *             md_output += "## Business Concepts\n\n"
 *             for concept in output_data['database_objects']['concepts']:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1102
 * 
 *         # Add relationships
 *         if 'relationships' in output_data and output_data['relationships']:             # <<<<<<<<<<<<<<
 *             md_output += "## Table Relationships\n\n"
 *             for rel in output_data['relationships']:
 */
  __pyx_t_7 = (__Pyx_PyDict_ContainsTF(__pyx_n_u_relationships, __pyx_v_output_data, Py_EQ)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1102, __pyx_L1_error)
  if (__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 1102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __pyx_t_7;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1103
 *         # Add relationships
 *         if 'relationships' in output_data and output_data['relationships']:
 *             md_output += "## Table Relationships\n\n"             # <<<<<<<<<<<<<<
 *             for rel in output_data['relationships']:
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 */
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Table_Relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1104
 *         if 'relationships' in output_data and output_data['relationships']:
 *             md_output += "## Table Relationships\n\n"
 *             for rel in output_data['relationships']:             # <<<<<<<<<<<<<<
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_5 = __pyx_t_1; __Pyx_INCREF(__pyx_t_5);
      __pyx_t_2 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1104, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_5);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1104, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1104, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1104, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_5);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1104, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1104, __pyx_L1_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1104, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_8(__pyx_t_5);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1104, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_rel, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":1105
 *             md_output += "## Table Relationships\n\n"
 *             for rel in output_data['relationships']:
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"             # <<<<<<<<<<<<<<
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 *                 if rel['description']:
 */
      __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u__41);
      __pyx_t_9 += 4;
      __Pyx_GIVEREF(__pyx_kp_u__41);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u__41);
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_source); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__42);
      __pyx_t_3 = (65535 > __pyx_t_3) ? 65535 : __pyx_t_3;
      __pyx_t_9 += 7;
      __Pyx_GIVEREF(__pyx_kp_u__42);
      PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__42);
      __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_target); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_INCREF(__pyx_kp_u__43);
      __pyx_t_9 += 3;
      __Pyx_GIVEREF(__pyx_kp_u__43);
      PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u__43);
      __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_1, 5, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":1106
 *             for rel in output_data['relationships']:
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"             # <<<<<<<<<<<<<<
 *                 if rel['description']:
 *                     md_output += f"  * Description: {rel['description']}\n"
 */
      __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u_Join_condition);
      __pyx_t_9 += 21;
      __Pyx_GIVEREF(__pyx_kp_u_Join_condition);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_Join_condition);
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_join_condition); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__44);
      __pyx_t_9 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__44);
      PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__44);
      __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "sQQideR/DatabaseContext.pyx":1107
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 *                 if rel['description']:             # <<<<<<<<<<<<<<
 *                     md_output += f"  * Description: {rel['description']}\n"
 *             md_output += "\n"
 */
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_description); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1107, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1107, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_6) {

        /* "sQQideR/DatabaseContext.pyx":1108
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 *                 if rel['description']:
 *                     md_output += f"  * Description: {rel['description']}\n"             # <<<<<<<<<<<<<<
 *             md_output += "\n"
 * 
 */
        __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1108, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = 0;
        __pyx_t_3 = 127;
        __Pyx_INCREF(__pyx_kp_u_Description);
        __pyx_t_9 += 17;
        __Pyx_GIVEREF(__pyx_kp_u_Description);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_Description);
        __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rel, __pyx_n_u_description); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1108, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1108, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_3;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_10);
        __pyx_t_10 = 0;
        __Pyx_INCREF(__pyx_kp_u__9);
        __pyx_t_9 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__9);
        PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__9);
        __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_1, 3, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1108, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1108, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "sQQideR/DatabaseContext.pyx":1107
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 *                 if rel['description']:             # <<<<<<<<<<<<<<
 *                     md_output += f"  * Description: {rel['description']}\n"
 *             md_output += "\n"
 */
      }

      /* "sQQideR/DatabaseContext.pyx":1104
 *         if 'relationships' in output_data and output_data['relationships']:
 *             md_output += "## Table Relationships\n\n"
 *             for rel in output_data['relationships']:             # <<<<<<<<<<<<<<
 *                 md_output += f"* **{rel['source']}**  **{rel['target']}**\n"
 *                 md_output += f"  * Join condition: `{rel['join_condition']}`\n"
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":1109
 *                 if rel['description']:
 *                     md_output += f"  * Description: {rel['description']}\n"
 *             md_output += "\n"             # <<<<<<<<<<<<<<
 * 
 *         # Add business rules
 */
    __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u__9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":1102
 * 
 *         # Add relationships
 *         if 'relationships' in output_data and output_data['relationships']:             # <<<<<<<<<<<<<<
 *             md_output += "## Table Relationships\n\n"
 *             for rel in output_data['relationships']:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1112
 * 
 *         # Add business rules
 *         if output_data['business_rules']:             # <<<<<<<<<<<<<<
 *             md_output += "## Business Rules\n\n"
 *             for rule in output_data['business_rules']:
 */
  __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_business_rules); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {

    /* "sQQideR/DatabaseContext.pyx":1113
 *         # Add business rules
 *         if output_data['business_rules']:
 *             md_output += "## Business Rules\n\n"             # <<<<<<<<<<<<<<
 *             for rule in output_data['business_rules']:
 *                 md_output += f"* {rule['description']}\n"
 */
    __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u_Business_Rules); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "sQQideR/DatabaseContext.pyx":1114
 *         if output_data['business_rules']:
 *             md_output += "## Business Rules\n\n"
 *             for rule in output_data['business_rules']:             # <<<<<<<<<<<<<<
 *                 md_output += f"* {rule['description']}\n"
 *             md_output += "\n"
 */
    __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_output_data, __pyx_n_u_business_rules); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
      __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1114, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1114, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_5); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1114, __pyx_L1_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1114, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1114, __pyx_L1_error)
            #endif
            if (__pyx_t_2 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_5); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 1114, __pyx_L1_error)
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1114, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1114, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_v_rule, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1115
 *             md_output += "## Business Rules\n\n"
 *             for rule in output_data['business_rules']:
 *                 md_output += f"* {rule['description']}\n"             # <<<<<<<<<<<<<<
 *             md_output += "\n"
 * 
 */
      __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = 0;
      __pyx_t_3 = 127;
      __Pyx_INCREF(__pyx_kp_u__45);
      __pyx_t_9 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__45);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u__45);
      __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rule, __pyx_n_u_description); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_3 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_3) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_3;
      __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__9);
      __pyx_t_9 += 1;
      __Pyx_GIVEREF(__pyx_kp_u__9);
      PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u__9);
      __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_5, 3, __pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "sQQideR/DatabaseContext.pyx":1114
 *         if output_data['business_rules']:
 *             md_output += "## Business Rules\n\n"
 *             for rule in output_data['business_rules']:             # <<<<<<<<<<<<<<
 *                 md_output += f"* {rule['description']}\n"
 *             md_output += "\n"
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1116
 *             for rule in output_data['business_rules']:
 *                 md_output += f"* {rule['description']}\n"
 *             md_output += "\n"             # <<<<<<<<<<<<<<
 * 
 *         return md_output
 */
    __pyx_t_1 = __Pyx_PyUnicode_ConcatInPlace(__pyx_v_md_output, __pyx_kp_u__9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_md_output, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1112
 * 
 *         # Add business rules
 *         if output_data['business_rules']:             # <<<<<<<<<<<<<<
 *             md_output += "## Business Rules\n\n"
 *             for rule in output_data['business_rules']:
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1118
 *             md_output += "\n"
 * 
 *         return md_output             # <<<<<<<<<<<<<<
 * 
 *     def _get_schema_summary(self) -> Dict:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_md_output);
  __pyx_r = __pyx_v_md_output;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":1059
 *         return output
 * 
 *     def format_as_markdown(self, output_data: Dict) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Format context information as Markdown.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService.format_as_markdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_md_output);
  __Pyx_XDECREF(__pyx_v_table);
  __Pyx_XDECREF(__pyx_v_column);
  __Pyx_XDECREF(__pyx_v_concept);
  __Pyx_XDECREF(__pyx_v_rel);
  __Pyx_XDECREF(__pyx_v_rule);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sQQideR/DatabaseContext.pyx":1120
 *         return md_output
 * 
 *     def _get_schema_summary(self) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Generate a summary of the database schema for AI context.
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_27_get_schema_summary(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_26_get_schema_summary, "\n        Generate a summary of the database schema for AI context.\n        \n        Returns:\n            Dictionary with schema information\n        ");
static PyMethodDef __pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_27_get_schema_summary = {"_get_schema_summary", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_27_get_schema_summary, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7sQQideR_15DatabaseContext_22DatabaseContextService_26_get_schema_summary};
static PyObject *__pyx_pw_7sQQideR_15DatabaseContext_22DatabaseContextService_27_get_schema_summary(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_schema_summary (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_self)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1120, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "_get_schema_summary") < 0)) __PYX_ERR(0, 1120, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_schema_summary", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1120, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService._get_schema_summary", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_26_get_schema_summary(__pyx_self, __pyx_v_self);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sQQideR_15DatabaseContext_22DatabaseContextService_26_get_schema_summary(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_v_schema = NULL;
  PyObject *__pyx_v_table_name = NULL;
  PyObject *__pyx_v_table_data = NULL;
  PyObject *__pyx_v_col_name = NULL;
  PyObject *__pyx_v_col_data = NULL;
  PyObject *__pyx_v_rel = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  unsigned int __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  PyObject *(*__pyx_t_17)(PyObject *);
  int __pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_schema_summary", 1);

  /* "sQQideR/DatabaseContext.pyx":1127
 *             Dictionary with schema information
 *         """
 *         if not self.metadata:             # <<<<<<<<<<<<<<
 *             return {}
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "sQQideR/DatabaseContext.pyx":1128
 *         """
 *         if not self.metadata:
 *             return {}             # <<<<<<<<<<<<<<
 * 
 *         schema = {
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "sQQideR/DatabaseContext.pyx":1127
 *             Dictionary with schema information
 *         """
 *         if not self.metadata:             # <<<<<<<<<<<<<<
 *             return {}
 * 
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1131
 * 
 *         schema = {
 *             "tables": {}             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tables, __pyx_t_4) < 0) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_schema = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":1135
 * 
 *         # Add basic table and column information
 *         for table_name, table_data in self.metadata.get("tables", {}).items():             # <<<<<<<<<<<<<<
 *             schema["tables"][table_name] = {
 *                 "description": table_data.get("description", ""),
 */
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = NULL;
  __pyx_t_11 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_11 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_10, __pyx_n_u_tables, __pyx_t_8};
    __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  if (unlikely(__pyx_t_4 == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 1135, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_dict_iterator(__pyx_t_4, 0, __pyx_n_s_items, (&__pyx_t_6), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_12 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_6, &__pyx_t_5, &__pyx_t_9, &__pyx_t_4, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_12 == 0)) break;
    if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(0, 1135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_table_name, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_table_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1137
 *         for table_name, table_data in self.metadata.get("tables", {}).items():
 *             schema["tables"][table_name] = {
 *                 "description": table_data.get("description", ""),             # <<<<<<<<<<<<<<
 *                 "columns": {}
 *             }
 */
    __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_table_data, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_description, __pyx_t_8) < 0) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":1138
 *             schema["tables"][table_name] = {
 *                 "description": table_data.get("description", ""),
 *                 "columns": {}             # <<<<<<<<<<<<<<
 *             }
 *             for col_name, col_data in table_data.get("columns", {}).items():
 */
    __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_columns, __pyx_t_8) < 0) __PYX_ERR(0, 1137, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "sQQideR/DatabaseContext.pyx":1136
 *         # Add basic table and column information
 *         for table_name, table_data in self.metadata.get("tables", {}).items():
 *             schema["tables"][table_name] = {             # <<<<<<<<<<<<<<
 *                 "description": table_data.get("description", ""),
 *                 "columns": {}
 */
    __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_schema, __pyx_n_u_tables); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (unlikely((PyObject_SetItem(__pyx_t_8, __pyx_v_table_name, __pyx_t_4) < 0))) __PYX_ERR(0, 1136, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1140
 *                 "columns": {}
 *             }
 *             for col_name, col_data in table_data.get("columns", {}).items():             # <<<<<<<<<<<<<<
 *                 schema["tables"][table_name]["columns"][col_name] = {
 *                     "description": col_data.get("description", ""),
 */
    __pyx_t_13 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_table_data, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_15 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_15)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_15);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_15, __pyx_n_u_columns, __pyx_t_10};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    if (unlikely(__pyx_t_8 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 1140, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_dict_iterator(__pyx_t_8, 0, __pyx_n_s_items, (&__pyx_t_14), (&__pyx_t_12)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_4);
    __pyx_t_4 = __pyx_t_9;
    __pyx_t_9 = 0;
    while (1) {
      __pyx_t_16 = __Pyx_dict_iter_next(__pyx_t_4, __pyx_t_14, &__pyx_t_13, &__pyx_t_9, &__pyx_t_8, NULL, __pyx_t_12);
      if (unlikely(__pyx_t_16 == 0)) break;
      if (unlikely(__pyx_t_16 == -1)) __PYX_ERR(0, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_XDECREF_SET(__pyx_v_col_name, __pyx_t_9);
      __pyx_t_9 = 0;
      __Pyx_XDECREF_SET(__pyx_v_col_data, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "sQQideR/DatabaseContext.pyx":1142
 *             for col_name, col_data in table_data.get("columns", {}).items():
 *                 schema["tables"][table_name]["columns"][col_name] = {
 *                     "description": col_data.get("description", ""),             # <<<<<<<<<<<<<<
 *                     "data_type": col_data.get("data_type", "")
 *                 }
 */
      __pyx_t_8 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_col_data, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_description, __pyx_t_10) < 0) __PYX_ERR(0, 1142, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "sQQideR/DatabaseContext.pyx":1143
 *                 schema["tables"][table_name]["columns"][col_name] = {
 *                     "description": col_data.get("description", ""),
 *                     "data_type": col_data.get("data_type", "")             # <<<<<<<<<<<<<<
 *                 }
 * 
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_col_data, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (PyDict_SetItem(__pyx_t_8, __pyx_n_u_data_type, __pyx_t_9) < 0) __PYX_ERR(0, 1142, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "sQQideR/DatabaseContext.pyx":1141
 *             }
 *             for col_name, col_data in table_data.get("columns", {}).items():
 *                 schema["tables"][table_name]["columns"][col_name] = {             # <<<<<<<<<<<<<<
 *                     "description": col_data.get("description", ""),
 *                     "data_type": col_data.get("data_type", "")
 */
      __pyx_t_9 = __Pyx_PyDict_GetItem(__pyx_v_schema, __pyx_n_u_tables); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_table_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_columns); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely((PyObject_SetItem(__pyx_t_9, __pyx_v_col_name, __pyx_t_8) < 0))) __PYX_ERR(0, 1141, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":1147
 * 
 *         # Add relationships
 *         schema["relationships"] = []             # <<<<<<<<<<<<<<
 *         for rel in self.metadata.get("relationships", []):
 *             schema["relationships"].append({
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyDict_SetItem(__pyx_v_schema, __pyx_n_u_relationships, __pyx_t_1) < 0))) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "sQQideR/DatabaseContext.pyx":1148
 *         # Add relationships
 *         schema["relationships"] = []
 *         for rel in self.metadata.get("relationships", []):             # <<<<<<<<<<<<<<
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_metadata); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = NULL;
  __pyx_t_11 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_11 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_n_u_relationships, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8);
    __pyx_t_6 = 0;
    __pyx_t_17 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_17 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1148, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_17)) {
      if (likely(PyList_CheckExact(__pyx_t_8))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1148, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 1148, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1148, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely((0 < 0))) __PYX_ERR(0, 1148, __pyx_L1_error)
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1148, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_17(__pyx_t_8);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1148, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_rel, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "sQQideR/DatabaseContext.pyx":1149
 *         schema["relationships"] = []
 *         for rel in self.metadata.get("relationships", []):
 *             schema["relationships"].append({             # <<<<<<<<<<<<<<
 *                 "from_table": rel.get("from_table", ""),
 *                 "to_table": rel.get("to_table", ""),
 */
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_schema, __pyx_n_u_relationships); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "sQQideR/DatabaseContext.pyx":1150
 *         for rel in self.metadata.get("relationships", []):
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),             # <<<<<<<<<<<<<<
 *                 "to_table": rel.get("to_table", ""),
 *                 "join_condition": rel.get("join_condition", "")
 */
    __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_rel, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_from_table, __pyx_t_10) < 0) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1151
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),
 *                 "to_table": rel.get("to_table", ""),             # <<<<<<<<<<<<<<
 *                 "join_condition": rel.get("join_condition", "")
 *             })
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_rel, __pyx_n_s_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_to_table, __pyx_t_9) < 0) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":1152
 *                 "from_table": rel.get("from_table", ""),
 *                 "to_table": rel.get("to_table", ""),
 *                 "join_condition": rel.get("join_condition", "")             # <<<<<<<<<<<<<<
 *             })
 * 
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_rel, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_u_join_condition, __pyx_t_10) < 0) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "sQQideR/DatabaseContext.pyx":1149
 *         schema["relationships"] = []
 *         for rel in self.metadata.get("relationships", []):
 *             schema["relationships"].append({             # <<<<<<<<<<<<<<
 *                 "from_table": rel.get("from_table", ""),
 *                 "to_table": rel.get("to_table", ""),
 */
    __pyx_t_18 = __Pyx_PyObject_Append(__pyx_t_1, __pyx_t_4); if (unlikely(__pyx_t_18 == ((int)-1))) __PYX_ERR(0, 1149, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1148
 *         # Add relationships
 *         schema["relationships"] = []
 *         for rel in self.metadata.get("relationships", []):             # <<<<<<<<<<<<<<
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),
 */
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sQQideR/DatabaseContext.pyx":1155
 *             })
 * 
 *         return schema             # <<<<<<<<<<<<<<
 * 
 * # Example usage of the service
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_schema);
  __pyx_r = __pyx_v_schema;
  goto __pyx_L0;

  /* "sQQideR/DatabaseContext.pyx":1120
 *         return md_output
 * 
 *     def _get_schema_summary(self) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Generate a summary of the database schema for AI context.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("sQQideR.DatabaseContext.DatabaseContextService._get_schema_summary", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_schema);
  __Pyx_XDECREF(__pyx_v_table_name);
  __Pyx_XDECREF(__pyx_v_table_data);
  __Pyx_XDECREF(__pyx_v_col_name);
  __Pyx_XDECREF(__pyx_v_col_data);
  __Pyx_XDECREF(__pyx_v_rel);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords[8];
static int __pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords = 0;
#endif

static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords)))) {
    o = (PyObject*)__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords[--__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords];
    memset(o, 0, sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(PyObject *o) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_pattern);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords)))) {
    __pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords[__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords++] = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)o;
  if (p->__pyx_v_pattern) {
    e = (*v)(p->__pyx_v_pattern, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords *)o;
  tmp = ((PyObject*)p->__pyx_v_pattern);
  p->__pyx_v_pattern = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords},
  {Py_tp_clear, (void *)__pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords},
  {Py_tp_new, (void *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords},
  {0, 0},
};
static PyType_Spec __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords_spec = {
  "sQQideR.DatabaseContext.__pyx_scope_struct__search_by_keywords",
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords_slots,
};
#else

static PyTypeObject __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords = {
  PyVarObject_HEAD_INIT(0, 0)
  "sQQideR.DatabaseContext.""__pyx_scope_struct__search_by_keywords", /*tp_name*/
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords, /*tp_traverse*/
  __pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr[8];
static int __pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr = 0;
#endif

static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr)))) {
    o = (PyObject*)__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr[--__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_genexpr_arg_0);
  Py_CLEAR(p->__pyx_v_k);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr)))) {
    __pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr[__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_genexpr_arg_0) {
    e = (*v)(p->__pyx_genexpr_arg_0, a); if (e) return e;
  }
  if (p->__pyx_v_k) {
    e = (*v)(p->__pyx_v_k, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr_spec = {
  "sQQideR.DatabaseContext.__pyx_scope_struct_1_genexpr",
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr_slots,
};
#else

static PyTypeObject __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "sQQideR.DatabaseContext.""__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql[8];
static int __pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql = 0;
#endif

static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql)))) {
    o = (PyObject*)__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql[--__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql];
    memset(o, 0, sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(PyObject *o) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_used_aliases);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql)))) {
    __pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql[__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql++] = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)o;
  if (p->__pyx_v_used_aliases) {
    e = (*v)(p->__pyx_v_used_aliases, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql *)o;
  tmp = ((PyObject*)p->__pyx_v_used_aliases);
  p->__pyx_v_used_aliases = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql},
  {Py_tp_clear, (void *)__pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql},
  {Py_tp_new, (void *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql},
  {0, 0},
};
static PyType_Spec __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql_spec = {
  "sQQideR.DatabaseContext.__pyx_scope_struct_2_generate_sql",
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql_slots,
};
#else

static PyTypeObject __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql = {
  PyVarObject_HEAD_INIT(0, 0)
  "sQQideR.DatabaseContext.""__pyx_scope_struct_2_generate_sql", /*tp_name*/
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql, /*tp_traverse*/
  __pyx_tp_clear_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl[8];
static int __pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl = 0;
#endif

static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl)))) {
    o = (PyObject*)__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl[--__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl];
    memset(o, 0, sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl));
    (void) PyObject_INIT(o, t);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl(PyObject *o) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  Py_CLEAR(p->__pyx_v_natural_language_query);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl)))) {
    __pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl[__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl++] = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl},
  {Py_tp_new, (void *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl},
  {0, 0},
};
static PyType_Spec __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl_spec = {
  "sQQideR.DatabaseContext.__pyx_scope_struct_3_generate_sql_from_nl",
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl_slots,
};
#else

static PyTypeObject __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl = {
  PyVarObject_HEAD_INIT(0, 0)
  "sQQideR.DatabaseContext.""__pyx_scope_struct_3_generate_sql_from_nl", /*tp_name*/
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr[8];
static int __pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr = 0;
#endif

static PyObject *__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr)))) {
    o = (PyObject*)__pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr[--__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr(PyObject *o) {
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_word);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr)))) {
    __pyx_freelist_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr[__pyx_freecount_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr++] = ((struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *p = (struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr_spec = {
  "sQQideR.DatabaseContext.__pyx_scope_struct_4_genexpr",
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr_slots,
};
#else

static PyTypeObject __pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "sQQideR.DatabaseContext.""__pyx_scope_struct_4_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_kp_u_1_Extract_the_key_business_doma, __pyx_k_1_Extract_the_key_business_doma, sizeof(__pyx_k_1_Extract_the_key_business_doma), 0, 1, 0, 0},
    {&__pyx_kp_u_1_Extract_the_key_business_doma_2, __pyx_k_1_Extract_the_key_business_doma_2, sizeof(__pyx_k_1_Extract_the_key_business_doma_2), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_Enhanced_Query, __pyx_k_AI_Enhanced_Query, sizeof(__pyx_k_AI_Enhanced_Query), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_SQL_generation_failed, __pyx_k_AI_SQL_generation_failed, sizeof(__pyx_k_AI_SQL_generation_failed), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_enhanced_query, __pyx_k_AI_enhanced_query, sizeof(__pyx_k_AI_enhanced_query), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_extracted_keywords, __pyx_k_AI_extracted_keywords, sizeof(__pyx_k_AI_extracted_keywords), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_history_context, __pyx_k_AI_history_context, sizeof(__pyx_k_AI_history_context), 0, 1, 0, 0},
    {&__pyx_kp_u_AI_identified_domains, __pyx_k_AI_identified_domains, sizeof(__pyx_k_AI_identified_domains), 0, 1, 0, 0},
    {&__pyx_kp_u_AND, __pyx_k_AND, sizeof(__pyx_k_AND), 0, 1, 0, 0},
    {&__pyx_n_s_Any, __pyx_k_Any, sizeof(__pyx_k_Any), 0, 0, 1, 1},
    {&__pyx_n_u_Assistant, __pyx_k_Assistant, sizeof(__pyx_k_Assistant), 0, 1, 0, 1},
    {&__pyx_kp_u_Attempting_to_load_enriched_proj, __pyx_k_Attempting_to_load_enriched_proj, sizeof(__pyx_k_Attempting_to_load_enriched_proj), 0, 1, 0, 0},
    {&__pyx_kp_u_Business_Concepts, __pyx_k_Business_Concepts, sizeof(__pyx_k_Business_Concepts), 0, 1, 0, 0},
    {&__pyx_kp_u_Business_Domains, __pyx_k_Business_Domains, sizeof(__pyx_k_Business_Domains), 0, 1, 0, 0},
    {&__pyx_kp_u_Business_Rules, __pyx_k_Business_Rules, sizeof(__pyx_k_Business_Rules), 0, 1, 0, 0},
    {&__pyx_n_s_ChatMessage, __pyx_k_ChatMessage, sizeof(__pyx_k_ChatMessage), 0, 0, 1, 1},
    {&__pyx_kp_u_Cleaned_response, __pyx_k_Cleaned_response, sizeof(__pyx_k_Cleaned_response), 0, 1, 0, 0},
    {&__pyx_kp_u_Column_Data_Type_Description, __pyx_k_Column_Data_Type_Description, sizeof(__pyx_k_Column_Data_Type_Description), 0, 1, 0, 0},
    {&__pyx_kp_u_Consider_this_conversation_hist, __pyx_k_Consider_this_conversation_hist, sizeof(__pyx_k_Consider_this_conversation_hist), 0, 1, 0, 0},
    {&__pyx_n_s_DatabaseContextService, __pyx_k_DatabaseContextService, sizeof(__pyx_k_DatabaseContextService), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService___init, __pyx_k_DatabaseContextService___init, sizeof(__pyx_k_DatabaseContextService___init), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService__get_sche, __pyx_k_DatabaseContextService__get_sche, sizeof(__pyx_k_DatabaseContextService__get_sche), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_build_gra, __pyx_k_DatabaseContextService_build_gra, sizeof(__pyx_k_DatabaseContextService_build_gra), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_find_path, __pyx_k_DatabaseContextService_find_path, sizeof(__pyx_k_DatabaseContextService_find_path), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_format_as, __pyx_k_DatabaseContextService_format_as, sizeof(__pyx_k_DatabaseContextService_format_as), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate, __pyx_k_DatabaseContextService_generate, sizeof(__pyx_k_DatabaseContextService_generate), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate_2, __pyx_k_DatabaseContextService_generate_2, sizeof(__pyx_k_DatabaseContextService_generate_2), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate_3, __pyx_k_DatabaseContextService_generate_3, sizeof(__pyx_k_DatabaseContextService_generate_3), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate_4, __pyx_k_DatabaseContextService_generate_4, sizeof(__pyx_k_DatabaseContextService_generate_4), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate_5, __pyx_k_DatabaseContextService_generate_5, sizeof(__pyx_k_DatabaseContextService_generate_5), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_generate_6, __pyx_k_DatabaseContextService_generate_6, sizeof(__pyx_k_DatabaseContextService_generate_6), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_get_relat, __pyx_k_DatabaseContextService_get_relat, sizeof(__pyx_k_DatabaseContextService_get_relat), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_load_meta, __pyx_k_DatabaseContextService_load_meta, sizeof(__pyx_k_DatabaseContextService_load_meta), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_process_q, __pyx_k_DatabaseContextService_process_q, sizeof(__pyx_k_DatabaseContextService_process_q), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_process_q_2, __pyx_k_DatabaseContextService_process_q_2, sizeof(__pyx_k_DatabaseContextService_process_q_2), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_process_q_3, __pyx_k_DatabaseContextService_process_q_3, sizeof(__pyx_k_DatabaseContextService_process_q_3), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_search_by, __pyx_k_DatabaseContextService_search_by, sizeof(__pyx_k_DatabaseContextService_search_by), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_search_by_2, __pyx_k_DatabaseContextService_search_by_2, sizeof(__pyx_k_DatabaseContextService_search_by_2), 0, 0, 1, 1},
    {&__pyx_n_s_DatabaseContextService_search_by_3, __pyx_k_DatabaseContextService_search_by_3, sizeof(__pyx_k_DatabaseContextService_search_by_3), 0, 0, 1, 1},
    {&__pyx_kp_u_Database_Context_for_Query, __pyx_k_Database_Context_for_Query, sizeof(__pyx_k_Database_Context_for_Query), 0, 1, 0, 0},
    {&__pyx_kp_u_Description, __pyx_k_Description, sizeof(__pyx_k_Description), 0, 1, 0, 0},
    {&__pyx_kp_u_Description_contains, __pyx_k_Description_contains, sizeof(__pyx_k_Description_contains), 0, 1, 0, 0},
    {&__pyx_kp_u_Description_partially_matches, __pyx_k_Description_partially_matches, sizeof(__pyx_k_Description_partially_matches), 0, 1, 0, 0},
    {&__pyx_n_s_DiGraph, __pyx_k_DiGraph, sizeof(__pyx_k_DiGraph), 0, 0, 1, 1},
    {&__pyx_n_s_Dict, __pyx_k_Dict, sizeof(__pyx_k_Dict), 0, 0, 1, 1},
    {&__pyx_kp_u_Error_enhancing_query_with_AI, __pyx_k_Error_enhancing_query_with_AI, sizeof(__pyx_k_Error_enhancing_query_with_AI), 0, 1, 0, 0},
    {&__pyx_kp_u_Error_parsing_AI_response, __pyx_k_Error_parsing_AI_response, sizeof(__pyx_k_Error_parsing_AI_response), 0, 1, 0, 0},
    {&__pyx_kp_u_Extracted_keywords, __pyx_k_Extracted_keywords, sizeof(__pyx_k_Extracted_keywords), 0, 1, 0, 0},
    {&__pyx_kp_u_FROM, __pyx_k_FROM, sizeof(__pyx_k_FROM), 0, 1, 0, 0},
    {&__pyx_kp_u_Failed_to_load_metadata_file_Ple, __pyx_k_Failed_to_load_metadata_file_Ple, sizeof(__pyx_k_Failed_to_load_metadata_file_Ple), 0, 1, 0, 0},
    {&__pyx_kp_u_Falling_back_to_rule_based_appr, __pyx_k_Falling_back_to_rule_based_appr, sizeof(__pyx_k_Falling_back_to_rule_based_appr), 0, 1, 0, 0},
    {&__pyx_n_s_FileNotFoundError, __pyx_k_FileNotFoundError, sizeof(__pyx_k_FileNotFoundError), 0, 0, 1, 1},
    {&__pyx_kp_u_Find_activities_with_excessive_l, __pyx_k_Find_activities_with_excessive_l, sizeof(__pyx_k_Find_activities_with_excessive_l), 0, 1, 0, 0},
    {&__pyx_kp_u_Found, __pyx_k_Found, sizeof(__pyx_k_Found), 0, 1, 0, 0},
    {&__pyx_n_s_G, __pyx_k_G, sizeof(__pyx_k_G), 0, 0, 1, 1},
    {&__pyx_kp_u_Generated_SQL, __pyx_k_Generated_SQL, sizeof(__pyx_k_Generated_SQL), 0, 1, 0, 0},
    {&__pyx_kp_u_Graph_built_with, __pyx_k_Graph_built_with, sizeof(__pyx_k_Graph_built_with), 0, 1, 0, 0},
    {&__pyx_kp_u_Graph_not_built_Call_build_graph, __pyx_k_Graph_not_built_Call_build_graph, sizeof(__pyx_k_Graph_not_built_Call_build_graph), 0, 1, 0, 0},
    {&__pyx_kp_u_ID_contains, __pyx_k_ID_contains, sizeof(__pyx_k_ID_contains), 0, 1, 0, 0},
    {&__pyx_n_s_IGNORECASE, __pyx_k_IGNORECASE, sizeof(__pyx_k_IGNORECASE), 0, 0, 1, 1},
    {&__pyx_kp_u_JOIN, __pyx_k_JOIN, sizeof(__pyx_k_JOIN), 0, 1, 0, 0},
    {&__pyx_n_s_JSONDecodeError, __pyx_k_JSONDecodeError, sizeof(__pyx_k_JSONDecodeError), 0, 0, 1, 1},
    {&__pyx_kp_u_Join_condition, __pyx_k_Join_condition, sizeof(__pyx_k_Join_condition), 0, 1, 0, 0},
    {&__pyx_n_s_List, __pyx_k_List, sizeof(__pyx_k_List), 0, 0, 1, 1},
    {&__pyx_kp_s_List_ChatMessage, __pyx_k_List_ChatMessage, sizeof(__pyx_k_List_ChatMessage), 0, 0, 1, 0},
    {&__pyx_kp_s_List_Tuple_str_str, __pyx_k_List_Tuple_str_str, sizeof(__pyx_k_List_Tuple_str_str), 0, 0, 1, 0},
    {&__pyx_kp_u_List_relationships_with_lag_grea, __pyx_k_List_relationships_with_lag_grea, sizeof(__pyx_k_List_relationships_with_lag_grea), 0, 1, 0, 0},
    {&__pyx_kp_s_List_str, __pyx_k_List_str, sizeof(__pyx_k_List_str), 0, 0, 1, 0},
    {&__pyx_kp_u_Loaded_metadata_from, __pyx_k_Loaded_metadata_from, sizeof(__pyx_k_Loaded_metadata_from), 0, 1, 0, 0},
    {&__pyx_kp_u_Markdown_Output, __pyx_k_Markdown_Output, sizeof(__pyx_k_Markdown_Output), 0, 1, 0, 0},
    {&__pyx_kp_u_Metadata_contains_sections, __pyx_k_Metadata_contains_sections, sizeof(__pyx_k_Metadata_contains_sections), 0, 1, 0, 0},
    {&__pyx_kp_u_Metadata_file_not_found, __pyx_k_Metadata_file_not_found, sizeof(__pyx_k_Metadata_file_not_found), 0, 1, 0, 0},
    {&__pyx_kp_u_Metadata_was_cleaned_at, __pyx_k_Metadata_was_cleaned_at, sizeof(__pyx_k_Metadata_was_cleaned_at), 0, 1, 0, 0},
    {&__pyx_n_s_Models, __pyx_k_Models, sizeof(__pyx_k_Models), 0, 0, 1, 1},
    {&__pyx_kp_u_Name_contains, __pyx_k_Name_contains, sizeof(__pyx_k_Name_contains), 0, 1, 0, 0},
    {&__pyx_kp_u_Name_partially_matches, __pyx_k_Name_partially_matches, sizeof(__pyx_k_Name_partially_matches), 0, 1, 0, 0},
    {&__pyx_n_s_NetworkXNoPath, __pyx_k_NetworkXNoPath, sizeof(__pyx_k_NetworkXNoPath), 0, 0, 1, 1},
    {&__pyx_kp_u_No_SQL_generated, __pyx_k_No_SQL_generated, sizeof(__pyx_k_No_SQL_generated), 0, 1, 0, 0},
    {&__pyx_kp_u_No_metadata_provided_Call_load_m, __pyx_k_No_metadata_provided_Call_load_m, sizeof(__pyx_k_No_metadata_provided_Call_load_m), 0, 1, 0, 0},
    {&__pyx_kp_u_No_tables_selected_for_query, __pyx_k_No_tables_selected_for_query, sizeof(__pyx_k_No_tables_selected_for_query), 0, 1, 0, 0},
    {&__pyx_kp_u_None, __pyx_k_None, sizeof(__pyx_k_None), 0, 1, 0, 0},
    {&__pyx_kp_u_ON, __pyx_k_ON, sizeof(__pyx_k_ON), 0, 1, 0, 0},
    {&__pyx_kp_u_ON_Missing_join_condition_verif, __pyx_k_ON_Missing_join_condition_verif, sizeof(__pyx_k_ON_Missing_join_condition_verif), 0, 1, 0, 0},
    {&__pyx_n_s_Optional, __pyx_k_Optional, sizeof(__pyx_k_Optional), 0, 0, 1, 1},
    {&__pyx_kp_s_Optional_List_str, __pyx_k_Optional_List_str, sizeof(__pyx_k_Optional_List_str), 0, 0, 1, 0},
    {&__pyx_kp_u_Processing_query, __pyx_k_Processing_query, sizeof(__pyx_k_Processing_query), 0, 1, 0, 0},
    {&__pyx_kp_u_Raw_response, __pyx_k_Raw_response, sizeof(__pyx_k_Raw_response), 0, 1, 0, 0},
    {&__pyx_kp_u_Relevant_Columns, __pyx_k_Relevant_Columns, sizeof(__pyx_k_Relevant_Columns), 0, 1, 0, 0},
    {&__pyx_kp_u_Relevant_Tables, __pyx_k_Relevant_Tables, sizeof(__pyx_k_Relevant_Tables), 0, 1, 0, 0},
    {&__pyx_kp_u_SELECT, __pyx_k_SELECT, sizeof(__pyx_k_SELECT), 0, 1, 0, 0},
    {&__pyx_n_s_SQLPrompt, __pyx_k_SQLPrompt, sizeof(__pyx_k_SQLPrompt), 0, 0, 1, 1},
    {&__pyx_kp_s_Service_for_providing_database, __pyx_k_Service_for_providing_database, sizeof(__pyx_k_Service_for_providing_database), 0, 0, 1, 0},
    {&__pyx_kp_u_Show_all_projects_with_negative, __pyx_k_Show_all_projects_with_negative, sizeof(__pyx_k_Show_all_projects_with_negative), 0, 1, 0, 0},
    {&__pyx_kp_u_Table_Relationships, __pyx_k_Table_Relationships, sizeof(__pyx_k_Table_Relationships), 0, 1, 0, 0},
    {&__pyx_kp_u_Tagged_with_keyword, __pyx_k_Tagged_with_keyword, sizeof(__pyx_k_Tagged_with_keyword), 0, 1, 0, 0},
    {&__pyx_kp_u_The_previous_SQL_query_was, __pyx_k_The_previous_SQL_query_was, sizeof(__pyx_k_The_previous_SQL_query_was), 0, 1, 0, 0},
    {&__pyx_n_s_Tuple, __pyx_k_Tuple, sizeof(__pyx_k_Tuple), 0, 0, 1, 1},
    {&__pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s, __pyx_k_Tuple_List_Tuple_str_Dict_Dict_s, sizeof(__pyx_k_Tuple_List_Tuple_str_Dict_Dict_s), 0, 0, 1, 0},
    {&__pyx_n_s_Union, __pyx_k_Union, sizeof(__pyx_k_Union), 0, 0, 1, 1},
    {&__pyx_n_u_User, __pyx_k_User, sizeof(__pyx_k_User), 0, 1, 0, 1},
    {&__pyx_kp_u_Using_basic_keyword_extraction, __pyx_k_Using_basic_keyword_extraction, sizeof(__pyx_k_Using_basic_keyword_extraction), 0, 1, 0, 0},
    {&__pyx_kp_u_Using_template_query, __pyx_k_Using_template_query, sizeof(__pyx_k_Using_template_query), 0, 1, 0, 0},
    {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
    {&__pyx_kp_u_WARNING_Missing_required_sectio, __pyx_k_WARNING_Missing_required_sectio, sizeof(__pyx_k_WARNING_Missing_required_sectio), 0, 1, 0, 0},
    {&__pyx_kp_u_WHERE, __pyx_k_WHERE, sizeof(__pyx_k_WHERE), 0, 1, 0, 0},
    {&__pyx_kp_u_What_are_the_activities_on_the_c, __pyx_k_What_are_the_activities_on_the_c, sizeof(__pyx_k_What_are_the_activities_on_the_c), 0, 1, 0, 0},
    {&__pyx_kp_u_You_are_an_expert_in_project_ma, __pyx_k_You_are_an_expert_in_project_ma, sizeof(__pyx_k_You_are_an_expert_in_project_ma), 0, 1, 0, 0},
    {&__pyx_kp_u__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0, 0},
    {&__pyx_kp_u__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 1, 0, 0},
    {&__pyx_kp_u__15, __pyx_k__15, sizeof(__pyx_k__15), 0, 1, 0, 0},
    {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
    {&__pyx_kp_u__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 1, 0, 0},
    {&__pyx_kp_u__23, __pyx_k__23, sizeof(__pyx_k__23), 0, 1, 0, 0},
    {&__pyx_kp_u__27, __pyx_k__27, sizeof(__pyx_k__27), 0, 1, 0, 0},
    {&__pyx_kp_u__28, __pyx_k__28, sizeof(__pyx_k__28), 0, 1, 0, 0},
    {&__pyx_kp_u__29, __pyx_k__29, sizeof(__pyx_k__29), 0, 1, 0, 0},
    {&__pyx_n_s__31, __pyx_k__31, sizeof(__pyx_k__31), 0, 0, 1, 1},
    {&__pyx_kp_u__31, __pyx_k__31, sizeof(__pyx_k__31), 0, 1, 0, 0},
    {&__pyx_kp_u__35, __pyx_k__35, sizeof(__pyx_k__35), 0, 1, 0, 0},
    {&__pyx_kp_u__36, __pyx_k__36, sizeof(__pyx_k__36), 0, 1, 0, 0},
    {&__pyx_kp_u__37, __pyx_k__37, sizeof(__pyx_k__37), 0, 1, 0, 0},
    {&__pyx_kp_u__38, __pyx_k__38, sizeof(__pyx_k__38), 0, 1, 0, 0},
    {&__pyx_kp_u__39, __pyx_k__39, sizeof(__pyx_k__39), 0, 1, 0, 0},
    {&__pyx_kp_u__40, __pyx_k__40, sizeof(__pyx_k__40), 0, 1, 0, 0},
    {&__pyx_kp_u__41, __pyx_k__41, sizeof(__pyx_k__41), 0, 1, 0, 0},
    {&__pyx_kp_u__42, __pyx_k__42, sizeof(__pyx_k__42), 0, 1, 0, 0},
    {&__pyx_kp_u__43, __pyx_k__43, sizeof(__pyx_k__43), 0, 1, 0, 0},
    {&__pyx_kp_u__44, __pyx_k__44, sizeof(__pyx_k__44), 0, 1, 0, 0},
    {&__pyx_kp_u__45, __pyx_k__45, sizeof(__pyx_k__45), 0, 1, 0, 0},
    {&__pyx_n_s__48, __pyx_k__48, sizeof(__pyx_k__48), 0, 0, 1, 1},
    {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
    {&__pyx_n_s__55, __pyx_k__55, sizeof(__pyx_k__55), 0, 0, 1, 1},
    {&__pyx_kp_u__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 1, 0, 0},
    {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
    {&__pyx_kp_u__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 1, 0, 0},
    {&__pyx_n_s_adapted_query, __pyx_k_adapted_query, sizeof(__pyx_k_adapted_query), 0, 0, 1, 1},
    {&__pyx_n_s_add_edge, __pyx_k_add_edge, sizeof(__pyx_k_add_edge), 0, 0, 1, 1},
    {&__pyx_n_s_add_node, __pyx_k_add_node, sizeof(__pyx_k_add_node), 0, 0, 1, 1},
    {&__pyx_n_s_ai_base, __pyx_k_ai_base, sizeof(__pyx_k_ai_base), 0, 0, 1, 1},
    {&__pyx_n_s_ai_context, __pyx_k_ai_context, sizeof(__pyx_k_ai_context), 0, 0, 1, 1},
    {&__pyx_n_u_ai_enhanced, __pyx_k_ai_enhanced, sizeof(__pyx_k_ai_enhanced), 0, 1, 0, 1},
    {&__pyx_n_s_alias, __pyx_k_alias, sizeof(__pyx_k_alias), 0, 0, 1, 1},
    {&__pyx_n_s_aliases, __pyx_k_aliases, sizeof(__pyx_k_aliases), 0, 0, 1, 1},
    {&__pyx_n_s_all_rules, __pyx_k_all_rules, sizeof(__pyx_k_all_rules), 0, 0, 1, 1},
    {&__pyx_n_s_all_tables, __pyx_k_all_tables, sizeof(__pyx_k_all_tables), 0, 0, 1, 1},
    {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
    {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio, __pyx_k_asyncio, sizeof(__pyx_k_asyncio), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_tasks, __pyx_k_asyncio_tasks, sizeof(__pyx_k_asyncio_tasks), 0, 0, 1, 1},
    {&__pyx_kp_u_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 1, 0, 0},
    {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
    {&__pyx_n_s_best_example, __pyx_k_best_example, sizeof(__pyx_k_best_example), 0, 0, 1, 1},
    {&__pyx_n_u_best_guess_sql, __pyx_k_best_guess_sql, sizeof(__pyx_k_best_guess_sql), 0, 1, 0, 1},
    {&__pyx_n_s_bool, __pyx_k_bool, sizeof(__pyx_k_bool), 0, 0, 1, 1},
    {&__pyx_n_s_build_graph, __pyx_k_build_graph, sizeof(__pyx_k_build_graph), 0, 0, 1, 1},
    {&__pyx_n_s_buisness_domains, __pyx_k_buisness_domains, sizeof(__pyx_k_buisness_domains), 0, 0, 1, 1},
    {&__pyx_n_u_business_concepts, __pyx_k_business_concepts, sizeof(__pyx_k_business_concepts), 0, 1, 0, 1},
    {&__pyx_n_u_business_domains, __pyx_k_business_domains, sizeof(__pyx_k_business_domains), 0, 1, 0, 1},
    {&__pyx_n_u_business_rules, __pyx_k_business_rules, sizeof(__pyx_k_business_rules), 0, 1, 0, 1},
    {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
    {&__pyx_n_s_chat_history, __pyx_k_chat_history, sizeof(__pyx_k_chat_history), 0, 0, 1, 1},
    {&__pyx_n_u_chat_history_context, __pyx_k_chat_history_context, sizeof(__pyx_k_chat_history_context), 0, 1, 0, 1},
    {&__pyx_n_s_class_getitem, __pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 0, 1, 1},
    {&__pyx_n_s_clean_response, __pyx_k_clean_response, sizeof(__pyx_k_clean_response), 0, 0, 1, 1},
    {&__pyx_n_s_cleaned_at, __pyx_k_cleaned_at, sizeof(__pyx_k_cleaned_at), 0, 0, 1, 1},
    {&__pyx_n_u_cleaned_at, __pyx_k_cleaned_at, sizeof(__pyx_k_cleaned_at), 0, 1, 0, 1},
    {&__pyx_n_u_cleaning_version, __pyx_k_cleaning_version, sizeof(__pyx_k_cleaning_version), 0, 1, 0, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
    {&__pyx_n_s_col_data, __pyx_k_col_data, sizeof(__pyx_k_col_data), 0, 0, 1, 1},
    {&__pyx_n_s_col_name, __pyx_k_col_name, sizeof(__pyx_k_col_name), 0, 0, 1, 1},
    {&__pyx_n_s_col_node_id, __pyx_k_col_node_id, sizeof(__pyx_k_col_node_id), 0, 0, 1, 1},
    {&__pyx_kp_u_column, __pyx_k_column, sizeof(__pyx_k_column), 0, 1, 0, 0},
    {&__pyx_n_s_column_2, __pyx_k_column_2, sizeof(__pyx_k_column_2), 0, 0, 1, 1},
    {&__pyx_n_u_column_2, __pyx_k_column_2, sizeof(__pyx_k_column_2), 0, 1, 0, 1},
    {&__pyx_n_u_column_statistics, __pyx_k_column_statistics, sizeof(__pyx_k_column_statistics), 0, 1, 0, 1},
    {&__pyx_n_s_columns, __pyx_k_columns, sizeof(__pyx_k_columns), 0, 0, 1, 1},
    {&__pyx_n_u_columns, __pyx_k_columns, sizeof(__pyx_k_columns), 0, 1, 0, 1},
    {&__pyx_n_s_compile, __pyx_k_compile, sizeof(__pyx_k_compile), 0, 0, 1, 1},
    {&__pyx_n_s_complexity, __pyx_k_complexity, sizeof(__pyx_k_complexity), 0, 0, 1, 1},
    {&__pyx_n_u_complexity, __pyx_k_complexity, sizeof(__pyx_k_complexity), 0, 1, 0, 1},
    {&__pyx_kp_u_concept, __pyx_k_concept, sizeof(__pyx_k_concept), 0, 1, 0, 0},
    {&__pyx_n_s_concept_2, __pyx_k_concept_2, sizeof(__pyx_k_concept_2), 0, 0, 1, 1},
    {&__pyx_n_u_concept_2, __pyx_k_concept_2, sizeof(__pyx_k_concept_2), 0, 1, 0, 1},
    {&__pyx_n_s_concept_data, __pyx_k_concept_data, sizeof(__pyx_k_concept_data), 0, 0, 1, 1},
    {&__pyx_n_s_concept_id, __pyx_k_concept_id, sizeof(__pyx_k_concept_id), 0, 0, 1, 1},
    {&__pyx_n_s_concepts, __pyx_k_concepts, sizeof(__pyx_k_concepts), 0, 0, 1, 1},
    {&__pyx_n_u_concepts, __pyx_k_concepts, sizeof(__pyx_k_concepts), 0, 1, 0, 1},
    {&__pyx_n_s_condition_nodes, __pyx_k_condition_nodes, sizeof(__pyx_k_condition_nodes), 0, 0, 1, 1},
    {&__pyx_n_s_content, __pyx_k_content, sizeof(__pyx_k_content), 0, 0, 1, 1},
    {&__pyx_n_s_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 0, 1, 1},
    {&__pyx_n_s_context_md, __pyx_k_context_md, sizeof(__pyx_k_context_md), 0, 0, 1, 1},
    {&__pyx_n_s_context_prompt, __pyx_k_context_prompt, sizeof(__pyx_k_context_prompt), 0, 0, 1, 1},
    {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
    {&__pyx_n_s_current_table, __pyx_k_current_table, sizeof(__pyx_k_current_table), 0, 0, 1, 1},
    {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
    {&__pyx_kp_u_d_4_d_2_d_2, __pyx_k_d_4_d_2_d_2, sizeof(__pyx_k_d_4_d_2_d_2), 0, 1, 0, 0},
    {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
    {&__pyx_n_u_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 1, 0, 1},
    {&__pyx_n_u_data_type, __pyx_k_data_type, sizeof(__pyx_k_data_type), 0, 1, 0, 1},
    {&__pyx_n_u_database_objects, __pyx_k_database_objects, sizeof(__pyx_k_database_objects), 0, 1, 0, 1},
    {&__pyx_n_u_database_schema, __pyx_k_database_schema, sizeof(__pyx_k_database_schema), 0, 1, 0, 1},
    {&__pyx_n_u_database_statistics, __pyx_k_database_statistics, sizeof(__pyx_k_database_statistics), 0, 1, 0, 1},
    {&__pyx_n_u_date, __pyx_k_date, sizeof(__pyx_k_date), 0, 1, 0, 1},
    {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
    {&__pyx_n_u_day, __pyx_k_day, sizeof(__pyx_k_day), 0, 1, 0, 1},
    {&__pyx_n_u_defines, __pyx_k_defines, sizeof(__pyx_k_defines), 0, 1, 0, 1},
    {&__pyx_n_s_definition, __pyx_k_definition, sizeof(__pyx_k_definition), 0, 0, 1, 1},
    {&__pyx_n_s_description, __pyx_k_description, sizeof(__pyx_k_description), 0, 0, 1, 1},
    {&__pyx_n_u_description, __pyx_k_description, sizeof(__pyx_k_description), 0, 1, 0, 1},
    {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
    {&__pyx_kp_u_disable, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
    {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
    {&__pyx_n_s_domain, __pyx_k_domain, sizeof(__pyx_k_domain), 0, 0, 1, 1},
    {&__pyx_n_u_domain, __pyx_k_domain, sizeof(__pyx_k_domain), 0, 1, 0, 1},
    {&__pyx_n_s_domain_results, __pyx_k_domain_results, sizeof(__pyx_k_domain_results), 0, 0, 1, 1},
    {&__pyx_n_s_e, __pyx_k_e, sizeof(__pyx_k_e), 0, 0, 1, 1},
    {&__pyx_n_s_edge_data, __pyx_k_edge_data, sizeof(__pyx_k_edge_data), 0, 0, 1, 1},
    {&__pyx_n_s_edges, __pyx_k_edges, sizeof(__pyx_k_edges), 0, 0, 1, 1},
    {&__pyx_kp_u_edges_2, __pyx_k_edges_2, sizeof(__pyx_k_edges_2), 0, 1, 0, 0},
    {&__pyx_kp_u_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
    {&__pyx_n_s_encoding, __pyx_k_encoding, sizeof(__pyx_k_encoding), 0, 0, 1, 1},
    {&__pyx_n_s_end_node, __pyx_k_end_node, sizeof(__pyx_k_end_node), 0, 0, 1, 1},
    {&__pyx_n_s_endswith, __pyx_k_endswith, sizeof(__pyx_k_endswith), 0, 0, 1, 1},
    {&__pyx_n_s_enhanced_query, __pyx_k_enhanced_query, sizeof(__pyx_k_enhanced_query), 0, 0, 1, 1},
    {&__pyx_n_u_enhanced_query, __pyx_k_enhanced_query, sizeof(__pyx_k_enhanced_query), 0, 1, 0, 1},
    {&__pyx_kp_u_enriched_project_metadata_cleane, __pyx_k_enriched_project_metadata_cleane, sizeof(__pyx_k_enriched_project_metadata_cleane), 0, 1, 0, 0},
    {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
    {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
    {&__pyx_n_s_escape, __pyx_k_escape, sizeof(__pyx_k_escape), 0, 0, 1, 1},
    {&__pyx_kp_u_example, __pyx_k_example, sizeof(__pyx_k_example), 0, 1, 0, 0},
    {&__pyx_n_s_example_2, __pyx_k_example_2, sizeof(__pyx_k_example_2), 0, 0, 1, 1},
    {&__pyx_n_u_example_2, __pyx_k_example_2, sizeof(__pyx_k_example_2), 0, 1, 0, 1},
    {&__pyx_n_s_example_data, __pyx_k_example_data, sizeof(__pyx_k_example_data), 0, 0, 1, 1},
    {&__pyx_n_s_example_id, __pyx_k_example_id, sizeof(__pyx_k_example_id), 0, 0, 1, 1},
    {&__pyx_n_s_example_node, __pyx_k_example_node, sizeof(__pyx_k_example_node), 0, 0, 1, 1},
    {&__pyx_n_s_example_nodes, __pyx_k_example_nodes, sizeof(__pyx_k_example_nodes), 0, 0, 1, 1},
    {&__pyx_n_s_example_queries, __pyx_k_example_queries, sizeof(__pyx_k_example_queries), 0, 0, 1, 1},
    {&__pyx_n_s_example_scores, __pyx_k_example_scores, sizeof(__pyx_k_example_scores), 0, 0, 1, 1},
    {&__pyx_n_s_exists, __pyx_k_exists, sizeof(__pyx_k_exists), 0, 0, 1, 1},
    {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
    {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
    {&__pyx_n_s_file_path, __pyx_k_file_path, sizeof(__pyx_k_file_path), 0, 0, 1, 1},
    {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
    {&__pyx_n_s_filtered_rules, __pyx_k_filtered_rules, sizeof(__pyx_k_filtered_rules), 0, 0, 1, 1},
    {&__pyx_n_s_find, __pyx_k_find, sizeof(__pyx_k_find), 0, 0, 1, 1},
    {&__pyx_n_s_find_path, __pyx_k_find_path, sizeof(__pyx_k_find_path), 0, 0, 1, 1},
    {&__pyx_n_s_first_newline, __pyx_k_first_newline, sizeof(__pyx_k_first_newline), 0, 0, 1, 1},
    {&__pyx_n_s_first_table, __pyx_k_first_table, sizeof(__pyx_k_first_table), 0, 0, 1, 1},
    {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
    {&__pyx_n_s_format_as_markdown, __pyx_k_format_as_markdown, sizeof(__pyx_k_format_as_markdown), 0, 0, 1, 1},
    {&__pyx_n_u_from_table, __pyx_k_from_table, sizeof(__pyx_k_from_table), 0, 1, 0, 1},
    {&__pyx_n_s_from_table_id, __pyx_k_from_table_id, sizeof(__pyx_k_from_table_id), 0, 0, 1, 1},
    {&__pyx_n_s_from_tables, __pyx_k_from_tables, sizeof(__pyx_k_from_tables), 0, 0, 1, 1},
    {&__pyx_kp_u_gc, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
    {&__pyx_n_s_generate_output, __pyx_k_generate_output, sizeof(__pyx_k_generate_output), 0, 0, 1, 1},
    {&__pyx_n_s_generate_sql, __pyx_k_generate_sql, sizeof(__pyx_k_generate_sql), 0, 0, 1, 1},
    {&__pyx_n_s_generate_sql_from_nl, __pyx_k_generate_sql_from_nl, sizeof(__pyx_k_generate_sql_from_nl), 0, 0, 1, 1},
    {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
    {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
    {&__pyx_n_s_get_alias, __pyx_k_get_alias, sizeof(__pyx_k_get_alias), 0, 0, 1, 1},
    {&__pyx_n_s_get_edge_data, __pyx_k_get_edge_data, sizeof(__pyx_k_get_edge_data), 0, 0, 1, 1},
    {&__pyx_n_s_get_related_rules, __pyx_k_get_related_rules, sizeof(__pyx_k_get_related_rules), 0, 0, 1, 1},
    {&__pyx_n_u_get_response, __pyx_k_get_response, sizeof(__pyx_k_get_response), 0, 1, 0, 1},
    {&__pyx_n_s_get_schema_summary, __pyx_k_get_schema_summary, sizeof(__pyx_k_get_schema_summary), 0, 0, 1, 1},
    {&__pyx_n_u_governed_by, __pyx_k_governed_by, sizeof(__pyx_k_governed_by), 0, 1, 0, 1},
    {&__pyx_n_s_graph, __pyx_k_graph, sizeof(__pyx_k_graph), 0, 0, 1, 1},
    {&__pyx_n_u_has_column, __pyx_k_has_column, sizeof(__pyx_k_has_column), 0, 1, 0, 1},
    {&__pyx_n_s_has_node, __pyx_k_has_node, sizeof(__pyx_k_has_node), 0, 0, 1, 1},
    {&__pyx_n_s_history_context, __pyx_k_history_context, sizeof(__pyx_k_history_context), 0, 0, 1, 1},
    {&__pyx_n_u_history_context, __pyx_k_history_context, sizeof(__pyx_k_history_context), 0, 1, 0, 1},
    {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
    {&__pyx_n_u_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 1, 0, 1},
    {&__pyx_n_u_implements, __pyx_k_implements, sizeof(__pyx_k_implements), 0, 1, 0, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_u_indexed, __pyx_k_indexed, sizeof(__pyx_k_indexed), 0, 1, 0, 1},
    {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
    {&__pyx_n_s_init_subclass, __pyx_k_init_subclass, sizeof(__pyx_k_init_subclass), 0, 0, 1, 1},
    {&__pyx_n_s_initializing, __pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 0, 1, 1},
    {&__pyx_n_s_inspect, __pyx_k_inspect, sizeof(__pyx_k_inspect), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_n_s_is_user, __pyx_k_is_user, sizeof(__pyx_k_is_user), 0, 0, 1, 1},
    {&__pyx_kp_u_isenabled, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
    {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
    {&__pyx_n_s_j, __pyx_k_j, sizeof(__pyx_k_j), 0, 0, 1, 1},
    {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
    {&__pyx_n_s_join_clauses, __pyx_k_join_clauses, sizeof(__pyx_k_join_clauses), 0, 0, 1, 1},
    {&__pyx_n_s_join_condition, __pyx_k_join_condition, sizeof(__pyx_k_join_condition), 0, 0, 1, 1},
    {&__pyx_n_u_join_condition, __pyx_k_join_condition, sizeof(__pyx_k_join_condition), 0, 1, 0, 1},
    {&__pyx_n_s_join_found, __pyx_k_join_found, sizeof(__pyx_k_join_found), 0, 0, 1, 1},
    {&__pyx_n_s_json, __pyx_k_json, sizeof(__pyx_k_json), 0, 0, 1, 1},
    {&__pyx_n_u_json, __pyx_k_json, sizeof(__pyx_k_json), 0, 1, 0, 1},
    {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
    {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
    {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
    {&__pyx_kp_u_keyword, __pyx_k_keyword, sizeof(__pyx_k_keyword), 0, 1, 0, 0},
    {&__pyx_n_s_keyword_2, __pyx_k_keyword_2, sizeof(__pyx_k_keyword_2), 0, 0, 1, 1},
    {&__pyx_n_u_keyword_2, __pyx_k_keyword_2, sizeof(__pyx_k_keyword_2), 0, 1, 0, 1},
    {&__pyx_n_s_keyword_id, __pyx_k_keyword_id, sizeof(__pyx_k_keyword_id), 0, 0, 1, 1},
    {&__pyx_n_s_keywords, __pyx_k_keywords, sizeof(__pyx_k_keywords), 0, 0, 1, 1},
    {&__pyx_n_u_keywords, __pyx_k_keywords, sizeof(__pyx_k_keywords), 0, 1, 0, 1},
    {&__pyx_n_s_load, __pyx_k_load, sizeof(__pyx_k_load), 0, 0, 1, 1},
    {&__pyx_n_s_load_metadata, __pyx_k_load_metadata, sizeof(__pyx_k_load_metadata), 0, 0, 1, 1},
    {&__pyx_n_s_loads, __pyx_k_loads, sizeof(__pyx_k_loads), 0, 0, 1, 1},
    {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_u_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 1, 0, 1},
    {&__pyx_n_u_markdown, __pyx_k_markdown, sizeof(__pyx_k_markdown), 0, 1, 0, 1},
    {&__pyx_n_s_match_reason, __pyx_k_match_reason, sizeof(__pyx_k_match_reason), 0, 0, 1, 1},
    {&__pyx_n_s_match_score, __pyx_k_match_score, sizeof(__pyx_k_match_score), 0, 0, 1, 1},
    {&__pyx_n_s_matched, __pyx_k_matched, sizeof(__pyx_k_matched), 0, 0, 1, 1},
    {&__pyx_n_s_matches, __pyx_k_matches, sizeof(__pyx_k_matches), 0, 0, 1, 1},
    {&__pyx_n_s_max, __pyx_k_max, sizeof(__pyx_k_max), 0, 0, 1, 1},
    {&__pyx_n_s_md_output, __pyx_k_md_output, sizeof(__pyx_k_md_output), 0, 0, 1, 1},
    {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
    {&__pyx_n_s_metadata, __pyx_k_metadata, sizeof(__pyx_k_metadata), 0, 0, 1, 1},
    {&__pyx_n_u_metadata_cleaning, __pyx_k_metadata_cleaning, sizeof(__pyx_k_metadata_cleaning), 0, 1, 0, 1},
    {&__pyx_n_s_metadata_file, __pyx_k_metadata_file, sizeof(__pyx_k_metadata_file), 0, 0, 1, 1},
    {&__pyx_n_s_metadata_sections, __pyx_k_metadata_sections, sizeof(__pyx_k_metadata_sections), 0, 0, 1, 1},
    {&__pyx_n_s_missing, __pyx_k_missing, sizeof(__pyx_k_missing), 0, 0, 1, 1},
    {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
    {&__pyx_n_s_msg, __pyx_k_msg, sizeof(__pyx_k_msg), 0, 0, 1, 1},
    {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_u_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 1, 0, 1},
    {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
    {&__pyx_n_s_natural_language_query, __pyx_k_natural_language_query, sizeof(__pyx_k_natural_language_query), 0, 0, 1, 1},
    {&__pyx_n_s_neighbor, __pyx_k_neighbor, sizeof(__pyx_k_neighbor), 0, 0, 1, 1},
    {&__pyx_n_s_neighbors, __pyx_k_neighbors, sizeof(__pyx_k_neighbors), 0, 0, 1, 1},
    {&__pyx_n_s_networkx, __pyx_k_networkx, sizeof(__pyx_k_networkx), 0, 0, 1, 1},
    {&__pyx_n_s_node, __pyx_k_node, sizeof(__pyx_k_node), 0, 0, 1, 1},
    {&__pyx_n_s_node_data, __pyx_k_node_data, sizeof(__pyx_k_node_data), 0, 0, 1, 1},
    {&__pyx_n_s_node_id, __pyx_k_node_id, sizeof(__pyx_k_node_id), 0, 0, 1, 1},
    {&__pyx_n_s_node_matches, __pyx_k_node_matches, sizeof(__pyx_k_node_matches), 0, 0, 1, 1},
    {&__pyx_n_s_node_props, __pyx_k_node_props, sizeof(__pyx_k_node_props), 0, 0, 1, 1},
    {&__pyx_n_s_node_rules, __pyx_k_node_rules, sizeof(__pyx_k_node_rules), 0, 0, 1, 1},
    {&__pyx_n_s_node_score, __pyx_k_node_score, sizeof(__pyx_k_node_score), 0, 0, 1, 1},
    {&__pyx_n_s_node_scores, __pyx_k_node_scores, sizeof(__pyx_k_node_scores), 0, 0, 1, 1},
    {&__pyx_n_s_nodes, __pyx_k_nodes, sizeof(__pyx_k_nodes), 0, 0, 1, 1},
    {&__pyx_kp_u_nodes_and, __pyx_k_nodes_and, sizeof(__pyx_k_nodes_and), 0, 1, 0, 0},
    {&__pyx_n_s_normalized_keywords, __pyx_k_normalized_keywords, sizeof(__pyx_k_normalized_keywords), 0, 0, 1, 1},
    {&__pyx_n_s_nx, __pyx_k_nx, sizeof(__pyx_k_nx), 0, 0, 1, 1},
    {&__pyx_kp_s_nx_DiGraph, __pyx_k_nx_DiGraph, sizeof(__pyx_k_nx_DiGraph), 0, 0, 1, 0},
    {&__pyx_n_s_open, __pyx_k_open, sizeof(__pyx_k_open), 0, 0, 1, 1},
    {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
    {&__pyx_n_s_os_path, __pyx_k_os_path, sizeof(__pyx_k_os_path), 0, 0, 1, 1},
    {&__pyx_n_s_output, __pyx_k_output, sizeof(__pyx_k_output), 0, 0, 1, 1},
    {&__pyx_n_s_output_data, __pyx_k_output_data, sizeof(__pyx_k_output_data), 0, 0, 1, 1},
    {&__pyx_n_s_output_format, __pyx_k_output_format, sizeof(__pyx_k_output_format), 0, 0, 1, 1},
    {&__pyx_n_s_parsed, __pyx_k_parsed, sizeof(__pyx_k_parsed), 0, 0, 1, 1},
    {&__pyx_n_s_path, __pyx_k_path, sizeof(__pyx_k_path), 0, 0, 1, 1},
    {&__pyx_n_s_pattern, __pyx_k_pattern, sizeof(__pyx_k_pattern), 0, 0, 1, 1},
    {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
    {&__pyx_n_s_prev, __pyx_k_prev, sizeof(__pyx_k_prev), 0, 0, 1, 1},
    {&__pyx_n_s_prev_table, __pyx_k_prev_table, sizeof(__pyx_k_prev_table), 0, 0, 1, 1},
    {&__pyx_n_s_previous_sql, __pyx_k_previous_sql, sizeof(__pyx_k_previous_sql), 0, 0, 1, 1},
    {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
    {&__pyx_n_s_process_query, __pyx_k_process_query, sizeof(__pyx_k_process_query), 0, 0, 1, 1},
    {&__pyx_n_s_process_query_with_ai, __pyx_k_process_query_with_ai, sizeof(__pyx_k_process_query_with_ai), 0, 0, 1, 1},
    {&__pyx_n_s_process_query_with_chat_history, __pyx_k_process_query_with_chat_history, sizeof(__pyx_k_process_query_with_chat_history), 0, 0, 1, 1},
    {&__pyx_n_s_prompt, __pyx_k_prompt, sizeof(__pyx_k_prompt), 0, 0, 1, 1},
    {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
    {&__pyx_n_s_query, __pyx_k_query, sizeof(__pyx_k_query), 0, 0, 1, 1},
    {&__pyx_n_u_query, __pyx_k_query, sizeof(__pyx_k_query), 0, 1, 0, 1},
    {&__pyx_n_u_query_examples, __pyx_k_query_examples, sizeof(__pyx_k_query_examples), 0, 1, 0, 1},
    {&__pyx_n_s_query_text, __pyx_k_query_text, sizeof(__pyx_k_query_text), 0, 0, 1, 1},
    {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
    {&__pyx_n_u_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 1, 0, 1},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 0, 1, 1},
    {&__pyx_n_u_references, __pyx_k_references, sizeof(__pyx_k_references), 0, 1, 0, 1},
    {&__pyx_n_s_rel, __pyx_k_rel, sizeof(__pyx_k_rel), 0, 0, 1, 1},
    {&__pyx_n_u_related_business_rules, __pyx_k_related_business_rules, sizeof(__pyx_k_related_business_rules), 0, 1, 0, 1},
    {&__pyx_n_u_related_to, __pyx_k_related_to, sizeof(__pyx_k_related_to), 0, 1, 0, 1},
    {&__pyx_n_s_relationship, __pyx_k_relationship, sizeof(__pyx_k_relationship), 0, 0, 1, 1},
    {&__pyx_n_s_relationship_count, __pyx_k_relationship_count, sizeof(__pyx_k_relationship_count), 0, 0, 1, 1},
    {&__pyx_n_u_relationships, __pyx_k_relationships, sizeof(__pyx_k_relationships), 0, 1, 0, 1},
    {&__pyx_kp_u_relationships_2, __pyx_k_relationships_2, sizeof(__pyx_k_relationships_2), 0, 1, 0, 0},
    {&__pyx_n_s_replace, __pyx_k_replace, sizeof(__pyx_k_replace), 0, 0, 1, 1},
    {&__pyx_n_s_required_sections, __pyx_k_required_sections, sizeof(__pyx_k_required_sections), 0, 0, 1, 1},
    {&__pyx_n_s_response, __pyx_k_response, sizeof(__pyx_k_response), 0, 0, 1, 1},
    {&__pyx_n_s_results, __pyx_k_results, sizeof(__pyx_k_results), 0, 0, 1, 1},
    {&__pyx_n_s_return, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
    {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
    {&__pyx_n_s_role, __pyx_k_role, sizeof(__pyx_k_role), 0, 0, 1, 1},
    {&__pyx_n_s_row_count, __pyx_k_row_count, sizeof(__pyx_k_row_count), 0, 0, 1, 1},
    {&__pyx_n_u_row_count, __pyx_k_row_count, sizeof(__pyx_k_row_count), 0, 1, 0, 1},
    {&__pyx_kp_u_rule, __pyx_k_rule, sizeof(__pyx_k_rule), 0, 1, 0, 0},
    {&__pyx_n_s_rule_2, __pyx_k_rule_2, sizeof(__pyx_k_rule_2), 0, 0, 1, 1},
    {&__pyx_n_u_rule_2, __pyx_k_rule_2, sizeof(__pyx_k_rule_2), 0, 1, 0, 1},
    {&__pyx_n_s_rule_data, __pyx_k_rule_data, sizeof(__pyx_k_rule_data), 0, 0, 1, 1},
    {&__pyx_n_s_rule_desc, __pyx_k_rule_desc, sizeof(__pyx_k_rule_desc), 0, 0, 1, 1},
    {&__pyx_n_s_rule_id, __pyx_k_rule_id, sizeof(__pyx_k_rule_id), 0, 0, 1, 1},
    {&__pyx_n_s_rule_node, __pyx_k_rule_node, sizeof(__pyx_k_rule_node), 0, 0, 1, 1},
    {&__pyx_n_s_rules, __pyx_k_rules, sizeof(__pyx_k_rules), 0, 0, 1, 1},
    {&__pyx_n_s_run, __pyx_k_run, sizeof(__pyx_k_run), 0, 0, 1, 1},
    {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
    {&__pyx_n_s_sQQideR_DatabaseContext, __pyx_k_sQQideR_DatabaseContext, sizeof(__pyx_k_sQQideR_DatabaseContext), 0, 0, 1, 1},
    {&__pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_k_sQQideR_DatabaseContext_pyx, sizeof(__pyx_k_sQQideR_DatabaseContext_pyx), 0, 0, 1, 0},
    {&__pyx_kp_u_s_LIMIT_s_d_s, __pyx_k_s_LIMIT_s_d_s, sizeof(__pyx_k_s_LIMIT_s_d_s), 0, 1, 0, 0},
    {&__pyx_n_s_schema, __pyx_k_schema, sizeof(__pyx_k_schema), 0, 0, 1, 1},
    {&__pyx_n_s_score, __pyx_k_score, sizeof(__pyx_k_score), 0, 0, 1, 1},
    {&__pyx_n_s_search, __pyx_k_search, sizeof(__pyx_k_search), 0, 0, 1, 1},
    {&__pyx_n_s_search_by_keywords, __pyx_k_search_by_keywords, sizeof(__pyx_k_search_by_keywords), 0, 0, 1, 1},
    {&__pyx_n_s_select_clause, __pyx_k_select_clause, sizeof(__pyx_k_select_clause), 0, 0, 1, 1},
    {&__pyx_n_s_selected_query, __pyx_k_selected_query, sizeof(__pyx_k_selected_query), 0, 0, 1, 1},
    {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
    {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
    {&__pyx_n_s_service, __pyx_k_service, sizeof(__pyx_k_service), 0, 0, 1, 1},
    {&__pyx_n_s_set_name, __pyx_k_set_name, sizeof(__pyx_k_set_name), 0, 0, 1, 1},
    {&__pyx_n_s_shortest_path, __pyx_k_shortest_path, sizeof(__pyx_k_shortest_path), 0, 0, 1, 1},
    {&__pyx_n_u_simple, __pyx_k_simple, sizeof(__pyx_k_simple), 0, 1, 0, 1},
    {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
    {&__pyx_n_s_sorted_nodes, __pyx_k_sorted_nodes, sizeof(__pyx_k_sorted_nodes), 0, 0, 1, 1},
    {&__pyx_n_u_source, __pyx_k_source, sizeof(__pyx_k_source), 0, 1, 0, 1},
    {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
    {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
    {&__pyx_n_s_sql, __pyx_k_sql, sizeof(__pyx_k_sql), 0, 0, 1, 1},
    {&__pyx_n_s_sql_lines, __pyx_k_sql_lines, sizeof(__pyx_k_sql_lines), 0, 0, 1, 1},
    {&__pyx_n_u_src, __pyx_k_src, sizeof(__pyx_k_src), 0, 1, 0, 1},
    {&__pyx_n_s_src_name, __pyx_k_src_name, sizeof(__pyx_k_src_name), 0, 0, 1, 1},
    {&__pyx_n_s_start_node, __pyx_k_start_node, sizeof(__pyx_k_start_node), 0, 0, 1, 1},
    {&__pyx_n_s_startswith, __pyx_k_startswith, sizeof(__pyx_k_startswith), 0, 0, 1, 1},
    {&__pyx_n_u_statistics, __pyx_k_statistics, sizeof(__pyx_k_statistics), 0, 1, 0, 1},
    {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
    {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
    {&__pyx_n_s_sub, __pyx_k_sub, sizeof(__pyx_k_sub), 0, 0, 1, 1},
    {&__pyx_n_s_suffix, __pyx_k_suffix, sizeof(__pyx_k_suffix), 0, 0, 1, 1},
    {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
    {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
    {&__pyx_kp_u_table, __pyx_k_table, sizeof(__pyx_k_table), 0, 1, 0, 0},
    {&__pyx_n_s_table_2, __pyx_k_table_2, sizeof(__pyx_k_table_2), 0, 0, 1, 1},
    {&__pyx_n_u_table_2, __pyx_k_table_2, sizeof(__pyx_k_table_2), 0, 1, 0, 1},
    {&__pyx_n_s_table_count, __pyx_k_table_count, sizeof(__pyx_k_table_count), 0, 0, 1, 1},
    {&__pyx_n_s_table_data, __pyx_k_table_data, sizeof(__pyx_k_table_data), 0, 0, 1, 1},
    {&__pyx_n_s_table_id, __pyx_k_table_id, sizeof(__pyx_k_table_id), 0, 0, 1, 1},
    {&__pyx_n_s_table_name, __pyx_k_table_name, sizeof(__pyx_k_table_name), 0, 0, 1, 1},
    {&__pyx_n_s_table_names, __pyx_k_table_names, sizeof(__pyx_k_table_names), 0, 0, 1, 1},
    {&__pyx_n_s_table_nodes, __pyx_k_table_nodes, sizeof(__pyx_k_table_nodes), 0, 0, 1, 1},
    {&__pyx_n_s_tables, __pyx_k_tables, sizeof(__pyx_k_tables), 0, 0, 1, 1},
    {&__pyx_n_u_tables, __pyx_k_tables, sizeof(__pyx_k_tables), 0, 1, 0, 1},
    {&__pyx_kp_u_tables_and, __pyx_k_tables_and, sizeof(__pyx_k_tables_and), 0, 1, 0, 0},
    {&__pyx_n_u_tags, __pyx_k_tags, sizeof(__pyx_k_tags), 0, 1, 0, 1},
    {&__pyx_n_s_target, __pyx_k_target, sizeof(__pyx_k_target), 0, 0, 1, 1},
    {&__pyx_n_u_target, __pyx_k_target, sizeof(__pyx_k_target), 0, 1, 0, 1},
    {&__pyx_n_s_targets, __pyx_k_targets, sizeof(__pyx_k_targets), 0, 0, 1, 1},
    {&__pyx_n_s_tbl, __pyx_k_tbl, sizeof(__pyx_k_tbl), 0, 0, 1, 1},
    {&__pyx_n_s_template_query, __pyx_k_template_query, sizeof(__pyx_k_template_query), 0, 0, 1, 1},
    {&__pyx_kp_u_term, __pyx_k_term, sizeof(__pyx_k_term), 0, 1, 0, 0},
    {&__pyx_n_s_term_2, __pyx_k_term_2, sizeof(__pyx_k_term_2), 0, 0, 1, 1},
    {&__pyx_n_u_term_2, __pyx_k_term_2, sizeof(__pyx_k_term_2), 0, 1, 0, 1},
    {&__pyx_n_s_term_id, __pyx_k_term_id, sizeof(__pyx_k_term_id), 0, 0, 1, 1},
    {&__pyx_n_u_terminology, __pyx_k_terminology, sizeof(__pyx_k_terminology), 0, 1, 0, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_tgt_name, __pyx_k_tgt_name, sizeof(__pyx_k_tgt_name), 0, 0, 1, 1},
    {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
    {&__pyx_n_u_to_table, __pyx_k_to_table, sizeof(__pyx_k_to_table), 0, 1, 0, 1},
    {&__pyx_n_s_to_table_id, __pyx_k_to_table_id, sizeof(__pyx_k_to_table_id), 0, 0, 1, 1},
    {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
    {&__pyx_n_u_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 1, 0, 1},
    {&__pyx_n_s_typing, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
    {&__pyx_n_s_unique_rules, __pyx_k_unique_rules, sizeof(__pyx_k_unique_rules), 0, 0, 1, 1},
    {&__pyx_n_u_unknown, __pyx_k_unknown, sizeof(__pyx_k_unknown), 0, 1, 0, 1},
    {&__pyx_n_s_used_aliases, __pyx_k_used_aliases, sizeof(__pyx_k_used_aliases), 0, 0, 1, 1},
    {&__pyx_n_s_user_prompt, __pyx_k_user_prompt, sizeof(__pyx_k_user_prompt), 0, 0, 1, 1},
    {&__pyx_kp_u_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0, 0},
    {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
    {&__pyx_n_s_verbose, __pyx_k_verbose, sizeof(__pyx_k_verbose), 0, 0, 1, 1},
    {&__pyx_kp_u_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 1, 0, 0},
    {&__pyx_n_s_version_2, __pyx_k_version_2, sizeof(__pyx_k_version_2), 0, 0, 1, 1},
    {&__pyx_n_s_where_clauses, __pyx_k_where_clauses, sizeof(__pyx_k_where_clauses), 0, 0, 1, 1},
    {&__pyx_n_s_word, __pyx_k_word, sizeof(__pyx_k_word), 0, 0, 1, 1},
    {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(0, 1164, __pyx_L1_error)
  __pyx_builtin_open = __Pyx_GetBuiltinName(__pyx_n_s_open); if (!__pyx_builtin_open) __PYX_ERR(0, 57, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 99, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 397, __pyx_L1_error)
  __pyx_builtin_sorted = __Pyx_GetBuiltinName(__pyx_n_s_sorted); if (!__pyx_builtin_sorted) __PYX_ERR(0, 521, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 619, __pyx_L1_error)
  __pyx_builtin_max = __Pyx_GetBuiltinName(__pyx_n_s_max); if (!__pyx_builtin_max) __PYX_ERR(0, 733, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "sQQideR/DatabaseContext.pyx":57
 *             raise FileNotFoundError(f"Metadata file not found: {filename}")
 * 
 *         with open(filename, 'r', encoding='utf-8') as f:  # Explicitly specify UTF-8 encoding             # <<<<<<<<<<<<<<
 *             self.metadata = json.load(f)
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "sQQideR/DatabaseContext.pyx":76
 *             # Additional info about enriched sections
 *             if "metadata_cleaning" in metadata_sections:
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")             # <<<<<<<<<<<<<<
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")
 */
  __pyx_tuple__3 = PyTuple_Pack(2, __pyx_n_u_cleaned_at, __pyx_n_u_unknown); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "sQQideR/DatabaseContext.pyx":77
 *             if "metadata_cleaning" in metadata_sections:
 *                 cleaned_at = self.metadata["metadata_cleaning"].get("cleaned_at", "unknown")
 *                 version = self.metadata["metadata_cleaning"].get("cleaning_version", "unknown")             # <<<<<<<<<<<<<<
 *                 print(f"  - Metadata was cleaned at {cleaned_at} (version {version})")
 * 
 */
  __pyx_tuple__4 = PyTuple_Pack(2, __pyx_n_u_cleaning_version, __pyx_n_u_unknown); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "sQQideR/DatabaseContext.pyx":99
 *         """
 *         if self.graph is None:
 *             raise ValueError("Graph not built. Call build_graph first.")             # <<<<<<<<<<<<<<
 * 
 *         # Get AI-enhanced query information
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Graph_not_built_Call_build_graph); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "sQQideR/DatabaseContext.pyx":152
 *                 # Remove any trailing markdown code block identifiers
 *                 if clean_response.endswith("```"):
 *                     clean_response = clean_response[:-3].strip()             # <<<<<<<<<<<<<<
 * 
 *                 try:
 */
  __pyx_slice__11 = PySlice_New(Py_None, __pyx_int_neg_3, Py_None); if (unlikely(!__pyx_slice__11)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__11);
  __Pyx_GIVEREF(__pyx_slice__11);

  /* "sQQideR/DatabaseContext.pyx":159
 * 
 *                     if self.verbose:
 *                         print(f"AI enhanced query: {parsed.get('enhanced_query', '')}")             # <<<<<<<<<<<<<<
 *                         print(f"AI extracted keywords: {parsed.get('keywords', [])}")
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 */
  __pyx_tuple__13 = PyTuple_Pack(2, __pyx_n_u_enhanced_query, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "sQQideR/DatabaseContext.pyx":163
 *                         print(f"AI identified domains: {parsed.get('business_domains', [])}")
 *                         if 'history_context' in parsed:
 *                             print(f"AI history context: {parsed.get('history_context', '')}")             # <<<<<<<<<<<<<<
 * 
 *                     enhanced_query = SQLPrompt(
 */
  __pyx_tuple__14 = PyTuple_Pack(2, __pyx_n_u_history_context, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "sQQideR/DatabaseContext.pyx":233
 *                         for neighbor in self.graph.neighbors(node_id):
 *                             if neighbor == concept:
 *                                 table_name = node_id.split(".")[0].replace("column:", "")             # <<<<<<<<<<<<<<
 *                                 table_id = f"table:{table_name}"
 *                                 if table_id not in tables:
 */
  __pyx_tuple__16 = PyTuple_Pack(2, __pyx_kp_u_column, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "sQQideR/DatabaseContext.pyx":319
 *         if metadata is None:
 *             if self.metadata is None:
 *                 raise ValueError("No metadata provided. Call load_metadata first or provide metadata.")             # <<<<<<<<<<<<<<
 *             metadata = self.metadata
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_u_No_metadata_provided_Call_load_m); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "sQQideR/DatabaseContext.pyx":333
 *                        description=table_data["description"],
 *                        domain=table_data["domain"],
 *                        row_count=table_data.get("row_count", 0))             # <<<<<<<<<<<<<<
 * 
 *             # Add columns as nodes and connect to tables
 */
  __pyx_tuple__18 = PyTuple_Pack(2, __pyx_n_u_row_count, __pyx_int_0); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "sQQideR/DatabaseContext.pyx":343
 *                     "description": col_data["description"],
 *                     "data_type": col_data["data_type"],
 *                     "indexed": col_data.get("indexed", False)             # <<<<<<<<<<<<<<
 *                 }
 *                 # Add statistics if available
 */
  __pyx_tuple__19 = PyTuple_Pack(2, __pyx_n_u_indexed, Py_False); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "sQQideR/DatabaseContext.pyx":382
 *                                        type="concept",
 *                                        name=concept,
 *                                        description=concept_data.get("description", ""))             # <<<<<<<<<<<<<<
 *                         G.add_edge(col_node_id, concept_id, relationship="implements")
 * 
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_u_description, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "sQQideR/DatabaseContext.pyx":403
 *                            query=example["query"],
 *                            description=example["description"],
 *                            complexity=example.get("complexity", "simple"))             # <<<<<<<<<<<<<<
 *                 # Connect examples to related tables
 *                 for table_name in metadata["tables"]:
 */
  __pyx_tuple__22 = PyTuple_Pack(2, __pyx_n_u_complexity, __pyx_n_u_simple); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "sQQideR/DatabaseContext.pyx":601
 *         used_aliases = set()
 * 
 *         def get_alias(name):             # <<<<<<<<<<<<<<
 *             base = name[0].lower()
 *             alias = base
 */
  __pyx_tuple__24 = PyTuple_Pack(4, __pyx_n_s_name, __pyx_n_s_base, __pyx_n_s_alias, __pyx_n_s_suffix); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_get_alias, 601, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 601, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":612
 * 
 *         for t in table_nodes:
 *             table_name = t.replace("table:", "")             # <<<<<<<<<<<<<<
 *             from_tables.append(table_name)
 *             aliases[table_name] = get_alias(table_name)
 */
  __pyx_tuple__26 = PyTuple_Pack(2, __pyx_kp_u_table, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "sQQideR/DatabaseContext.pyx":758
 * 
 *         # Remove any literal date strings (e.g., '2025-04-05') if no date or day is provided
 *         if not any(word in natural_language_query.lower() for word in ["date", "day"]):             # <<<<<<<<<<<<<<
 *             sql = re.sub(r"'(\d{4}-\d{2}-\d{2})'", "?", sql)
 * 
 */
  __pyx_tuple__30 = PyTuple_Pack(2, __pyx_n_u_date, __pyx_n_u_day); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "sQQideR/DatabaseContext.pyx":968
 *                             "description": self.graph.nodes[t].get("description", "")} for t in tables],
 *                 "columns": [{"id": c, "name": c.replace("column:", ""),
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),             # <<<<<<<<<<<<<<
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),
 */
  __pyx_tuple__32 = PyTuple_Pack(2, __pyx_n_u_data_type, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "sQQideR/DatabaseContext.pyx":970
 *                             "data_type": self.graph.nodes[c].get("data_type", ""),
 *                             "description": self.graph.nodes[c].get("description", "")} for c in columns],
 *                 "concepts": [{"id": c, "name": c.replace("concept:", ""),             # <<<<<<<<<<<<<<
 *                              "description": self.graph.nodes[c].get("description", "")} for c in concepts],
 *             },
 */
  __pyx_tuple__33 = PyTuple_Pack(2, __pyx_kp_u_concept, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 970, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "sQQideR/DatabaseContext.pyx":989
 *                             "source": tables[i].replace("table:", ""),
 *                             "target": tables[j].replace("table:", ""),
 *                             "join_condition": edge_data.get("join_condition", ""),             # <<<<<<<<<<<<<<
 *                             "description": edge_data.get("description", "")
 *                         })
 */
  __pyx_tuple__34 = PyTuple_Pack(2, __pyx_n_u_join_condition, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "sQQideR/DatabaseContext.pyx":1150
 *         for rel in self.metadata.get("relationships", []):
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),             # <<<<<<<<<<<<<<
 *                 "to_table": rel.get("to_table", ""),
 *                 "join_condition": rel.get("join_condition", "")
 */
  __pyx_tuple__46 = PyTuple_Pack(2, __pyx_n_u_from_table, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "sQQideR/DatabaseContext.pyx":1151
 *             schema["relationships"].append({
 *                 "from_table": rel.get("from_table", ""),
 *                 "to_table": rel.get("to_table", ""),             # <<<<<<<<<<<<<<
 *                 "join_condition": rel.get("join_condition", "")
 *             })
 */
  __pyx_tuple__47 = PyTuple_Pack(2, __pyx_n_u_to_table, __pyx_kp_u__12); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "sQQideR/DatabaseContext.pyx":12
 * import networkx as nx
 * import re
 * import os.path             # <<<<<<<<<<<<<<
 * import datetime
 * import asyncio
 */
  __pyx_tuple__49 = PyTuple_Pack(2, __pyx_n_s_os, __pyx_n_s_path); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "sQQideR/DatabaseContext.pyx":27
 *     """
 * 
 *     def __init__(self, metadata_file: str = None, verbose: bool = False, ai_base=None):             # <<<<<<<<<<<<<<
 *         """
 *         Initialize the service with optional metadata file.
 */
  __pyx_tuple__50 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_metadata_file, __pyx_n_s_verbose, __pyx_n_s_ai_base); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_init, 27, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_tuple__52 = PyTuple_Pack(3, Py_None, ((PyObject *)Py_False), Py_None); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "sQQideR/DatabaseContext.pyx":44
 *             self.load_metadata(metadata_file)
 * 
 *     def load_metadata(self, filename: str) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Load metadata from a JSON file.
 */
  __pyx_tuple__53 = PyTuple_Pack(11, __pyx_n_s_self, __pyx_n_s_filename, __pyx_n_s_f, __pyx_n_s_metadata_sections, __pyx_n_s_table_count, __pyx_n_s_relationship_count, __pyx_n_s_required_sections, __pyx_n_s_missing, __pyx_n_s_cleaned_at, __pyx_n_s_version_2, __pyx_n_s_s); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);
  __pyx_codeobj__54 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__53, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_load_metadata, 44, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__54)) __PYX_ERR(0, 44, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":84
 *     # ...existing code...
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,             # <<<<<<<<<<<<<<
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:
 *         """
 */
  __pyx_tuple__56 = PyTuple_Pack(40, __pyx_n_s_self, __pyx_n_s_query_text, __pyx_n_s_chat_history, __pyx_n_s_previous_sql, __pyx_n_s_output_format, __pyx_n_s_enhanced_query, __pyx_n_s_context_prompt, __pyx_n_s_msg, __pyx_n_s_role, __pyx_n_s_response, __pyx_n_s_clean_response, __pyx_n_s_first_newline, __pyx_n_s_parsed, __pyx_n_s_e, __pyx_n_s_keywords, __pyx_n_s_results, __pyx_n_s_matches, __pyx_n_s_ai_context, __pyx_n_s_tables, __pyx_n_s_columns, __pyx_n_s_concepts, __pyx_n_s_node_id, __pyx_n_s_data, __pyx_n_s_domain, __pyx_n_s_domain_results, __pyx_n_s__55, __pyx_n_s_concept_2, __pyx_n_s_neighbor, __pyx_n_s_table_name, __pyx_n_s_table_id, __pyx_n_s_all_tables, __pyx_n_s_all_rules, __pyx_n_s_node_rules, __pyx_n_s_unique_rules, __pyx_n_s_rule_id, __pyx_n_s_rule_desc, __pyx_n_s_filtered_rules, __pyx_n_s_sql, __pyx_n_s_output, __pyx_n_s_word); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);
  __pyx_codeobj__57 = (PyObject*)__Pyx_PyCode_New(5, 0, 0, 40, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__56, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_process_query_with_chat_history, 84, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__57)) __PYX_ERR(0, 84, __pyx_L1_error)
  __pyx_tuple__58 = PyTuple_Pack(3, Py_None, Py_None, ((PyObject*)__pyx_n_u_json)); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "sQQideR/DatabaseContext.pyx":285
 *         return output
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",             # <<<<<<<<<<<<<<
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:
 *         """
 */
  __pyx_tuple__59 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_query_text, __pyx_n_s_output_format, __pyx_n_s_chat_history, __pyx_n_s_previous_sql); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  __pyx_codeobj__60 = (PyObject*)__Pyx_PyCode_New(5, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__59, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_process_query, 285, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__60)) __PYX_ERR(0, 285, __pyx_L1_error)
  __pyx_tuple__61 = PyTuple_Pack(3, ((PyObject*)__pyx_n_u_json), Py_None, Py_None); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "sQQideR/DatabaseContext.pyx":307
 * 
 * 
 *     def build_graph(self, metadata: Dict = None) -> nx.DiGraph:             # <<<<<<<<<<<<<<
 *         """
 *         Build a graph from the metadata.
 */
  __pyx_tuple__62 = PyTuple_Pack(29, __pyx_n_s_self, __pyx_n_s_metadata, __pyx_n_s_G, __pyx_n_s_table_name, __pyx_n_s_table_data, __pyx_n_s_node_id, __pyx_n_s_col_name, __pyx_n_s_col_data, __pyx_n_s_col_node_id, __pyx_n_s_node_props, __pyx_n_s_rule_2, __pyx_n_s_rule_id, __pyx_n_s_rule_data, __pyx_n_s_concept_2, __pyx_n_s_concept_id, __pyx_n_s_concept_data, __pyx_n_s_rel, __pyx_n_s_from_table_id, __pyx_n_s_to_table_id, __pyx_n_s_i, __pyx_n_s_example_2, __pyx_n_s_example_id, __pyx_n_s_keyword_2, __pyx_n_s_targets, __pyx_n_s_keyword_id, __pyx_n_s_target, __pyx_n_s_term_2, __pyx_n_s_definition, __pyx_n_s_term_id); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 29, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_build_graph, 307, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_tuple__64 = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */
  __pyx_tuple__65 = PyTuple_Pack(20, __pyx_n_s_self, __pyx_n_s_keywords, __pyx_n_s_results, __pyx_n_s_matches, __pyx_n_s_normalized_keywords, __pyx_n_s_node_scores, __pyx_n_s_node_id, __pyx_n_s_data, __pyx_n_s_node_score, __pyx_n_s_node_matches, __pyx_n_s_keyword_2, __pyx_n_s_pattern, __pyx_n_s_matched, __pyx_n_s_match_reason, __pyx_n_s_match_score, __pyx_n_s_sorted_nodes, __pyx_n_s_score, __pyx_n_s_k, __pyx_n_s_genexpr, __pyx_n_s_genexpr); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);
  __pyx_codeobj__66 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 20, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__65, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_search_by_keywords, 446, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__66)) __PYX_ERR(0, 446, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":529
 *         return results, matches
 * 
 *     def get_related_rules(self, node_id: str) -> List[Tuple[str, str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Get business rules related to a node.
 */
  __pyx_tuple__67 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_node_id, __pyx_n_s_rules, __pyx_n_s_neighbor, __pyx_n_s_rule_node); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);
  __pyx_codeobj__68 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__67, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_get_related_rules, 529, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__68)) __PYX_ERR(0, 529, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":559
 *         return rules
 * 
 *     def find_path(self, start_node: str, end_node: str) -> Optional[List[str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Find a path between two nodes in the graph.
 */
  __pyx_tuple__69 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_start_node, __pyx_n_s_end_node, __pyx_n_s_path); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);
  __pyx_codeobj__70 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__69, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_find_path, 559, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__70)) __PYX_ERR(0, 559, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */
  __pyx_tuple__71 = PyTuple_Pack(29, __pyx_n_s_self, __pyx_n_s_table_nodes, __pyx_n_s_condition_nodes, __pyx_n_s_from_tables, __pyx_n_s_aliases, __pyx_n_s_used_aliases, __pyx_n_s_get_alias, __pyx_n_s_get_alias, __pyx_n_s_t, __pyx_n_s_table_name, __pyx_n_s_join_clauses, __pyx_n_s_i, __pyx_n_s_current_table, __pyx_n_s_join_found, __pyx_n_s_prev, __pyx_n_s_prev_table, __pyx_n_s_edge_data, __pyx_n_s_join_condition, __pyx_n_s_tbl, __pyx_n_s_alias, __pyx_n_s_where_clauses, __pyx_n_s_node, __pyx_n_s_node_data, __pyx_n_s_col_name, __pyx_n_s_sql_lines, __pyx_n_s_select_clause, __pyx_n_s_first_table, __pyx_n_s_sql, __pyx_n_s_alias); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);
  __pyx_codeobj__72 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 29, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__71, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_generate_sql, 579, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__72)) __PYX_ERR(0, 579, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */
  __pyx_tuple__73 = PyTuple_Pack(26, __pyx_n_s_self, __pyx_n_s_natural_language_query, __pyx_n_s_context, __pyx_n_s_e, __pyx_n_s_keywords, __pyx_n_s_example_scores, __pyx_n_s_example_nodes, __pyx_n_s_example_node, __pyx_n_s_example_data, __pyx_n_s_score, __pyx_n_s_keyword_2, __pyx_n_s_best_example, __pyx_n_s_template_query, __pyx_n_s_adapted_query, __pyx_n_s_sql, __pyx_n_s_tables, __pyx_n_s_columns, __pyx_n_s_results, __pyx_n_s_matches, __pyx_n_s_node_id, __pyx_n_s_data, __pyx_n_s_word, __pyx_n_s_n, __pyx_n_s_d, __pyx_n_s_genexpr, __pyx_n_s_genexpr); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);
  __pyx_codeobj__74 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 26, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__73, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_generate_sql_from_nl, 675, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__74)) __PYX_ERR(0, 675, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":766
 *         return sql
 * 
 *     def process_query_with_ai(self, query_text: str, output_format: str = "json") -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query using AI enhancement and return context information.
 */
  __pyx_tuple__75 = PyTuple_Pack(36, __pyx_n_s_self, __pyx_n_s_query_text, __pyx_n_s_output_format, __pyx_n_s_enhanced_query, __pyx_n_s_prompt, __pyx_n_s_response, __pyx_n_s_clean_response, __pyx_n_s_first_newline, __pyx_n_s_parsed, __pyx_n_s_e, __pyx_n_s_keywords, __pyx_n_s_results, __pyx_n_s_matches, __pyx_n_s_ai_context, __pyx_n_s_tables, __pyx_n_s_columns, __pyx_n_s_concepts, __pyx_n_s_node_id, __pyx_n_s_data, __pyx_n_s_domain, __pyx_n_s_domain_results, __pyx_n_s__55, __pyx_n_s_concept_2, __pyx_n_s_neighbor, __pyx_n_s_table_name, __pyx_n_s_table_id, __pyx_n_s_all_tables, __pyx_n_s_all_rules, __pyx_n_s_node_rules, __pyx_n_s_unique_rules, __pyx_n_s_rule_id, __pyx_n_s_rule_desc, __pyx_n_s_filtered_rules, __pyx_n_s_sql, __pyx_n_s_output, __pyx_n_s_word); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);
  __pyx_codeobj__76 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 36, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__75, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_process_query_with_ai, 766, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__76)) __PYX_ERR(0, 766, __pyx_L1_error)
  __pyx_tuple__77 = PyTuple_Pack(1, ((PyObject*)__pyx_n_u_json)); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "sQQideR/DatabaseContext.pyx":942
 *         return output
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,             # <<<<<<<<<<<<<<
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:
 */
  __pyx_tuple__78 = PyTuple_Pack(29, __pyx_n_s_self, __pyx_n_s_query_text, __pyx_n_s_results, __pyx_n_s_matches, __pyx_n_s_tables, __pyx_n_s_columns, __pyx_n_s_concepts, __pyx_n_s_rules, __pyx_n_s_sql, __pyx_n_s_all_tables, __pyx_n_s_output, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_edge_data, __pyx_n_s_table_names, __pyx_n_s_src_name, __pyx_n_s_tgt_name, __pyx_n_s_relationship, __pyx_n_s_rel, __pyx_n_s_table_2, __pyx_n_s_table_name, __pyx_n_s_column_2, __pyx_n_s_col_name, __pyx_n_s_t, __pyx_n_s_c, __pyx_n_s_c, __pyx_n_s_r, __pyx_n_s_t, __pyx_n_s_t); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);
  __pyx_codeobj__79 = (PyObject*)__Pyx_PyCode_New(10, 0, 0, 29, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__78, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_generate_output, 942, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__79)) __PYX_ERR(0, 942, __pyx_L1_error)
  __pyx_tuple__80 = PyTuple_Pack(2, Py_None, Py_None); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "sQQideR/DatabaseContext.pyx":1059
 *         return output
 * 
 *     def format_as_markdown(self, output_data: Dict) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Format context information as Markdown.
 */
  __pyx_tuple__81 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_output_data, __pyx_n_s_md_output, __pyx_n_s_table_2, __pyx_n_s_column_2, __pyx_n_s_concept_2, __pyx_n_s_rel, __pyx_n_s_rule_2); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);
  __pyx_codeobj__82 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__81, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_format_as_markdown, 1059, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__82)) __PYX_ERR(0, 1059, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":1120
 *         return md_output
 * 
 *     def _get_schema_summary(self) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Generate a summary of the database schema for AI context.
 */
  __pyx_tuple__83 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_schema, __pyx_n_s_table_name, __pyx_n_s_table_data, __pyx_n_s_col_name, __pyx_n_s_col_data, __pyx_n_s_rel); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);
  __pyx_codeobj__84 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__83, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_sQQideR_DatabaseContext_pyx, __pyx_n_s_get_schema_summary, 1120, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__84)) __PYX_ERR(0, 1120, __pyx_L1_error)

  /* "sQQideR/DatabaseContext.pyx":1164
 *     # Load metadata from the cleaned file
 *     try:
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")             # <<<<<<<<<<<<<<
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 */
  __pyx_tuple__85 = PyTuple_Pack(1, __pyx_kp_u_Attempting_to_load_enriched_proj); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(0, 1164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "sQQideR/DatabaseContext.pyx":1166
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")             # <<<<<<<<<<<<<<
 *         if not os.path.exists(file_path):
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 */
  __pyx_tuple__86 = PyTuple_Pack(3, __pyx_n_u_src, __pyx_n_u_data, __pyx_kp_u_enriched_project_metadata_cleane); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(0, 1166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "sQQideR/DatabaseContext.pyx":1168
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 *         if not os.path.exists(file_path):
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")             # <<<<<<<<<<<<<<
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:
 */
  __pyx_tuple__87 = PyTuple_Pack(2, __pyx_n_u_data, __pyx_kp_u_enriched_project_metadata_cleane); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "sQQideR/DatabaseContext.pyx":1171
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:
 *         print("Failed to load metadata file. Please ensure the file exists.")             # <<<<<<<<<<<<<<
 *         raise e
 * 
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_kp_u_Failed_to_load_metadata_file_Ple); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "sQQideR/DatabaseContext.pyx":1194
 *     # Get context in Markdown format
 *     context_md = service.process_query(selected_query, output_format="markdown")
 *     print("\nMarkdown Output:")             # <<<<<<<<<<<<<<
 *     print(context_md["markdown"])
 * 
 */
  __pyx_tuple__89 = PyTuple_Pack(1, __pyx_kp_u_Markdown_Output); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(0, 1194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "sQQideR/DatabaseContext.pyx":1198
 * 
 *     # Show the generated SQL
 *     sql = context.get("best_guess_sql", "No SQL generated")             # <<<<<<<<<<<<<<
 *     print(f"\nGenerated SQL: {sql}")
 */
  __pyx_tuple__90 = PyTuple_Pack(2, __pyx_n_u_best_guess_sql, __pyx_kp_u_No_SQL_generated); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(0, 1198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  __pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_get.method_name = &__pyx_n_s_get;
  __pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_items.method_name = &__pyx_n_s_items;
  __pyx_umethod_PyDict_Type_keys.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_keys.method_name = &__pyx_n_s_keys;
  __pyx_umethod_PyList_Type_copy.type = (PyObject*)&PyList_Type;
  __pyx_umethod_PyList_Type_copy.method_name = &__pyx_n_s_copy;
  __pyx_umethod_PyUnicode_Type_lower.type = (PyObject*)&PyUnicode_Type;
  __pyx_umethod_PyUnicode_Type_lower.method_name = &__pyx_n_s_lower;
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_3 = PyInt_FromLong(-3); if (unlikely(!__pyx_int_neg_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  return 0;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords_spec, NULL); if (unlikely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords)) __PYX_ERR(0, 446, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords_spec, __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  #else
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords = &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords->tp_dictoffset && __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct__search_by_keywords->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr_spec, NULL); if (unlikely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr)) __PYX_ERR(0, 497, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr_spec, __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  #else
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr = &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr->tp_dictoffset && __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_1_genexpr->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql_spec, NULL); if (unlikely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql)) __PYX_ERR(0, 579, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql_spec, __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  #else
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql = &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql->tp_dictoffset && __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_2_generate_sql->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl_spec, NULL); if (unlikely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl)) __PYX_ERR(0, 675, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl_spec, __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl) < 0) __PYX_ERR(0, 675, __pyx_L1_error)
  #else
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl = &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl) < 0) __PYX_ERR(0, 675, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl->tp_dictoffset && __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_3_generate_sql_from_nl->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr_spec, NULL); if (unlikely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr)) __PYX_ERR(0, 758, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr_spec, __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr) < 0) __PYX_ERR(0, 758, __pyx_L1_error)
  #else
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr = &__pyx_type_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr) < 0) __PYX_ERR(0, 758, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr->tp_dictoffset && __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7sQQideR_15DatabaseContext___pyx_scope_struct_4_genexpr->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_DatabaseContext(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_DatabaseContext},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "DatabaseContext",
      __pyx_k_Database_Context_Service_A_serv, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initDatabaseContext(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initDatabaseContext(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_DatabaseContext(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_DatabaseContext(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_DatabaseContext(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'DatabaseContext' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("DatabaseContext", __pyx_methods, __pyx_k_Database_Context_Service_A_serv, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "DatabaseContext" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_DatabaseContext(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_sQQideR__DatabaseContext) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "sQQideR.DatabaseContext")) {
      if (unlikely((PyDict_SetItemString(modules, "sQQideR.DatabaseContext", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely((__Pyx_modinit_type_init_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "sQQideR/DatabaseContext.pyx":9
 * """
 * 
 * import json             # <<<<<<<<<<<<<<
 * import networkx as nx
 * import re
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_json, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_json, __pyx_t_2) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":10
 * 
 * import json
 * import networkx as nx             # <<<<<<<<<<<<<<
 * import re
 * import os.path
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_networkx, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nx, __pyx_t_2) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":11
 * import json
 * import networkx as nx
 * import re             # <<<<<<<<<<<<<<
 * import os.path
 * import datetime
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_re, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re, __pyx_t_2) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":12
 * import networkx as nx
 * import re
 * import os.path             # <<<<<<<<<<<<<<
 * import datetime
 * import asyncio
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_os_path, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_2) < 0) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":13
 * import re
 * import os.path
 * import datetime             # <<<<<<<<<<<<<<
 * import asyncio
 * from typing import Dict, List, Tuple, Optional, Any, Union
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_datetime, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_2) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":14
 * import os.path
 * import datetime
 * import asyncio             # <<<<<<<<<<<<<<
 * from typing import Dict, List, Tuple, Optional, Any, Union
 * from Models import SQLPrompt, ChatMessage
 */
  __pyx_t_2 = __Pyx_patch_asyncio(__Pyx_ImportDottedModule(__pyx_n_s_asyncio, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_asyncio, __pyx_t_2) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":15
 * import datetime
 * import asyncio
 * from typing import Dict, List, Tuple, Optional, Any, Union             # <<<<<<<<<<<<<<
 * from Models import SQLPrompt, ChatMessage
 * 
 */
  __pyx_t_2 = PyList_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Dict);
  __Pyx_GIVEREF(__pyx_n_s_Dict);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Dict)) __PYX_ERR(0, 15, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_List);
  __Pyx_GIVEREF(__pyx_n_s_List);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_List)) __PYX_ERR(0, 15, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Tuple);
  __Pyx_GIVEREF(__pyx_n_s_Tuple);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_Tuple)) __PYX_ERR(0, 15, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Optional);
  __Pyx_GIVEREF(__pyx_n_s_Optional);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_Optional)) __PYX_ERR(0, 15, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Any);
  __Pyx_GIVEREF(__pyx_n_s_Any);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_s_Any)) __PYX_ERR(0, 15, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_Union);
  __Pyx_GIVEREF(__pyx_n_s_Union);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_s_Union)) __PYX_ERR(0, 15, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_typing, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_Dict); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Dict, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_List); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_List, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_Tuple); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Tuple, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_Optional); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Optional, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_Any); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Any, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_Union); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Union, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":16
 * import asyncio
 * from typing import Dict, List, Tuple, Optional, Any, Union
 * from Models import SQLPrompt, ChatMessage             # <<<<<<<<<<<<<<
 * 
 * class DatabaseContextService:
 */
  __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_SQLPrompt);
  __Pyx_GIVEREF(__pyx_n_s_SQLPrompt);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_SQLPrompt)) __PYX_ERR(0, 16, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_ChatMessage);
  __Pyx_GIVEREF(__pyx_n_s_ChatMessage);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_ChatMessage)) __PYX_ERR(0, 16, __pyx_L1_error);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_Models, __pyx_t_3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_SQLPrompt); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SQLPrompt, __pyx_t_3) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ChatMessage); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ChatMessage, __pyx_t_3) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":18
 * from Models import SQLPrompt, ChatMessage
 * 
 * class DatabaseContextService:             # <<<<<<<<<<<<<<
 *     """
 *     Service for providing database context from natural language queries.
 */
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_DatabaseContextService, __pyx_n_s_DatabaseContextService, (PyObject *) NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_kp_s_Service_for_providing_database); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "sQQideR/DatabaseContext.pyx":27
 *     """
 * 
 *     def __init__(self, metadata_file: str = None, verbose: bool = False, ai_base=None):             # <<<<<<<<<<<<<<
 *         """
 *         Initialize the service with optional metadata file.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_metadata_file, __pyx_n_s_str) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_verbose, __pyx_n_s_bool) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_1__init__, 0, __pyx_n_s_DatabaseContextService___init, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__51)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__52);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":44
 *             self.load_metadata(metadata_file)
 * 
 *     def load_metadata(self, filename: str) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Load metadata from a JSON file.
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_filename, __pyx_n_s_str) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_3load_metadata, 0, __pyx_n_s_DatabaseContextService_load_meta, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__54)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_load_metadata, __pyx_t_3) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":84
 *     # ...existing code...
 * 
 *     def process_query_with_chat_history(self, query_text: str, chat_history: List[ChatMessage] = None,             # <<<<<<<<<<<<<<
 *                                         previous_sql: str = None, output_format: str = "json") -> Dict:
 *         """
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_query_text, __pyx_n_s_str) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_chat_history, __pyx_kp_s_List_ChatMessage) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_previous_sql, __pyx_n_s_str) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_output_format, __pyx_n_s_str) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_5process_query_with_chat_history, 0, __pyx_n_s_DatabaseContextService_process_q, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__57)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__58);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_process_query_with_chat_history, __pyx_t_4) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":285
 *         return output
 * 
 *     def process_query(self, query_text: str, output_format: str = "json",             # <<<<<<<<<<<<<<
 *                       chat_history: List[ChatMessage] = None, previous_sql: str = None) -> Dict:
 *         """
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_query_text, __pyx_n_s_str) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_output_format, __pyx_n_s_str) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_chat_history, __pyx_kp_s_List_ChatMessage) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_previous_sql, __pyx_n_s_str) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_7process_query, 0, __pyx_n_s_DatabaseContextService_process_q_2, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__60)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_tuple__61);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_process_query, __pyx_t_3) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":307
 * 
 * 
 *     def build_graph(self, metadata: Dict = None) -> nx.DiGraph:             # <<<<<<<<<<<<<<
 *         """
 *         Build a graph from the metadata.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_metadata, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_kp_s_nx_DiGraph) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_9build_graph, 0, __pyx_n_s_DatabaseContextService_build_gra, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__63)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__64);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_build_graph, __pyx_t_4) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":446
 *         return G
 * 
 *     def search_by_keywords(self, keywords: List[str]) -> Tuple[List[Tuple[str, Dict]], Dict[str, List[str]]]:             # <<<<<<<<<<<<<<
 *         """
 *         Search the graph for nodes matching the given keywords.
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_keywords, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_kp_s_Tuple_List_Tuple_str_Dict_Dict_s) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_11search_by_keywords, 0, __pyx_n_s_DatabaseContextService_search_by_3, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__66)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_search_by_keywords, __pyx_t_3) < 0) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":529
 *         return results, matches
 * 
 *     def get_related_rules(self, node_id: str) -> List[Tuple[str, str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Get business rules related to a node.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_node_id, __pyx_n_s_str) < 0) __PYX_ERR(0, 529, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_kp_s_List_Tuple_str_str) < 0) __PYX_ERR(0, 529, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_13get_related_rules, 0, __pyx_n_s_DatabaseContextService_get_relat, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__68)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_get_related_rules, __pyx_t_4) < 0) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":559
 *         return rules
 * 
 *     def find_path(self, start_node: str, end_node: str) -> Optional[List[str]]:             # <<<<<<<<<<<<<<
 *         """
 *         Find a path between two nodes in the graph.
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_start_node, __pyx_n_s_str) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_end_node, __pyx_n_s_str) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_kp_s_Optional_List_str) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_15find_path, 0, __pyx_n_s_DatabaseContextService_find_path, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__70)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_find_path, __pyx_t_3) < 0) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":579
 *             return None
 * 
 *     def generate_sql(self, table_nodes: List[str], condition_nodes: List[str] = None) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate an improved SQL query based on selected tables and conditions.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_table_nodes, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_condition_nodes, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_str) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_17generate_sql, 0, __pyx_n_s_DatabaseContextService_generate_4, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__72)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__64);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_generate_sql, __pyx_t_4) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":675
 *         return sql
 * 
 *     def generate_sql_from_nl(self, natural_language_query: str) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Generate SQL based on natural language query using graph knowledge and examples.
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_natural_language_query, __pyx_n_s_str) < 0) __PYX_ERR(0, 675, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_str) < 0) __PYX_ERR(0, 675, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_19generate_sql_from_nl, 0, __pyx_n_s_DatabaseContextService_generate_5, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__74)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_generate_sql_from_nl, __pyx_t_3) < 0) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":766
 *         return sql
 * 
 *     def process_query_with_ai(self, query_text: str, output_format: str = "json") -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Process a natural language query using AI enhancement and return context information.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_query_text, __pyx_n_s_str) < 0) __PYX_ERR(0, 766, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_output_format, __pyx_n_s_str) < 0) __PYX_ERR(0, 766, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 766, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_21process_query_with_ai, 0, __pyx_n_s_DatabaseContextService_process_q_3, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__76)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_tuple__77);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_process_query_with_ai, __pyx_t_4) < 0) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":942
 *         return output
 * 
 *     def generate_output(self, query_text: str, results: List, matches: Dict,             # <<<<<<<<<<<<<<
 *                         tables: List[str], columns: List[str], concepts: List[str],
 *                         rules: List[str], sql: str = None, all_tables: List[str] = None) -> Dict:
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_query_text, __pyx_n_s_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_results, __pyx_n_s_List) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_matches, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_tables, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_columns, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_concepts, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_rules, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_sql, __pyx_n_s_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_all_tables, __pyx_kp_s_List_str) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_23generate_output, 0, __pyx_n_s_DatabaseContextService_generate_6, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__79)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_tuple__80);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_generate_output, __pyx_t_3) < 0) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":1059
 *         return output
 * 
 *     def format_as_markdown(self, output_data: Dict) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Format context information as Markdown.
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_output_data, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 1059, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_str) < 0) __PYX_ERR(0, 1059, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_25format_as_markdown, 0, __pyx_n_s_DatabaseContextService_format_as, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__82)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_format_as_markdown, __pyx_t_4) < 0) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "sQQideR/DatabaseContext.pyx":1120
 *         return md_output
 * 
 *     def _get_schema_summary(self) -> Dict:             # <<<<<<<<<<<<<<
 *         """
 *         Generate a summary of the database schema for AI context.
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_Dict) < 0) __PYX_ERR(0, 1120, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7sQQideR_15DatabaseContext_22DatabaseContextService_27_get_schema_summary, 0, __pyx_n_s_DatabaseContextService__get_sche, NULL, __pyx_n_s_sQQideR_DatabaseContext, __pyx_d, ((PyObject *)__pyx_codeobj__84)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_get_schema_summary, __pyx_t_3) < 0) __PYX_ERR(0, 1120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sQQideR/DatabaseContext.pyx":18
 * from Models import SQLPrompt, ChatMessage
 * 
 * class DatabaseContextService:             # <<<<<<<<<<<<<<
 *     """
 *     Service for providing database context from natural language queries.
 */
  __pyx_t_3 = __Pyx_Py3ClassCreate(((PyObject*)&PyType_Type), __pyx_n_s_DatabaseContextService, __pyx_empty_tuple, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DatabaseContextService, __pyx_t_3) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sQQideR/DatabaseContext.pyx":1158
 * 
 * # Example usage of the service
 * if __name__ == "__main__":             # <<<<<<<<<<<<<<
 *     # Initialize the service
 *     service = DatabaseContextService(verbose=True)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_n_u_main, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "sQQideR/DatabaseContext.pyx":1160
 * if __name__ == "__main__":
 *     # Initialize the service
 *     service = DatabaseContextService(verbose=True)             # <<<<<<<<<<<<<<
 * 
 *     # Load metadata from the cleaned file
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_DatabaseContextService); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_verbose, Py_True) < 0) __PYX_ERR(0, 1160, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_service, __pyx_t_4) < 0) __PYX_ERR(0, 1160, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1163
 * 
 *     # Load metadata from the cleaned file
 *     try:             # <<<<<<<<<<<<<<
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "sQQideR/DatabaseContext.pyx":1164
 *     # Load metadata from the cleaned file
 *     try:
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")             # <<<<<<<<<<<<<<
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__85, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1164, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "sQQideR/DatabaseContext.pyx":1166
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")             # <<<<<<<<<<<<<<
 *         if not os.path.exists(file_path):
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_os); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1166, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_path); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1166, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_join); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1166, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__86, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1166, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (PyDict_SetItem(__pyx_d, __pyx_n_s_file_path, __pyx_t_3) < 0) __PYX_ERR(0, 1166, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":1167
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 *         if not os.path.exists(file_path):             # <<<<<<<<<<<<<<
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 *         service.load_metadata(file_path)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_os); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_path); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_exists); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_file_path); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1167, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_8 = (!__pyx_t_5);
        if (__pyx_t_8) {

          /* "sQQideR/DatabaseContext.pyx":1168
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 *         if not os.path.exists(file_path):
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")             # <<<<<<<<<<<<<<
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:
 */
          __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_os); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1168, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_path); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1168, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_join); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1168, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__87, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1168, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (PyDict_SetItem(__pyx_d, __pyx_n_s_file_path, __pyx_t_4) < 0) __PYX_ERR(0, 1168, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "sQQideR/DatabaseContext.pyx":1167
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 *         file_path = os.path.join("src", "data", "enriched_project_metadata_cleaned.json")
 *         if not os.path.exists(file_path):             # <<<<<<<<<<<<<<
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 *         service.load_metadata(file_path)
 */
        }

        /* "sQQideR/DatabaseContext.pyx":1169
 *         if not os.path.exists(file_path):
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 *         service.load_metadata(file_path)             # <<<<<<<<<<<<<<
 *     except FileNotFoundError as e:
 *         print("Failed to load metadata file. Please ensure the file exists.")
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_service); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1169, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_load_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1169, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_file_path); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1169, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1169, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "sQQideR/DatabaseContext.pyx":1163
 * 
 *     # Load metadata from the cleaned file
 *     try:             # <<<<<<<<<<<<<<
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 */
      }
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L8_try_end;
      __pyx_L3_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "sQQideR/DatabaseContext.pyx":1170
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:             # <<<<<<<<<<<<<<
 *         print("Failed to load metadata file. Please ensure the file exists.")
 *         raise e
 */
      __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_4, &__pyx_t_2);
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_FileNotFoundError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1170, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_9);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_ErrRestore(__pyx_t_3, __pyx_t_4, __pyx_t_2);
      __pyx_t_3 = 0; __pyx_t_4 = 0; __pyx_t_2 = 0;
      if (__pyx_t_10) {
        __Pyx_AddTraceback("sQQideR.DatabaseContext", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 1170, __pyx_L5_except_error)
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_3);
        if (PyDict_SetItem(__pyx_d, __pyx_n_s_e, __pyx_t_4) < 0) __PYX_ERR(0, 1170, __pyx_L5_except_error)
        /*try:*/ {

          /* "sQQideR/DatabaseContext.pyx":1171
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:
 *         print("Failed to load metadata file. Please ensure the file exists.")             # <<<<<<<<<<<<<<
 *         raise e
 * 
 */
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__88, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1171, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

          /* "sQQideR/DatabaseContext.pyx":1172
 *     except FileNotFoundError as e:
 *         print("Failed to load metadata file. Please ensure the file exists.")
 *         raise e             # <<<<<<<<<<<<<<
 * 
 *     # Build the graph
 */
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_e); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1172, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_Raise(__pyx_t_9, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __PYX_ERR(0, 1172, __pyx_L15_error)
        }

        /* "sQQideR/DatabaseContext.pyx":1170
 *             file_path = os.path.join("data", "enriched_project_metadata_cleaned.json")
 *         service.load_metadata(file_path)
 *     except FileNotFoundError as e:             # <<<<<<<<<<<<<<
 *         print("Failed to load metadata file. Please ensure the file exists.")
 *         raise e
 */
        /*finally:*/ {
          __pyx_L15_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
            if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __pyx_t_10 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
            {
              if (unlikely(__Pyx_PyObject_DelAttrStr(__pyx_m, __pyx_n_s_e) < 0)) { if (likely(PyErr_ExceptionMatches(PyExc_AttributeError))) PyErr_Clear(); else __PYX_ERR(0, 1170, __pyx_L20_error) }
            }
            if (PY_MAJOR_VERSION >= 3) {
              __Pyx_XGIVEREF(__pyx_t_16);
              __Pyx_XGIVEREF(__pyx_t_17);
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            }
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
            __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
            __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
            goto __pyx_L5_except_error;
            __pyx_L20_error:;
            if (PY_MAJOR_VERSION >= 3) {
              __Pyx_XGIVEREF(__pyx_t_16);
              __Pyx_XGIVEREF(__pyx_t_17);
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            }
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
            goto __pyx_L5_except_error;
          }
        }
      }
      goto __pyx_L5_except_error;

      /* "sQQideR/DatabaseContext.pyx":1163
 * 
 *     # Load metadata from the cleaned file
 *     try:             # <<<<<<<<<<<<<<
 *         print("Attempting to load enriched_project_metadata_cleaned.json...")
 *         #the metadata file is located in src/data we need to make sure it works on all platforms
 */
      __pyx_L5_except_error:;
      __Pyx_XGIVEREF(__pyx_t_1);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_6, __pyx_t_7);
      goto __pyx_L1_error;
      __pyx_L8_try_end:;
    }

    /* "sQQideR/DatabaseContext.pyx":1175
 * 
 *     # Build the graph
 *     service.build_graph()             # <<<<<<<<<<<<<<
 * 
 *     # Define some example queries
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_service); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_build_graph); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":1178
 * 
 *     # Define some example queries
 *     example_queries = [             # <<<<<<<<<<<<<<
 *         "Show all projects with negative float",
 *         "List relationships with lag greater than 44 days",
 */
    __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_u_Show_all_projects_with_negative);
    __Pyx_GIVEREF(__pyx_kp_u_Show_all_projects_with_negative);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Show_all_projects_with_negative)) __PYX_ERR(0, 1178, __pyx_L1_error);
    __Pyx_INCREF(__pyx_kp_u_List_relationships_with_lag_grea);
    __Pyx_GIVEREF(__pyx_kp_u_List_relationships_with_lag_grea);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_kp_u_List_relationships_with_lag_grea)) __PYX_ERR(0, 1178, __pyx_L1_error);
    __Pyx_INCREF(__pyx_kp_u_What_are_the_activities_on_the_c);
    __Pyx_GIVEREF(__pyx_kp_u_What_are_the_activities_on_the_c);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_What_are_the_activities_on_the_c)) __PYX_ERR(0, 1178, __pyx_L1_error);
    __Pyx_INCREF(__pyx_kp_u_Find_activities_with_excessive_l);
    __Pyx_GIVEREF(__pyx_kp_u_Find_activities_with_excessive_l);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 3, __pyx_kp_u_Find_activities_with_excessive_l)) __PYX_ERR(0, 1178, __pyx_L1_error);
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_example_queries, __pyx_t_3) < 0) __PYX_ERR(0, 1178, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":1186
 * 
 *     # Process a selected query
 *     selected_query = example_queries[0]  # Change index to try different queries             # <<<<<<<<<<<<<<
 *     print(f"\nProcessing query: {selected_query}")
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_example_queries); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_selected_query, __pyx_t_4) < 0) __PYX_ERR(0, 1186, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1187
 *     # Process a selected query
 *     selected_query = example_queries[0]  # Change index to try different queries
 *     print(f"\nProcessing query: {selected_query}")             # <<<<<<<<<<<<<<
 * 
 *     # Get context in JSON format
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_selected_query); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Processing_query, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "sQQideR/DatabaseContext.pyx":1190
 * 
 *     # Get context in JSON format
 *     context = service.process_query(selected_query, output_format="json")             # <<<<<<<<<<<<<<
 * 
 *     # Get context in Markdown format
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_service); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_process_query); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_selected_query); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_output_format, __pyx_n_u_json) < 0) __PYX_ERR(0, 1190, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_context, __pyx_t_9) < 0) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":1193
 * 
 *     # Get context in Markdown format
 *     context_md = service.process_query(selected_query, output_format="markdown")             # <<<<<<<<<<<<<<
 *     print("\nMarkdown Output:")
 *     print(context_md["markdown"])
 */
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_service); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_process_query); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_selected_query); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_9)) __PYX_ERR(0, 1193, __pyx_L1_error);
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_output_format, __pyx_n_u_markdown) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_context_md, __pyx_t_4) < 0) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1194
 *     # Get context in Markdown format
 *     context_md = service.process_query(selected_query, output_format="markdown")
 *     print("\nMarkdown Output:")             # <<<<<<<<<<<<<<
 *     print(context_md["markdown"])
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__89, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1195
 *     context_md = service.process_query(selected_query, output_format="markdown")
 *     print("\nMarkdown Output:")
 *     print(context_md["markdown"])             # <<<<<<<<<<<<<<
 * 
 *     # Show the generated SQL
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_context_md); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_u_markdown); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1198
 * 
 *     # Show the generated SQL
 *     sql = context.get("best_guess_sql", "No SQL generated")             # <<<<<<<<<<<<<<
 *     print(f"\nGenerated SQL: {sql}")
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_context); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_get); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__90, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_sql, __pyx_t_4) < 0) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "sQQideR/DatabaseContext.pyx":1199
 *     # Show the generated SQL
 *     sql = context.get("best_guess_sql", "No SQL generated")
 *     print(f"\nGenerated SQL: {sql}")             # <<<<<<<<<<<<<<
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_sql); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Generated_SQL, __pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "sQQideR/DatabaseContext.pyx":1158
 * 
 * # Example usage of the service
 * if __name__ == "__main__":             # <<<<<<<<<<<<<<
 *     # Initialize the service
 *     service = DatabaseContextService(verbose=True)
 */
  }

  /* "sQQideR/DatabaseContext.pyx":1
 * """             # <<<<<<<<<<<<<<
 * Database Context Service
 * 
 */
  __pyx_t_9 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_9) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init sQQideR.DatabaseContext", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init sQQideR.DatabaseContext");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    type_name = __Pyx_PyType_GetName(type);
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")", name, type_name, obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return with_error ? __Pyx_PyObject_GetAttrStr(obj, attr_name) : __Pyx_PyObject_GetAttrStrNoError(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSsize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(Py_ssize_t)*3+2];
    char *dpos, *end = digits + sizeof(Py_ssize_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    Py_ssize_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (Py_ssize_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (Py_ssize_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (Py_ssize_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    CYTHON_MAYBE_UNUSED_VAR(max_char);
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    kind_shift = (result_ukind == 4) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    CYTHON_UNUSED_VAR(value_count);
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
static int
__Pyx_unicode_modifiable(PyObject *unicode)
{
    if (Py_REFCNT(unicode) != 1)
        return 0;
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (__Pyx_unicode_modifiable(left)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* SliceTupleAndList */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_crop_slice(Py_ssize_t* _start, Py_ssize_t* _stop, Py_ssize_t* _length) {
    Py_ssize_t start = *_start, stop = *_stop, length = *_length;
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    *_length = stop - start;
    *_start = start;
    *_stop = stop;
}
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length = PyList_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    if (length <= 0) {
        return PyList_New(0);
    }
    return __Pyx_PyList_FromArray(((PyListObject*)src)->ob_item + start, length);
}
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length = PyTuple_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    return __Pyx_PyTuple_FromArray(((PyTupleObject*)src)->ob_item + start, length);
}
#endif

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* PyIntCompare */
static CYTHON_INLINE int __Pyx_PyInt_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 0;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        return (a != b);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) != 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 1;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 1;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal != 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
        return ((double)a != (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* UnpackUnboundCMethod */
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
     METH_VARARGS | METH_KEYWORDS,
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #else
    if (likely(!__Pyx_CyOrPyCFunction_Check(method)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            target->method = unbound_method;
        }
    }
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_keys */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_keys, d);
    else
        return PyDict_Keys(d);
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (!r) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#elif PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if ((PY_VERSION_HEX >= 0x030600B1) && flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            #else
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            #endif
        } else if ((PY_VERSION_HEX >= 0x030700A0) && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
        
            x = (long)((unsigned long)a + (unsigned long)b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op1))) {
            return __Pyx_NewRef(op2);
        }
        if (likely(__Pyx_PyLong_IsCompact(op1))) {
            a = __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
            double result;
            
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyObjectCall2Args */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C00A2)
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C00A2
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_GetMethod;
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* RaiseUnboundLocalError */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* RaiseClosureNameError */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc;
    __Pyx_PyThreadState_declare
    #ifdef __Pyx_StopAsyncIteration_USED
    int is_async_stopiteration = 0;
    #endif
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    cur_exc = PyErr_Occurred();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        #ifdef __Pyx_StopAsyncIteration_USED
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, __Pyx_PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else
        #endif
            return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(val);
    Py_XDECREF(tb);
    PyErr_SetString(PyExc_RuntimeError,
        #ifdef __Pyx_StopAsyncIteration_USED
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        #endif
        "generator raised StopIteration");
}

/* py_dict_items */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_items, d);
    else
        return PyDict_Items(d);
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if PY_VERSION_HEX >= 0x030d00A4
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_n_s_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = PyTuple_GET_SIZE(substrings);
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* pyfrozenset_new */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if ((PY_VERSION_HEX >= 0x031000A1) || likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_USE_TYPE_SLOTS
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

/* PySetContains */
static int __Pyx_PySet_ContainsUnhashable(PyObject *set, PyObject *key) {
    int result = -1;
    if (PySet_Check(key) && PyErr_ExceptionMatches(PyExc_TypeError)) {
        PyObject *tmpkey;
        PyErr_Clear();
        tmpkey = __Pyx_PyFrozenSet_New(key);
        if (tmpkey != NULL) {
            result = PySet_Contains(set, tmpkey);
            Py_DECREF(tmpkey);
        }
    }
    return result;
}
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq) {
    int result = PySet_Contains(set, key);
    if (unlikely(result < 0)) {
        result = __Pyx_PySet_ContainsUnhashable(set, key);
    }
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_MACROS
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (n < 0) return -1;
#endif
    for (i = 1; i < n; i++)
    {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        PyTypeObject *b;
#if PY_MAJOR_VERSION < 3
        if (PyClass_Check(b0))
        {
            PyErr_Format(PyExc_TypeError, "base class '%.200s' is an old-style class",
                         PyString_AS_STRING(((PyClassObject*)b0)->cl_name));
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !(CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY)
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !(CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API) || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if PY_VERSION_HEX >= 0x030700a1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400)
        gc = PyImport_GetModule(__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    __Pyx_TypeName type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, attr_name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(attr_name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportDottedModule */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
    if (likely(PyTuple_GET_SIZE(parts_tuple) == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
#if PY_MAJOR_VERSION < 3
        PyExc_ImportError,
        "No module named '%s'", PyString_AS_STRING(partial_name));
#else
#if PY_VERSION_HEX >= 0x030600B1
        PyExc_ModuleNotFoundError,
#else
        PyExc_ImportError,
#endif
        "No module named '%U'", partial_name);
#endif
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
    nparts = PyTuple_GET_SIZE(parts_tuple);
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = PySequence_ITEM(parts_tuple, i);
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
#endif
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if PY_MAJOR_VERSION < 3
    PyObject *module, *from_list, *star = __pyx_n_s__48;
    CYTHON_UNUSED_VAR(parts_tuple);
    from_list = PyList_New(1);
    if (unlikely(!from_list))
        return NULL;
    Py_INCREF(star);
    PyList_SET_ITEM(from_list, 0, star);
    module = __Pyx_Import(name, from_list, 0);
    Py_DECREF(from_list);
    return module;
#else
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
#endif
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030400B1
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_n_s_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_n_s_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchInspect */
static PyObject* __Pyx_patch_inspect(PyObject* module) {
#if defined(__Pyx_Generator_USED) && (!defined(CYTHON_PATCH_INSPECT) || CYTHON_PATCH_INSPECT)
    static int inspect_patched = 0;
    if (unlikely((!inspect_patched) && module)) {
        module = __Pyx_Coroutine_patch_module(
            module, ""
"old_types = getattr(_module.isgenerator, '_cython_generator_types', None)\n"
"if old_types is None or not isinstance(old_types, set):\n"
"    old_types = set()\n"
"    def cy_wrap(orig_func, type=type, cython_generator_types=old_types):\n"
"        def cy_isgenerator(obj): return type(obj) in cython_generator_types or orig_func(obj)\n"
"        cy_isgenerator._cython_generator_types = cython_generator_types\n"
"        return cy_isgenerator\n"
"    _module.isgenerator = cy_wrap(_module.isgenerator)\n"
"old_types.add(_cython_generator_type)\n"
        );
        inspect_patched = 1;
    }
#else
    if ((0)) return __Pyx_Coroutine_patch_module(module, NULL);
#endif
    return module;
}

/* PatchAsyncIO */
static PyObject* __Pyx_patch_asyncio(PyObject* module) {
#if PY_VERSION_HEX < 0x030500B2 &&\
        (defined(__Pyx_Coroutine_USED) || defined(__Pyx_Generator_USED)) &&\
        (!defined(CYTHON_PATCH_ASYNCIO) || CYTHON_PATCH_ASYNCIO)
    PyObject *patch_module = NULL;
    static int asyncio_patched = 0;
    if (unlikely((!asyncio_patched) && module)) {
        PyObject *package;
        package = __Pyx_Import(__pyx_n_s_asyncio_coroutines, NULL, 0);
        if (package) {
            patch_module = __Pyx_Coroutine_patch_module(
                PyObject_GetAttrString(package, "coroutines"), ""
"try:\n"
"    coro_types = _module._COROUTINE_TYPES\n"
"except AttributeError: pass\n"
"else:\n"
"    if _cython_coroutine_type is not None and _cython_coroutine_type not in coro_types:\n"
"        coro_types = tuple(coro_types) + (_cython_coroutine_type,)\n"
"    if _cython_generator_type is not None and _cython_generator_type not in coro_types:\n"
"        coro_types = tuple(coro_types) + (_cython_generator_type,)\n"
"_module._COROUTINE_TYPES = coro_types\n"
            );
        } else {
            PyErr_Clear();
            package = __Pyx_Import(__pyx_n_s_asyncio_tasks, NULL, 0);
            if (unlikely(!package)) goto asyncio_done;
            patch_module = __Pyx_Coroutine_patch_module(
                PyObject_GetAttrString(package, "tasks"), ""
"if hasattr(_module, 'iscoroutine'):\n"
"    old_types = getattr(_module.iscoroutine, '_cython_coroutine_types', None)\n"
"    if old_types is None or not isinstance(old_types, set):\n"
"        old_types = set()\n"
"        def cy_wrap(orig_func, type=type, cython_coroutine_types=old_types):\n"
"            def cy_iscoroutine(obj): return type(obj) in cython_coroutine_types or orig_func(obj)\n"
"            cy_iscoroutine._cython_coroutine_types = cython_coroutine_types\n"
"            return cy_iscoroutine\n"
"        _module.iscoroutine = cy_wrap(_module.iscoroutine)\n"
"    if _cython_coroutine_type is not None:\n"
"        old_types.add(_cython_coroutine_type)\n"
"    if _cython_generator_type is not None:\n"
"        old_types.add(_cython_generator_type)\n"
            );
        }
        Py_DECREF(package);
        if (unlikely(!patch_module)) goto ignore;
asyncio_done:
        PyErr_Clear();
        asyncio_patched = 1;
#ifdef __Pyx_Generator_USED
        {
            PyObject *inspect_module;
            if (patch_module) {
                inspect_module = PyObject_GetAttr(patch_module, __pyx_n_s_inspect);
                Py_DECREF(patch_module);
            } else {
                inspect_module = __Pyx_Import(__pyx_n_s_inspect, NULL, 0);
            }
            if (unlikely(!inspect_module)) goto ignore;
            inspect_module = __Pyx_patch_inspect(inspect_module);
            if (unlikely(!inspect_module)) {
                Py_DECREF(module);
                module = NULL;
            }
            Py_XDECREF(inspect_module);
        }
#else
        if ((0)) return __Pyx_patch_inspect(module);
#endif
    }
    return module;
ignore:
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch asyncio package with custom generator type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    if ((0)) return __Pyx_patch_inspect(__Pyx_Coroutine_patch_module(module, NULL));
#endif
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_kp_u__15);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_MACROS
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
#if PY_VERSION_HEX >= 0x03030000
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
#else
    CYTHON_MAYBE_UNUSED_VAR(qualname);
#endif
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
#if PY_VERSION_HEX < 0x030600A4 && CYTHON_PEP487_INIT_SUBCLASS
static int __Pyx_SetNamesPEP487(PyObject *type_obj) {
    PyTypeObject *type = (PyTypeObject*) type_obj;
    PyObject *names_to_set, *key, *value, *set_name, *tmp;
    Py_ssize_t i = 0;
#if CYTHON_USE_TYPE_SLOTS
    names_to_set = PyDict_Copy(type->tp_dict);
#else
    {
        PyObject *d = PyObject_GetAttr(type_obj, __pyx_n_s_dict);
        names_to_set = NULL;
        if (likely(d)) {
            PyObject *names_to_set = PyDict_New();
            int ret = likely(names_to_set) ? PyDict_Update(names_to_set, d) : -1;
            Py_DECREF(d);
            if (unlikely(ret < 0))
                Py_CLEAR(names_to_set);
        }
    }
#endif
    if (unlikely(names_to_set == NULL))
        goto bad;
    while (PyDict_Next(names_to_set, &i, &key, &value)) {
        set_name = __Pyx_PyObject_LookupSpecialNoError(value, __pyx_n_s_set_name);
        if (unlikely(set_name != NULL)) {
            tmp = __Pyx_PyObject_Call2Args(set_name, type_obj, key);
            Py_DECREF(set_name);
            if (unlikely(tmp == NULL)) {
                __Pyx_TypeName value_type_name =
                    __Pyx_PyType_GetName(Py_TYPE(value));
                __Pyx_TypeName type_name = __Pyx_PyType_GetName(type);
                PyErr_Format(PyExc_RuntimeError,
#if PY_MAJOR_VERSION >= 3
                    "Error calling __set_name__ on '" __Pyx_FMT_TYPENAME "' instance %R " "in '" __Pyx_FMT_TYPENAME "'",
                    value_type_name, key, type_name);
#else
                    "Error calling __set_name__ on '" __Pyx_FMT_TYPENAME "' instance %.100s in '" __Pyx_FMT_TYPENAME "'",
                    value_type_name,
                    PyString_Check(key) ? PyString_AS_STRING(key) : "?",
                    type_name);
#endif
                goto bad;
            } else {
                Py_DECREF(tmp);
            }
        }
        else if (unlikely(PyErr_Occurred())) {
            goto bad;
        }
    }
    Py_DECREF(names_to_set);
    return 0;
bad:
    Py_XDECREF(names_to_set);
    return -1;
}
static PyObject *__Pyx_InitSubclassPEP487(PyObject *type_obj, PyObject *mkw) {
#if CYTHON_USE_TYPE_SLOTS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyTypeObject *type = (PyTypeObject*) type_obj;
    PyObject *mro = type->tp_mro;
    Py_ssize_t i, nbases;
    if (unlikely(!mro)) goto done;
    (void) &__Pyx_GetBuiltinName;
    Py_INCREF(mro);
    nbases = PyTuple_GET_SIZE(mro);
    assert(PyTuple_GET_ITEM(mro, 0) == type_obj);
    for (i = 1; i < nbases-1; i++) {
        PyObject *base, *dict, *meth;
        base = PyTuple_GET_ITEM(mro, i);
        dict = ((PyTypeObject *)base)->tp_dict;
        meth = __Pyx_PyDict_GetItemStrWithError(dict, __pyx_n_s_init_subclass);
        if (unlikely(meth)) {
            descrgetfunc f = Py_TYPE(meth)->tp_descr_get;
            PyObject *res;
            Py_INCREF(meth);
            if (likely(f)) {
                res = f(meth, NULL, type_obj);
                Py_DECREF(meth);
                if (unlikely(!res)) goto bad;
                meth = res;
            }
            res = __Pyx_PyObject_FastCallDict(meth, NULL, 0, mkw);
            Py_DECREF(meth);
            if (unlikely(!res)) goto bad;
            Py_DECREF(res);
            goto done;
        } else if (unlikely(PyErr_Occurred())) {
            goto bad;
        }
    }
done:
    Py_XDECREF(mro);
    return type_obj;
bad:
    Py_XDECREF(mro);
    Py_DECREF(type_obj);
    return NULL;
#else
    PyObject *super_type, *super, *func, *res;
#if CYTHON_COMPILING_IN_PYPY && !defined(PySuper_Type)
    super_type = __Pyx_GetBuiltinName(__pyx_n_s_super);
#else
    super_type = (PyObject*) &PySuper_Type;
    (void) &__Pyx_GetBuiltinName;
#endif
    super = likely(super_type) ? __Pyx_PyObject_Call2Args(super_type, type_obj, type_obj) : NULL;
#if CYTHON_COMPILING_IN_PYPY && !defined(PySuper_Type)
    Py_XDECREF(super_type);
#endif
    if (unlikely(!super)) {
        Py_CLEAR(type_obj);
        goto done;
    }
    func = __Pyx_PyObject_GetAttrStrNoError(super, __pyx_n_s_init_subclass);
    Py_DECREF(super);
    if (likely(!func)) {
        if (unlikely(PyErr_Occurred()))
            Py_CLEAR(type_obj);
        goto done;
    }
    res = __Pyx_PyObject_FastCallDict(func, NULL, 0, mkw);
    Py_DECREF(func);
    if (unlikely(!res))
        Py_CLEAR(type_obj);
    Py_XDECREF(res);
done:
    return type_obj;
#endif
}
#endif
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET,
#if PY_VERSION_HEX < 0x030600A4
        (metaclass == (PyObject*)&PyType_Type) ? NULL : mkw
#else
        mkw
#endif
    );
    Py_XDECREF(owned_metaclass);
#if PY_VERSION_HEX < 0x030600A4 && CYTHON_PEP487_INIT_SUBCLASS
    if (likely(result) && likely(PyType_Check(result))) {
        if (unlikely(__Pyx_SetNamesPEP487(result) < 0)) {
            Py_CLEAR(result);
        } else {
            result = __Pyx_InitSubclassPEP487(result, mkw);
        }
    }
#else
    (void) &__Pyx_GetBuiltinName;
#endif
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name_2);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__31);
    }
    return name;
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CoroutineBase */
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(PyObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body(self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject* gen, PyObject *retval) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval))
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    gen->is_running = 0;
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        Py_TYPE(gen)->tp_del(self);
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (likely(--self->ob_refcnt == 0)) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context)
{
    PyObject *frame = self->gi_frame;
    CYTHON_UNUSED_VAR(context);
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (unlikely(!module)) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) __Pyx_PyObject_GenericGetAttrNoDict},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    __pyx_GeneratorType_slots
};
#else
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if PY_VERSION_HEX >= 0x030d00A4
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_Generator_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_GeneratorType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
#endif
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
